Program Tool;

{$IFDEF LINUX}
{$DEFINE NOACTIVEX}
{$ENDIF}

{$APPTYPE CONSOLE}

Uses
{$IFDEF MSWINDOWS}
  Windows, ComObj,
{$ENDIF}

{$IFDEF NOACTIVEX}
  icewarpservercom,
{$ENDIF}
  StringUnit, SysUtils, toolunit, toolhelp, Variants;

Procedure ParamError;
Begin                  
  Writeln('Parameters error.');
  ExitCode := R_ERROR_PARAMS;
End;

Var
  FParam, FValue: String;

// *** Account Actions ***

Procedure ProcessAccountAction(FMode: TProcessItem; Email: ShortString; ParamIndex: Longint);
Var
  FLUser, FUser: TXAccountObject;
  FDom, FDomain: String;
  FI: Longint;
  FQuery: String;
  TryAgain: Boolean;

Procedure ProcessAccount(Email: ShortString; Paramindex: Longint; Mode: TProcessItem; Open: Boolean);
Var
  FExport: String;
  FUsr: TXAccountObject;
Begin
  If Open Or (Mode = piDelete) Then
    FUsr := FUser
  Else
    FUsr := FLUser;

  If Open Or (Mode = piDelete) Then
    If Not FUsr.Open(Email) Then
    Begin
      Writeln(Format(AccountNotFound, [Email]));
      ExitCode := R_ERROR_API_FAILURE;
      Exit;
    End;

  // Primary email
  Email := FUsr.EmailAddress;

  Case Mode Of
    piDisplay: Writeln(Email, True);
    piValidate:
    Begin
      GetNextParam(ParamIndex, FParam, FValue);
      If FUsr.ValidateData(FParam) Then Exit;
      Writeln(Email, True);
    End;
    piExport: FExport := Email + cComma;
    piDelete:
    Begin
      FUser.Delete;
      Writeln(Format(AccountDeleted, [Email]));
      Exit;
    End;
  End;

  // Parameters
  While GetNextParam(ParamIndex, FParam, FValue, Mode <> piModify) Do
    Try
      Case Mode Of
        piModify: FUsr.SetProperty(FParam, FValue);
        piDisplay, piExport:
        Begin
          If Pos(cAsterisk, FParam) = $00 Then
            FValue := FUsr.GetProperty(FParam)
          Else
            FValue := cCRLF + FUsr.GetProperties(FParam);

          If Mode = piDisplay Then
            Writeln(FParam + cDisplaySep + FValue, True)
          Else
            FExport := FExport + CommaString(FValue) + cComma;
        End;
      End;
    Except
      On E: Exception Do Writeln(E.Message);
    End;

  Case Mode of
    piModify:
    Begin
      If FUsr.Save Then
        Writeln(Format(AccountUpdated, [Email]))
      Else
      Begin
        Writeln(Format(AccountProblem, [Email]));
        ExitCode := R_ERROR_API_FAILURE;
      End;
    End;
    piDisplay, piValidate: Writeln('', True);
    piExport: Writeln(FExport, True);
  End;
End;

Procedure ImportUsers(FileName: String);
Var
  ODom: TXDomainObject;
  FDom, LastDom, Email, S: String;
  FC, FP: Longint;
  F: System.Text;
  FExists: Boolean;
Begin
  FP := ParamIndex;

  // Import file
  If Length(FileName) = $00 Then Exit;

  // Open
  AssignFile(F, FileName);
  {$I-}
  FileMode := fmOpenRead;
  ResetBOM(F);
  {$I+}
  If IOResult = $00 Then
  Begin
    // For each line
    While Not Eof(F) Do
    Begin
      Readln(F, S);
      If Length(S) = $00 Then Continue;
      S := S + cComma;

      // Name
      Email := Copy(S, $01, Pos(cComma, S) - $01);

      // Open or new
      If Not FUser.Open(Email) Then
      Begin
        // Check if domain exists
        FC := Pos(cAt, Email);
        If FC <> $00 Then
        Begin
          // Name
          FDom := Copy(Email, FC + $01, Length(Email) - FC);

          // Check if already checked
          If FDom <> LastDom Then
          Begin
            // Open
            ODom := TXDomainObject(TXVariant(FAPI.OpenDomain(FDom)));
            FExists := IDispatch(ODom) <> Nil;

            // Not exists create
            If Not FExists Then
            Begin
              ODom := TXDomainObject(TXVariant(FAPI.NewDomain(FDom)));
              ODom.ApplyTemplate('');
              If Not ODom.Save Then
              Begin
                Writeln(Format(DomainProblem, [FDom]));
                Continue;
              End;

            End;
          End;

          // Set
          LastDom := FDom;
        End;

        FUser.New(Email);

        // Set proper type
        If FindParam(ParamIndex, 'U_Type', FValue) Then FUser.SetProperty('U_Type', FValue);

        // Apply template
        FUser.ApplyTemplate(Template);
      End;

      // Properties
      ParamIndex := FP;
      While GetNextParam(ParamIndex, FParam, FValue, True) Do
      Try
        FValue := GetCommaString(S, ParamIndex - FP);
        FUser.SetProperty(FParam, FValue);
      Except
        On E: Exception Do Writeln(E.Message);
      End;

      // Save
      If FUser.Save Then
        Writeln(Format(AccountCreated, [Email]))
      Else
        Writeln(Format(AccountProblem, [Email]));
    End;

    CloseFile(F);
  End;
End;

Begin
  // Default user instance
  FUser := TXAccountObject(TXVariant(CreateObject('IceWarpServer.AccountObject')));
  FUser.TokenHandle := TokenHandle;

  If FMode = piCreate Then
  Begin
    // Already exists
    If FUser.Open(Email) Then
    Begin
      Writeln(Format(AccountExists, [Email]));
      ExitCode := R_ERROR_API_FAILURE;
      Exit;
    End;

    // Create user
    FUser.New(Email);

    // Set proper type
    If FindParam(ParamIndex, 'U_Type', FValue) Then FUser.SetProperty('U_Type', FValue);

    // Apply template
    FUser.ApplyTemplate(Template);

    While GetNextParam(ParamIndex, FParam, FValue) Do
    Try
      FUser.SetProperty(FParam, FValue);
    Except
      On E: Exception Do Writeln(E.Message);
    End;

    If FUser.Save Then
      Writeln(Format(AccountCreated, [Email]))
    Else
    Begin
      Writeln(Format(AccountProblem, [Email]));
      ExitCode := R_ERROR_API_FAILURE;
    End;
  End
  Else

  If (FMode = piModify) Or (FMode = piDisplay) Or (FMode = piValidate) Or (FMode = piExport) Or (FMode = piDelete) Then
  Begin
    // If not any mask then modify one single domain
    If Not IsStringMask(Email) Then ProcessAccount(Email, ParamIndex, FMode, True)

    // Else check each domain
    Else
    Begin
      // Parse domain and alias
      FDom := '';
      FI := Pos(cAT, Email);
      If FI > $00 Then
      Begin
        FDom := Copy(Email, FI + $01, Length(Email) - FI);
        Email := Copy(Email, $01, FI - $01);
      End;

      // Prepare query
      FQuery := Email;
      FQuery := StringReplace(FQuery, cAsterisk, cPercent, [rfReplaceAll]);
      FQuery := StringReplace(FQuery, cAsk, cDownLine, [rfReplaceAll]);
      FQuery := Format(cUALIASLIKE, [FQuery]);
      If Length(Filter) > $00 Then FQuery := FQuery + ' AND (' + Filter + ')';

      // New User instance
      FLUser := TXAccountObject(TXVariant(CreateObject('IceWarpServer.AccountObject')));
      FLUser.TokenHandle := TokenHandle;

      // Get every domain
      For FI := $01 To FAPI.GetDomainCount Do
      Begin
        // Get the domain
        FDomain := FDom;
        If (FDom = cAsterisk) Or (FDom = '') Then FDomain := FAPI.GetDomain(FI - $01);

        While True Do
        Begin
          TryAgain := False;

          // For every user meeting the search criteria
          If FLUser.FindInitQuery(FDomain, FQuery) Then
          Begin
            While FLUser.FindNext Do
            Begin
              // Email
              Email := FLUser.EmailAddress;

              // Free iteration if Delete
              TryAgain := FMode = piDelete;
              If TryAgain Then FLUser.FindDone;

              // Process
              ProcessAccount(Email, ParamIndex, FMode, False);

              // Break if delete
              If TryAgain Then Break;
            End;

            // Do not free if delete
            If Not TryAgain Then FLUser.FindDone;
          End;

          // Try again
          If Not TryAgain Then Break;
        End;

        // Continue if all
        If FDom <> cAsterisk Then Break;
      End;

      Unset(TXObject(FLUser));
    End;
  End
  Else

  If FMode = piImport Then
  Begin
    ImportUsers(Email);
  End;


  Unset(TXObject(FUser));
End;

// *** Remote Account Actions ***

Procedure ProcessRemoteAccountAction(FMode: TProcessItem; ID: ShortString; ParamIndex: Longint);
Var
  FRemote: TXRemoteAccountObject;
  FExport, FDomain: String;
  FI: Longint;

Procedure ImportRemoteAccounts(FileName: String);
Var
  ID, S: String;
  FP: Longint;
  F: System.Text;
Begin
  FP := ParamIndex;

  // Import file
  If Length(FileName) = $00 Then Exit;

  // Open
  AssignFile(F, FileName);
  {$I-}
  FileMode := fmOpenRead;
  ResetBOM(F);
  {$I+}
  If IOResult = $00 Then
  Begin
    // For each line
    While Not Eof(F) Do
    Begin
      Readln(F, S);
      If Length(S) = $00 Then Continue;

      // Name
      ID := Copy(S, $01, Pos(cComma, S) - $01);

      // Create
      FRemote.New;
      FRemote.ApplyTemplate(Template);

      // Properties
      ParamIndex := FP;
      While GetNextParam(ParamIndex, FParam, FValue, True) Do
      Try
        FValue := GetCommaString(S, ParamIndex - FP);
        FRemote.SetProperty(FParam, FValue);
      Except
        On E: Exception Do Writeln(E.Message);
      End;

      // Save
      FRemote.Save;
    End;

    CloseFile(F);
  End;
End;

Procedure ProcessRemoteAccount(ID: ShortString; Paramindex: Longint; Mode: TProcessItem; Open: Boolean);
Begin
  If Open Or (Mode = piDelete) Then
    If Not FRemote.Open(StrToIntDef(ID, $00)) Then
    Begin
      Writeln(Format(AccountNotFound, [ID]));
      ExitCode := R_ERROR_API_FAILURE;
      Exit;
    End;

  Case Mode Of
    piDisplay: Writeln(ID, True);
    piExport: FExport := ID + cComma;
    piDelete:
    Begin
      FRemote.Delete;
      Writeln(Format(AccountDeleted, [ID]));
      Exit;
    End;
  End;

  // Parameters
  While GetNextParam(ParamIndex, FParam, FValue, Mode <> piModify) Do
    Try
      Case Mode Of
        piModify: FRemote.SetProperty(FParam, FValue);
        piDisplay, piExport:
        Begin
          If Pos(cAsterisk, FParam) = $00 Then
            FValue := FRemote.GetProperty(FParam)
          Else
            FValue := cCRLF + FRemote.GetProperties(FParam);

          If Mode = piDisplay Then
            Writeln(FParam + cDisplaySep + FValue, True)
          Else
            FExport := FExport + CommaString(FValue) + cComma;
        End;
      End;
    Except
      On E: Exception Do Writeln(E.Message);
    End;

  Case Mode of
    piModify:
    Begin
      If FRemote.Save Then
        Writeln(Format(AccountUpdated, [ID]))
      Else
      Begin
        Writeln(Format(AccountProblem, [ID]));
        ExitCode := R_ERROR_API_FAILURE;
      End;
    End;
    piDisplay: Writeln('', True);
    piExport: Writeln(FExport, True);
  End;
End;

Begin
  // Default user instance
  FRemote := TXRemoteAccountObject(TXVariant(CreateObject('IceWarpServer.RemoteAccountObject')));
  FRemote.TokenHandle := TokenHandle;

  // Parse domain and id
  FDomain := cAsterisk;
  FI := Pos(cAT, ID);
  If FI > $00 Then
  Begin
    FDomain := Copy(ID, FI + $01, Length(ID) - FI);
    ID := Copy(ID, $01, FI - $01);
  End;

  If FMode = piCreate Then
  Begin
    // Create new
    FRemote.New;
    FRemote.ApplyTemplate(Template);
    While GetNextParam(ParamIndex, FParam, FValue) Do
    Try
      FRemote.SetProperty(FParam, FValue);
    Except
      On E: Exception Do Writeln(E.Message);
    End;

    // Set new ID
    ID := IntToStr(FRemote.Count + $01);

    If FRemote.Save Then
      Writeln(Format(AccountCreated, [ID]))
    Else
    Begin
      Writeln(Format(AccountProblem, [ID]));
      ExitCode := R_ERROR_API_FAILURE;
    End;
  End
  Else

  If (FMode = piModify) Or (FMode = piDisplay) Or (FMode = piDelete) Or (FMode = piExport) Then
  Begin
    // If not any mask then modify one single account
    If Not IsStringMask(ID) Then ProcessRemoteAccount(ID, ParamIndex, FMode, True)

    // Else check each account
    Else
    Begin
      // Get every remote
      For FI := FRemote.Count DownTo $01 Do
      Begin
        // Open
        FRemote.Open(FI - $01);

        // Check if strings match then modify
        If MatchStrings(FRemote.GetProperty('RA_DomainString'), FDomain) And MatchStrings(IntToStr(FI - $01), ID) Then ProcessRemoteAccount(IntToStr(FI - $01), ParamIndex, FMode, False);
      End;
    End;
  End
  Else
  If FMode = piImport Then
  Begin
    ImportRemoteAccounts(ID);
  End;

  Unset(TXObject(FRemote));
End;

// *** Domain Actions ***

Procedure ProcessDomainAction(FMode: TProcessItem; Domain: ShortString; ParamIndex: Longint);
Var
  FDomain: TXDomainObject;
  FDom: String;
  FI: Longint;

Procedure ProcessDomain(Domain: ShortString; Paramindex: Longint; Mode: TProcessItem);
Var
  FExport: String;
  FExists: Boolean;
Begin
  FDomain := TXDomainObject(TXVariant(FAPI.OpenDomain(Domain)));
  FExists := IDispatch(FDomain) <> Nil;

  // Check if exists
  If Not FExists Then
  Begin
    Writeln(Format(DomainNotFound, [Domain]));
    ExitCode := R_ERROR_API_FAILURE;
    Exit;
  End;

  Case Mode Of
    piDisplay: Writeln(Domain, True);
    piExport: FExport := Domain + cComma;
    piDelete:
    Begin
      If FAPI.DeleteDomain(Domain) Then Writeln(Format(DomainDeleted, [Domain]));
      Exit;
    End;
  End;

  // Parameters
  While GetNextParam(ParamIndex, FParam, FValue, Mode <> piModify) Do
  Try
    Case Mode Of
      piModify: FDomain.SetProperty(FParam, FValue);
      piDisplay, piExport:
      Begin
        If Pos(cAsterisk, FParam) = $00 Then
          FValue := FDomain.GetProperty(FParam)
        Else
          FValue := cCRLF + FDomain.GetProperties(FParam);

        If Mode = piDisplay Then
          Writeln(FParam + cDisplaySep + FValue, True)
        Else
          FExport := FExport + CommaString(FValue) + cComma;
      End;
    End;
  Except
    On E: Exception Do Writeln(E.Message);
  End;

  Case Mode Of
    piModify:
    Begin
      If FDomain.Save Then
        Writeln(Format(DomainUpdated, [Domain]))
      Else
      Begin
        Writeln(Format(DomainProblem, [Domain]));
        ExitCode := R_ERROR_API_FAILURE;
      End;
    End;
    piDisplay: Writeln('', True);
    piExport: Writeln(FExport, True);
  End;
End;

Procedure ImportDomains(FileName: String);
Var
  Domain, S: String;
  FP: Longint;
  F: System.Text;
Begin
  FP := ParamIndex;

  // Import file
  If Length(FileName) = $00 Then Exit;

  // Open
  AssignFile(F, FileName);
  {$I-}
  FileMode := fmOpenRead;
  ResetBOM(F);
  {$I+}
  If IOResult = $00 Then
  Begin
    // For each line
    While Not Eof(F) Do
    Begin
      Readln(F, S);
      If Length(S) = $00 Then Continue;

      // Name
      Domain := StrIndex(S,0,cComma);

      // Open or create
      FDomain := TXDomainObject(TXVariant(FAPI.OpenDomain(Domain)));
      If IDispatch(FDomain) = Nil Then
      Begin
        FDomain := TXDomainObject(TXVariant(FAPI.NewDomain(Domain)));
        FDomain.ApplyTemplate(Template);
      End;

      // Properties
      ParamIndex := FP;
      While GetNextParam(ParamIndex, FParam, FValue, True) Do
      Try
        FValue := GetCommaString(S, ParamIndex - FP);
        FDomain.SetProperty(FParam, FValue);
      Except
        On E: Exception Do Writeln(E.Message);
      End;

      // Save
      FDomain.Save;

      Writeln(Format(DomainCreated, [Domain]));
    End;

    CloseFile(F);
  End;
End;

Var
  FExists: Boolean;
Begin
  If FMode = piCreate Then
  Begin
    // Already exists
    FDomain := TXDomainObject(TXVariant(FAPI.OpenDomain(Domain)));
    FExists := IDispatch(FDomain) <> Nil;
    Unset(TXObject(FDomain));
    If FExists Then
    Begin
      Writeln(Format(DomainExists, [Domain]));
      ExitCode := R_ERROR_API_FAILURE;
      Exit;
    End;

    // Create domain
    FDomain := TXDomainObject(TXVariant(FAPI.NewDomain(Domain)));
    FDomain.ApplyTemplate(Template);
    While GetNextParam(ParamIndex, FParam, FValue) Do
    Try
      FDomain.SetProperty(FParam, FValue);
    Except
      On E: Exception Do Writeln(E.Message);
    End;
    If FDomain.Save Then
      Writeln(Format(DomainCreated, [Domain]))
    Else
    Begin
      Writeln(Format(DomainProblem, [Domain]));
      ExitCode := R_ERROR_API_FAILURE;
    End;
  End
  Else

  If (FMode = piModify) Or (FMode = piDisplay) Or (FMode = piExport) Or (FMode = piDelete) Then
  Begin
    // If not any mask then modify one single domain
    If Not IsStringMask(Domain) Then ProcessDomain(Domain, ParamIndex, FMode)

    // Else check each domain
    Else
      // Get every domain
      For FI := FAPI.GetDomainCount DownTo $01 Do
      Begin
        FDom := FAPI.GetDomain(FI - $01);

        // Check if strings match then modify
        If MatchStrings(FDom, Domain) Then ProcessDomain(FDom, ParamIndex, FMode);
      End;
  End

  Else
  If FMode = piImport Then
  Begin
    // Import domains
    ImportDomains(Domain);
  End;
End;

// *** System Actions ***

Procedure ProcessSystemAction(FMode: TProcessItem; ParamIndex: Longint);
Begin
  If FMode = piModify Then
  Begin
      While GetNextParam(ParamIndex, FParam, FValue) Do
      Try
        FAPI.SetProperty(FParam, FValue);
      Except
        On E: Exception Do Writeln(E.Message);
      End;
    If FAPI.Save Then
      Writeln(SettingsUpdated)
    Else
    Begin
      Writeln(SettingsProblem);
      ExitCode := R_ERROR_API_FAILURE;
    End;
  End
  Else

  If FMode = piDisplay Then
  Begin
    While GetNextParam(ParamIndex, FParam, FValue, True) Do
    Try
      If Pos(cAsterisk, FParam) = $00 Then
        FValue := FAPI.GetProperty(FParam)
      Else
        FValue := cCRLF + FAPI.GetProperties(FParam);
        
      Writeln(FParam + cDisplaySep + FValue, True);
    Except
      On E: Exception Do Writeln(E.Message);
    End;
  End;
End;

// *** Service Actions ***

Procedure ProcessServiceAction(FMode: TProcessItem; Service: ShortString; ParamIndex: Longint);
Var
  FService: TXServiceObject;
Begin
  If FMode = piDisplay Then
  Begin
    FService := TXServiceObject(TXVariant(CreateObject('IceWarpServer.StatisticsObject')));
    FService.TokenHandle := TokenHandle;
    FService.Poll(Service);

    While GetNextParam(ParamIndex, FParam, FValue, True) Do
    Try
      If Pos(cAsterisk, FParam) = $00 Then
        FValue := FService.GetProperty(FParam)
      Else
        FValue := cCRLF + FService.GetProperties(FParam);
        
      Writeln(FParam + cDisplaySep + FValue, True);
    Except
      On E: Exception Do Writeln(E.Message);
    End;
  End;
End;

// *** Tables Actions ***

Procedure ProcessTables(FMode: TProcessItem; Service: Longint; ConnectionString: ShortString);
Begin
  If FMode = piCreate Then
  Begin
    If FAPI.CreateTables(Service, ConnectionString) Then
      Writeln(SettingsUpdated)
    Else
    Begin
      Writeln(SettingsProblem);
      ExitCode := R_ERROR_API_FAILURE;
    End;
  End
  Else

  If FMode = piDelete Then
  Begin
    If FAPI.DropTables(Service) Then
      Writeln(SettingsUpdated)
    Else
    Begin
      Writeln(SettingsProblem);
      ExitCode := R_ERROR_API_FAILURE;
    End;
  End;
End;

// *** Backup Actions ***

Procedure ProcessBackupAction(FMode: TProcessItem; FileName: ShortString; Param: Longint);
Begin
  If (FMode = piCreate) Or (FMode = piExport) Then
  Begin
    If FAPI.BackupConfig(FileName) Then
      Writeln(Format(BackupFinished, [FileName]))
    Else
      Writeln(BackupProblem);
  End
  Else

  If FMode = piImport Then
  Begin
    If FAPI.RestoreConfig(FileName, ParamStr(Param)) Then
      Writeln(Format(RestoreFinished, [FileName]))
    Else
      Writeln(RestoreProblem);
  End;
End;

// *** Storage Actions ***

Procedure ProcessStorageAction(FMode: TProcessItem; ToDB: ShortString; Param: Longint);
Begin
  If FMode = piModify Then        
  Begin
    If FAPI.ConvertStorage(ToDB = IntToStr(Longint(True))) Then
    Begin
      Writeln(StorageOk);
      FAPI.UpdateCOnfiguration;
    End
    Else
      Writeln(StorageFailed);
  End;
End;

Procedure ToolRun; Forward;

// *** File Actions ***

Procedure ProcessBatch(BatchFile, Prefix: Shortstring);
Var
  F: System.Text;
  S: String;
Begin
  If Length(BatchFile) = $00 Then Exit;

  AssignFile(F, BatchFile);
  {$I-}
  ResetBOM(F);
  {$I+}
  If IOResult = $00 Then
  Begin
    While Not Eof(F) Do
    Begin
      Readln(F, S);

      // Section title
      If Pos(cSolidL, S) = $01 Then Continue;
      // Comment
      If Pos(cComment, S) = $01 Then Continue;
      // Blank
      If Length(S) = $00 Then Continue;

      If ParseBatchLine(Prefix + S) Then ToolRun;
    End;

    CloseFile(F);
  End;
End;

Procedure ProcessFiles(SubParam: ShortString; Param: ShortString);
Begin
  // Batch
  If SubParam = CBATCH Then ProcessBatch(Param, ParamStr(ParamOptions + $04));
End;

Procedure ProcessUpgrades(Previous: ShortString);
Begin
  // Call upgrade functions
  FAPI.ProcessUpgradeTasks(Previous);
End;

Procedure ProcessSearch(Str: ShortString);
Begin
  Writeln(FAPI.LocateString(Str));
End;

Procedure ToolRun;
Var
  SubParam: ShortString;
  Command: TProcessItem;
Begin
  Command := GetProcessMode(UpperCase(ParamStr(ParamOptions + $01)));
  SubParam := UpperCase(ParamStr(ParamOptions + $02));

  // Search functions
  If Command = piSearch Then ProcessSearch(SubParam) Else

  // Upgrade functions
  If Command = piUpgrade Then ProcessUpgrades(SubParam) Else

  // Account functions
  If SubParam = CACCOUNT Then ProcessAccountAction(Command, ParamStr(ParamOptions + $03), $04) Else

  // Remote account functions
  If SubParam = CREMOTEACCOUNT Then ProcessRemoteAccountAction(Command, ParamStr(ParamOptions + $03), $04) Else

  // Domain functions
  If SubParam = CDOMAIN Then ProcessDomainAction(Command, ParamStr(ParamOptions + $03), $04) Else

  // Tables functions
  If SubParam = CTABLES Then ProcessTables(Command, StrToInt(ParamStr(ParamOptions + $03)), ParamStr(ParamOptions + $04)) Else

  // System functions
  If SubParam = CSYSTEM Then ProcessSystemAction(Command, $03) Else

  // User statistics
  If SubParam = CUSERSTATISTICS Then Writeln(FAPI.GetUserStatistics(ParamStr(ParamOptions + $03), ParamStr(ParamOptions + $04), ParamStr(ParamOptions + $05))) Else

  // Service functions
  If SubParam = CSERVICE Then ProcessServiceAction(Command, ParamStr(ParamOptions + $03), $04) Else

  // Service functions
  If SubParam = CBACKUP Then ProcessBackupAction(Command, ParamStr(ParamOptions + $03), $04) Else

  // Storage functions
  If SubParam = CSTORAGE Then ProcessStorageAction(Command, ParamStr(ParamOptions + $03), $04) Else

  // File functions
  If Command = piFile Then ProcessFiles(SubParam, ParamStr(ParamOptions + $03)) Else

  // Error
  ParamError;
End;

Procedure ProcessOptions;
Const
  ParamChars = [cSlash, cDash];
Var
  FI: Longint;
  Param: String;

Procedure ProcessSwitch(Switch: String; Var Index: Longint);
Var
  Param, Value: String;
  FE: Longint;

Function GetValue: String;
Begin
  Result := Value;
  If Length(Result) = $00 Then
  Begin
    Inc(Index);
    Result := ParamStr(Index);
  End;
End;

Begin
  // Parase parm and value
  Param := Switch;
  Value := '';
  FE := Pos(cEqual, Switch);
  If FE <> $00 Then
  Begin
    Value := Copy(Switch, FE + $01, Length(Switch) - FE);
    Param := Copy(Switch, $01, FE - $01);
  End;
  Param := UpperCase(Param);

  // Options
  If (Param = CQUIET) Or (Param = CQ) Then QuietMode := True Else
  If (Param = CRPC) Or (Param = CREMOTE) Or (Param = CR) Then CreateRPC(GetValue) Else
  If (Param = CPATH) Or (Param = CP) Then InitPath := GetValue Else
  If (Param = CFILTER) Or (Param = CF)  Then Filter := GetValue Else
  If (Param = CTEMPLATE) Or (Param = CM) Then Template := GetValue Else
  If (Param = CH) Or (Param = CHELP) Or (Param = CASK) Then Begin Help(ParamStr(FI + $01)); Halt($00); End Else
  If (Param = CT) Or (Param = CTUTORIAL) Then Help('tutorial') Else
  If (Param = CV) Or (Param = CVERSION) Then Help('version', False) Else
  ;
End;

Begin
  // Parse options parameters
  FI := $00;
  While True Do
  Begin
    Inc(FI);
    Param := ParamStr(FI);

    // Is option
    If (Length(Param) >= $02) And (Param[$01] In ParamChars) Then
    Begin
      // Parse param
      Delete(Param, $01, $01);
      If Param[$01] In ParamChars Then Delete(Param, $01, $01);

      // Match parameters
      ProcessSwitch(Param, FI);
      ParamOptions := FI;
    End
    Else
      Break;
  End;

  // Help
  If ParamCount = $00 Then
  Begin
    Help;
    Halt($00);
  End;

  If ParamOptions = ParamCount Then Halt($00);

  // Help
  If ParamCount <= ParamOptions + $01 Then
  Begin
    Help(ParamStr(ParamOptions + $01));
    Halt($00);
  End;
End;

Begin
  ParseParameters;
  ProcessOptions;

  APIInit;

  ToolRun;

  APIDone;
End.
