Unit toolunit;

Interface

{$IFDEF LINUX}
{$DEFINE NOACTIVEX}
{$ENDIF}

Uses
  SysUtils,
{$IFDEF NOACTIVEX}
  icewarpservercom,
{$ENDIF}
  SystemUnit, StringUnit, MimeUnit;

Const
  cComment = ';';
  cProto = '://';
  cDisplaySep = ': ';

  R_ERROR_API_INITIALIZATION = 1;
  R_ERROR_PARAMS = 2;
  R_ERROR_API_FAILURE = 3;

Type
  TProcessItem = (piModify, piDisplay, piExport, piDelete, piImport, piSearch, piFile, piUpgrade, piCreate, piValidate);

{$IFNDEF NOACTIVEX}
  TXAPIObject = OleVariant;
  TXDomainObject = OleVariant;
  TXRemoteAccountObject = OleVariant;
  TXAccountObject = OleVariant;
  TXTokenObject = OleVariant;
  TXServiceObject = OleVariant;

  TXObject = OleVariant;
  TXVariant = OleVariant;

{$ELSE}
  TXAPIObject = TAPIObject;
  TXDomainObject = TDomainObject;
  TXRemoteAccountObject = TRemoteAccountObject;
  TXAccountObject = TAccountObject;
  TXTokenObject = TTokenObject;
  TXServiceObject = TStatisticsObject;

  TXObject = TObject;
  TXVariant = Longint;

  IDispatch = Pointer;

Const
  Unassigned = Nil;
{$ENDIF}

Function CreateObject(Obj: String): OleVariant;

Procedure Unset(Var Obj: TXObject);

Procedure APIInit;

Procedure APIDone;

Procedure CreateRPC(RPC: String);

Function GetCommaString(S: String; Index: Longint): String;

Function GetNextParam(Var Index: Longint; Var FParam, FValue: String; NoValue: Boolean = False): Boolean;

Function FindParam(Var Index: Longint; const Param: String; Var Value: String): Boolean;

Function GetProcessMode(Command: String): TProcessItem;

Function CommaString(Str: String): String;

Procedure ParseParameters;

Function ParamStr(Index: Longint): String;

Function ParamCount: Longint;

Function ParseBatchLine(Line: String): Boolean;

Procedure Writeln(Str: String = ''; Force: Boolean = False);

Function IsStringMask(const S: String): Boolean;

Var
  FAPI: TXAPIObject;
  FToken: TXTokenObject;
  TokenHandle: Longint = $00;
  QuietMode: Boolean = False;
  ParamOptions: Longint = $00;
  InitPath: String = '';
  Filter: String = '';
  Template: String = '';

Const
  // Commmands
  CCREATE = 'CREATE';
  CMODIFY = 'MODIFY';
  CSET = 'SET';
  CNEW = 'NEW';
  CADD = 'ADD';
  CDELETE = 'DELETE';
  CDISPLAY = 'DISPLAY';
  CLIST = 'LIST';
  CGET = 'GET';
  CFILE = 'FILE';
  CEXPORT = 'EXPORT';
  CIMPORT = 'IMPORT';
  CUPGRADE = 'UPGRADE';
  CSEARCH = 'SEARCH';
  CCHECK = 'CHECK';
  CVALIDATE = 'VALIDATE';

  // Parameters
  CACCOUNT = 'ACCOUNT';
  CREMOTEACCOUNT = 'REMOTEACCOUNT';
  CDOMAIN = 'DOMAIN';
  CTABLES = 'TABLES';
  CSYSTEM = 'SYSTEM';
  CUSERSTATISTICS = 'USERSTATISTICS';
  CBATCH = 'BATCH';
  CSERVICE = 'SERVICE';
  CBACKUP = 'BACKUP';
  CSTORAGE = 'STORAGE';

  // Options
  CQUIET = 'QUIET';
  CQ = 'Q';

  CHELP = 'HELP';
  CH = 'H';

  CRPC = 'RPC';
  CREMOTE = 'REMOTE';
  CR = 'R';

  CPATH = 'PATH';
  CP = 'P';

  CFILTER = 'FILTER';
  CF = 'F';

  CTEMPLATE = 'TEMPLATE';
  CM = 'M';

  CTUTORIAL = 'TUTORIAL';
  CT = 'T';

  CVERSION = 'VERSION';
  CV = 'V';

  CouldNotInitAPI = 'Could not initialize the API';
  CouldNotInitRemoteAPI = 'Could not connect or authenticate to the remote server';

  SettingsUpdated = 'Settings updated.';
  SettingsProblem = 'Settings save problem.';

  BackupFinished = 'Backup to %s finished.';
  BackupProblem = 'Backup problem.';
  RestoreFinished = 'Restore from %s finished.';
  RestoreProblem = 'Restore problem';

  DomainDeleted = 'Domain %s deleted.';
  DomainNotFound = 'Domain %s not found.';
  DomainCreated = 'Domain %s created.';
  DomainUpdated = 'Domain %s updated.';
  DomainExists = 'Domain %s already exists.';
  DomainProblem = 'Domain %s save problem.';

  AccountDeleted = 'Account %s deleted.';
  AccountNotFound = 'Account %s not found.';
  AccountCreated = 'Account %s created.';
  AccountUpdated = 'Account %s updated.';
  AccountExists = 'Account %s already exists.';
  AccountProblem = 'Account %s save problem.';

  StorageOk = 'Storage conversion finished.';
  StorageFailed = 'Storage conversion problem.';


  cUALIAS = 'u_alias';
  cUALIASLIKE = '( (u_alias like ''%0:s'') or (u_alias like ''%0:s;%%'') )';

Type
  TParameters = Array Of String;

Var
  Parameters: TParameters = Nil;

Implementation

Uses
{$IFDEF MSWINDOWS}
  Windows, ActiveX, ComObj,
{$ENDIF}
  Variants;

{$IFDEF NOACTIVEX}
Function CoInitialize(Reserved: Pointer): Longint;
Begin
  Result := $00;
End;

Function CreateObject(Obj: String): OleVariant;
Begin
  Result := Longint(CreateClassObject(Obj));
End;

Procedure Unset(Var Obj: TXObject);
Begin
  Obj.Free;
End;

{$ELSE}
Function CreateObject(Obj: String): OleVariant;
Begin
  CoInitialize(Nil);

  Result := CreateOleObject(Obj);
End;

Procedure Unset(Var Obj: TXObject);
Begin
  Obj := Unassigned;
End;
{$ENDIF}

Procedure APIInit;
Var
  FTokenError: Boolean;
Begin
  // Global API instance
  Try
    FAPI := Unassigned;
    FAPI := TXAPIObject(TXVariant(CreateObject('IceWarpServer.APIObject')));
    FAPI.TokenHandle := TokenHandle;

    // Init path
    if Length(InitPath) > $00 Then FAPI.Init(InitPath);
  Except
  End;

  // Token error
  FTokenError := ((IDispatch(FToken) <> Nil) And (FToken.Counter = $00));

  // Check if instance created and if token set whether it has been acquired
  If (IDispatch(FAPI) = Nil) Or FTokenError Then
  Begin
    If FTokenError Then
      Writeln(CouldNotInitRemoteAPI)
    Else
      Writeln(CouldNotInitAPI);

    APIDone;

    Halt(R_ERROR_API_INITIALIZATION);
  End;
End;

Procedure APIDone;
Begin
  FAPI.Done;
  Unset(TXObject(FAPI));
End;

Procedure CreateRPC(RPC: String);
Var
  URL: String;
Begin
  // No RPC given
  If Length(RPC) = $00 Then
  Begin
    // Get connection info
    Write('Enter the connection information (admin,pass,server): ');
    Readln(RPC);
  End;

  // Get file contents
  If Length(RPC) > $00 Then If Pos(cComma, RPC) = $00 Then RPC := LoadFileToString(RPC);

  // No proto
  If Pos(cProto, RPC) = $00 Then
  Begin
    // URL
    URL := URLEncode(GetCommaString(RPC, $00)) + cColon + URLEncode(GetCommaString(RPC, $01)) + cAt + GetCommaString(RPC, $02);

    // SSL support
    If GetCommaString(RPC, $03) <> '' Then URL := 'https://' + URL;
  End

  // URL
  Else
    URL := RPC;

  Try
    // Create token object
    FToken := Unassigned;
    FToken := TXTokenObject(TXVariant(CreateObject('IceWarpServer.TokenObject')));

    // Set URL
    FToken.URL := URL;

    // Get token handle
    TokenHandle := FToken.TokenHandle;
  Except
  End;
End;

Function GetCommaString(S: String; Index: Longint): String;
Var
  FC, FL, FI: Longint;
Begin
  Result := '';
  FL := $01;
  FC := $00;
  S := S + cComma;

  // For every char
  For FI := $01 To Length(S) Do
    // If comma
    If S[FI] = cComma Then
    Begin
      // Do we have our index
      If Index = FC Then
      Begin
        Result := Copy(S, FL, FI - FL);
        Break;
      End;

      // Inc counters
      Inc(FC);
      FL := FI + $01;
    End;
End;

Function FindParam(Var Index: Longint; const Param: String; Var Value: String): Boolean;
Var
  FIndex: Longint;
  FParam, FValue: String;
Begin
  Result := False;
  FIndex := Index;

  // For each param
  While GetNextParam(Index, FParam, FValue) Do
    // Match
    If LowerCase(FParam) = LowerCase(Param) Then
    Try
      // True
      Result := True;

      // Get value
      Value := FValue;
      
      Break;
    Except
    End;

  Index := FIndex;
End;

Function GetNextParam(Var Index: Longint; Var FParam, FValue: String; NoValue: Boolean = False): Boolean;
Var
  FL: Longint;
Begin
  Result := False;

  FParam := ParamStr(ParamOptions + Index);
  Inc(Index);

  If Not NoValue Then
  Begin
    FL := Pos(cEqual, FParam);
    If FL = $00 Then
    Begin
      FValue := ParamStr(ParamOptions + Index);
      Inc(Index);
    End
    Else
    Begin
      FValue := Copy(FParam, FL + $01, Length(FParam) - FL);
      FParam := Copy(FParam, $01, FL - $01);
    End;
  End;

  If Length(FParam) > $00 Then Result := True;
End;

Function GetProcessMode(Command: String): TProcessItem;
Begin
  Result := piDisplay;
  If (Command = CMODIFY) Or (Command = CSET) Then Result := piModify Else
  If (Command = CDISPLAY) Or (Command = CLIST) Or (Command = CGET) Then Result := piDisplay Else
  If (Command = CCHECK) Or (Command = CVALIDATE) Then Result := piValidate Else
  If (Command = CDELETE) Then Result := piDelete Else
  If (Command = CCREATE) Or (Command = CNEW) Or (Command = CADD) Then Result := piCreate Else
  If (Command = CSEARCH) Then Result := piSearch Else              
  If (Command = CIMPORT) Then Result := piImport Else
  If (Command = CUPGRADE) Then Result := piUpgrade Else
  If (Command = CFILE) Then Result := piFile Else
  If (Command = CEXPORT) Then Result := piExport;
End;

Function CommaString(Str: String): String;
Begin
  Result := Str;

  // Check quot
  If Pos(cSpeech, Result) <> $00 Then Result := StringReplace(Result, cSpeech, cBackSlash + cSpeech, [rfReplaceAll]);

  // Escape commas
  If Pos(cComma, Result) <> $00 Then Result := cSpeech + Result + cSpeech;
End;

Function ParamStr(Index: Longint): String;
Begin
  Result := '';
  If Index < Length(Parameters) Then Result := Parameters[Index];
End;

Function ParamCount: Longint;
Var
  FL: Longint;
Begin
  Result := $00;

  FL := Length(Parameters);
  If FL > $01 Then Result := FL - $01;
End;

Procedure ParseParameters;
Var
  FI: Longint;
Begin
  SetLength(Parameters, System.ParamCount + $01);
  For FI := $01 To Length(Parameters) Do Parameters[FI - $01] := System.ParamStr(FI - $01);
End;

Function ParseBatchLine(Line: String): Boolean;

{$IFNDEF MSWINDOWS}
Function CharNext(P: PChar): PChar;
Begin
  Result := P;
  Inc(Result);
End;
{$ENDIF}

function GetParamStr(P: PChar; var Param: string): PChar;
var
  i, Len: Integer;
  Start, S, Q: PChar;
begin
  while True do
  begin
    while (P[0] <> #0) and (P[0] <= ' ') do
      P := CharNext(P);
    if (P[0] = '"') and (P[1] = '"') then Inc(P, 2) else Break;
  end;
  Len := 0;
  Start := P;
  while P[0] > ' ' do
  begin
    if P[0] = '"' then
    begin
      P := CharNext(P);
      while (P[0] <> #0) and (P[0] <> '"') do
      begin
        Q := CharNext(P);
        Inc(Len, Q - P);
        P := Q;
      end;
      if P[0] <> #0 then
        P := CharNext(P);
    end
    else
    begin
      Q := CharNext(P);
      Inc(Len, Q - P);
      P := Q;
    end;
  end;

  SetLength(Param, Len);

  P := Start;
  S := Pointer(Param);
  i := 0;
  while P[0] > ' ' do
  begin
    if P[0] = '"' then
    begin
      P := CharNext(P);
      while (P[0] <> #0) and (P[0] <> '"') do
      begin
        Q := CharNext(P);
        while P < Q do
        begin
          S[i] := P^;
          Inc(P);
          Inc(i);
        end;
      end;
      if P[0] <> #0 then P := CharNext(P);
    end
    else
    begin
      Q := CharNext(P);
      while P < Q do
      begin
        S[i] := P^;
        Inc(P);
        Inc(i);
      end;
    end;
  end;

  Result := P;
end;

var
  P: PChar;
  FS, S: string;
  FI, FL: Longint;
begin
  Result := False;

  If Length(Line) = $00 Then Exit;

  // We have an equal character
  FI := Pos(cEqual, Line);
  If FI <> $00 Then
  Begin
    FL := Pos('"', Line);

    // Value with the equal sign
    If (FL = $00) Or (FI < FL) Then
    Begin
      // Blank value
      If Length(Line) = FI Then Exit;

      // Replace value
      Line[FI] := ' ';
      Insert('"', Line, FI + $01);
      Line := Line + '"';

      // Parameter
      FS := Copy(Line, $01, FI);
      FS := StringReplace(FS, '.', '_', [rfReplaceAll]);
      Delete(Line, $01, FI);
      Line := FS + Line;
    End;
  End;

  // Command name
  SetLength(Parameters, $01);
  Parameters[$00] := System.ParamStr($00);
  ParamOptions := $00;

  // Reset
  P := PChar(Line);
  FI := $00;

  // Loop
  While True Do
  Begin
    // Get next string
    P := GetParamStr(P, S);
    If Length(S) = $00 Then Break;

    // Count
    Inc(FI);

    // Add new item
    SetLength(Parameters, FI + $01);
    Parameters[FI] := S;
    Result := True;
  End;
End;

Procedure Writeln(Str: String = ''; Force: Boolean = False);
Begin
  If Not QuietMode Or Force Then System.Writeln(Str);
End;

Function IsStringMask(const S: String): Boolean;
Begin
  Result := (Pos(cAsterisk, S) <> $00) Or (Pos(cAsk, S) <> $00);
End;

End.
