<?php
/**
 *  @Class        :ConversionTool
 *  @Brief        :This class is used to convert old webmail contact lists
 *                 stored in files into groupware database
 *  @Author       :Pavel Perna(pavel@icewarp.com)
 *  @Revision     :20070626 23:37    
 *    
 **/

//Tool libraries
require_once ('../inc/conversion/defines.php');
require_once ('../inc/conversion/tools.php');
require_once ('../inc/conversion/item.php');


class ConversionTool
{
  private $aUsers;
  
  private $sPath;
  private $sInstallPath;
  
  
/**
 *  @Method        :Constructor
 *  @Desctiption   :Initializes API objects and retrieves
 *                  superuser username and password  
 **/
  public function __construct(){
    global $oAPI;
    
    //Super user login
    $this->superLogin();
    
    //Useful path
    $this->sInstallPath = $oAPI->getProperty("C_InstallPath");
    
    if($_REQUEST['webmailroot']!=''){
      $this->sPath = $_REQUEST['webmailroot'];
      //Fix path
      $end = substr($this->sPath,-1,1);
      if($end!='/' && $end!='\\'){
        $this->sPath.=FDR_DELIMITTER; 
      }
    }else{
      $this->sPath = $this->sInstallPath.OLD_WM_USERPATH;
    }
    


    
  
  }

/**
 *  @Method        :Constructor
 *  @Desctiption   :Logouts from groupware, ending message 
 **/
  public function __destruct(){
    //Close super user's access
    $this->superLogout();
    
    dmp("<br/><br/>Processing completed .. terminating script");
  }

/**
 *  @Method        :getUsers
 *  @Desctiption   :Process all folder in old webmail config directory
 *                 :and store the userpaths
 **/
  private function getUsers()
  {
    global $oAPI;
    
    $aUsers = array();
    $sFolders = icewarp_get_folder_list($this->sPath,0);
    $aFolders = Tools::explode_j("\r\n",$sFolders);
    
    $bSubFolders = false;  
    
    
    foreach($aFolders as $sFolder){
      $aDomains[$sFolder]=str_replace(array($this->sPath,FDR_DELIMITTER),"",$sFolder);
      $sSubFolders = icewarp_get_folder_list($sFolder,0);
      $aSubFolders = Tools::explode_j("\r\n",$sSubFolders);
  
      //FORMAT : users/{domain}/{user} no user@nerakdemo.com
      if (strpos($sFolder,"@")===false)
        foreach($aSubFolders as $sSubFolder){
          if (strpos($sSubFolder,"@")===false){
            $aUsers[str_replace(array($sFolder,FDR_DELIMITTER),"",$sSubFolder)."@".$aDomains[$sFolder]] = $sSubFolder;
          }
      }
  
    }
    return $aUsers;
  }

/**
 *  @Method        :getOldMailboxes_r
 *  @Desctiption   :Recursively checks for folders with the 
 *                  "(.*)@(.*)\.(.*)" matching name, to be copied
 *                  later to the new user dir.If the address.dat file
 *                  is present in the folder, create contacts for that
 *                  users as well     
 *  @Param  $aOldUsers[out] - very old webmail configuration folder locations
 *  @Param  $path           - path to be processed recursively                
 **/
  private function getOldMailboxes_r(&$aOldUsers,$path = false){
      global $oAPI,$oAccount;
      
      $sMailboxPath = $oAPI->getProperty("C_System_Storage_Dir_MailPath");
      $aOldMailboxes = array();
      
      if ($path === false){
        $aOldUsers = array();
        $path = $this->sPath;
      }
      
      $sFolders = icewarp_get_folder_list($path,0);
      $aFolders = Tools::explode_j("\r\n",$sFolders);

      foreach($aFolders as $sFolder){
        $sUser = str_replace(array($path,FDR_DELIMITTER),"",$sFolder);
        $process = false;

        //Process only {user}@{domain2}.{domain1} folders
        if(eregi("(.*)@(.*)",$sUser,$aMatches)){
          $sUser = $aMatches[1].'@'.$aMatches[2];
          
          
          if (file_exists($sFolder.ADDRESS_FILE) || file_exists($sFolder.GROUPS_FILE))
            $aOldUsers[$sUser] = $sFolder;

          //$aLines = explode("\r\n",@file_get_contents($sFolder.MAILBOX_FILE));
          //$sFromPath = $aLines[0];
          $process = true;
        } else{
            $info = explode(FDR_DELIMITTER,str_replace($this->sPath,'',$sFolder));
            $sUser = $info[1].'@'.$info[0];
            if($info[0] && $info[1] && !$info[2])
              $process = true;
        }
        if($process){
          if($oAccount->Open($sUser)) $userExists = true;
          else $userExists = false;
          //Auto create account
          if (ACCOUNT_AUTO_CREATE){
            if (!isset($oAccount->EmailAddress)){
              $oAccount->New_($sUser);
              $oAccount->SetProperty("u_type",0);
              $oAccount->Save();
            }
          }
          
          //If account exists
          if ($userExists){
            $sTargetPath = $oAccount->getProperty("u_fullmailboxpath");  
            
            
            if ($sTargetPath!=$sMailboxPath){
              $aOldMailboxes[$sFolder]['folder'] = $sTargetPath;
              $aOldMailboxes[$sFolder]['configpath'] = $sFolder;  
            }
          }
        }
        if (is_dir($sFolder))
          $aOldMailboxes = array_merge($aOldMailboxes,$this->getOldMailboxes_r($aOldUsers,$sFolder));
      }

      return $aOldMailboxes;
  }


/**
 *  @Method        :getUserGroupIDs
 *  @Desctiption   :Retrieves existing user owner info and
 *                  main group info
 **/ 
  private function getUserGroupIDs()
  {
    global $oGWAPI;
    $sid = $oGWAPI->sessid; 

    $users = $oGWAPI->GetOwners();
    foreach($users as $user){
  
      $aUsers[$user['OWN_EMAIL']]['DATA'] = $user;
  
      $groups = $oGWAPI->FunctionCall("GetGroupList",$sid,$user['OWN_EMAIL']);
      $groups = $oGWAPI->ParseParamLine($groups);
  
      $aUsers[$user['OWN_EMAIL']]['GROUP'] = $groups[0];
      
    }
    
    return $aUsers;
  }

/**3
 *  @Method         :loadCalendarConfig
 *  @Desctiption    :Loads data from CALENDAR_CFG_FILE to the array
 *  @Param   $sPath :Path where to look for the CALENDAR_CFG_FILE 
 **/ 
  private function loadCalendarCongif($sPath){
    $content = explode("\r\n",@file_get_contents($sPath.CALENDAR_CFG_FILE));
    foreach($content as $line){
      $line = explode("=",$line,2);
      $result[$line[0]] = $line[1];
    }
    
    return $result;
  }

/**
 *  @Method         :getOwnerFromFile
 *  @Desctiption    :Loads owner data from CALENDAR_CFG_FILE
 *  @Param   $sPath :Path where to look for the CALENDAR_CFG_FILE 
 *  @Param   $sUser :Username to be created in GW 
 **/ 
  private function getOwnerFromFile($sUser,$sPath)
  { 
    $aCalendarCfg = $this->loadCalendarCongif($sPath);
    
    //Owner email
    $owner['OWN_EMAIL'] = $sUser;
    
    //Day start & day end
    $owner['OWNDAYSTART'] = $aCalendarCfg['OWNDAYSTART']?$aCalendarCfg['OWNDAYSTART']:GW_DAY_START;
    $owner['OWNDAYEND'] =   $aCalendarCfg['OWNDAYEND']?$aCalendarCfg['OWNDAYEND']:GW_DAY_END;
      
    if ($aCalendarCfg['OWNDATEFORMAT'])
      $owner['OWNDATEFORMAT'] = $aCalendarCfg['OWNDATEFORMAT'];
    
    return $owner;
     
  }

/**
 *  @Method         :getGroupFromFile
 *  @Desctiption    :Loads group data from CALENDAR_CFG_FILE
 *  @Param   $aOwner:Owner data read from CALENDAR_CFG_FILE 
 *  @Param   $sUser :Username to be created in GW 
 **/ 
  private function getGroupFromFile($aOwner,$sPath)
  {
    $aCalendarCfg = $this->loadCalendarCongif($sPath);
    
    //Owner email
    $group['GRPSHAREMODE'] = $aCalendarCfg['GRPSHAREMODE'];
    $group['GRPDAILYEVENTSEMAIL'] = $group['GRPREMINDEREMAIL'] = $aOwner['OWN_EMAIL'];
    
    return $group;
    
    
  }


/**
 *  @Method         :getGroupFromFile
 *  @Desctiption    :Loads group data from CALENDAR_CFG_FILE
 *  @Param   $aOwner:Owner data read from CALENDAR_CFG_FILE 
 *  @Param   $sUsername :Username to be created in GW 
 **/ 
  private function createUserOwner($sUsername,$aOwner)
  {
    global $oGWAPI;
    //Todo : set owner data(API function required)
    return $oGWAPI->FunctionCall("CreateUser",$oGWAPI->sessid,$sUsername);
  }


/**
 *  @Method               :update UserGroup
 *  @Desctiption          :edit group, created while creating new user
 *  @Param   $sUsername   :Username
 *  @Param   $aGroup      :Group Data
 **/ 
private function updateUserGroup($sUsername,$aGroup)
{
  global $oGWAPI;
  
  $sGroups = $oGWAPI->FunctionCall("GetGroupList",$oGWAPI->sessid,$sUsername);
  $aGroups = $oGWAPI->ParseParamLine($sGroups);

  $aExistingGroup = $aGroups[0];
  $sGroupID = $aExistingGroup['GRP_ID'];
  
  foreach($aExistingGroup as $sGroupKey => $sGroupField)
    $aGroup[$sGroupKey] = $sGroupField;

  $sGroup = $oGWAPI->CreateParamLine($aGroup);
  if ($oGWAPI->FunctionCall("AddGroup",$oGWAPI->sessid,$sGroup,$sGroupID))
    return $aGroup;

}

/**
 *  @Method               :autoCreateGroups
 *  @Desctiption          :Create groups for users, that had never been logged in
 *  @Param   $aOld        :List of old users (retrieved by getUsers) 
 *  @Param   $aNew        :List of new users (retrieved by getUsersGroupIDs) 
 **/ 
private function autoCreateGroups($aOld,&$aNew,$sGlobalAddress = false)
{
  global $oAccount;
  
  if ($sGlobalAddress) $aOld[$sGlobalAddress] = '';
  foreach($aOld as $sUsername => $sPath)
  {
    $aNewUser = &$aNew[$sUsername];

    //Create user group and owner
    if (!$aNewUser){
      $aNewOwner = $this->getOwnerFromFile($sUsername,$sPath);
      $aNewGroup = $this->getGroupFromFile($aNewOwner,$sPath);

      if ($oAccount->Open($sUsername)){
        dmp("Auto creating data for user $sUsername in GW database");
        if (!$sNewOwnerID = $this->createUserOwner($sUsername,$aNewOwner))
          $this->errors["OWNER_CREATE"] = $aNewOwner;
        else 
          $aNewOwner['OWN_ID'] = $sNewOwnerID;
          

        dmp(" >>> oConversionTool->createUserOwner($sUsername)");
        dmp(" <<< ".($sNewOwnerID?"1":"0")." ($sNewOwnerID)");
          

        if (!$aNewGroup = $this->updateUserGroup($sUsername,$aNewGroup))
          $this->errror["GROUP_EDIT"] = $aNewGroup;
        else
          $aNewGroup['GRPOWN_ID'] = $sNewOwnerID;

        dmp(" >>> oConversionTool->updateUserGroup($sUsername)");
        dmp(" <<< ".($aNewGroup?"1":"0")." ()");
                
        $aNewUser['DATA'] = $aNewOwner;
        $aNewUser['GROUP'] = $aNewGroup;
      }
    }
  }
  

}

public function processFolder($aOldUsers,$aNewUsers,$sGlobalAddress)
{
  global $oGWAPI; 

  
  //Copy contacts
  foreach($aOldUsers as $sUsername => $sOldWMDir){
    dmp("USER[".(++$i)."](<b>$sUsername</b>)");
    
    //Convert contacts    
    $oConversionItem = new ConversionItem($sUsername,$sOldWMDir);
    $aContacts = $oConversionItem->loadContactsFromFile($iSkipped);
    $aGroups = $oConversionItem->loadGroupsFromFile($iSkipped);

    
    dmp(" >>>> oConversionItem->loadContactsFromFile($sOldWMDir);");
    if (!$aNewUsers[$sUsername]['GROUP']){
      $iSkipped = $iSkipped + count($aContacts);
      unset($aContacts);
    }
    dmp(" <<<< 1 count[".(count($aContacts)+$iSkipped)."] process(".count($aContacts).") ToBeSkipped($iSkipped)");
    
    if(($aContacts || $aGroups) && $aNewUsers[$sUsername]['GROUP']){
      //Open group && folder to work with
      $sGID = $oGWAPI->OpenGroup($aNewUsers[$sUsername]['GROUP']['GRP_ID']);
      $sFID = $oGWAPI->OpenFolder($sGID,"Contacts");

      //Call main conversion method
      $oConversionItem->convert($sFID,$aContacts,$aGroups);
  
      //Close GW access to user folder & group
      $oGWAPI->CloseFolder($sFID);
      $oGWAPI->CloseGroup($sGID); 
      unset($aContacts); 
    }
    unset($oConversionItem);
    
  }
}

/**
 *  @Method               :convert
 *  @Desctiption          :Main function of the conversion tool , process all required
 **/ 
public function convert($sGlobalAddress = false)
{
  global $oGWAPI;
  
  dmp("<b>Contacts conversion</b><br/>===================");
  //Get old users and get new users
  $aOldUsers = $this->getUsers();
  $aNewUsers = $this->getUserGroupIDs();

  dmp(" >>>> this->getOldUsers();");
  dmp(" <<<< 1 count[".count($aOldUsers)."]");
  

  //Retrieve old mailbox paths and very old webmail user address.dat file locations
  $aOldMailboxes = $this->getOldMailboxes_r($aVeryOldUsers);
  dmp(" >>>> this->getVeryOldUsers();");
  dmp(" <<<< 1 count[".count($aVeryOldUsers)."]");  
  
  $this->autoCreateGroups($aOldUsers,$aNewUsers,$sGlobalAddress);
  $this->autoCreateGroups($aVeryOldUsers,$aNewUsers,$sGlobalAddress);
  
  //Process old and very old mailboxes
  $this->processFolder($aOldUsers,$aNewUsers,$sGlobalAddress);
  $this->processFolder($aVeryOldUsers,$aNewUsers,$sGlobalAddress);

  //Copy global address book
  if($sGlobalAddress && file_exists($this->sInstallPath.OLD_WM_CONFIGPATH.ADDRESS_FILE)){

      $oConversionItem = new ConversionItem($sGlobalAddress,$this->sInstallPath.OLD_WM_CONFIGPATH);
      $aGlobalContacts = $oConversionItem->loadContactsFromFile($iSkipped);

      $sGID = $oGWAPI->OpenGroup($aNewUsers[$sGlobalAddress]['GROUP']['GRP_ID']);
      $sFID = $oGWAPI->OpenFolder($sGID,"Contacts");
         
      //Call main conversion method
      $oConversionItem->convert($sFID,$aGlobalContacts,$aNewUsers[$sGlobalAddress]['GROUP']['GRPSHAREMODE']);
      
      $oGWAPI->CloseFolder($sFID);
      $oGWAPI->CloseGroup($sGID);
       
      unset($aContacts); 
      unset($oConversionItem);      
  }

  //Found in folders (.*)@(.*)
  dmp("<br/><br/><b>Mailbox conversion</b><br/>===================");

  if ($aOldMailboxes)
    foreach($aOldMailboxes as $from => $to){
        dmp("USER[".(++$i)."](<b>$sUsername</b>)");

      if (($from!=$to['folder']) && !file_exists($to['configpath'].DONE_FILE2)){
        dmp("Copying mailboxpath <i>$from</i> => <i>".$to['folder']."</i>");
        Tools::dircopy($from,$to['folder']);
        file_put_contents($to['configpath'].DONE_FILE2,"");
      }else 
      dmp("Mailbox path already copied, or same as old");
    }
  
}

/**
 * @Method        :SuperLogin
 * @Desctiption   :Logs the owner of the script as superuser in
 *                order to have rights to open all user groups
 *                and to be able to import the contacts into GW
 **/    
private function superLogin()
{
    global $oAPI,$oGWAPI;
    
    //Super user login
    $oGWAPI->user = $oAPI->getProperty("C_GW_SuperUser");
    $oGWAPI->pass = $oAPI->getProperty("C_GW_SuperPass");
    $oGWAPI->Login();
    
}

/**
 * @Method        :SuperLogout
 * @Desctiption   :Logs the owner out after running a conversion proces
 **/   
private function superLogout()
{
  global $oGWAPI;
  
  $oGWAPI->Logout();
}



}
