<?php
require_once 'account.php';
require_once 'api.php';
require_once 'domain.php';
require_once 'remoteaccount.php';
require_once 'security.php';

//DEBUG
define('DEBUG', 0);

//LOG
define('COMMUNICATION_LOG', 0);

//Version
define('WEBMAIL_DB_VERSION', 1);

//Some folders
define('SETTINGS_FOLDER','~webmail/');
define('SETTINGS_FOLDER_NO_SLASH','~webmail');
define('WM_CONFIGPATH','../../../config/_webmail/');
define('RSA_CONFIGPATH','data/');

//Some files
define('SETTINGS_FILE','settings');
define('COOKIE_FILE','cookies');
define('CERTIFICATE_FILE','cert.pem');

// SESSION PATH
define('sessiondir', './sessions/');
define('loggingdir', './logs/');
define('loggingext', '.log');
define('sessionext', '.ses');
define('sessionexpire', 240); // min

// LOGIN
define('heshexpire', 60);	// sec

// TEMPLATES
define('templateext', '.tpl');

// SETTINGS & STORAGE
define('restriction_suffix','_restriction');
define('storeageext', '.xml');
define('filterext', '.xml');
define('allowdadmin', 1);	 // domain admin can edit global setttings like full admin

// ACCOUNTS
define('accountsfile', 'accounts.xml');

// URL locations
define('DOWNLOAD_PHP','server/download.php');

// Messages
define('messageext', '.msg');
define('localpopmessageext', '.tmp');

//Synchronization mail messages
define('MAX_CHUNK_SIZE',1000);
define('CHUNK_SIZE',1000);

define('UNAVAILABLE_FILE', 'unavailable.tmp');
define('MAX_EXECUTION_TIME', 960);

//Useful constants
define('CR',"\r");
define('LF',"\n");

define('CRLF',"\r\n");
define('UNIX1900',2415019);


// PHP5 FUNCTIONS
/*
@Brief: Logging class
@ Class for handling exceptions , it handles critical and non-critical errors
@Author Pavel Perna
@coding standard added by cenek.svoboda@icewarp.com on 27th of April 2k8 (no change of logic)
*/
class LogError {
	/**
	@Brief : Logs an error to log file
	@param $file - file to log an error
	$param $message an error message to log
	*/
	public static function Error($message,$type = "Warning"){
		if(!COMMUNICATION_LOG){
			return;
		}
		//Get file name
		$date = date("Ymd");
		$file = loggingdir.'c'.$date.loggingext;
		//Gets an error message
		list($msec, $time) = explode(" ", microtime());
		$buffer = 'ERROR    '.date('H:i:s.',$time).substr($msec,2,6)."\r\n".$type.": ".$message."\r\n";
		//Opens log file for appending or creates a new one
		if (!file_exists($file)) {
			$fp = fopen($file,"w+");
		} else {
			$fp = fopen($file,"a+");
		}
		//Write the message
		@fwrite($fp,$buffer);
		//Close the file
		fclose($fp);
		return true;
	}
	public static function CommunicationLog($message){
		if(!COMMUNICATION_LOG){
			return;
		}
		//Get file name
		$date = date("Ymd");
		$file = loggingdir.'c'.$date.loggingext;
		//Gets an error message
		$buffer = $message;
		//Opens log file for appending or creates a new one
		if (!file_exists($file)) {
			$fp = fopen($file,"w+");
		} else {
			$fp = fopen($file,"a+");
		}
		//Write the message
		@fwrite($fp,$buffer);
		//Close the file
		fclose($fp);
		return true;
	}
}
/*
@Brief: Exception class
Class for handling exceptions , it handles critical and non-critical errors
@Author Pavel Perna
*/
class Exc extends Exception {
	/**
	@Brief : constructor
	@params:
	msg /STRING: Description of error
	code /INT: Error code
	*/
	public function __construct($msg = "Default exception caught",$code = '',$showLocation = true,$logError = true){
		$this->WMCode = $code;
		parent::__construct($msg,intval($code));
		$this->wmcode = $code;
		$this->showLocation = $showLocation;
		//Log error
		if ($logError) {
			LogError::Error($this->Message(),"Error"); 
		}
	}
	/**
	@Brief : Generates an error message
	*/
	public function Message()
	{
		$ret = $this->message;
		if ($this->showLocation) {
			$ret.=" in file:".$this->file." on line:".$this->line." found in function <b>".$this->function."</b> and class <b>".$this->class."</b>\r\n";
		}
		return $ret;
	}
	public function XMLResponse()
	{
		global $_iqdata;
		$template['message'] = $this->message;
		if ($this->showLocation) {
			$template['message'].=" in file:".$this->file." on line:".$this->line;
		}
		$template['id'] = $_iqdata['id'];
		$template['eid'] = strval($this->wmcode);
		$template['sid'] = $_iqdata['sid'];
		return trim(template('./inc/templates/xmlrequest_error.tpl',$template));
	}
}
//Var_dump on DOMElement
function dump_r($o)
{
	if ($o->childNodes) {
		for ($i = 0; $i <  $o->childNodes->length; $i++) {
			if (!$o->childNodes->item($i)->tagName && $o->nodeName) {
				$r[$o->childNodes->item($i)->tagName?$o->childNodes->item($i)->tagName:$o->nodeName] = $o->childNodes->item($i)->nodeValue;
			}
			if ($o->childNodes->item($i)->childNodes) {
				$r[$o->nodeName][$i] = dump_r($o->childNodes->item($i));
			}
		}
		return $r;
	} else {
		return $o;
	}
}
function dmp($o)
{
	$r = dump_r($o);
	print_r($r);
}
//Output buffering
function log_buffer(&$buffer, $type, $attributes = array())
{
	if (!COMMUNICATION_LOG) {
		return;
	}
	static $start,$msec,$id,$sid;
	global $_iqdata;
	switch (strtoupper($type)) {
		case 'RESPONSE':
		case 'ERROR':
			list($mesec, $end) = explode(" ", microtime());
			$duration = $end - $start + $mesec - $msec;
			$CommunicationLog='----------------------------------------------------------------------------------------------------------------'."\r\n";
			$CommunicationLog.=strtoupper($type).' '.date('H:i:s.',$end).substr($mesec,2,6).', IQ ID: '.$id.', SID: '.$sid.' Duration: '.$duration."\r\n";
			break;
		case 'REQUEST':
			$id = $attributes['id'];
			$sid = $attributes['sid'];
			list($msec, $start) = explode(" ", microtime());
			if (!$id) {
				$id = rand();
			}
			$CommunicationLog="\r\n=================================================================================================================\r\n";
			$CommunicationLog.='REQUEST  '.date('H:i:s.',$start).substr($msec,2,6).', IQ ID: '.$id.', SID: '.$sid."\r\n";
			break;
	}
	if ($buffer) {
		$CommunicationLog.=$buffer."\r\n";
	}
	$_iqdata = $attributes;
	LogError::CommunicationLog($CommunicationLog);
}
//	 function __autoload($className) {
//		  require_once(dirname($_SERVER['SCRIPT_FILENAME'])."/inc/$className.php");
//	 }
?>
