<?php

  // Debug
  define(isdebug, 0);
  define(debugfile, '/temp/rpc.log');

  function do_log($filename, $body)
  {
    // Open log file
    $file=@fopen($filename,'a');
    if ($file)
    {
      fputs($file,date('Y-m-d H:i:s '));
      fputs($file,$body."\r\n");
      fclose($file);
    }
  }

  // Parses the xml rpc request into an array of calls with methods and params and returns this list
  //
  // $request XML RPC
  //
  // returns array[METHOD]
  //              [PARAMS]
  //

  function rpc_request(&$request) {

    // Parse xml 
    $xml = new SimpleXMLElement($request);

    if ((!$xml) || ($xml->getName() != 'methodCall'))
      exit('Invalid XML request');

    // read method name
    $xml_methodName = $xml->methodName;
    if ($xml_methodName) {
      $result[METHOD] = (string)$xml_methodName;
      $result[PARAMS] = array();
    } else
      exit('Invalid XML request');

    // read parameters
    // array and struct are not supported
    $xml_params = $xml->params;
    if (!$xml_params)
      exit('Invalid XML request');

    $idx = 0;
    foreach ($xml_params->param as $xml_param) {
      $xml_value = $xml_param->value;
      if (!$xml_value)
        exit('Invalid XML request');

      // test for value type
      $xml_value_type = $xml_value->children();
      if ($xml_value_type) {
        switch ($xml_value_type->getName()) {
          case 'array':
          case 'struct':
            continue 2;
        }
        $xml_value = $xml_value_type;
      }

      // save value
      $result[PARAMS][$idx] = (string)$xml_value;
      $idx++;
    }

    // Return
    return $result;
  }

  // Reads the output array and creates the xml rpc response
  //
  // $output array[METHOD]
  //              [PARAMS]

  //
  // returns xml rpc

  function rpc_result(&$output) {

    // Set response
    $result = sprintf('<methodResponse><params><param><value>%s</value></param></params></methodResponse>', 
      htmlspecialchars($output[PARAMS][0])) . "\r\n";
    
    // Return
    return $result;
  }


  // Processes the RPC session
  //
  // $handler is the name of the handler function to be called

  function rpc_process($handler) {

    //Read requested data
    $request = file_get_contents("php://input");

    // Gzip
    $gzip = $_SERVER['HTTP_CONTENT_ENCODING'] == "gzip";
    if ($gzip) $request = gzuncompress($request);

    // Debug request
    if (isdebug) do_log(debugfile, "<<<\r\n" . $request);

    // RPC parse
    $call = rpc_request($request);

    // Call handler
    $output = call_user_func($handler, $call);

    // RPC get result
    $result = rpc_result($output);  

    // Debug response
    if (isdebug) do_log(debugfile, ">>>\r\n" . $result);

    // Gzip
    if ($gzip) {
      header('Content-Encoding: gzip');
      $result = gzcompress($result);
    }

    // Headers
    header('Content-Type: text/html');
    header('Content-Length: ' . strlen($result));

    // Body
    print($result);  
  }
?>
