<?php
//Implementace chovn SyncML synchronizace
//Implementation behavior of SyncML synchronization

include_once 'datastore.php';
include_once 'info.php';
include_once 'xml.php';

$sync_types=array('Two-Way Normal Sync','Two-Way Slow Sync','One-Way Update Sync from Client Only','One-Way Refresh Sync from Client Only','One-Way Update Sync from Server Only','One-Way Refresh Sync from Server Only','One-Way Slow Sync from Client Only','One-Way Slow Sync from Server Only','Reset Client');

//************************************************************
// Pomocn tda pro zapouzen ovovac sti SyncML
// Supplementary class for SyncML creddentials encapsulation
//************************************************************

class syncml_cred
{
  var $_meta_type;
  var $_data;

//************************************************************

  function syncml_cred($xml)
  {
    $this->_meta_type=$xml->get_data('Meta/Type');
    $this->_data=$xml->get_data('Data');
  }

//************************************************************

  function get_type()
  {
    return $this->_meta_type;
  }
  
//************************************************************
  
  function get_data()
  {
    return $this->_data;
  }
}

//************************************************************
// Pomocn tda zapouzdujc SyncML hlaviku
// Supplementary class for SyncML header encapsulation
//************************************************************

class syncml_header
{
  var $_fields;
  var $_cred;

//************************************************************

  function syncml_header($xml)
  {
		$this->_fields['VerDTD']=$xml->get_data('VerDTD');
		$this->_fields['SessionID']=$xml->get_data('SessionID');
    $this->_fields['MsgID']=$xml->get_data('MsgID');
    $this->_fields['Target_LocURI']=$xml->get_data('Target/LocURI');
    $this->_fields['Source_LocURI']=$xml->get_data('Source/LocURI');
    $this->_fields['Source_LocName']=$xml->get_data('Source/LocName');
    $this->_fields['MaxMsgSize']=$xml->get_data('Meta/MaxMsgSize');
    $this->_fields['MaxObjSize']=$xml->get_data('Meta/MaxObjSize');

    if ($xml->get_child('Cred'))
      $this->_cred=new syncml_cred($xml->get_child('Cred'));
  }

//************************************************************

  function get_field($name)
  {
    return $this->_fields[$name];
  }

//************************************************************

  function &get_cred()
  {
    return $this->_cred;
  }
}

//************************************************************
// Hlavn tda realizujc SyncML synchronizaci
// Main class implementing SyncML synchronization
//************************************************************

class syncml_session
{
  var $_datastore;
  var $_info;

	var $_version;
	var $_session_id;
  var $_content_type;
  var $_rcvd_msg_id;
  var $_next_msg_id;
  var $_next_cmd_id;
  var $_target_uri;
  var $_source_uri;
  var $_message_size;
  var $_object_size;
  var $_credentials;

  var $_username;
	var $_auth_type;
	var $_authenticated;
  var $_challenge;
  var $_next_nonce;
  var $_params;

  var $_sync_stage;
  var $_alerts;
  var $_need_get;
	var $_db_names;
	var $_filters;
  var $_syncs;
  var $_more_data;
  var $_status;
  var $_ip;

//************************************************************

  function syncml_session($config_list,$content_type)
  {
		$this->_datastore=new datastore();
		$this->_info=new Info();
		$this->_content_type=$content_type;
		$this->_next_msg_id=1;
    $this->_message_size=$config_list['MaxMsgSize'];
    $this->_object_size=$config_list['MaxObjSize'];
    $this->_credential=array();
    $this->_auth_type=$config_list['AuthType'];
		$this->_authenticated=false;
		$this->_parametres=array();
		$this->_sync_stage=1;
    $this->_alerts=array();
    $this->_need_get=false;
    $this->_filters=array();
    $this->_syncs=array();
    $this->_status=array();
    $this->_more_data=array();
  }

//************************************************************

	function get_version()
	{
		return $this->_version;
	}

//************************************************************
	
	function get_IP()
	{
		return $this->_ip;
	}

//************************************************************

	function get_info()
	{
		$info=$this->_info->get_text();
		$this->_info->clear_text();
		return $info;
	}

//************************************************************

	function get_summary()
	{
		$this->_info->final_summary();
		return $this->_info->get_text();
	}

//************************************************************

  function handle_header($header,$body,&$response_tree)
  {
    //Aktualizace daju o stavu session podle pijat hlaviky
    //Update session variables from requested header
		$this->_version=$header->get_field('VerDTD');
		$this->_session_id=$header->get_field('SessionID');
    $this->_rcvd_msg_id=$header->get_field('MsgID');
    $this->_target_uri=$header->get_field('Target_LocURI');
    $this->_source_uri=$header->get_field('Source_LocURI');
    if ($locname=$header->get_field('Source_LocName'))
      $this->_username=$locname;
    if ($size=$header->get_field('MaxMsgSize'))
      $this->_message_size=$size;
    if ($size=$header->get_field('MaxObjSize'))
      $this->_object_size=$size;
      
    $this->_next_cmd_id=1;

		//Vytvoen spojen na Merak API a na databzi
		//Create connection to Merak API and to database
		$this->_datastore->connect_api();
		$this->_datastore->connect_db();
		
    //Nastaven max. velikosti dat, kter lze odeslat v jedn zprv
    //Set the maximum size of data, we can send to client in one message
    $this->_datastore->set_max_place($this->_message_size*.5);
    
    //Nastaven max. velikosti objektu, kter lze odeslat
    //Set the maximum size of object, we can send to client
    $this->_datastore->set_max_object($this->_object_size);

    //Vytvoen odpovdi na pijatou hlaviku
    //Make response for requested header
    $response=&$response_tree->new_child('Status',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $response->new_child_data('MsgRef',NULL,$this->_rcvd_msg_id);
    $response->new_child_data('CmdRef',NULL,0);
    $response->new_child_data('Cmd',NULL,'SyncHdr');
    $response->new_child_data('TargetRef',NULL,$this->_target_uri);
    $response->new_child_data('SourceRef',NULL,$this->_source_uri);

		//Je session ovena?
    //Is session authenticated?
    if ($this->_authenticated)
		{
			//Oven je OK
      //Authentication is OK
      $response->new_child_data('Data',NULL,200);

			return;
		}

		$this->_info->add_delimiter('-',70);
		$this->_info->add_text('AUTHENTICATION');
      
		//Obsahuje hlavika sekci <CRED>?
    //Has requested header a <CRED> section?
    if ($cred=&$header->get_cred())
    {
			$auth_type=($cred->get_type()=='syncml:auth-md5'?2:1);

      $this->_info->add_text('- Client sent credentials for '.(($auth_type==1)?'Basic':'MD5').' authentication type');
        
			//Client se ovuje pomoc MD5?
			//Does clint authenticate with MD5?
			if ($auth_type==2)
			{
				//Naten next nonce a uvolnn z databze
				//Read next nonce and release from database
				$next_nonce=$this->_datastore->next_nonce_get($this->_username,$this->_source_uri);

			  if (!$this->_next_nonce)
					$this->_next_nonce=$next_nonce;
			}
				
			//Odpovd typ oven klienta poadovanmu?
			//Does client's authentication type correspond to required type?
			if ($this->_auth_type&$auth_type)
			{
				//Klient se ovuje pomoc b64 nebo m MD5 next nonce?
				//Does client authenticate with b64 or have MD5 next nonce?
				if ($auth_type==1 || $this->_next_nonce)
      	{
					$cred_data=$cred->get_data();
        	if ($auth_type==1)
        	{
						//Oven b64
						//b64 authentication
						$cred_data=base64_decode($cred_data);
          	$p=strcspn($cred_data,':');
          	$this->_username=substr($cred_data,0,$p);
          	$password=substr($cred_data,$p+1,strlen($cred_data)-$p-1);
          	$this->_authenticated=$this->_datastore->login($this->_username,$password);
        	}
        	else
					{
        		//Oven MD5
        		//MD5 authentication
          	$this->_authenticated=$this->_datastore->login($this->_username,$cred_data,'SYNCML-MD5',$this->_next_nonce);

            //Vytvoen vzvy k oven klienta typu MD5 v pt synchronizaci
    				//Create challenge for client's MD5 type authentication in next synchronization
    				$chal=&$response->new_child('Chal',NULL);
    				$meta=&$chal->new_child('Meta',NULL);
						$meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),'syncml:auth-md5');
      			$meta->new_child_data('Format',array('xmlns'=>'syncml:metinf'),'b64');
      			$this->_next_nonce=md5('shit',true);
      			$meta->new_child_data('NextNonce',array('xmlns'=>'syncml:metinf'),base64_encode($this->_next_nonce));

            $this->_datastore->next_nonce_set($this->_username,$this->_source_uri,$this->_next_nonce);

						$this->_info->add_text('- Server sends challenge for MD5 authentication in the next synchronization');

					}

					$this->_info->add_text('- Username: '.$this->_username);

					if ($this->_authenticated)
        	{
          	//Pstup povolen
          	//Authorized
          	$response->new_child_data('Data',NULL,212);

            $this->_info->add_text('- Client is successfully authenticated!');
					}
					else
        	{
          	//Pstup odepen
          	//Unauthorized
          	$response->new_child_data('Data',NULL,401);

            $this->_info->add_text('- Bad username or password!');
					}
						
					return;
				}
			}
			else
				$this->_info->add_text('- Server accepts '.(($auth_type==2)?'Basic':'MD5').' authentication type only!');
    }
		else
		{
			$xml_commands=&$body->children();
			foreach($xml_commands as $xml_command)
        if (($xml_command->get_name()=='Status') && ($xml_command->get_data('Cmd')=='SyncHdr'))
      	{
					$chal=&$xml_command->get_child('Chal');
					if ($chal)
					{
					  $type=$chal->get_data('Meta/Type');
						$format=$chal->get_data('Meta/Format');
						$nonce=$chal->get_data('Meta/NextNonce');

						$source=$xml_command->get_data('SourceRef');
						$pos=strpos($source,'?');
						if ($pos)
						{
						  $account=substr($source,$pos+1);

							$start_pos=strpos(strtolower($this->_target_uri),'push=');
							if ($start_pos)
							{
								$end_pos=strpos($this->_target_uri,'&');
								if ($end_pos>$start_pos)
									$device=substr($this->_target_uri,$start_pos+5,$end_pos-$start_pos-5);
								else
                  $device=substr($this->_target_uri,$start_pos+5);
							}
							
							//CALL API
							$password='1234';
do_log($account);
do_log($device);
do_log($nonce);
						  
						  if ($type=='syncml:auth-md5')
							  $data=md5(base64_encode(md5($account.':'.$password,true)).':'.($format=='b64'?base64_decode($nonce):$nonce),true);
						  else
							  $data=$account.':'.$password;
						}
						
						$this->_credentials=array('type'=>$type,'format'=>$format,'data'=>($format=='b64'?base64_encode($data):$data));

            $response->new_child_data('Data',NULL,200);
            
            $alert=&$response_tree->new_child('Alert',NULL);
  					$alert->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
  					$alert->new_child_data('Data',NULL,'206');
  					$item=&$alert->new_child('Item',NULL);
  					$source=&$item->new_child('Source',NULL);
  					$source->new_child_data('LocURI',NULL,'Events');
						return;
					}

					break;
				}
		}
		  
    //Vytvoen vzvy k oven klientova
    //Create challenge for client's authentication
    $chal=&$response->new_child('Chal',NULL);
    $meta=&$chal->new_child('Meta',NULL);

		if ($this->_auth_type>1)
		{
			//Vzva k oven MD5
			//MD5 authentication challenge
			$meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),'syncml:auth-md5');
      $meta->new_child_data('Format',array('xmlns'=>'syncml:metinf'),'b64');
      $this->_next_nonce=md5(uniqid(''),true);
      $this->_datastore->next_nonce_set($this->_username,$this->_source_uri,$this->_next_nonce);
      $meta->new_child_data('NextNonce',array('xmlns'=>'syncml:metinf'),base64_encode($this->_next_nonce));

			$this->_info->add_text('- Server sends challenge for MD5 authentication');
		}
		else
		{
			//Vzva k oven b64
			//b64 authentication challenge
			$meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),'syncml:auth-basic');
      $meta->new_child_data('Format',array('xmlns'=>'syncml:metinf'),'b64');

      $this->_info->add_text('- Server sends challenge for Basic authentication');
		}

		$response->new_child_data('Data',NULL,407);
        
    //Jsme v challenge fzi
    //We're in challenge state
    $this->_challenge=true;
  }

//************************************************************

  function make_response_header(&$header)
  {
    //Vytvoen hlaviky odpovdi
    //Make response header
    $header->new_child_data('VerDTD',NULL,$this->_version);
    $header->new_child_data('VerProto',NULL,'SyncML/'.$this->_version);
    $header->new_child_data('SessionID',NULL,$this->_session_id);
    $header->new_child_data('MsgID',NULL,$this->_next_msg_id++);
    $target=&$header->new_child('Target',NULL);
    $target->new_child_data('LocURI',NULL,$this->_source_uri);
    $source=&$header->new_child('Source',NULL);
    $source->new_child_data('LocURI',NULL,$this->_target_uri);

		$this->_status[$this->_next_msg_id-1]=array();

		//Obsahuje URI poadavku parametry?
		//Does request URI contain parametres?
		$pos=strpos($_SERVER['REQUEST_URI'],'?');
    if ($pos!==false)
    {
			//Oddlen parametr pro dal zprvy v session
			//Parametres separation for next messages in session
			$uri=substr($_SERVER['REQUEST_URI'],0,$pos);
			
			$params=substr($_SERVER['REQUEST_URI'],$pos+1);
			
			//Jsou parametry oddleny  + ? (Funambol m problm s kdovnm standartnho oddlovae &)
			//Are parametres separated by + ? (Funambol has problem with standart separator & encoding)
			$pos=strpos($params,'#');
			if ($pos!==false)
			{
				//Parsovn parametr ve tvaru a=b+c=d...
				//Parse parametres in form a=b+c=d...
				$params_array=explode('#',$params);
				foreach($params_array as $param)
				{
					$param_array=explode('=',$param);
					$_REQUEST[$param_array[0]]=$param_array[1];
				}
			}

			foreach($_REQUEST as $key=>$value)
				$this->_params[strtoupper($key)]=strtolower($value);

			if (isset($this->_params['RESET']))
			  $this->_params['SYNC']=9;
		}
		else
		  $uri=$_SERVER['REQUEST_URI'];

    //Vytvoen URL serveru
    //Create server's URL
    $host=$_SERVER['HTTP_HOST'];
    if ($pos=strpos($host,":"))
      $host=substr($host,0,$pos);
      
    $uri=($_SERVER['HTTPS']=='ON'?'https://':'http://').$host.($_SERVER['SERVER_PORT']==80?'':(':'.$_SERVER['SERVER_PORT'])).$uri;

    $header->new_child_data('RespURI',NULL,$uri.'?sid='.session_id());

		if ($this->_credentials['data'])
		{
		  $cred=&$header->new_child('Cred',NULL);
		  $meta=&$cred->new_child('Meta',NULL);
    	$meta->new_child_data('Format',array('xmlns'=>'syncml:metinf'),$this->_credentials['format']);
    	$meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),$this->_credentials['type']);
    	$cred->new_child_data('Data',NULL,$this->_credentials['data']);

			$this->_credentials=array();
		}
		
		$meta=&$header->new_child('Meta',NULL);
    $meta->new_child_data('MaxMsgSize',array('xmlns'=>'syncml:metinf'),$this->_message_size);
    $meta->new_child_data('MaxObjSize',array('xmlns'=>'syncml:metinf'),$this->_object_size);
  }

//************************************************************

  function handle_commands($request_syncbody,&$response_tree)
  {
    //Pedpokldme, e nae zprva bude posledn
    //We expect, that our message will be last
    $server_final=true;
    $last_message=true;
		
		//Poslal nm klient zprvu s Final pkazem?
    //Did client send us message with Final command?
    $xml_commands=&$request_syncbody->children();
		$client_final=($xml_commands[count($xml_commands)-1]->get_name()=='Final');

		if (($this->_sync_stage==1) && ($this->_challenge))
		{
      //Vzvu k oven poslme pouze jednou
			//We send authentication challenge only once
			$this->_challenge=false;

			$last_message=false;
		}

    //Obsluha pkaz Alert, Get, Map, Put, Results, Status a Sync
    //Handle commands Alert, Get, Map, Put, Status and Sync
    foreach($xml_commands as $xml_command)
      switch($xml_command->get_name())
      {
        case 'Alert':
          $this->handle_command_alert($xml_command,$response_tree);

          //Budeme nco synchronizovat nebo vyzvme klienta k oven?
          //Will we synchronize anything or send authentication challenge to client?
          if (($this->_sync_stage==1) && (count($this->_alerts) || $this->_challenge))
          {
						//Vzvu k oven poslme pouze jednou
						//We send authentication challenge only once
						$this->_challenge=false;

						$last_message=false;
          }
          break;

        case 'Get':
					//Pikaz zpracujeme pozdji
          //We handle command later
          $get=$xml_command;

					break;
					
        case 'Map':
          $this->handle_command_map($xml_command,$response_tree);

					//Neposlal nm klient vechny Map pkazy?
					//Didn't client send us all Map commands?
					if (!$client_final)
            $last_message=false;
            
          break;
          
        case 'Put':
          //Pikaz zpracujeme pozdji
          //We handle command later
          $this->make_put_status($xml_command,$response_tree);
					$put=$xml_command;
					break;
					
				case 'Results':
				  $this->make_put_status($xml_command,$response_tree,true);
				  $this->handle_command_put($xml_command,true);
					break;

				case 'Status':
				  $this->handle_command_status($xml_command);
				  break;
				  
        case 'Sync':
					//Konme incializan fzi?
					//Are we at the end of initialization phase?
					if ($this->_sync_stage==1)
					{
						$this->_datastore->push_response($this->_alerts);
						$this->_sync_stage++;
					}

          $this->handle_command_sync($xml_command,$response_tree);

					//Zmny na serveru poleme v pt zprv
    			//Server's changes we'll send in next message
					$server_final=false;
					$last_message=false;
      }

    //Vynucen zpracovan pkaz v poad Alert, Put a Get
    //Enforcement of processing commands in order Alert, Put and Get
    if ($put)
    {
      $this->handle_command_put($put);
      //$this->_need_get=false;
		}

		if ($get)
		  $this->handle_command_get($get,$response_tree);
		
		//Explicitn vydn DevInfo
		//Explicit request for DevInfo
		if ($this->_need_get)
		{
			$this->make_command_get($response_tree);

      //Vydn DevInfo poslme pouze jednou
			//We send request for DevInfo only once
			$this->_need_get=false;
		}

    //Jsme ve fzi posln zmn ze serveru?
		//Are we in the phase of sending changes from the server?
		if ($this->_sync_stage==3)
		{
			$server_final=$this->make_response_syncs($response_tree);

			if ($server_final)
			  $this->_sync_stage++;
				
			$last_message=false;
		}

		//Je package klienta kompletn, tj. obsahuje pkaz Final?
    //Is client's package complete: has it command Final?
		if ($client_final)
		{
			//Konme fzi posln zmn klienta?
			//Are we at the end of client's changes sending phase?
			if ($this->_sync_stage==2)
		  {
				//Obsluha synchronizac typu "pouze od klienta"
				//Handle synchronizations which are "from client only"
				foreach($this->_syncs as $target=>$x)
          if (($this->_alerts[$target]==3) || ($this->_alerts[$target]==4) || ($this->_alerts[$target]==7))
            $this->make_response_sync($target,$response_tree);
          
				//Pokud jsme synchronizovali jen typem "pouze client", konme
				//If we synchronized client only type only, we end
				if (!count($this->_syncs))
				{
					$server_final=true;
					$last_message=true;
				}
				else
      		$this->_sync_stage++;
			}
		}
    else
		{
			//Vytvoen pkazu Alert pro zasln dal zprvy
      //Create command Alert for next message
      $response=&$response_tree->new_child('Alert',NULL);
      $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
      $response->new_child_data('Data',NULL,222);
      $item=&$response->new_child('Item',NULL);
      $target=&$item->new_child('Target',NULL);
      $target->new_child_data('LocURI',NULL,$this->_source_uri);
      $source=&$item->new_child('Source',NULL);
      $source->new_child_data('LocURI',NULL,$this->_target_uri);
		}
			
		//Vymazn informac pro zpracovn Status pkazu
		//Delete informations for Status command handle
		unset($this->_status[$this->_next_msg_id-2]);
		
		$this->_datastore->_imap->close();
			
    if ($server_final)
    {
      //Pidn ukonovacho pkazu Final
      //Add command Final for package finalizing
      $response_tree->new_child('Final',NULL);

      if ($last_message)
      {
        //Konec synchronizace
        //End of synchronization
        $this->_datastore->logout();
        return true;
      }
    }
    return false;
  }

//************************************************************

	function get_sync_type($client,$url,$anchors)
	{
		$result['status']=200;

		//Volba typu synchronizace klientem
		//Client's choose type of synchronization
		if ($client>=201 && $client<=205)
		{

			//Mme synchronizaci, pro kterou nepotebujeme ovit synchronizan odkazy?
			//Have we synchronization we needn't check sync anchors for?
			if ($client!=202 && $client!=204)
			{
        $result['alert']=$client;
				return $result;
			}
		}
		else

			//Volba typu synchronizace parametrem
			//Choose type of synchronization by parameter
			if ($url>=2 && $url<=9)
			{
				//Mme synchronizaci, pro kterou nepotebujeme ovit synchronizan odkazy?
				//Have we synchronization we needn't check sync anchors for?
				if ($url!=3 && $url!=5)
		  	{
					//Chceme pomalou synchronizaci?
					//Do we want slow synchronization?
					if ($url==2 || $url==8)
					  $result['status']=508;

          $result['alert']=$url+199;
					return $result;
				}

				$client=$url+199;
			}
			else
		  	$client=200;

    //Oven monosti navzat synchronizaci na posledn
    //Verification possibility of normal (incremental) synchronization
		if (!$anchors)
			switch($client)
			{
				case 200:
			  	$result['status']=508;
			  	$result['alert']=201;
			  	break;

				case 202:
			  	$result['alert']=206;
					break;
					
				case 204:
				  $result['status']=508;
			  	$result['alert']=207;
			}
		else
		  $result['alert']=$client;

		return $result;
	}

//************************************************************

	function get_filters($xml_command,$target)
	{
    //Mme SyncML 1.2 filtry?
		//Have we SyncML 1.2 filters?
		if ($filter=&$xml_command->get_child('Item/Target/Filter'))
		{
			//Zpracovn zznamovho filtru
			//Process record filter
			$record_data=$filter->get_data('Record/Item/Data');
			if ($record_data)
			{
				$date=substr($record_data,strpos($record_data,';')+1);
				$year=substr($date,0,4);
				$month=substr($date,4,2);
				$day=substr($date,6,2);
        $time=strtotime($year.'-'.$month.'-'.$day);

				if ($time!=-1)
					$record_filter=$time;
			}

			//Zpracovn vlastnost polokovho filtru
			//Process properties of field filter
			$field_property=&$filter->get_child('Field/Item/Data/Property');
			$max_size=$field_property->get_data('MaxSize');

			$field_filter='header';
			$field_info='Header only';

			$children=&$field_property->children();
			for($i=0;$i<$field_property->children_count();$i++)
			{
    	  $tag=&$children[$i];
				if ($tag->get_name()=='PropParam')
				{
					$param_name=$tag->get_data('ParamName');

					if ($param_name=='texttype' && $field_filter=='header')
					{
						$field_filter='body';
						$field_info='Without attachments';
					}

					if ($param_name=='attachtype')
					{
						$field_filter='all';
						$field_info='';
					}
				}
			}
				
			$this->_filters[$target]=array('record'=>$record_filter,'size'=>$max_size,'field'=>$field_filter);
		}
		else

			//Mme Synthesis filtry?
			//Have we Synthesis filters?
			if ($pos=strpos($target,'?'))
			{
				//Zpracovn zadan velikosti mail
				//Process set size of mails
				$start=strpos($target,'li(');
				if ($start)
				{
					$end=strpos($target,')',$start);
					$size=substr($target,$start+3,$end-$start-3);
					if ($size)
					{
						$max_size=$size;
						$field_filter='body';
						$field_info='Without attachments';
					}
					else
					{
					  $field_filter='header';
					  $field_info='Header only';
					}
				}
				else
					$field_filter='all';

				//Zpracovn zadanho asu
				//Process set time
				$start=strpos($target,'dr(-');
				if ($start)
				{
					$end=strpos($target,',',$start);
					$days=substr($target,$start+4,$end-$start-4);
					$time=strtotime(date('Y-m-d',time()-$days*24*60*60));

					if ($time!=-1)
					  $record_filter=$time;
				}

				$this->_filters[$target]=array('record'=>$record_filter,'size'=>$max_size,'field'=>$field_filter);
			}

		if ($this->_filters[$target])
		{
      if ($time!=-1)
				$this->_info->add_text('- Time filter:        '.date("m/d/Y",$time));
    	if ($max_size)
				$this->_info->add_text('- Size filter:        '.$max_size.'B');
    	if ($field_info)
				$this->_info->add_text('- Field filter:       '.$field_info);
		}
	}

//************************************************************

  function handle_command_alert($xml_command,&$response_tree)
  {
		global $item_types;
		global $sync_types;

		if ($xml_command->get_data('Data')=='745')
		{
			$this->_ip=$xml_command->get_data('Item/Data');
			
			//****************************************
			$this->_info->add_delimiter('-',70);
    	$this->_info->add_text('ALERT for new IP address:'.$this->_ip);
    	//****************************************
    
			return;
		}
		
		//Vytvoen odpovdi na pkaz Alert
    //Make response for command Alert
    $response=&$response_tree->new_child('Status',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $response->new_child_data('MsgRef',NULL,$this->_rcvd_msg_id);
    $response->new_child_data('CmdRef',NULL,$xml_command->get_data('CmdID'));
    $response->new_child_data('Cmd',NULL,'Alert');
    $target=$xml_command->get_data('Item/Target/LocURI');
    $response->new_child_data('TargetRef',NULL,$target);
    $source=$xml_command->get_data('Item/Source/LocURI');
    $response->new_child_data('SourceRef',NULL,$source);

    //Je session ovena?
    //Is session authenticated?
		if (!$this->_authenticated)
		{
		  //Schz oven session
		  //Missing session credentials
      $response->new_child_data('Data',NULL,407);

			return;
		}

		//Poslal nm klient Alert pro dal zprvu?
    //Did client send us Alert for next message?
		if ($xml_command->get_data('Data')=='222')
		{
      $response->new_child_data('Data',NULL,200);

      return;
		}
		
		//****************************************
		$this->_info->add_delimiter('-',70);
    $this->_info->add_text('ALERT for '.((substr($target,0,2)=='./')?substr($target,2):$target));
    //****************************************
    
		//Poslal nm klient stejn jmno databze pro vce typ?
    //Did client sent us the same name of database for several types?
		if ($this->_alert[$target])
		{
      //Databze neexistuje
      //Database not found
      $response->new_child_data('Data',NULL,404);
      $this->_info->add_text('- ERROR: Used folder name!');

			return;
		}
		
    //Poslal nm klient platn jmno databze?
    //Did client sent us a valid name of database?
		$datastore_type=$this->_datastore->set_datastore_type($target);
		if (!$datastore_type)
		{
      //Databze neexistuje
      //Database not found
      $response->new_child_data('Data',NULL,404);
      $this->_info->add_text('- ERROR: Unknown folder name!');
      
			return;
		}

    //Poslal nm klient svj Next odkaz?
    //Did client sent us its Next anchor?
		$remote_next_anchor=$xml_command->get_data('Item/Meta/Anchor/Next');
		if (!isset($remote_next_anchor))
		{
      //Nepln pkaz
      //Incomplete command
      $response->new_child_data('Data',NULL,412);
      $this->_info->add_text("- ERROR: Client's next anchor missing!");

			return;
		}
		
		$this->_datastore->device_registration($this->_username,$this->_source_uri,$this->_params);
    $this->_datastore->get_anchors(&$local_last_anchor,&$remote_last_anchor,&$local_next_anchor,$remote_next_anchor);

		//Potebujeme pkaz Get?
		//Do we need command Get?
		$saved_format=$this->_datastore->get_saved_format();
		if (!$saved_format)
			$this->_need_get=true;

    //Zapamatovn odpovdajcch si jmen databz pro zpracovn pkazu Put
		//Remember database names for handle command Put
		$this->_db_names[$source]=$target;

    //Volba typu synchronizace
		//Choose type of synchronization
		$client_sync=$xml_command->get_data('Data');
		$anchors=$xml_command->get_data('Item/Meta/Anchor/Last')==$remote_last_anchor;
    $sync_type=$this->get_sync_type($client_sync,$this->_params['SYNC'],$anchors);
		$status_data=$sync_type['status'];
		$alert_data=$sync_type['alert'];
		$this->_alerts[$target]=$alert_data-199;
				
		//Simulace jednostrannch pomalch synchronizac
		//One way slow synchronizations simulation
		switch($alert_data)
		{
			case 206:
				$alert_data=203;
				break;
					  
			case 207:
				$alert_data=201;
				
			case 208:
			  $alert_data=205;
		}

		//****************************************
		$this->_info->add_text('- Type of items:      '.$item_types[$datastore_type]);
    $this->_info->add_text('- Saved format:       '.($saved_format?$saved_format:'Unknown!'));
    $this->_info->add_text("- Client's sync type: ".$sync_types[$client_sync-200]);
		if ($this->_params['SYNC']>=1 && $this->_params['SYNC']<=9)
			$this->_info->add_text("- URL's sync type:    ".$sync_types[$this->_params['SYNC']-1]);
		$this->_info->add_text('- Sync anchors '.($anchors?'OK':'mismatch').'!');
    $this->_info->add_text("- FINAL sync type:    ".$sync_types[$this->_alerts[$target]-1]);
		//****************************************

		//Podpora jednoduchch filtr pro maily
		//Simple mail's filters support
		if ($datastore_type=='M')
			$this->get_filters($xml_command,$target);
			
		//Dokonen odpovdi na pkaz Alert
		//Response finalization for command Alert
		$response->new_child_data('Data',NULL,$status_data);
		$item=&$response->new_child('Item',NULL);
    $data=&$item->new_child('Data',NULL);
    $anchor=&$data->new_child('Anchor',array('xmlns'=>'syncml:metinf'));
    $anchor->new_child_data('Next',NULL,$remote_next_anchor);

    //Vytvoen pkazu Alert pro klienta
    //Create command Alert for client
    $response=&$response_tree->new_child('Alert',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $response->new_child_data('Data',NULL,$alert_data);
    $item=&$response->new_child('Item',NULL);
    $target=&$item->new_child('Target',NULL);
		$source=&$item->new_child('Source',NULL);
    $target->new_child_data('LocURI',NULL,$xml_command->get_data('Item/Source/LocURI'));
		$source->new_child_data('LocURI',NULL,$xml_command->get_data('Item/Target/LocURI'));
		$meta=&$item->new_child('Meta',NULL);
    $anchor=&$meta->new_child('Anchor',array('xmlns'=>'syncml:metinf'));
		$anchor->new_child_data('Last',NULL,$local_last_anchor);
  	$anchor->new_child_data('Next',NULL,$local_next_anchor);
		$meta->new_child_data('MaxObjSize',array('xmlns'=>'syncml:metinf'),$this->_object_size);
	}
	
//************************************************************
  
  function display_message(&$response_tree,$message)
  {
    $response=&$response_tree->new_child('Alert',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $response->new_child_data('Data',NULL,100);
    $item=&$response->new_child('Item',NULL);
    $item->new_child_data('Data',NULL,$message);
	}

//************************************************************

  function handle_command_get($xml_command,&$response_tree)
  {
		global $item_types;
		global $datastore_formats;
		global $format_versions;
    
    //Je session ovena?
    //Is session authenticated?
    if ($this->_authenticated)
    {
    	//Vytvoen odpovdi na pkaz Get
    	//Make response for command Get
    	$response=&$response_tree->new_child('Results',NULL);
    	$response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    	$response->new_child_data('MsgRef',NULL,$this->_rcvd_msg_id);
    	$response->new_child_data('CmdRef',NULL,$xml_command->get_data('CmdID'));
    	$meta=&$response->new_child('Meta',NULL);
    	$meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),$xml_command->get_data('Meta/Type'));
    	$item=&$response->new_child ('Item',NULL);
    	$source=&$item->new_child('Source',NULL);
    	$source->new_child_data('LocURI',NULL,$xml_command->get_data('Item/Target/LocURI'));
    	$data=&$item->new_child('Data',NULL);

    	$devinf=&$data->new_child('DevInf',array('xmlns'=>'syncml:devinf'));
    	$devinf->new_child_data('VerDTD',NULL,$this->_version);
    	$devinf->new_child_data('Man',NULL,'IceWarp Ltd.');
    	$devinf->new_child_data('Mod',NULL,'Merak Mail Server');

			$host=$_SERVER['HTTP_HOST'];
			if ($pos=strpos($host,":"))
      	$host=substr($host,0,$pos);

			$devinf->new_child_data('DevID',NULL,$host);
    	$devinf->new_child_data('DevTyp',NULL,'server');

    	$devinf->new_child('SupportLargeObjs',NULL);
    
      //****************************************
      $this->_info->add_delimiter('-',70);
      $this->_info->add_text('RESULTS from Server');
      //****************************************
      
    	//Vrcen devinf pro kadou databzi
    	//Return devinf for each database
    	foreach($this->_db_names as $name)
    	{
      	$datastore=&$devinf->new_child('DataStore',NULL);
      	$datastore->new_child_data('SourceRef',NULL,$name);

				$display_name=(substr($name,0,2)=='./')?substr($name,2):$name;
      	$datastore->new_child_data('DisplayName',NULL,($pos=strpos($display_name,'?'))?substr($display_name,0,$pos):$display_name);

				//Zskn formtu dat
				//Data format determination
				$type=$this->_datastore->set_datastore_type($name);
				$format=$this->_datastore->get_datastore_format();
				$version=$this->_datastore->get_format_version();
				
				if ($format_versions[$format])
					if ($datastore_formats[$type]['pref']==$format)
						$alt_format=$datastore_formats[$type]['vob'];
					else
						$alt_format=$datastore_formats[$type]['pref'];

      	$rxpref=&$datastore->new_child('Rx-Pref',NULL);
      	$rxpref->new_child_data('CTType',NULL,$format);
      	$rxpref->new_child_data('VerCT',NULL,$version);
      	
      	if ($alt_format)
      	{
      		$rx=&$datastore->new_child('Rx',NULL);
      		$rx->new_child_data('CTType',NULL,$alt_format);
      		$rx->new_child_data('VerCT',NULL,$format_versions[$alt_format]); 		
				}
				
      	$txpref=&$datastore->new_child('Tx-Pref',NULL);
      	$txpref->new_child_data('CTType',NULL,$format);
      	$txpref->new_child_data('VerCT',NULL,$version);

				if ($alt_format)
				{
					$tx=&$datastore->new_child('Tx',NULL);
      		$tx->new_child_data('CTType',NULL,$alt_format);
      		$tx->new_child_data('VerCT',NULL,$format_versions[$alt_format]);
				}
							
        //****************************************
      	$this->_info->add_text(sprintf('% -28s','- Format for '.$item_types[$type].' type:').$format.($version?(' v'.$version):''));
        //****************************************
        
    		//Podporovan typy synchronizac
    		//Supported sync types
				$synccap=&$datastore->new_child('SyncCap',NULL);
				for($i=1;$i<=7;$i++)
					$synccap->new_child_data('SyncType',NULL,$i);

			}

			/*
			$support=array('BEGIN'=>array('VCALENDAR','VEVENT','VTODO'),'END'=>array('VCALENDAR','VEVENT','VTODO'),'VERSION'=>array('1.0'),'SUMMARY','CATEGORIES','CLASS','LOCATION','DESCRIPTION','DTSTART','DTEND','TRANSP','RRULE','EXDATE','VALARM','ATTENDEE','COMPLETED','DUE','PRIORITY','STATUS');
			$ctcap=&$devinf->new_child('CTCap',NULL);
			$ctcap->new_child_data('CTType',NULL,'text/x-vcalendar');
			foreach($support as $propname=>$valenums)
				if (is_array($valenums))
				{
          $ctcap->new_child_data('PropName',NULL,$propname);
					foreach($valenums as $valenum)
					  $ctcap->new_child_data('ValEnum',NULL,$valenum);
				}
				else
				  $ctcap->new_child_data('PropName',NULL,$valenums);
			*/
		}
		else
		{
      //Schz oven session
		  //Missing session credentials
			$response=&$response_tree->new_child('Status',NULL);
    	$response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    	$response->new_child_data('MsgRef',NULL,$this->_rcvd_msg_id);
    	$response->new_child_data('CmdRef',NULL,$xml_command->get_data('CmdID'));
    	$response->new_child_data('Cmd',NULL,'Get');
    	$response->new_child_data('Data',NULL,407);
		}
	}

//************************************************************

	function make_command_get(&$response_tree)
	{
    //Vytvoen pkazu Get
    //Make command Get
    $response=&$response_tree->new_child('Get',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $meta=&$response->new_child('Meta',NULL);
    $meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),'application/vnd.syncml-devinf+'.$this->_content_type);
		$item=&$response->new_child('Item',NULL);
		$target=&$item->new_child('Target',NULL);
		$target->new_child_data('LocURI',NULL,'./devinf'.(($this->_version=='1.2')?'12':'11'));
		
		//****************************************
		$this->_info->add_delimiter('-',70);
    $this->_info->add_text('GET from Server');
    //****************************************
	}
	
//************************************************************

  function handle_command_map($xml_command,&$response_tree)
  {
    //Vytvoen odpovdi na pkaz Map
    //Make response for command Map
    $response=&$response_tree->new_child('Status',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $response->new_child_data('MsgRef',NULL,$this->_rcvd_msg_id);
    $response->new_child_data('CmdRef',NULL,$xml_command->get_data('CmdID'));
    $response->new_child_data('Cmd',NULL,'Map');
    $target=$xml_command->get_data('Target/LocURI');
    $response->new_child_data('TargetRef',NULL,$target);
    $response->new_child_data('SourceRef',NULL,$xml_command->get_data('Source/LocURI'));

    $datastore_type=$this->_datastore->set_datastore_type($target);
		$format=$this->_datastore->get_datastore_format();
		
    $children=&$xml_command->children();
    for($i=0;$i<$xml_command->children_count();$i++)
    {
      $cmd=&$children[$i];
      $cmd_name=$cmd->get_name();
      if ($cmd_name=='MapItem')
      {
				$guid=$cmd->get_data('Target/LocURI');

				//Oeten lenho chovn Funambolu: identifikace Inboxu v Id poloky
        //Handle mad Funambol's behavior: Inbox identification in item Id
        if ($format=='application/vnd.omads-email+xml')
        	$this->_datastore->map_data_update(substr($guid,2),$cmd->get_data('Source/LocURI'));
				else
				  $this->_datastore->map_data_update($guid,$cmd->get_data('Source/LocURI'));
			}
		}
    $response->new_child_data('Data',NULL,200);

    //Ukonen synchronizace tj. aktualizace synchronizanch odkaz
    //End of synchronization: synchronization's anchors update
    //$this->_datastore->end_sync();
  }

//************************************************************

	function make_put_status($xml_command,&$response_tree,$results=false)
	{
    //Vytvoen odpovdi na pkaz Put
    //Make response for command Put
    $response=&$response_tree->new_child('Status',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $response->new_child_data('MsgRef',NULL,$this->_rcvd_msg_id);
    $response->new_child_data('CmdRef',NULL,$xml_command->get_data('CmdID'));
    $response->new_child_data('Cmd',NULL,($results?'Results':'Put'));

    //Je session ovena?
    //Is session authenticated?
		if ($this->_authenticated)
		{
			$response->new_child_data('SourceRef',NULL,$xml_command->get_data('Item/Source/LocURI'));
    	$response->new_child_data('Data',NULL,200);
    }
    else

			//Schz oven session
		  //Missing session credentials
		  $response->new_child_data('Data',NULL,407);

	}
	
//************************************************************

  function handle_command_put($xml_command,$results=false)
  {
		global $item_types;

		//Je session ovena?
    //Is session authenticated?
		if (!$this->_authenticated)
		{
		  $this->_need_get=true;
		  return;
		}

		//****************************************
    $this->_info->add_delimiter('-',70);
    $this->_info->add_text(($results?'RESULTS':'PUT').' from Client');
    //****************************************
      
		//Zpracovn DevInf
		//Process DevInf
		$devinf=&$xml_command->get_child('Item/Data/DevInf');
		$man=$devinf->get_data('Man');
		$children=&$devinf->children();
		for($i=0;$i<$devinf->children_count();$i++)
		{
    	$tag=&$children[$i];
			if ($tag->get_name()=='DataStore')
			{
				$source=$tag->get_data('SourceRef');
				$format=$tag->get_data('Rx-Pref/CTType');
				$version=$tag->get_data('Rx-Pref/VerCT');
					
				//Znme uvedenou databzi?
				//Do we know the database?
				if ($this->_db_names[$source])
				{
					if (strpos($format,'-zip-'))
					{
            $format=$tag->get_data('Rx/CTType');
					  $version=$tag->get_data('Rx/VerCT');
					}
					
					if ((!$version)&&($format=='text/x-vcalendar'))
					  $version='1.0';
					  
					//Uloen formtu dat
					//Save data format
					$type=$this->_datastore->set_datastore_type($this->_db_names[$source]);
					$this->_datastore->set_datastore_format($format,$version);

					/*
					if ($type=='E' && $man=='NOKIA')
						$this->_datastore->set_datastore_filter('VALARM');
					*/

          //****************************************
          $this->_info->add_text(sprintf('% -28s','- Format for '.$item_types[$type].' type:').$format.($version?(' v'.$version):''));
          //****************************************
				}
			}
		}
		
		$this->_need_get=false;
	}

//************************************************************

	function handle_command_status($xml_command)
	{
		$cmd_name=$xml_command->get_data('Cmd');
    if (($cmd_name=='Add')||($cmd_name=='Delete')||($cmd_name=='Replace'))
    {
			$cmdref=$xml_command->get_data('CmdRef');
			$target=$this->_status[$this->_next_msg_id-2][$cmdref];
			$this->_info->set_info($target,'server');
			$result=$xml_command->get_data('Data');
			
			$children=&$xml_command->children();
      for($i=0;$i<$xml_command->children_count();$i++)
      {
				$tag_name=&$children[$i]->get_name();
				if ($tag_name=='Item' || $tag_name=='SourceRef' || $tag_name=='TargetRef')
          $this->_info->{'item_'.strtolower($cmd_name)}($result);
			}
		}
	}

//************************************************************

  function handle_command_sync($xml_command,&$response_tree)
  {
		//Vytvoen odpovdi na pkaz Sync
    //Make response for command Sync
    $response=&$response_tree->new_child('Status',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $response->new_child_data('MsgRef',NULL,$this->_rcvd_msg_id);
    $response->new_child_data('CmdRef',NULL,$xml_command->get_data('CmdID'));
    $response->new_child_data('Cmd',NULL,'Sync');
    $target=$xml_command->get_data('Target/LocURI');
    $response->new_child_data('TargetRef',NULL,$target);
    $source=$xml_command->get_data('Source/LocURI');
    $response->new_child_data('SourceRef',NULL,$source);

    //****************************************
    $this->_info->add_delimiter('-',70);
    $this->_info->add_text('SYNC from Client for '.((substr($target,0,2)=='./')?substr($target,2):$target));
    $this->_info->set_info($target,'client');
    //****************************************
    
		//Nastaven typu synchronizace
    //Set the type of synchronization
    if ($this->_alerts[$target]==1 || $this->_alerts[$target]==3 || $this->_alerts[$target]==5)
			$this->_datastore->set_slow_sync(false);
    else
      $this->_datastore->set_slow_sync(true);

		if ($this->_alerts[$target]==8 || $this->_alerts[$target]==9)
      $this->_datastore->set_special_sync(true);
    else
      $this->_datastore->set_special_sync(false);

    //Pokraujeme v synchronizaci i ji zanme?
    //Do we continue or begin the synchronization?
    if ($this->_syncs[$target]) 
    {
      $sync_ok=true;
      $this->_datastore->set_datastore_type($target,true);
    }
    else
    {
      $sync_ok=$this->_datastore->start_sync($target);
      $this->_syncs[$target]=$source;
    }
    
		//Nastaven stavu synchronizace
		//Set synchronization status
    if ($sync_ok)
      $status=200;
    else
    {
      $status=500;

      //****************************************
			$this->_info->add_text('- ERROR: No response from server!');
			$this->_info->add_delimiter('-',70);
			//****************************************
    }
    
    $response->new_child_data('Data',NULL,$status);

    //Konec synchronizace, pokud je chyba na stran serveru
    //End sync if server failure
    if (!$sync_ok) 
      return;

    //Vytvoen odpovdi na podpkazy Add, Delete a Replace vetn jejich vykonn
    //Make response for subcommands Add, Delete and Replace with their processing
    $children=&$xml_command->children();
    for($i=0;$i<$xml_command->children_count();$i++)
    {
      $cmd=&$children[$i];
      $cmd_name=$cmd->get_name();
      if (($cmd_name=='Add')||($cmd_name=='Delete')||($cmd_name=='Replace'))
      {
        $response=&$response_tree->new_child('Status',NULL);
        $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
        $response->new_child_data('MsgRef',NULL,$this->_rcvd_msg_id);
        $response->new_child_data('CmdRef',NULL,$cmd->get_data('CmdID'));
        $response->new_child_data('Cmd',NULL,$cmd_name);

				$encode=$cmd->get_data('Meta/Format');
        $this->_datastore->set_datastore_format($cmd->get_data('Meta/Type'));
				
        $cmd_children=&$cmd->children();
        for($j=0;$j<$cmd->children_count();$j++)
        {
          $item=&$cmd_children[$j];
          if ($item->get_name()=='Item')
          { 
            $result=0;

            $this->_datastore->set_datastore_format($item->get_data('Meta/Type'));
            $this->_datastore->set_datastore_encode(($tmp=$item->get_data('Meta/Format'))?$tmp:$encode);
            
						$locuri=$item->get_data('Source/LocURI');
            $data=$item->get_data('Data');

            //Mme nov velk objekt nebo ho ji zpracovvme?
            //Have we new large object or it's handling already?
            if (($moredata=$item->get_child('MoreData'))||$this->_more_data['Size'])
            {
              //Mme nov velk objekt?
              //Have we new large object?
              if ($size=$item->get_data('Meta/Size'))
              {
                $this->_more_data['LocURI']=$locuri;
                $this->_more_data['Data']=$data;
                $this->_more_data['Size']=$size-strlen($data);
              }
              else

								//Pokrujeme ve zpracovn sprvnho velkho objektu?
                //Do we continue handling correct large object?
                if ($locuri==$this->_more_data['LocURI'])
                {
                   $this->_more_data['Data'].=$data;
                   $this->_more_data['Size']-=strlen($data);
                }

							//Oekvme dal st velkho objektu?
              //Do we wait for next part of large object?
              if ($moredata)
                $result=213;
              else
              {
                //Je velikost pijatho velkho objektu v podku?
                //Is size of large object correct?
                if ($this->_more_data['Size'])
                {
                  $result=424;

                  //****************************************
									$this->_info->add_error($locuri);
									//****************************************
								}
								else

							    //Uloen kompletnho velkho objektu
                  //Set complete large object
    		          $data=$this->_more_data['Data'];

                //Konme se zpracovnm velkho objektu
                //End of large object's handling
                $this->_more_data=array();
              }
            }
            
            //Pokud nemme rozpracovan velk objekt a ani nedolo k chyb, pak poslame data do Datastore
            //If we haven't in-process large object or any error, we send data to Datastore
            if (!$result)
						{
              $result=$this->_datastore->{'client_'.strtolower($cmd_name)}($locuri,$data);

              //****************************************
							if ($result==500)
							  $this->_info->add_error($locuri,$data);
							//****************************************
            }
            
            $response->new_child_data('SourceRef',NULL,$locuri);
            $response->new_child_data('Data',NULL,$result);

            //****************************************
            $this->_info->{'item_'.strtolower($cmd_name)}($result);
            //****************************************
					}
        }
      }
    }
    
    $this->_info->client_partial_summary();
  }

//************************************************************

	function make_response_syncs(&$response_tree)
	{
		//Sname se poslat v prvnm Sync pkazu vechny zmny
    //Try to send all changes in first Sync command
		$this->_datastore->set_all_items();
		
		//Pokud mme msto ve zprv a v pedchozm Sync pkazu jsme poslali vechny poloky, pokraujeme dalm Sync pkazem
		//If we have free place in message and we sent all items in previous Sync command, we continue in next Sync command
		foreach($this->_syncs as $target=>$x)
      if ($this->_datastore->get_free_place() && $this->_datastore->get_all_items())
				$this->make_response_sync($target,$response_tree);
			else
			  break;
			  	
		if (count($this->_syncs))
			return false;
		else
		  return true;
	}

//************************************************************

  function make_response_sync($target_db,&$response_tree)
  {
		//Pokraujeme v jednostrann synchronizaci od klienta?
		//Do we continue in one-way synchronization from client?
		if ($this->_alerts[$target_db]==3 || $this->_alerts[$target_db]==4 || $this->_alerts[$target_db]==7)
		{
		  //Pokraujeme v jednostrann obnovovac synchronizaci od klienta?
		  //Do we continue in one-way refresh synchronization from client?
		  if ($this->_alerts[$target_db]==4)
		  {
		    $this->_datastore->set_datastore_type($target_db,true);
		    $this->_datastore->server_refresh_delete();
		  }
		  else
		    $this->_datastore->set_datastore_type($target_db);
		    
		  $this->_datastore->end_sync();
		  
      //Vyjmeme synchronizaci z fronty
      //Remove the synchronization from the queue
      unset($this->_syncs[$target_db]);
			
      return;
		}
		
		//Vytvoen hlaviky pkazu Sync pro klienta
    //Create header of command Sync for client
    $response=&$response_tree->new_child('Sync',NULL);
    $response->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
    $target=&$response->new_child('Target',NULL);
    $target->new_child_data('LocURI',NULL,$this->_syncs[$target_db]);
    $source=&$response->new_child('Source',NULL);
    $source->new_child_data('LocURI',NULL,$target_db);

    $this->_info->add_delimiter('-',70);
    $this->_info->add_text('SYNC from Server for '.((substr($target_db,0,2)=='./')?substr($target_db,2):$target_db));
    
    //Nastaven typu synchronizace
    //Set the type of synchronization
    if ($this->_alerts[$target_db]==2 || $this->_alerts[$target_db]==6 || $this->_alerts[$target_db]==8 || $this->_alerts[$target_db]==9)
      $this->_datastore->set_slow_sync(true);
    else
      $this->_datastore->set_slow_sync(false);

    if ($this->_alerts[$target_db]==9)
      $this->_datastore->set_special_sync(true);
    else
      $this->_datastore->set_special_sync(false);

		//Nastaven filtr synchronizace
		//Set synchronization filters
		$this->_datastore->set_filters($this->_filters[$target_db]);
		
    $this->_datastore->set_datastore_type($target_db,true);

    //Sname se poslat vechny zmny
    //Try to send all changes
    $this->_datastore->set_all_items();
    
    //Nastavme kdovn HTML specilnch znak
    //Set HTML special chars encoding
    $this->_datastore->set_htmlspecialchars();

		$format=$this->_datastore->get_datastore_format();
		$encode=$this->_datastore->get_datastore_encode();
		
    //Vytvoen podpkaz Add, Delete, Replace tj. seznam zmn na serveru
    //Create subcommands Add, Delete, Replace: lists of server's changes
    $add_list=$this->_datastore->server_add();
		if ($add_list)
    {
      $cmd=&$response->new_child('Add',NULL);
      $cmd->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
      $meta=&$cmd->new_child('Meta', NULL);
      $meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),$format);
      if ($encode)
        $meta->new_child_data('Format',array('xmlns'=>'syncml:metinf'),$encode);
      
      foreach($add_list as $add)
      {
        $item=&$cmd->new_child('Item',NULL);
        $source=&$item->new_child('Source',NULL);
        
        //Oeten lenho chovn Funambolu: identifikace Inboxu v Id poloky
        //Handle mad Funambol's behavior: Inbox identification in item Id
        if ($format=='application/vnd.omads-email+xml')
       		$source->new_child_data('LocURI',NULL,'I/'.$add['guid']);
				else
				  $source->new_child_data('LocURI',NULL,$add['guid']);
				
        if ($add['format'] || $add['size'])
        {
					$meta=&$item->new_child('Meta', NULL);

					if ($add['format'])
            $meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),$format);

					if ($add['size'])
						$meta->new_child_data('Size',array('xmlns'=>'syncml:metinf'),$add['size']);
				}
				
        $item->new_child_data('Data',NULL,$add['item'],false);

				if ($add['moredata'])
          $item->new_child('MoreData',NULL);
      }
      
      $this->_status[$this->_next_msg_id-1][$this->_next_cmd_id-1]=$target_db;
    }
    
    $this->_info->add_text(sprintf('- ADD     Sent:% 4d',count($add_list)));

    $delete_list=$this->_datastore->server_delete();
    if ($delete_list)
    {
      $cmd=&$response->new_child('Delete',NULL);
      $cmd->new_child_data('CmdID',NULL,$this->_next_cmd_id++);

			foreach($delete_list as $delete)
      {
        $item=&$cmd->new_child('Item',NULL);
        $target=&$item->new_child('Target',NULL);
        $target->new_child_data('LocURI',NULL,$delete);
      }
      
      $this->_status[$this->_next_msg_id-1][$this->_next_cmd_id-1]=$target_db;
    }
    
    $this->_info->add_text(sprintf('- DELETE  Sent:% 4d',count($delete_list)));
    
    $replace_list=$this->_datastore->server_replace();
    if ($replace_list)
    {
      $cmd=&$response->new_child('Replace',NULL);
      $cmd->new_child_data('CmdID',NULL,$this->_next_cmd_id++);
      $meta=&$cmd->new_child('Meta',NULL);
      $meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),$format);
      if ($encode)
        $meta->new_child_data('Format',array('xmlns'=>'syncml:metinf'),$encode);
        
      foreach($replace_list as $replace)
      {
        $item=&$cmd->new_child('Item',NULL);
        $target=&$item->new_child('Target',NULL);
        $target->new_child_data('LocURI',NULL,$replace['luid']);

        if ($replace['format'] || $replace['size'])
        {
					$meta=&$item->new_child('Meta', NULL);

					if ($replace['format'])
            $meta->new_child_data('Type',array('xmlns'=>'syncml:metinf'),$format);

					if ($replace['size'])
						$meta->new_child_data('Size',array('xmlns'=>'syncml:metinf'),$replace['size']);
				}
				
				$item->new_child_data('Data',NULL,$replace['item'],false);
				
				if ($replace['moredata'])
          $item->new_child('MoreData',NULL);
      }
      
      $this->_status[$this->_next_msg_id-1][$this->_next_cmd_id-1]=$target_db;
    }
    
    $this->_info->add_text(sprintf('- REPLACE Sent:% 4d',count($replace_list)));

		//Poslali jsme klientovi vechny zmny?
    //Do we send all changes to client?
    if ($this->_datastore->get_all_items())
    {
			//Pokud jsme klientovi nco pidali, ekme na pkaz Map
      //If we add to client anything, we wait for command Map
      //if (!$add_list)

		    //Ukonen synchronizace tj. aktualizace synchronizanch odkaz
        //End of synchronization: synchronization's anchors update
        $this->_datastore->end_sync();
			
      //Vyjmeme synchronizaci z fronty
      //Remove the synchronization from the queue
      unset($this->_syncs[$target_db]);
    }
  }
}
?>
