<?php
include_once 'webdav.php';

//************************************************************
// Hlavn tda realizujc CalDAV protokol
// Main class implementing CalDAV protocol
//************************************************************

class caldav_session extends webdav_session
{
  var $_version;
  
  function caldav_session()
  {
    $this->webdav_session();
  }

//************************************************************

  function if_match()
  {
    //Vracme edit counter
    //We return edit counter
    if ($if_match=$_SERVER['HTTP_IF_MATCH'])
    {
			$if_match=trim($if_match,'"');
      $p=strpos($if_match,':');
      if ($p!==false)
      	return substr($if_match,$p+1,strlen($if_match)-$p-1);
    }
  }

//************************************************************
  
  function handle_delete()
  { 
    //Aktualizujeme stvajc poloku?
    //Do we update existing item?    
    $if_match=$this->if_match();
    
    $status=$this->_datastore->delete_item($this->_folder,$this->_local_id,$if_match);
    
    $this->http_response($status,$headers);          
  }

//************************************************************
  
  function handle_move()
  {    
    $destination=$_SERVER['HTTP_DESTINATION'];
    $status=$this->_datastore->move_item($this->_folder,$this->_local_id,$destination);
    
    $this->http_response($status);          
  }

//************************************************************
  
  function handle_mkcalendar()
  {    
    $status=$this->_datastore->mkcalendar($this->_folder, $this->_body);
    
    $this->http_response($status);          
  }

//************************************************************
  
  function handle_mkcol()
  {    
    return $this->handle_mkcalendar();
  }

//************************************************************
  
  function handle_get()
  {
    $result=$this->_datastore->get_item($this->_folder,$this->_local_id);
    
    //Existuje poloka, kterou chceme vrtit?
    //Does item we want to get exist?
    if ($result['item'])
      $headers=array('Content-Type: '.$result['type'],'ETag: '.$this->_local_id.':'.$result['counter']);
    else
      $headers=array();
    
    $result['data']=str_replace('.ics','',$result['data']);
    $this->http_response($result['status'],$headers,$result['data']);
  }

//************************************************************
  
  function handle_head() 
  {
    $this->handle_get();
  }

//************************************************************
  
  function handle_put()
  {            
    //Vytvme novou poloku?
    //Do we create new item?
    if ($_SERVER['HTTP_IF_NONE_MATCH']=='*')
      $verify=';VERIFY';
        
    //Aktualizujeme stvajc poloku?
    //Do we update existing item?
    $if_match=$this->if_match();  
    
    $result=$this->_datastore->put_item($this->_folder,$this->_local_id,$this->_body,$verify,$if_match);  

    $this->http_response($result['status'],array('ETag: '.$result['etag']));
  }

//************************************************************
  
  function handle_report()
  {    
    $request=text_to_xml($this->_body);
    $report_type=$request->get_name();
    
    switch($report_type)
    {
      case 'calendar-query':
      case 'principal-property-search':
      case 'propfind':
				$result=$this->item_query_report($request);
        break;
        
      case 'calendar-multiget':
        $result=$this->calendar_multiget_report($request);
        break;
        
      default:
        $this->http_response('405 Method not allowed');
        return;  
    }
		    
    $this->caldav_multistatus($result);   
  }

//************************************************************
  
  function item_query_report($request)
  {
    $result=array();
		$prop=&$request->get_child('prop');
		
		if ($prop->get_child('principal-collection-set'))
			$result['*']['D:principal-collection-set'][0]['D:href'][0]='/webdav/'.$this->_email;
			
		if ($prop->get_child('calendar-home-set'))
			$result['*']['C:calendar-home-set'][0]['D:href'][0]='/webdav/'.$this->_email.'/'.($this->_users?'':'Events/');
			
		if ($prop->get_child('calendar-user-address-set'))
		{
			$result['*']['C:calendar-user-address-set'][0]['D:href'][0]='/webdav/users/'.$this->_email.'/';
			$result['*']['C:calendar-user-address-set'][0]['D:href'][1]='mailto:'.$this->_email;
		}
		
		if ($prop->get_child('schedule-inbox-URL'))
			$result['*']['C:schedule-inbox-URL'][0]['D:href'][0]='/webdav/'.$this->_email.'/Events/INBOX';
			

		if ($this->_local_id && $prop->get_child('displayname'))
			$result['*']['D:displayname'][0]=$this->_local_id;
			
		//Chce klient vrtit kolekce?
    //Does client want to collections?
    $resourcetype=&$prop->get_child('resourcetype');
    $this->_datastore->set_folder($resourcetype?true:false);

    //Chce klient vrtit etag?
    //Does client want to send etag?
    $etag=&$prop->get_child('getetag');
    $this->_datastore->set_etag($etag?true:false);
    
    //Chce klient vrtit data?
    //Does client want to send data?
    $calendar_data=&$prop->get_child('calendar-data');
    $this->_datastore->set_data($calendar_data?true:false);
    
    //Chce klient vrtit opakujc se data vicekrt?
    //Does client want to expand repeating data?
    if ($calendar_data)
      $expand=&$calendar_data->get_child('expand');
    $this->_datastore->set_expand($expand?true:false);
    
    //Chce klient vrtit konkrtn typ dat ci data z njakho asovho intervalu?
    //Does client want to send concreete type objects or data from any time range?
    $filter=&$request->get_child('filter');
    if ($filter)
    {
      $comp_filter=&$filter->get_child('comp-filter/comp-filter');
      $time_range=&$filter->get_child('comp-filter/comp-filter/time-range');

			$text_match=$filter->get_data('comp-filter/comp-filter/prop-filter/text-match');
		}
    
    $this->_datastore->set_object_type($comp_filter->_attribs);
    $this->_datastore->set_text_match($text_match);
    $this->_datastore->set_julian_range($time_range->_attribs);

		//Nastaven hloubky rekurze
		//Recursion depth set
    if (!isset($this->_depth))
      $this->_depth='infinity';
    $this->_datastore->set_depth($this->_depth);

    $this->_datastore->get_items($this->_folder,$this->_local_id,&$result);   

		return $result;
  }

//************************************************************

  function calendar_multiget_report($request)
  {
    $result=array();

		$children=&$request->children();
    foreach($children as $child)
    {
			if ($child->get_name()!='href')
        continue;
        
			$uri=$child->_children[0]->get_data();
			$fields=explode('/',$uri);

    	if ($uri[strlen($uri)-1]=='/')
    	  continue;
    	  
      $local_id=$fields[count($fields)-1];
			unset($fields[count($fields)-1]);
			
			unset($fields[0]);
    	unset($fields[1]);
    	unset($fields[2]);
    	
      $folder=implode("\\",$fields);

			$result[$local_id]=$this->_datastore->get_item($folder,$local_id);
			$result[$local_id]['folder']=$folder;
		}
    
    return $result;
  }

//************************************************************
  
  function array_to_xml(&$array,&$xml)
  {
		foreach($array as $tag=>$order)
			foreach($order as $content)
				if (is_array($content))
					$this->array_to_xml($content,$xml->new_child($tag,NULL));
				else
					$xml->new_child_data($tag,NULL,$content);
	}

//************************************************************

  function caldav_multistatus($results)
  {
    $multistatus=new xml_tree_node('D:multistatus',array('xmlns:D'=>'DAV:','xmlns:G'=>'http://groupdav.org/','xmlns:C'=>'urn:ietf:params:xml:ns:caldav'));
    foreach($results as $id=>$result)
    {
      $response=&$multistatus->new_child('D:response',NULL);
      
      if ($id=='*')
      {
        $response->new_child_data('D:href',NULL,$this->_server.($this->_folder?(str_replace('\\','/',$result['folder']).'/'):'').$this->_local_id);

				$propstat=&$response->new_child('D:propstat',NULL);
        $prop=&$propstat->new_child('D:prop',NULL);

				$this->array_to_xml($result,$prop);
				
				$propstat->new_child_data('D:status',NULL,'HTTP/1.1 200 OK');
				continue;
			}
			
			if ($result['collection'])
			{
        $folder=str_replace('\\','/',$id);

				$response->new_child_data('D:href',NULL,$this->_server.$this->_email.'/'.$folder.'/');

        $propstat=&$response->new_child('D:propstat',NULL);
        $prop=&$propstat->new_child('D:prop',NULL);

        $prop->new_child_data('D:displayname',NULL,$folder);

        $resourcetype=&$prop->new_child('D:resourcetype',NULL);
        $resourcetype->new_child('D:collection',NULL);
        $resourcetype->new_child('G:'.$result['collection']);

				if (strpos(' EJT',$result['type']))
				  $resourcetype->new_child('C:calendar',NULL);

        $propstat->new_child_data('D:status',NULL,'HTTP/1.1 200 OK');
			}
			else
      {
        $response->new_child_data('D:href',NULL,$this->_server.$this->_email.'/'.str_replace('\\','/',$result['folder']).'/'.$id);
        
        //Existuje poloka, kterou chceme vrtit?
        //Does item we want to get exist?
        if ($result['data'])
        {
          $propstat=&$response->new_child('D:propstat',NULL);
          $prop=&$propstat->new_child('D:prop',NULL);
          
          $prop->new_child_data('D:getetag',NULL,$id.':'.$result['counter']);
          $prop->new_child_data('D:getcontenttype',NULL,$result['type']);
          
          //Mme poloku, jej obsah chceme vrtit?
          //Have we item we wants to get its content?
          if ($result['data']!='*')
					{
						$result['data']=preg_replace("#(UID:..)([^.]{0,})(.ics)#i","\\1\\2",$result['data']);
						$prop->new_child_data('C:calendar-data',NULL,$result['data']);
          }
          
          $propstat->new_child_data('D:status',NULL,'HTTP/1.1 200 OK');
        }
        else
          $response->new_child_data('D:status',NULL,'HTTP/1.1 404 Resource not found');
      }
    }

    $response_str='<?xml version="1.0" encoding="UTF-8"?'.'>'."\r\n".$multistatus->print_node(0,false);
    $headers=array('Content-Type: '.$_SERVER['CONTENT_TYPE']);  

    $this->http_response('207 Multi-Status',$headers,$response_str);
  }

//************************************************************
  
  function handle_propfind()
  {
    $this->handle_report();
  }
}
?>
