<?php
/*
  Logging
  -------
  Lets you write debug text to a text file. See the attributes to specify the format of logging.

  // Init logs and logs some data
  $debug = new debug_class();
  $debug->init("/tmp/debug.log");
  $debug->log("data");
  $debug->done();

  
  Output buffering
  ----------------
  Allows you to capture all output the scripts create and send this output to a text file.
  The original output will be still sent to the client that requested the session.
  
  // Init output buffering
  $debug = new debug_class();
  $debug->init("", "/tmp/output.log");
  // Do some work here
  $debug->done();

  
  Other usage
  -----------
  //  See the attributes below to set special behavior
  $debug = new debug_class();
  $debug->LOG_APPEND=false;
  $debug->OUTPUT_APPEND=false;
  ...
*/

class debug_class {    
  // Public
  // Enables logs
  var $LOG=false;
  var $LOG_FILE="";

  // Log attributes
  var $LOG_APPEND=true;
  var $LOG_DATE=true;
  var $LOG_DELIMITER=" ";
  var $LOG_NEWLINE=false;
  var $LOG_TRIM=true;

  // Enables double output buffering 
  var $OUTPUT=false;
  var $OUTPUT_FILE="";
  var $OUTPUT_APPEND=true;
  var $OUTPUT_HEADERS=true;
  
  // $logfile - if specified enables the logging and sets the filename
  // $outputfile - if specified enables the output buffering and sets the filename  
  function init($logfile="",$outputfile="")
  {
    // Logging
    if ($logfile) {
      $this->LOG=true;
      $this->LOG_FILE=$logfile;
    }
    // Logging
    if ($this->LOG) if (!$this->LOG_APPEND) unlink($this->LOG_FILE);

    // Output
    if ($outputfile) {
      $this->OUTPUT=true;
      $this->OUTPUT_FILE=$outputfile;
    }
    // Output buffering
    if ($this->OUTPUT) {
      if (!$this->OUTPUT_APPEND) unlink($this->OUTPUT_FILE);

      // Create
      ob_start();
    }
  } 
  
  function done()
  {
    global $DEBUG_OUTPUT;
    
    if ($this->OUTPUT) {
      // Get data
      $data=ob_get_contents();
      ob_end_clean();
      
      // Print
      print($data);
      
      // Write data
      $file=@fopen($this->OUTPUT_FILE,"a");
      if ($file)
      {
        // Headers
        if ($this->OUTPUT_HEADERS) $data=@headers_list().$data;
        
        // Write
        fputs($file, $data . "\r\n");
        fclose($file);
      }
    }
  }
  
  // $text - text that will be logged to file
  function log($text)
  {
    // Check
    if (!$this->LOG) return false;
    
    // Open log file
    $file=@fopen($this->LOG_FILE,"a");
    if ($file)
    {
      // Attributes
      if ($this->LOG_DATE) $str.=date("Y-m-d H:i:s ");
      if ($this->LOG_DELIMITER) $str.=$this->LOG_DELIMITER;
      if ($this->LOG_NEWLINE) $str.="\r\n";
      if ($this->LOG_TRIM) $text=trim($text);
      
      // Write log
      fputs($file, $str . $text . "\r\n");
      fclose($file);
    }
  }
}
?>