/*
	Public:
		- sFrom
		- sTo
		- sCc	
		- sBcc
		- sSubject
		- aAttachments
		- sBody
		
		- isHtml()
		- setHtml()
		
		- onSentCallback  - array
		- onDisposeCallback - array
	Private:
		- __id
*/
function NewMessage(id) {
	if (id)
		this.__id = id;
		
	this.onSentCallback = [];
	this.onDisposeCallback  = [];
};

NewMessage.compose = function(aRecipients) {
	var newMessage = new NewMessage();

	if (typeof aRecipients == 'object'){
	    if (aRecipients.to)
	    	newMessage.sTo = aRecipients.to;
	    if (aRecipients.bc)
	    	newMessage.sBc = aRecipients.bc;
	    if (aRecipients.bcc)
	    	newMessage.sBcc = aRecipients.bcc;
    }

	newMessage.addSignature();
	gui._create('frm_compose', 'frm_compose', '', '', newMessage);
};

NewMessage.prototype.isHtml = function() {
	if (!Is.Defined(this.__bHtml))
		this.__bHtml = parseInt(GWOthers.getItem('MAIL_SETTINGS_DEFAULT', 'html_message'))>0 ? true : false;
		
	return this.__bHtml;
};

NewMessage.prototype.setHtml = function(bHtml) {
	// set HTML composing if bHtml is true, otherwise ignore the bHtml and signify
	// isHtml according to MAIL_SETTINGS_DEFAULT
	this.__bHtml = bHtml?true:false;
};

NewMessage.prototype.save = function(bKeepAttachments, aNotify) {
	var aMessageIQ = this.__loadCommonMessageIQ(bKeepAttachments);

	if (!this.__id) {
		this.__id = [];
		var sDraftFolder = Path.split(GWOthers.getItem('DEFAULT_FOLDERS', 'drafts'));
		this.__id[0] = sDraftFolder[0];
		this.__id[1] = sDraftFolder[1];
	}

	aMessageIQ['to'] = this.sTo;
	aMessageIQ['cc'] = this.sCc;
	aMessageIQ['bcc'] = this.sBcc;

	aMessageIQ['action']['email'] = 'false';
	aMessageIQ['action']['save'] = {"aid": this.__id[0], "fid": this.__id[1]};
	if (this.__id[2]) aMessageIQ['action']['save']['iid'] = this.__id[2];

	storage.library('wm_messages');
	return message.add(aMessageIQ, 'dummy', '', [this, '__messageSaved', [aNotify]]);
};

NewMessage.prototype.send = function(bKeepAttachments, aNotify, aSettingsDefault) {

	var aMessageIQ = this.__loadCommonMessageIQ(bKeepAttachments, aSettingsDefault);

	if (this.sTo == '' && this.sCc == '' && this.sBcc == '')
		return false;

	aMessageIQ['action']['email'] = 'true';

	if ((typeof aSettingsDefault == 'object' && parseInt(aSettingsDefault['save_sent_message'])>0) || (typeof aSettingsDefault != 'object' && parseInt(GWOthers.getItem('MAIL_SETTINGS_DEFAULT', 'save_sent_message'))>0)) {
		var sSentFolder = Path.split(aSettingsDefault['sent'] || GWOthers.getItem('DEFAULT_FOLDERS', 'sent'));
		aMessageIQ['action']['save'] = {"aid": sSentFolder[0], "fid": sSentFolder[1]};
	}

    if (GWOthers.getItem('MAIL_SETTINGS_GENERAL', 'smtp_relay')>0)
    	aMessageIQ['action']['smtp_relay'] = 'true';

	var aDistribList = MailAddress.findDistribList({'to': this.sTo, 'cc': this.sCc, 'bcc': this.sBcc});

	for (var sTag in aDistribList)
		aMessageIQ[sTag] = aDistribList[sTag];

	var rst = GWOthers.getItem('RESTRICTIONS', 'disable_gw_types') || '';

	if (rst.indexOf('c')<0 && window.sPrimaryAccountGW && parseInt(GWOthers.getItem('MAIL_SETTINGS_GENERAL', 'auto_recipient_to_addressbook')))
		aMessageIQ['action']['auto_addressbook'] = 'true';

	storage.library('wm_messages');

	//LOADING gui.frm_main.hmenu1._loading(1);

window.kokoko = true;

	return message.add(aMessageIQ, 'dummy', '' , [this, '__messageSent', [aNotify]]);
};

NewMessage.prototype.dispose = function() {
	if (typeof this.onDisposeCallback == 'object' && this.onDisposeCallback.length>0)
		for(var i in this.onDisposeCallback)
			executeCallbackFunction(this.onDisposeCallback[i]);
};

NewMessage.prototype.addSignature = function() {
	var signature = GWOthers.getItem('SIGNATURE', 'text');

	if (signature) {
		if (!Is.Defined(this.sBody))
			this.sBody = '';
		signature = signature.split('\n').join('<br/>');

		if (GWOthers.getItem('SIGNATURE', 'to_top')>0)
			this.sBody = signature + '<br/>' + this.sBody;
		else
			this.sBody += '<br/>' + signature;
	}
};

NewMessage.prototype.__loadCommonMessageIQ = function(bKeepAttachments, aSettings) {
	var aMessageIQ = {};

	if (typeof aSettings != 'object')
		aSettings = GWOthers.get('MAIL_SETTINGS_DEFAULT', 'storage')['VALUES'];

	aMessageIQ['action'] = {
		"im": 'false',
		"keep": (bKeepAttachments) ? 'true' : 'false',
		"html_body": (this.isHtml()) ? 'true' : 'false'
	};

	aMessageIQ['from'] = aSettings['from'] || getPrimaryAccountFromAddress();
	aMessageIQ['subject'] = this.sSubject;
	
	var aHeaders = [];
	if (parseInt(aSettings['read_confirmation'])) {
		aHeaders.push('X-Confirm-Reading-To: ' + sPrimaryAccount);
		aHeaders.push('Disposition-Notification-To: ' + sPrimaryAccount);
	}

	if (parseInt(aSettings['priority']))
		aHeaders.push('X-Priority: ' + aSettings['priority']);

	if (aSettings['reply_to_address'])
		aHeaders.push('Reply-To: ' + aSettings['reply_to_address']);

	if (!Is.Empty(aHeaders))
		aMessageIQ['headers'] = aHeaders;
	
	aMessageIQ['body'] = this.sBody;

	if (aSettings['charset']) aMessageIQ['charset'] = aSettings['charset'];

	if (parseInt(aSettings['encrypt']))
		aMessageIQ['action']['encrypt'] = 'true';
	if (parseInt(aSettings['sign']))
		aMessageIQ['action']['sign'] = 'true';
	
	if (this.aAttachments) {
		var aAttachments = this.aAttachments['attachments'];
		var sUploadFolId = this.aAttachments['folder'];
		
		var aAttachResult = [];
	
		for (var n in aAttachments)
			aAttachResult[n] = {"class":(aAttachments[n]['class'] ? aAttachments[n]['class'] : 'file'),"type":'normal',
				"fullpath":(aAttachments[n]['class'] ? aAttachments[n]['fullpath'] : sUploadFolId + '/' + aAttachments[n]['id'])};
				
		if (!Is.Empty(aAttachResult))
			aMessageIQ['attachments'] = aAttachResult;
	}
	
	return aMessageIQ;
};

NewMessage.prototype.__messageSaved = function(bOK, sUId, sError, aNotify) {

	if (bOK && sUId > -1) {
		var bFirstTime;
		if (this.__id[2] && this.__id[2] == sUId)
			bFirstTime = false;
		else{
			bFirstTime = true;
			this.__id[2] = sUId;

			//Remove saved email from Drafts folder after Sent
			this.onSentCallback.push([Item.remove, [makeIDSFromID(this.__id), true]]);
		}

		//Refresh Draft folder if active
		Item.__refreshView(this.__id,true);
		
  		if (aNotify)
			executeCallbackFunction(aNotify, true, bFirstTime);
	}
	else
	if (aNotify)
		executeCallbackFunction(aNotify, false, false);
};

NewMessage.prototype.__messageSent = function(bOK, sUId, sError, aNotify) {

	//LOADING gui.frm_main.hmenu1._loading();
	
	if (bOK && !sError && typeof this.onSentCallback == 'object' && this.onSentCallback.length)
		for(var i in this.onSentCallback)
			executeCallbackFunction(this.onSentCallback[i]);

	if (aNotify) executeCallbackFunction(aNotify, bOK, sUId, sError);
};