/**
 * Class for querying and manipulating with emails already stored in mail folders.
 *
 * WARNING! Because of the instance is listening on dataset when you finish work and the instance
 * is no more needet you have to call this.dispose() function in order not to leak.
 *
 * @param	id			[aid, fid, iid]
 * @param	mailInfo	Optional. If you specify extended attributes no server query is proceeded.
 *
 * @see	OldMessage.send()
 * @see	OldMessage.forward()
 * @see OldMessage.reply()
 *
 * @see	OldMessage.getColor()
 * @see OldMessage.setColor()
 *
 * @see	this.dispose()
 */
function OldMessage(id, mailInfo) {
	this.__id = id;

	// Listen on the dataset.
	// When the dataset is changed, function this.__update() is called.
	// Function this.__update() assures that OldMessage is always synchronized with dataset 'items'.
	// If in the dataset there is a message with 'id', private field this.__datasetItems is set pointing
	// to that. Otherwise the this.__datasetItems is left undefined.
	// @see	this.__update()
	dataSet.obey(this, null, 'items');

	// If there was no message with 'id' in dataset, exception is thrown.
	// This is because we don't want to gather basic attributes from the server.
	var aValues;
	if (!this.__datasetItems)
	    aValues = OldMessage.__FULLMAIL_VALUES;
	else
        aValues = OldMessage.__MAILINFO_VALUES;

	// mailInfo is optional parameter. User can specify extended attributes of the message
	// when creating instance of class OldMessage which saves query to the server.
	// TODO	Consider asynchronous communication
	if (mailInfo)
		this.__mailInfo = mailInfo;
	else {
		this.__mailInfo = WMItems.list(
			{"aid": this.__id[0], "fid": this.__id[1], "iid": this.__id[2], "values": aValues}
		)[this.__id[0]][this.__id[1]][this.__id[2]];
		delete this.__mailInfo['/'];
		
        if (!this.__datasetItems)
			this.__datasetItems = this.__mailInfo;
	}
};

OldMessage.__MAILINFO_VALUES = [
	'TO', 'HTML', 'CC', 'BCC', 'REPLY_TO', 'ATTACHMENTS', 'FLAGS', 'HAS_ATTACHMENT', 'PRIORITY',
	'STATIC_FLAGS', 'SMIME_STATUS', 'CONFIRM_ADDR'
];

OldMessage.__FULLMAIL_VALUES = [
	'TO', 'FROM', 'HTML', 'CC', 'BCC', 'REPLY_TO', 'ATTACHMENTS', 'FLAGS', 'HAS_ATTACHMENT', 'PRIORITY',
	'STATIC_FLAGS', 'SUBJECT', 'SMIME_STATUS', 'CONFIRM_ADDR', 'FLAGS', 'DATE', 'SIZE'
];

OldMessage.edit = function(id) {
	try{
		var oldMessage = new OldMessage(id);
		var newMessage = new NewMessage(id);
	}
	catch(er){
		return;
	}

	if (oldMessage.getTo() == 'undisclosed-recipients:;')
		newMessage.sTo = '';
	else
		newMessage.sTo = oldMessage.getTo();
		
	newMessage.sCc = oldMessage.getCc();
	newMessage.sBcc = oldMessage.getBcc();
	newMessage.sSubject = oldMessage.getSubject();
	newMessage.setHtml(oldMessage.isHtml());
	newMessage.sBody = oldMessage.getBody();
	newMessage.aAttachments = oldMessage.copyAttachments(id);

	oldMessage.dispose();

	newMessage.onSentCallback.push([Item.remove, [makeIDSFromID(id), true]]);

	gui._create('frm_compose', 'frm_compose', '', '', newMessage);
};

OldMessage.forward = function(id, bForwardAsMessage) {
	try{
		var oldMessage = new OldMessage(id);
		var newMessage = new NewMessage();
	}
	catch(er){
		return;
	}

	if (!Is.Defined(bForwardAsMessage)) {
		storage.library('gw_others');
		bForwardAsMessage = (GWOthers.getItem('MAIL_SETTINGS_GENERAL', 'forward_messages') != 'inline');
	}

	newMessage.sSubject = OldMessage.__prefixParser(oldMessage.getSubject(), 'Fw');

	if (bForwardAsMessage) {
		newMessage.aAttachments = {'attachments': [{'values': {'class': 'item', 'fullpath': id[0]+'/'+id[1]+'/'+id[2], 'name':'mesage_1.eml', 'size': oldMessage.getSize()}}]};
	} else {

		newMessage.sBody = oldMessage.quoteMessage();

		newMessage.aAttachments = oldMessage.copyAttachments(id);
		newMessage.addSignature();
	}

	newMessage.setHtml(oldMessage.isHtml());

	//if not embanded email
	if (id[2].indexOf('|')<0)
		newMessage.onSentCallback.push([oldMessage, oldMessage.setForwarded]);

	newMessage.onDisposeCallback.push([oldMessage, oldMessage.dispose]);

	gui._create('frm_compose', 'frm_compose', '', '', newMessage);
};

OldMessage.prototype.quoteMessage = function(bRe) {
	var str = '';
	if (this.isHtml()){

		if (bRe)
		    str = '<br /><blockquote class="reply_block" style="border-left: 2px solid blue; margin-left: 1em; padding-left: 1em">';
		else
			str = '<br />';
		
		str +=	'<hr size="1" />'+getLang('EMAIL::REPLY_MESSAGE_HTML');
		str +=  '<br />'+getLang('EMAIL::FROM')+' '+this.getFrom().escapeHTML();
		if (this.getTo()) str +=  '<br />'+getLang('EMAIL::TO')+' '+this.getTo().escapeHTML();
		if (this.getCc()) str +=  '<br />'+getLang('EMAIL::CC')+' '+this.getCc().escapeHTML();
		str +=  '<br />'+getLang('EMAIL::DATE')+' '+(new Date(parseInt(this.getDate())*1000)).toWMString(false);
		if (this.getSubject()) str +=  '<br />'+getLang('EMAIL::SUBJECT')+' '+this.getSubject().escapeHTML();
		str +=	'<br /><br />' + this.getBody();

		if (bRe)
			str += '</blockquote>';
	}
	else{
		str =	"\r\n<br />"+getLang('EMAIL::REPLY_MESSAGE_HTML');
		str +=  "\r\n<br />"+getLang('EMAIL::FROM')+' '+this.getFrom().escapeHTML();
		if (this.getTo()) str +=  "\r\n<br />"+getLang('EMAIL::TO')+' '+this.getTo().escapeHTML();
		if (this.getCc()) str +=  "\r\n<br />"+getLang('EMAIL::CC')+' '+this.getCc().escapeHTML();
		str +=  "\r\n<br />"+getLang('EMAIL::DATE')+' '+(new Date(parseInt(this.getDate())*1000)).toWMString(false);
		if (this.getSubject()) str +=  "\r\n<br />"+getLang('EMAIL::SUBJECT')+' '+this.getSubject().escapeHTML();
		str +=	"\r\n<br />\r\n<br />" + this.getBody();
		
		//Add reply char
		if (bRe)
			str = str.replace(/<[\/]?(br)[^>^<]*>/gm,'<br \>&gt; ');
	}

	return str;
};

OldMessage.reply = function(id, bReplyToAll) {

	function extractMail(sMails, sMyMail) {
		if (!sMails) return '';

		var sMailsLow = sMails.toLowerCase();
		sMyMail = sMyMail.toLowerCase();

		var nPos = sMailsLow.indexOf(sMyMail);
		if (nPos == -1)
			return sMails;

		var aMails = [];
		var sMail = '';
		var bQuote = false;
		var sQuote;
		var bFind = false;
		var n = 0;

		while(n < sMails.length) {
			if (!bQuote) {
				if (sMails.charAt(n) == '"' || sMails.charAt(n) == "'") {
					bQuote = true;
					sQuote = sMails.charAt(n);
				} else {
				  	if (sMails.charAt(n) == ',' || sMails.charAt(n) == ';') {
						if (!bFind && sMail)
							aMails.push(sMail);

						sMail = '';
						bFind = false;
						n++;
						continue;
					} else if (nPos == n) {
						sMail = '';
						bFind = true;
						n += sMyMail.length;
						nPos = sMailsLow.indexOf(sMyMail,n);
						continue;
					}
				}
			} else {
				if (sMails.charAt(n-1) != '\\' && sMails.charAt(n) == sQuote)
					bQuote = false;
			}

			sMail = sMail + sMails.charAt(n);
			n++;
		}

		if (!bFind && sMail)
			aMails.push(sMail);

		return aMails.join(', ');
	}


	try{
		var oldMessage = new OldMessage(id);
		var newMessage = new NewMessage();
	}
	catch(er){
		return;
	}

	if (bReplyToAll) {
		var sTo = extractMail(oldMessage.getTo(), id[0]);
		newMessage.sTo = oldMessage.getReply() + (sTo ? ','+sTo : '');
		newMessage.sCc = oldMessage.getCc();
	}
	else
		newMessage.sTo = oldMessage.getReply();

	newMessage.sSubject = OldMessage.__prefixParser(oldMessage.getSubject(), 'Re');
	newMessage.setHtml(oldMessage.isHtml());
	newMessage.sBody = oldMessage.quoteMessage(true);

   	//if not embanded email
	if (id[2].indexOf('|')<0)
		newMessage.onSentCallback.push([oldMessage, oldMessage.setAnswered]);
		
	newMessage.onDisposeCallback.push([oldMessage, oldMessage.dispose]);
	newMessage.addSignature();

	gui._create('frm_compose', 'frm_compose', '', '', newMessage);
};

OldMessage.redirect = function(id) {
	var eForm = gui._create('frm_addaddress', 'frm_addaddress', '', '', [OldMessage.__redirect, [id]],
		{'to': "DATAGRID_ITEMS_VIEW::TO"}, {}, 'to', true
	);

	eForm.ok._onclick = function(){
		this._parent.__hide();
		this._parent.__ok = true;
		this._parent._onclose();
	};
};

OldMessage.whitelistSender = function(ids) {
	gui._create('frm_confirm', 'frm_confirm', '', '', [OldMessage.__blackOrWhiteList, [ids, true]],
		'CONFIRMATION::WHITELIST_SENDER_CONFIRMATION', 'CONFIRMATION::WHITELIST_SENDER'
	);
};

OldMessage.blacklistSender = function(ids) {
	gui._create('frm_confirm', 'frm_confirm', '', '', [OldMessage.__blackOrWhiteList, [ids, false]],
		'CONFIRMATION::BLACKLIST_SENDER_CONFIRMATION', 'CONFIRMATION::BLACKLIST_SENDER'
	);
};

OldMessage.deliver = function(ids) { OldMessage.__quarantineAction(ids, 'deliver'); };
OldMessage.whitelist = function(ids) { OldMessage.__quarantineAction(ids, 'whitelist'); };
OldMessage.blacklist = function(ids) { OldMessage.__quarantineAction(ids, 'blacklist'); };
OldMessage.__quarantineAction = function(ids, sAction){
	WMItems.quarantine({'aid': ids[0], 'fid': ids[1], 'iid': ids[2], 'action': sAction}, 'items', '', 'folders');
	Item.__removeFromDataset(ids);
};

/**
 * @param	id			[aid, fid, iid]
 * @param	sColor		OldMessage.RED .. OldMessage.CLEAR
 */
OldMessage.setColor = function(id, sColor) {
	var sOldColor = OldMessage.getColor(id);

	if (sOldColor && sOldColor != sColor) {
		// TODO rollback on error
		dataSet.add('items', id.concat(['COLOR']), sColor);
		WMItems.add(id, {'values': {'COLOR': sColor}}, 'items');
	}
};

OldMessage.getColor = function(id) { return dataSet.get('items', id.concat(['COLOR'])); };

OldMessage.open = function(id,bFullHeaders) {

	if (dataSet.get('mailview', id)) return;

	if (dataSet.get('folders',[id[0],id[1],'TYPE']) == 'M' || dataSet.get('folders',[id[0],id[1],'TYPE']) == 'R'){
		//if you try to open email which is unavailable already
		try{
			var message = new OldMessage(id,[]);
			// Async list item (refresh mailview) and refresh folders (folder tree)
			WMItems.list({"aid": id[0], "fid": id[1], "iid": id[2], "values": OldMessage.__MAILINFO_VALUES},'mailview', '', 'folders', [OldMessage.__mailviewCallback, [message, message.hasFlag('SEEN')]]);
		}
		catch(er){}
	}
	else
		WMItems.list({"aid": id[0], "fid": id[1], "iid": id[2], "values": OldMessage.__MAILINFO_VALUES},'mailview', '', 'folders');
};


OldMessage.openwindow = function(id) {
	var sDataSet = 'mailview_window';

    for(var no = 0;;no++)
		if (!dataSet.get(sDataSet+'/'+no)){
            sDataSet += '/'+no;
			break;
		}

	if (id[2].indexOf('|')>-1)
		WMItems.list(
			{"aid": id[0], "fid": id[1], "iid": id[2], "values": OldMessage.__FULLMAIL_VALUES},
			sDataSet);
	else{
		var message = new OldMessage(id,[]);
		WMItems.list(
			{"aid": id[0], "fid": id[1], "iid": id[2], "values": OldMessage.__FULLMAIL_VALUES},
			sDataSet,'','folders',[OldMessage.__mailviewCallback, [message, message.hasFlag('SEEN')]]);
	}

	gui._create('frm_mail', 'frm_mail', '','', sDataSet);
};


OldMessage.markAsRead = function(ids) {
	WMItems.setFlag({"aid": ids[0], "fid": ids[1], "iid": ids[2]}, {'SEEN': true}, 'items', 'folders');
};

OldMessage.markAsUnread = function(ids) {
	WMItems.setFlag({"aid": ids[0], "fid": ids[1], "iid": ids[2]}, {'SEEN': false}, 'items', 'folders');
};

OldMessage.__blackOrWhiteList = function(ids, bWhitelist) {
	var sFolder = (bWhitelist) ? 'SPAM_QUEUE/Whitelist' : 'SPAM_QUEUE/Blacklist';

	/*
	var curCount = parseInt(dataSet.get('folders', [sPrimaryAccount, sFolder, 'TOTAL']));
	if (!Is.Number(curCount))
		curCount = 0;
	*/

	for (var i in ids[2]) {
		var sender = dataSet.get('items', [ids[0], ids[1], ids[2][i], 'FROM']);
		if (!sender || !(sender = MailAddress.splitEmailsAndNames(sender))) continue;
		sender = sender[0]['email'];

		WMItems.add([sPrimaryAccount, sFolder], {'values': {'EMAIL': sender}}, 'items');
		//curCount++;
	}

	//dataSet.add('folders', [sPrimaryAccount, sFolder, 'TOTAL'], curCount);

	// Get SPAMbox
	var sSpam = (new gw_others).get('DEFAULT_FOLDERS','storage')['VALUES']['spam'];
        sSpam = sSpam.substr(sSpam.indexOf('/')+1);

	// Move to Inbox
	if (bWhitelist && ids[1] == sSpam)
		Item.__copyOrMoveItems (sPrimaryAccount, 'INBOX', 'move', ids);
	else
	// Move to Spam
	if (!bWhitelist && ids[1] != sSpam){
		if (!dataSet.get('folders',[sSpam]))
			dataSet.add('folders',[sSpam],{'TYPE':'M','RIGHTS':'rw','NAME':getLang('COMMON_FOLDERS::SPAM'),'ACCESS':'rwmd'});

		Item.__copyOrMoveItems (sPrimaryAccount, sSpam, 'move', ids);
	}
};

OldMessage.__mailviewCallback = function(message, bPrevSeen, mailInfo) {

	message.__synchronizeFlags(mailInfo[message.__id[0]][message.__id[1]][message.__id[2]]);

	if (!bPrevSeen && message.hasFlag('SEEN') && message.getConfirmAddress()) {
		gui._create('frm_confirm', 'frm_confirm', '', '',
			[OldMessage.__createReadingConfirmation, [message]],
			'CONFIRMATION::CREATE_READING_CONFIRMATION_TITLE',
			'CONFIRMATION::CREATE_READING_CONFIRMATION'
		);
	}
	else
		message.dispose();

	return;
};

OldMessage.prototype.__synchronizeFlags = function(mailInfo) {

	this.__mailInfo = mailInfo;
	var datasetItems = this.__datasetItems;

	function testAndSet(sItemName) {
		if (datasetItems[sItemName] != mailInfo[sItemName]) {
			datasetItems[sItemName] = mailInfo[sItemName];
			return true;
		}
		return false;
	}

	var bChanged = false;
	bChanged |= testAndSet('FLAGS');
	bChanged |= testAndSet('HAS_ATTACHMENT');
	bChanged |= testAndSet('SMIME_STATUS');
	bChanged |= testAndSet('PRIORITY');

	if (bChanged && dataSet.get('items', this.__id))
		dataSet.update('items', this.__id);
};


OldMessage.__redirect = function(bOK, aAddresses, id, eForm) {
	if (bOK && eForm) {
		var aMessageInfo = {"aid": id[0], "fid": id[1], "iid": id[2]};
		var aResult = MailAddress.findDistribList({'to': aAddresses['to'].join(', ')});
		for (var sTag in aResult)
			aMessageInfo[sTag] = aResult[sTag];


		WMItems.redirect(aMessageInfo, 'dummy','',[OldMessage.__redirect_result,[eForm]]);
	}
};

OldMessage.__redirect_result = function(eForm,aError){

	eForm.__ok = false;

	if (aError){
		eForm.__show();
		if (aError[0])
            alert (getLang('ALERTS::'+aError[0].toUpperCase())+"\n"+aError[1]);
		else
			alert (aError[1]);
	}
	else
		eForm._destruct();
};

/**
 * read confirmation
 **/
OldMessage.__createReadingConfirmation = function(oldMessage) {

	function setMessageVariables(sText, oldMessage) {
		var date = new Date(oldMessage.getDate()*1000);
		var sdate = date.toGMTString();
		var from = oldMessage.getFrom() ? oldMessage.getFrom() : '';
		var to = oldMessage.getTo() ? oldMessage.getTo() : '';
		var subject = oldMessage.getSubject() ? oldMessage.getSubject() : '';
		sText = sText.replace(/%FROM%/g, from).replace(/\n/g,"<br />").replace(/%TO%/g, to).replace(/%DATE%/g, sdate).replace(/%SUBJECT%/g, subject);

		//FormatDateTime parsing
		sText = sText.replace(/%FormatDateTime ([^%]+)[^%]*%/gm, function($0,$1){ return dateFormat(date, $1); });

		return sText;
	}

	var newMessage = new NewMessage();

//	newMessage.sFrom = getPrimaryAccountFromAddress();//oldMessage.getTo();
	newMessage.sTo = oldMessage.getFrom();
	newMessage.sSubject = setMessageVariables(GWOthers.getItem('READ_CONFIRMATION', 'subject'), oldMessage);
	newMessage.sBody = setMessageVariables(GWOthers.getItem('READ_CONFIRMATION', 'text'), oldMessage);

	oldMessage.dispose();

	newMessage.send(false,'',{'read_confirmation':false,'priority':4});
};

OldMessage.prototype.getFrom = function() { return toString(this.__datasetItems.FROM); };

OldMessage.prototype.getReply = function() {
	if (this.__mailInfo.REPLY_TO)
		return toString(this.__mailInfo.REPLY_TO);
	else
		return this.getFrom();
};
OldMessage.prototype.getTo = function() {
	return toString(this.__datasetItems.TO);
};
OldMessage.prototype.getSubject = function() { return toString(this.__datasetItems.SUBJECT); };
OldMessage.prototype.getFlags = function() { return this.__datasetItems.FLAGS; };
OldMessage.prototype.getDate = function() { return this.__datasetItems.DATE; };
OldMessage.prototype.getSize = function() { return this.__datasetItems.SIZE; };
//OldMessage.prototype.hasAttachments = function() { return (this.__datasetItems.HAS_ATTACHMENT == 'true'); };

OldMessage.prototype.hasAttachments = function() { return (this.__mailInfo.HAS_ATTACHMENT == 'true'); };
OldMessage.prototype.getCc = function() { return toString(this.__mailInfo.CC); };
OldMessage.prototype.getBcc = function() { return toString(this.__mailInfo.BCC); };
OldMessage.prototype.getBody = function() { return toString(this.__mailInfo.HTML); };
OldMessage.prototype.isHtml = function() { return (this.getStaticFlags() & 1) ? true : false; };
OldMessage.prototype.getAttachments = function() { return this.__mailInfo.ATTACHMENTS; };
OldMessage.prototype.getStaticFlags = function() { return this.__mailInfo.STATIC_FLAGS; };
OldMessage.prototype.getConfirmAddress = function() { return toString(this.__mailInfo.CONFIRM_ADDR); };

OldMessage.prototype.copyAttachments = function(id) {
	var result = {'attachments': []};

	if (this.hasAttachments()) {
		var aOldAttachments = this.getAttachments();
		var aAttachFrame, aAttachResult;

		for(var sAttId in aOldAttachments) {
			aAttachFrame = aOldAttachments[sAttId]['values'];
			aAttachResult = {};
			aAttachResult['class'] = 'attachment';
			aAttachResult['fullpath'] = id[0] + '/' + id[1] + '/' + id[2] +'/' + sAttId;

			for(var sTag in aAttachFrame)
				aAttachResult[sTag.toLowerCase()] = aAttachFrame[sTag];

			result['attachments'].push({'values': aAttachResult});
		}
	}

	return result;
};

OldMessage.prototype.hasFlag = function(sFlagName) {
	var nFlag;
	if ((nFlag = this.getFlags()) == undefined) return false;

	return WMItems.hasFlag(nFlag, sFlagName);
};

OldMessage.prototype.setFlags = function(aFlagNames) {
	var bUpdateFolders = false;
	var aFlagPairs = {};

	for (var i in aFlagNames) {
		if (aFlagNames[i] == 'SEEN')
			bUpdateFolders = true;

		aFlagPairs[aFlagNames[i]] = true;
	}


	if (bUpdateFolders)
		WMItems.setFlag({"aid": this.__id[0], "fid": this.__id[1], "iid": [this.__id[2]]}, aFlagPairs, 'items', 'folders');
	else
		WMItems.setFlag({"aid": this.__id[0], "fid": this.__id[1], "iid": [this.__id[2]]}, aFlagPairs, 'items');
};

OldMessage.prototype.setAnswered = function() {
	if (!this.hasFlag('SEEN'))
		this.setFlags(['ANSWERED', 'SEEN']);
	else
		this.setFlags(['ANSWERED']);
};

OldMessage.prototype.setForwarded = function() {
	if (!this.hasFlag('SEEN')) {
		this.setFlags(['FORWARDED', 'SEEN']);
	} else {
		this.setFlags(['FORWARDED']);
	}
};

OldMessage.prototype.dispose = function() {
	dataSet.disobey(this);
};

OldMessage.__prefixParser = function(sSubject, sType) {
	if (typeof sSubject == 'undefined')
		sSubject = '';

	var aSubjectSplit = sSubject.split(':');
	var sLast = sType;
	var sLastLow = (sLast ? sLast.toLowerCase() : '');
	var nCounter = 1;
	var aResult = [];
	var nInArray = count(aSubjectSplit);
	var nItem = 0;

	var aParcSplit,sShort,sShortLow,nNumber,bEnd;

	for(var n in aSubjectSplit) {
		if (bEnd) {
			aResult.push(aSubjectSplit[n].trim());
		} else {
			aParcSplit = aSubjectSplit[n].split('[');
			sShort = aParcSplit[0].trim();
			sShortLow = (sShort ? sShort.toLowerCase() : '');
			nItem++;

			if (aParcSplit[1]) {
				nNumber = aParcSplit[1].substr(0,aParcSplit[1].indexOf(']')).trim()*1;
			} else {
				nNumber = 1;
			}

			if (sShort.length < 2 || sShort.length > 3 || isNaN(nNumber) || nNumber <= 0 || nItem == nInArray) {
				if (sLast) {
					if (nCounter>1) {
						aResult.push(sLast+'['+nCounter+']');
					} else {
						aResult.push(sLast);
					}
				}

				aResult.push(aSubjectSplit[n].trim());
				bEnd = true;
			} else {
				if (sLastLow == sShortLow) {
					nCounter += nNumber;
				} else {
					if (sLast) {
						if (nCounter>1) {
							aResult.push(sLast+'['+nCounter+']');
						} else {
							aResult.push(sLast);
						}
					}

					sLast = sShort;
					sLastLow = sShortLow;
					nCounter = nNumber;
				}
			}
		}
	}
	return aResult.join(': ');
};

OldMessage.prototype.__update = function() {
	var datasetItems = dataSet.get('items', this.__id);
	if (datasetItems)
		this.__datasetItems = datasetItems;
};