/*********** DataSet *************
 * @brief : Data storage provides binding functions between objects and data
 * @status: final
 * @date  : 6.3.2006 19:09:40
 *
 * pridat DUMP metodu
 **/
function cDataSet(){
	this.aDataSets = {};    // data storage (public)
	this.aListeners = {};   // listeners storage (private)
};

/**
 * @brief: add new data into storage
 * @scope: PUBLIC
 * @param: Data      - array or string data
 *         sDName    - dataset name
 *         aDPath    - array containing path ['folder','subfolder',...] dataset.folder.subfodler
 *         bNoUpdate - do not call update method after change
 *         sUpdater  - name of caller (this object will not be updated)
 *
 * @date : 26.5.2006 15:15:41
 **/
cDataSet.prototype.add = function(sDName,aDPath,Data,bNoUpdate,sUpdater) {

	var bDiffer = !compareObj(this.get(sDName,aDPath),Data,true);

	if (Is.Array(aDPath)){
		// create array path structure in dataset
		this.aDataSets[sDName] = mkArrayPath(aDPath,this.aDataSets[sDName]);
		try {
			// store data
			var sStr = 'this.aDataSets["'+ sDName +'"]["' + aDPath.join('"]["') +'"]';
			eval(sStr + '= Data;');
		}
		catch(e){
			return false;
		}
	}
	// store data
	else
		this.aDataSets[sDName] = Data;

	// check if new data are different to originals
	if (bDiffer){
		if (!bNoUpdate) this.update(sDName,aDPath,sUpdater);
		return true;
	}
	else
		return false;
};

/**
 * @brief: remove data from storage
 * @scope: PUBLIC
 * @date : 26.5.2006 15:15:47
 **/
cDataSet.prototype.remove = function(sDName,aDPath,bNoUpdate,sUpdater) {

	if (!aDPath)
		delete this.aDataSets[sDName];
	else
	if (Is.Array(aDPath))
		try {
			eval('delete this.aDataSets["'+ sDName +'"]["' + aDPath.join('"]["')+'"]');
		}
		catch(e){
			return false;
		}

	if (!bNoUpdate)
		this.update(sDName,aDPath,sUpdater);
		
	return true;
};

/**
 * @brief: retrieve data from storage
 * @scope: PUBLIC
 * @data : 20.4.2006 15:08:58
 **/
cDataSet.prototype.get = function (sDName,aDPath) {
	if (!sDName) return '';
	if (Is.Array(aDPath)){
		try {
			var sStr = 'this.aDataSets["'+ sDName +'"]["' + aDPath.join('"]["')+'"]';
			var aData = eval(sStr);
			return aData;
		}
		catch(e){
			return '';
		}
	}
	else{
		return this.aDataSets[sDName];
	}
};

/**
 * @brief: add object as listener for data storage, Listening obj. must contains __update() method!
 * @scope: PUBLIC
 * @data : 15.4.2006 15:08:58
 **/
cDataSet.prototype.obey = function(oListener,sObjMethod,sDName,bNoUpdate) {
	if (typeof oListener != 'object' || !sDName)
		return false;

	oListener[sObjMethod || '_listener'] = sDName;

	if (!this.aListeners[sDName]) this.aListeners[sDName] = [];
	this.aListeners[sDName].push(oListener);

	if (typeof this.aDataSets[sDName] != 'undefined' && !bNoUpdate)
		oListener.__update(sDName);

	return true;
};

/**
 * @brief: remove object from listeners
 * @scope: PUBLIC
 * @data : 15.4.2006 15:08:58
 **/
cDataSet.prototype.disobey = function(oListener,sProperty) {
	if (!sProperty) sProperty = '_listener';
	if (typeof oListener != 'object' || !oListener[sProperty] || !this.aListeners[oListener[sProperty]]) return false;

	var i;
	if ((i = inArray(this.aListeners[oListener[sProperty]],oListener))!=-1)
		delete this.aListeners[oListener[sProperty]][i];

	return true;
};

/**
 * @brief: execute update() method in all binded listeners
 * @scope: PUBLIC
 * @data : 26.6.2006 10:50:34
 **/
cDataSet.prototype.update = function(sName,aDPath,sUpdater) {

	if (!sName || !this.aListeners[sName]) return;
		for(var i in this.aListeners[sName]) {
			try {
				var o = this.aListeners[sName][i];
				if (sUpdater) continue;
				this.aListeners[sName][i].__update (sName, aDPath);
		}
		catch(e){}
	}
};

//////////// GLOBAL ////////////////
dataSet = new cDataSet();