_me = frm_add_folder.prototype;
function frm_add_folder(){};

/**
 * @brief   Form for adding folder.
 * Form has three input objects:
 *    - text input for name of the folder (folder must
 *    match this regular expr. /^ *[a-zA-Z0-9#\.\_\-]+[ a-zA-Z0-9#\.\_\-]*$/).
 *    - select box for folder type specification (mail, contact, event, journal, note, task).
 *    - folder three.
 * 'OK' button is active only if the name of the folder match the regular expr. above. 
 *     
 * @param[in]  sAccountID  [string] Only folders from this account will be listed because of
 * moving folders between accounts is disabled. 
 * @param[in]  sFolderID   [string] Optional. Forder with this ID will be preselected. 
 * 
 * @see  objects/frm_add_folder.xml
 * @see  inc/frm_add_folder.js
 */
_me.__constructor = function(sAccountID, sFolderID){
	var me = this;

	if (!Is.Defined(sAccountID)) return;

	this._title('FORM_FOLDERS::CREATE_NEW_FOLDER');
	this._size(300,400,true);

	// Create formular from template
	this._draw('frm_add_folder', 'main');
	this.x_btn_ok._disabled(true);

	function test (){
        var isError = me.input_name._checkError.length>0;
		if (!isError && me.select_type._value() == 'R' && me.input_channel._checkError.length>0)
		    isError = true;

		me.x_btn_ok._disabled(isError);
	};

    this.select_type._onchange = function(){
		this._parent.input_channel._disabled(this._value()!='R');
		test();
	};

	this.input_channel._onerror = function(isError) {
		test();
	};

	this.input_name._onerror = function(isError) {
        test();
	};
	
	this.input_name._onsubmit = function(){
        test();
        if (!me.x_btn_ok._disabled())
        	me.x_btn_ok._onclick();
	};
	this.input_channel._onsubmit = function(){
        test();
        if (!me.x_btn_ok._disabled())
        	me.x_btn_ok._onclick();
	};


    var sDefaultType;
	if (Is.Defined(sFolderID))
		// Preselect combo with parent type as default
		sDefaultType = dataSet.get('folders',[sAccountID,sFolderID,'TYPE']);
	else
	if (sAccountID == sPrimaryAccount+'_rss')
        sDefaultType = 'R';
        
        
	// Preselect folder
	sFullFolderPath = sAccountID + ((sFolderID) ? '/'+sFolderID : '');

	//Fill TYPE option
	var aFill,Typ;
	if (sPrimaryAccountGW>0){
		var dgw = GWOthers.getItem('RESTRICTIONS', 'disable_gw_types');

		aFill = {};
		//if (sDefaultType!='A'){}
			aFill.M = getLang('FOLDER_TYPES::MAIL');
			Typ = 'M';

		if(!dgw || dgw.indexOf('c')<0){
			aFill.C = getLang('FOLDER_TYPES::CONTACT');
			Typ = Typ || 'C';
		}
		if(!dgw || dgw.indexOf('e')<0){
			aFill.E = getLang('FOLDER_TYPES::EVENT');
			Typ = Typ || 'E';
		}
		if(!dgw || dgw.indexOf('j')<0){
			aFill.J = getLang('FOLDER_TYPES::JOURNAL');
			Typ = Typ || 'J';
		}
		if(!dgw || dgw.indexOf('n')<0){
			aFill.N = getLang('FOLDER_TYPES::NOTE');
			Typ = Typ || 'N';
		}
		if(!dgw || dgw.indexOf('t')<0){
			aFill.T = getLang('FOLDER_TYPES::TASK');
			Typ = Typ || 'T';
		}
		if(!dgw || dgw.indexOf('f')<0){
			aFill.F = getLang('FOLDER_TYPES::FILE');
			Typ = Typ || 'F';
		}
		if(!dgw || dgw.indexOf('r')<0){
			aFill.R = getLang('FOLDER_TYPES::RSS');
			Typ = Typ || 'R';
		}
	}
	else{
		aFill = {M:getLang('FOLDER_TYPES::MAIL')};
		Typ = 'M';
	}

	this.select_type._fill(aFill);
	this.select_type._value(sDefaultType && "MRCEJNTF".indexOf(sDefaultType)>-1?sDefaultType:Typ);


	//if folder is not listed then select account
	if (!this.tree_folder._setActive(sFullFolderPath)){
		sFullFolderPath = sAccountID;
		this.tree_folder._setActive(sFullFolderPath)
	}


	//this.tree_folder._openup('minus',sFullFolderPath);

	// Disable/Enable 'OK' button depending on reg. expr. of folder name. Initial
	// value of obj_input is empty, so initially 'OK' button should be disabled.
	// TODO dodelat restrictions
	// this.x_btn_ok._disabled(this.input_name.__check());

	// This function is triggered when 'OK' button is pressed
	this.x_btn_ok._onclick = function() {

		var aPath = Path.split(me.tree_folder.__activeNode);
		var sAccountName = aPath[0];
		var sFolderName = aPath[1];

		test();
		if (this._disabled())
		    return false;

		if (sFolderName && !dataSet.get('folders',[sAccountName, sFolderName])){
			me.tree_folder.__activeNode = '';
			alert(getLang('FORM_FOLDERS::CREATE_ERROR'));
            return;
		}

		sFolderName = ((sFolderName) ? sFolderName+'/' : '') + me.input_name._value().trim();

		if (!dataSet.get('folders',[sAccountName,sFolderName])){
			var sFolderType = me.select_type._value();
			var out = {'name':sFolderName,'type':sFolderType};

			if (sFolderType == 'R'){
				out.aid = sPrimaryAccount+'_rss';
				if (out.aid != sAccountName)
					out.name = me.input_name._value();

				out.channel = me.input_channel._value();
			}
			else
				out.aid = sAccountName;

			folders.add(out,'folders');

			me._destruct();
		}
		else
		    alert(getLang('ALERTS::FOLDER_EXIST'));
	}
};