_me = frm_edit_account.prototype;
function frm_edit_account(){};

/**
 * @brief   Creates "Edit account" or "Add account" dialog.
 * @param[in]  sAccountID [string]   Optional. Account to be edited. When not specified,
 * form for new account is created. 
 */ 
_me.__constructor = function(aResponse, sAccountID, aData) {
	var me = this;

	this._modal(true);
	this.__sAccountID = sAccountID;
	this.__aResponse  = aResponse;
	
	if (Is.Defined(sAccountID)) {
		this._title('POPUP_ACCOUNTS::EDIT_ACCOUNT');
		this._draw('frm_edit_account', 'main');
	} else {
		this._title('POPUP_ACCOUNTS::ADD_ACCOUNT');
		this._draw('frm_add_account', 'main',{sPrimaryAccount:"!^"+sPrimaryAccount.quoteMeta().quoteMeta()+"$"});
	}

	this._size(400,'',true);

    // Create 'OK' button
	this._create('x_btn_ok', 'obj_button', 'footer');
	this.x_btn_ok._value('FORM_BUTTONS::OK');
	this.x_btn_ok._onclick = function() {
		this._disabled(true);
		me._title('POPUP_ACCOUNTS::VALIDATING');
        setTimeout('try{'+me._pathName+'.__validate()}catch(e){}', 10);
	}

	// Create 'CANCEL' button
	this._create('x_btn_cancel', 'obj_button', 'footer');
	this.x_btn_cancel._value('FORM_BUTTONS::CANCEL');
	this.x_btn_cancel._onclick = function() { me._destruct(); }


	if (Is.Defined(sAccountID) && Is.Defined(aData))
		this._loadSettings(aData);
	else
		// Set port depending on protocol (IMAP => 143, POP3 => 110)
		this._setDefaultPort();


	// Set callback functions (onerror and onchange)
	if (!Is.Defined(sAccountID)) this.EMAIL._onerror = function(isError) { me.__bIsInputEmailValid = !isError; me.__enableOrDisableOK(); };
	this.USERNAME._onerror = function(isError) { me.__bIsInputUsernameValid = !isError; me.__enableOrDisableOK(); };
	this.PROTOCOL._onchange = function() { me._setDefaultPort(); }
	this.SERVER._onerror = function(isError) { me.__bIsInputServerValid = !isError; me.__enableOrDisableOK(); }
	this.PORT._onerror = function(isError) { me.__bIsInputPortValid = !isError; me.__enableOrDisableOK(); }
	this.PASSWORD._onerror = function(isError) { me.__bIsInputPassValid= !isError; me.__enableOrDisableOK(); }
	
	// Get states of inputs (valid/invalid)
	this.__bIsInputEmailValid = (!sAccountID) ? this.EMAIL.__check() : true;
	this.__bIsInputUsernameValid = this.USERNAME.__check();
	this.__bIsInputServerValid = this.SERVER.__check();
	this.__bIsInputPortValid = this.PORT.__check();
	this.__bIsInputPassValid = sAccountID ? true : false;
	
	// Disable/Enable 'OK' button depending on input validity.
	this.__enableOrDisableOK();
};

_me.__validate = function(){

	if(WMAccounts.test({EMAIL: this.__sAccountID || this.EMAIL._value(),USERNAME:this.USERNAME._value(),PROTOCOL:this.PROTOCOL._value(),SERVER:this.SERVER._value(),PORT:this.PORT._value(),PASSWORD:this.PASSWORD._value()})){
		var aValues = {};
		storeDataFromForm(this, aValues);
		executeCallbackFunction(this.__aResponse, aValues,  this.__sAccountID);
		this._destruct();
	}
	else{
		alert(getLang('POPUP_ACCOUNTS::INVALID'));

		if (Is.Defined(this.__sAccountID))
			this._title('POPUP_ACCOUNTS::EDIT_ACCOUNT');
		else
			this._title('POPUP_ACCOUNTS::ADD_ACCOUNT');
			
		this.x_btn_ok._disabled(0);
	}
};

/**
 * @brief   Enable or disable OK button depending on inputs validity.   
 */
_me.__enableOrDisableOK = function() {
	var b = !(this.__bIsInputEmailValid && this.__bIsInputUsernameValid &&
			this.__bIsInputServerValid && this.__bIsInputPortValid && this.__bIsInputPassValid);
			
	//this.x_btn_test._disabled(b);
	this.x_btn_ok._disabled(b);
};

/**
 * @brief   Set default port depending on selected protocol.
 */ 
_me._setDefaultPort = function() {

	switch (this.PROTOCOL._value()) {
		case 'imap':
			this.PORT._value(143);
			this.PORT.__check();
			break;
		case 'pop3':
			this.PORT._value(110);
			this.PORT.__check();
			break;
	}
};

/**
 * @brief   Load settings from account into form.
 * @param[in]   sAccountID  [string]    Account from which the settings should be loaded.
 */
_me._loadSettings = function(aValues) {
	loadDataIntoForm(this, aValues);
};