/**
 * @brief : Graphic User Interface builder
 *          Abstract class for GUI Objects, instances of this class
 *          constructs object tree of whole application
 *
 * @date  : 25.3.2006 16:36:27
 * @status: in development
 *
 * - load page
 * - load css
 * - load js
 * - create container??
 * - load template
 * - exe template
 * - PRINT as invisible
 * - exe JS
 *
 * DODELAT
 *   - zmena xml structury: zrusit onload/extension|function a pridat root/extend + root/onload
 *
 */

_me = cObject.prototype;
function cObject(sName,sType,oParent){
	// PUBLIC
	this._name		= sName;
	this._type		= sType || 'document';
	this._parent	= oParent;
	this._pathName	= this.__genPathName();

	if (!sType && !oParent){

		var home = mkElement('div',{id:sName});
		    home.style.width = "100%";
		    home.style.height = "100%";
		    home.style.overflow = "hidden";
			document.getElementsByTagName('body')[0].appendChild(home);
	        home = null;
		//document.getElementsByTagName('body')[0].id = sName;
		
		var me = this;

		//DOCUMENT EVENT Handlers
		function evn (e){
		    var e = e || window.event;
			me.__exeEvent(e.type,e);

			//stops F5 (refresh) propagation to browser
			//it caused unwanted refresh(es) because of placement in NB keyboard
			if ((e.type == 'keydown') && e.keyCode == 116){
				e.cancelBubble = true;
				e.preventDefault();
				try{e.stopPropagation();}catch(r){}
				return false;
			}
		};
		document.onclick = evn;
		document.onmousedown = evn;
		document.onmouseup = evn;
		document.onmousemove = evn;
		document.onkeydown = evn;
		document.onkeyup = evn;
		
        this.__loading_obj;			//path to loading obj
		this.__loading_counter = 0; //loading counter
		this._loading = function(b){
			if (this.__loading_obj){
				if (b)
					this.__loading_counter++;
				else
				if (this.__loading_counter>0)
	            	this.__loading_counter--;

				try{
					this.__loading_obj._loading(this.__loading_counter);
				}
				catch(r){}
			}
			else
				this.__loading_counter = 0;
		};
	}

	// for main gui obj. only
	this._anchor;
	this._anchors  = {'main':this._pathName};
	this._template = '';

	// PRIVATE
	/* destructors */
	this._destructors = [];		// array of destructor methods           Predelat na PROTECTED!!

	/* dataSet properties */
	this._listener;				// dataset listener
	this._listenerPath;			// Array - dataset listener path (optional)

	/* saving properties */
	this._saver = null;			// dataset for storage obj value
	this._saverPath = null;		// Array - dataset path for storage obj value (optional)
	this._skipsaving = false;	// do not save obj. value at all
	this._noupdate = false;		// update dataset after _saveme()

	/* update or refresh */
	this._norefresh = false;	// in update, fill methods
	this._updateBuffer = false;	// True if there was any update during "norefresh" state



	/* events handlers */
	this._events = {};
};

/**
 * getPos() is unable to retrieve scroll offset, so this recursove function collect offsets of parent objects
 * All objects with scrollbars in it must have __scrolloffset([anchor]) method *
 **/
_me._getScrollPosition = function(){
    var x=0,y=0,tmp,obj = this._parent;
    var sAnchor = this._anchor;
	do{
		if (obj.__scrolloffset){
			tmp = obj.__scrolloffset(sAnchor);
			x += tmp.x;
			y += tmp.y;
		}
		sAnchor = obj._anchor;
	}
	while((obj = obj._parent));
	
	return {x:x,y:y};
};

/**
 * create object structure
 * !! if object is unique than existing instance is destroyed. May be Attr destroy=true for <unigue>...
 */
_me._create = function (sName,sType,sTarget,sClass){

	/* load object XML descriptor from storage cache */
	var aObj  = storage.object(sType);

	/* check parent type for allowed type */
	if (aObj['PARENTS']){
		var str,er = true,pt = this._type;
		for (var i in aObj['PARENTS'][0]['OBJ']){
			str = aObj['PARENTS'][0]['OBJ'][i]['VALUE'];
			if (pt == str){
				er = false;
				break;
			}
		}
		if (er){
			throw new Error("gui._create() -  OBJ "+ sType +"\n disallowed parent "+ pt);
			return false;
		}
	}

	/* create name for new cObject instnce */
	if (aObj['UNIQUE'] && typeof(aObj['UNIQUE'][0]['VALUE']) != 'undefined'){
		if (typeof this[sName]!='undefined') {
			this[sName]._destruct();
			delete this[sName];
		}
	}
	else
		sName = this.__genName(sName);

	/* create clone of cObject class */
	var oNew = this[sName] = new cObject(sName,sType,this);

	/* add anchors pointes (as text representations) */
	if (aObj['ANCHORS'] && typeof(aObj['ANCHORS'][0]['ELM']) != 'undefined'){
		var sAnchor = '',aAnchors = {};
		for (var i in aObj['ANCHORS'][0]['ELM']){
			sAnchor = aObj['ANCHORS'][0]['ELM'][i]['VALUE'];
			aAnchors[sAnchor] = this._pathName + "." + sName +sAnchor;
		}
		oNew['_anchors'] = aAnchors;
	}

	/* create _main htmlobj in parent's anchor */
	if (!sTarget || !this._anchors[sTarget]) sTarget = 'main';

	try{
		var main = this._getAnchor(sTarget);
	}
	catch(e){
		throw Exception('Anchor "'+ sTarget +'" doesn\'t exists in "'+ this._type +'" object');
	}

	oNew['_anchor'] = sTarget;

	/* TYPE */
	var chld;
	if (!aObj['TYPE']) aObj['TYPE'] = [{'VALUE':'block'}];

	switch(aObj['TYPE'][0]['VALUE']){
	case 'inline':
		chld = mkElement('span');
		break;
	case 'tr':
		chld = mkElement('tr');
		break;
	case 'td':
		chld = mkElement('td');
		break;
	case 'form'	:
		chld = mkElement('form');
		chld.onsubmit = function(){return false;}
		chld.name = oNew._pathName;
		chld.style.margin = 0;
		chld.style.padding = 0;
		break;
	default: chld = mkElement('div');
	}
	chld.id = oNew._pathName;

	oNew['_main'] = chld;
	oNew['_css'] = sClass;
	
	//must be before className because of MSIE6 slowdown
	main.appendChild(chld);

	/*	Container's CSS can be set as name attr. of <object> tag
		"<type>" is always used as class descriptor */
	chld.className = (aObj['ATTRIBUTES'] && aObj['ATTRIBUTES']['CSS']?aObj['ATTRIBUTES']['CSS']+' ':'') + sType + (sClass?' '+sClass:'');

	main = chld = null;

	/* add template name (just template name from XML) */
	if (aObj['TEMPLATE'] && typeof(aObj['TEMPLATE'][0]['VALUE']) != 'undefined'){
		oNew['_template'] = aObj['TEMPLATE'][0]['VALUE'];
		/* DRAW */
		if (Is.Defined(this._parent) && Is.Defined(this._parent._aTemplateData))
			oNew._draw(null,null,this._parent._aTemplateData);
		else if (Is.Defined(this._aTemplateData))
			oNew._draw(null,null,this._aTemplateData);
		else
			oNew._draw();
	}

	/* Prepare Arguments for __construcor */
	if (arguments.length>4){
		var sArg=[],n=0,aArg = [];
		for (var i = 4;i<arguments.length;i++){
			aArg.push(arguments[i]);
			sArg.push('aArg['+n+']');
			n++;
		}
		sArg = sArg.join(',');
	}
	else{
		var aArg = null;
		var sArg = '';
	}

	/* inherit defined methods from extensions and execute their __constructor methods */
	if (aObj['LIBRARY']){
		var obj,sobj;
		for (var i in aObj['LIBRARY']){

			//try{
			if (aObj['LIBRARY'][i]['ATTRIBUTES'] && aObj['LIBRARY'][i]['ATTRIBUTES']['CLASS'])
				sobj = aObj['LIBRARY'][i]['ATTRIBUTES']['CLASS'];
			else
				sobj = aObj['LIBRARY'][i]['VALUE'];

			obj = eval(sobj);
			if (typeof obj == 'function'){
				inherits(oNew,obj);
				if (typeof obj.prototype.__constructor != 'undefined')
					eval("oNew.__constructor("+sArg+")");
			}
			//}catch(e){}
			obj = null;
		}
	}

	/* add destructors
	may be created before obj. inicialization...!? */
	if (aObj['ONUNLOAD']){
		for (var i in aObj['ONUNLOAD'])
			oNew._add_destructor(aObj['ONUNLOAD'][i]['VALUE'],aObj['ONUNLOAD'][i]['ATTRIBUTES']);
	}

	/*	execute onload functions
		example: fname(this[sName]) - for posting current object into onload function */
	if (aObj['ONLOAD']){
		for (var i in aObj['ONLOAD']){
			try {
				eval(aObj['ONLOAD'][i]['VALUE']);
			}
			catch(e){
				throw Exception("Cant execute onload event:\r\n" + aObj['ONLOAD'][i]['VALUE']);
			}
		}
	}

	return oNew;
};


/**
 * @brief: obj name generator, useful for generated forms etc.
 * @scope: PRIVATE
 * @date : 1.5.2006 17:51:44
 **/
_me.__genName = function(name){
	var sApx = '';
	for(var i=0;;i++){
		if(typeof this[name+sApx] == 'undefined') return name+sApx;
		sApx = "_"+i;
	}
};


/**
* @brief: obj path generator, path is used for HTML elm ID and is passed into template as {_ins}
* @scope: PRIVATE
* @date : 1.5.2006 17:53:31
**/
_me.__genPathName = function(){
	var sPN = this._name;
	if (this._parent && this._parent._pathName)
		sPN =  this._parent._pathName + '.' + sPN;   // jeste ui anchoru
	return sPN;
};

	 /**
	  * @param: sTmpName  - optional, name of template
	  * 	    Anchor    - optional, can by string or directly DOM element
	  **/
	_me._draw = function (sTmpName,Anchor,aData){

        if (typeof aData != 'object')
            aData = {"_ins":this._pathName};
        else
			aData._ins = this._pathName;
            
		/* retrieve template */
		var sHtml = template.tmp((sTmpName?sTmpName:this._template),aData);

/*
if (window.console)
	window.console.log((sTmpName?sTmpName:this._template) +' '+ sHtml);
else
	alert("Temlplate "+(sTmpName?sTmpName:this._template)+"\n"+sHtml.escapeHTML()+"\n*");
*/
	
		/* parse <OBJ>
		   <obj> can be parsed from gui objects only because of _ins var requirement */
		var id,obj = [],tmpObj = [],MSIE,xTpl;
		if (sHtml.indexOf('<obj ')>-1){

			// MSIE works as XML
			if (typeof ActiveXObject !='undefined'){//SAFARI currentBrowser()=='Safari' window.console.log(sHtml);
                MSIE = true;
				xTpl = XMLTools.Str2XML('<root>'+ sHtml.replace(/\&/g,'&amp;') +'</root>');
    		}
			// Others as XHTML
			else{
				MSIE = false;
				xTpl = mkElement('backquote');

				//Safari title tag hack
				if (currentBrowser()=='Safari' && sHtml.indexOf('<title')>-1)
					sHtml = sHtml.replace(/\<title/ig,'<safari_title');

				xTpl.innerHTML = sHtml;
			}

			var aList = xTpl.getElementsByTagName("obj");

			if (MSIE && !aList.length) throw Exception("Syntax error in template: "+(sTmpName?sTmpName:this._template));

			var ep,etmp,sType,sName,sCSS,sWidth,sHeight,iTabindex,oid = 0;

				/* PRIVATE, <ITEM> -> Array parser
				   @param: etmp - parent dom element */

				function parseitem(etmp,bDebug){
					var etmp2,key,out = [],n = etmp.getElementsByTagName("item");

					if (!n.length){
						n = null;
						return etmp.textContent || (typeof etmp.text == 'string'?etmp.text.unescapeHTML():'');
					}
					for (var i = 0,l = n.length; i < l ;i++){
						if  ((key = n[i].getAttribute('key')))
							out[key] = parseitem(n[i]);
						else
							out.push(parseitem(n[i]));
					}
					n = null;
					return out;
				};
				


			var prevObj = [];
			for (var i =0 ;i < aList.length;i++){

				ep = aList[i].parentNode;
				sType = aList[i].getAttribute('type');
				sName = aList[i].getAttribute('name');

				if (!sName || !sType) continue;

				// nasted <obj>
				if (ep.tagName.toLowerCase() == 'obj'){
					sAnchor = aList[i].getAttribute('anchor');
					sAnchor = sAnchor || 'main';
				}
				// <obj> in template
				else{
					// set id to parent HTML elm. & create anchor in parent gui object
					if ((id = ep.getAttribute('id'))){
						sAnchor = inArray(this._anchors,id);
						if (sAnchor == -1){
							sAnchor = Math.rand();
							this._anchors[sAnchor] = id;
						}
					}
					else{
						id = (this._pathName || '') + Math.rand();
						ep.setAttribute('id',id);
						sAnchor = Math.rand();
						this._anchors[sAnchor] = id;
					}
				}

				// CREATE OBJ FOR ARRAY
				tmpObj = {"type":sType,"name":sName,"anchor":sAnchor};

				if ((sCSS = aList[i].getAttribute('css'))) tmpObj['css'] = sCSS;
				if ((sWidth = aList[i].getAttribute('width'))) tmpObj['width'] = sWidth;
				if ((sHeight = aList[i].getAttribute('height'))) tmpObj['height'] = sHeight;
				if ((iTabindex = aList[i].getAttribute('tabindex'))) tmpObj['tabindex'] = iTabindex;
				if (aList[i].getAttribute('focus')) tmpObj['focus'] = true;
				if (aList[i].getAttribute('ondemand')) tmpObj['ondemand'] = true;

				// PARSE OBJECT'S PROPERTIES
				for (var nlen = aList[i].childNodes.length -1; nlen>=0;nlen--){

					etmp = aList[i].childNodes[nlen];

					switch ((etmp.tagName?etmp.tagName.toLowerCase():'')){
					case 'init' :	tmpObj['init'] = parseitem (etmp);
									break;
                    case 'disabled':
									var tval = etmp.textContent || (typeof etmp.text == 'string'?etmp.text.unescapeHTML():'');
									if (tval && (tval!='false' || tval!='0'))
										tmpObj['disabled'] = true;
										
									break;
					case 'safari_title': //safari is removing <title> tag
					case 'title':
									tmpObj['title'] = parseitem (etmp);
									break;
					case 'draw' :	tmpObj['draw'] = [etmp.getAttribute('form'),etmp.getAttribute('anchor') || 'main',parseitem (etmp)];
									break;
					case 'fill' :	tmpObj['fill'] = parseitem (etmp);
									break;
					case 'value':
									tmpObj['value'] = parseitem (etmp);
									break;
					case 'restrictions': tmpObj['restrictions'] = parseitem (etmp,1);
									break;
					}
				}
				etmp = null;

				// APPEND OBJ INTO ARRAY
				if ((ep.tagName).toLowerCase() == 'obj'){

					// child
					for (var iLPos = aList.length-1;iLPos>=0;iLPos--)
						if (aList[iLPos]===ep) break;

					if (prevObj[iLPos]['objects'])
						prevObj[iLPos]['objects'].push(tmpObj);
					else
						prevObj[iLPos]['objects'] = [tmpObj];
				}
				else
					obj[++oid] = tmpObj;

				prevObj.push(tmpObj);
				tmpObj = null;
			}
			prevObj = null;

			// REMOVE <OBJ> ELEMENTS FROM TEMPLATE
			for(var j = aList.length-1; j>=0;j--){
				var oTextNode = aList[0].ownerDocument.createTextNode("");
				//aList[j].parentNode.replaceChild(oTextNode,aList[j]);
				aList[j].parentNode.appendChild(oTextNode);
				aList[j].parentNode.removeChild(aList[j]);
				oTextNode = null;
			}

			if (MSIE){
				sHtml = XMLTools.XML2Str(xTpl);
				sHtml = sHtml.substring(6,sHtml.lastIndexOf('</root>'));
			}
			else
				sHtml = xTpl.innerHTML;

//SAFARI    alert(sshtml +"\n----------\n"+ sHtml);

			ep = null; // xTplDoc = null;

			xTpl = null;
		}
		else
			var sAnchor = Anchor;

		/* inner template into page */
		if (sTmpName && sAnchor){
			if (typeof sAnchor != 'object')
				Anchor = this._getAnchor(Anchor);

			Anchor.innerHTML = sHtml;
		}
		else
			this._main.innerHTML = sHtml;

		/* add GUI objects from template */
		delete aData['_ins'];
		if (count(aData) > 0)
			this.__addObjects(obj,null,aData);
		else
			this.__addObjects(obj);
	};


_me.__addObjects = function(obj,str,aData){
	str = str || this._pathName;
	var newObj,sInit;

	for (var i in obj){

		// init
		sInit = '';
		if (obj[i]['init']){
			//pole
			if (typeof obj[i]['init'] == 'object'){
				for (var j in obj[i]['init'])
					sInit += ",obj[i]['init']['"+ j.replace('"','\\"') +"']";
			}
			//string
			else
				sInit = ',"'+obj[i]['init'].replace('"','\\"')+'"';
		}

		if (count(aData) > 0){
			this._aTemplateData = aData;
			eval(str + '._aTemplateData = aData');
		}
			
		newObj = eval(str + '._create(obj[i]["name"],obj[i]["type"],obj[i]["anchor"],"'+ (obj[i]['css'] || '') +'"'+ sInit + ')');

		// _title
		if (obj[i]['title'] && Is.Function(newObj._title))
			newObj._title(obj[i]['title']);

		// _fill
		if (obj[i]['fill'] && Is.Function(newObj._fill))
			newObj._fill(obj[i]['fill']);

		// _value
		if (obj[i]['value'] && Is.Function(newObj._value))
			newObj._value(obj[i]['value']);

		// FOCUS
		if (obj[i]['focus'] && Is.Function(newObj._focus))
			newObj._focus(obj[i]['focus']);

		// DISABLED
		if (obj[i]['disabled'] && Is.Function(newObj._disabled))
			newObj._disabled(obj[i]['disabled']);
			
		// TABINDEX
		if (obj[i]['tabindex'] && Is.Function(newObj._tabindex))
			newObj._tabindex(obj[i]['focus']);

		// WIDTH & HEIGHT
		if ((obj[i]['width'] || obj[i]['height']) && Is.Function(newObj._size))
			newObj._size(obj[i]['width'],obj[i]['height']);

		// RESTRICTIONS
		if ((obj[i]['restrictions']) && Is.Function(newObj._restrict)){
			var j,stmp = '';

			if (typeof obj[i]['restrictions'] == 'object'){
				for(j in obj[i]['restrictions'])
					stmp += (stmp?',':'')+'"'+obj[i]['restrictions'][j].trim()+'","'+j+'"';
			}
			else
			if (typeof obj[i]['restrictions'] == 'string')
				stmp = '"'+obj[i]['restrictions'].trim()+'"';

			try{
				if (stmp)
					eval('newObj._restrict('+ stmp +');');
			}
			catch(er){
				throw "invalid input array for restrictions:\n"+stmp;
			}
		}

		// _draw
		if (obj[i]['draw'] && Is.Function(newObj._draw)){
			if (typeof newObj.__drawTpl != 'undefined' && !newObj._isActive && obj[i]['ondemand']) {
				newObj.__drawTpl = obj[i]['draw'];
				newObj.__drawData = aData;
			}
			else{
				aData = arrConcat(aData,obj[i]['draw'][2]);
				newObj._draw(obj[i]['draw'][0],obj[i]['draw'][1],aData);
				if (newObj._isActive && newObj._active) newObj._active(true);
			}
		}

		// nasted objects
		if (obj[i]['objects'] && obj[i]['objects'].length){
			if (typeof newObj.__drawObj != 'undefined' && !newObj._isActive && obj[i]['ondemand']) {
				newObj.__drawObj = obj[i]['objects'];
				newObj.__drawData = aData;
			}
			else{
				this.__addObjects(obj[i]['objects'],newObj._pathName,aData);
				if (newObj._isActive && newObj._active) newObj._active(true);
			}
		}
	}
};


/**
 *
 * @param: sType - event type
 * 				oEvn	- [object,"method",['arg1','arg2',...]]
 **/
_me._obeyEvent = function(sType,oEvn){

	if (this._events[sType]){
		//check if already doesnt exist
		this._disobeyEvent(sType,oEvn);
	}
	else
		this._events[sType] = [];

	this._events[sType].push(oEvn);
};


_me._disobeyEvent = function(sType,oEvn){

	var obj = getCallbackFunction(oEvn);

	for (var i in this._events[sType])
		if (getCallbackFunction(this._events[sType][i]) === obj)
			this._events[sType].splice(i,1);

};


_me.__exeEvent = function(sType,e,arg){
	for (var j in this._events[sType]){
		if (Is.Array(this._events[sType][j])){
			if (!executeCallbackFunction(this._events[sType][j], e, arg))
				this._events[sType].splice(j,1);
		}
		else
			this._events[sType].splice(j,1);
	}
};


/**
* @brief: debug function which returns all child nodes in GUI tree
**/
_me._debugTree = function (){
	var a = {};
	for(var i in this){
		if (i.indexOf('_')==0) continue;
		a[i +" ("+ this[i]._type+")"] = this[i]._debugTree();
	}
	return a;
};


/**
 * @brief : return Array of child objects
 * @param : sAnchor - optional, return childs from this anchor only
 *          sType   - optional, return objects with given type
 * @output: Array of child objects
 * @date  : 30.6.2006 13:39:41
 **/
_me._getChildObjects = function(sAnchor,sType){
	var aOut = [];
	for (var i in this){
		if (i.indexOf('_') == 0 || (sAnchor && this[i]._anchor != sAnchor) || (sType && this[i]._type != sType)) continue;
		aOut.push(this[i]);
	}
	return aOut;
};


/**
 * @brief: destruct all child objects
 * @param: sAnchor
 **/
_me._clean = function(sAnchor){
	var sAnchor = sAnchor || 'main';
	if (!this._anchors[sAnchor]) return false;
	
	var aObj = this._getChildObjects(sAnchor);
	for (var i in aObj)
		aObj[i]._destruct();

	return true;
};


_me._getAnchor = function(sAnchor){
	return document.getElementById(this._anchors[sAnchor]) || (sAnchor == 'main'?this._main:null);
};


/**
 * obey on given dataset object
 */
_me._listen = function(sDataSet,aDataPath){
	this._listener = sDataSet;
	if (typeof aDataPath == 'object') this._listenerPath = aDataPath;
	dataSet.obey(this,'_listener',sDataSet);
};


_me._save = function(sDataSet,aDataPath){
this._saver = sDataSet;
if (typeof aDataPath == 'object') this._saverPath = aDataPath;
dataSet.obey(this,'_saver',sDataSet);
};


_me._saveme = function (noupd){
	if (this._skipsaving) return '';
	if (this._noupdate) noupd = this._noupdate;

	if (this._saver){
		dataSet.add(this._saver, this._saverPath, this._value(), noupd, this._pathName);
		return this._saver;
	}
	else
	if (this._listener){
		dataSet.add(this._listener, this._listenerPath, this._value(), noupd, this._pathName);
		return this._listener;
	}
};


_me._add_destructor = function (sMethod,aProperties){
	if(!sMethod) return false;
	this._destructors[sMethod] = aProperties;
};


_me._remove_destructor = function (sMethod){
	delete this._destructors[sMethod];
};


/**
 * @breif: destruct object
 * @date : 30.6.2006 13:58:51
 **/
_me._destruct = function(){
	this._destructed = true;
	// try to destruct already destructed object
	if (!this._parent[this._name]) return false;

	/*
	1. STEP
	execute all destructors
	useful for _saveall method etc.
	*/
	for (var val in this._destructors){
		if (Is.Function(this[val]))
			eval('this.'+ val +"("+ (Is.Array(this._destructors[val])?this._destructors[val].join(','):'') +");");
	}
	/*
	 2. STEP
	 excecute destructors of all childs
	*/
	for (var i in this){
		if (i.indexOf('_')==0 || typeof this[i] != 'object' || this[i]==null || typeof this[i]._destruct != 'function') continue;
		this[i]._destruct();
		delete this[i];
	}

	// remove listeners
	if (this._listener)
		dataSet.disobey(this);
	if (this._listener_data)
		dataSet.disobey(this,'_listener_data');

	// remove HTML
	try{
		this._main.parentNode.removeChild(this._main);
	}
	catch(er){}
	
	// remove instance
	this._parent[this._name] = null;
	delete this._parent[this._name];
};
//////////////////////////////

gui = new cObject('gui');
