/**
 * @brief  : This class provides storage for GUI data and takes care about JS and CSS loading into page
 * @require: httprequest, browser_ext
 * @status : final
 * @date   : 6.3.2006 13:59:51
 **/
function cStorage(){
	this.aStorage = {"css":{},"library":{},"language":{},"template":{},"object":{}};

	this.aStorage.library = {
	//preloaded in startscript.js
	'client/inc/debug':true,'client/inc/object_ext':true,'client/inc/browser_ext':true,'client/inc/template':true,'client/inc/xmltools':true,
	'client/inc/httprequest':true,'client/inc/dataset':true,'client/inc/storage':true,'client/inc/gui':true,'client/inc/wm_generic':true,
	'client/inc/wm_auth':true,'client/inc/wm_accounts':true,'client/inc/wm_folders':true,'client/inc/wm_items':true,'client/inc/wm_settings':true,'client/inc/wm_storage':true,'client/inc/init':true,
	'client/inc/gw_others':true,

	//obj from Login screen
	'client/inc/obj_container_generic':true,'client/inc/obj_form_generic':true,'client/inc/obj_form_restrict':true,'client/inc/obj_label':true,'client/inc/obj_tabs':true,'client/inc/obj_tab':true,'client/inc/obj_input':true,'client/inc/obj_button':true,
	'client/inc/obj_select':true,'client/inc/obj_loader':true,'client/inc/frm_login':true
	};

	this.bCheckNames = false;

	this.request = new cHttpRequest();

//	this.request.bSync = true;
};

/**
 * @brief: Append CSS file into page or enabled previously appended one
 * @date: 6.3.2006 13:56:53
 */
cStorage.prototype.css = function(sName) {

	// validate given name
	var aAllowed = [];
	if (this.bCheckNames && inArray(aAllowed,sName)<0)
		throw new Error("cStorage.css() - disallowed filename: " + sName);

	// MSIE workaround because 31 link tag limitation  (style.css contains all *.css)
	if (typeof this.aStorage.css['style'] != 'undefined') return true;

	//check cache
	if (typeof this.aStorage.css[sName] != 'undefined') {
		if (this.aStorage.css[sName] == 'disabled') {
			document.getElementById('css_' + sName).disabled = false;
			this.aStorage.css[sName] = 'enabled';
		}
		return true;
	}

	//append CSS into page
	var cssobj = mkElement('link',{"rel":'stylesheet',"type":'text/css',"href":'client/skins/'+GWOthers.getItem('LAYOUT_SETTINGS', 'skin')+'/css/'+sName+'.css'});
	cssobj.id = 'css_' + sName;
	document.getElementsByTagName('head')[0].appendChild(cssobj);
	cssobj = null;
	//save name to cache
	this.aStorage.css[sName]='enabled';

	return true;
};


/**
 * @brief: disable given css in document
 * (remove doesnt work in MSIE - needs some research here!)
 * @date: 6.3.2006 13:56:21
 *
cStorage.prototype.remove_css = function (sName) {
	if (typeof this.aStorage.css[sName] != 'undefined') {
		if (this.aStorage.css[sName] == 'enabled') {
			document.getElementById('css_' + sName).disabled = true;
			this.aStorage.css[sName] = 'disabled';
		}
		return true;
	}
	return false;
};
/**/

/**
 * @brief: Append library into page
 * (little odd behaviour in Safari, "var" definitions aren't eval as public!)
 */
cStorage.prototype.library = function(sName,sPath,bASync) {

	var sFile;
	switch (sPath){
		case 'skin':
			sFile = 'client/skins/'+GWOthers.getItem('LAYOUT_SETTINGS', 'skin')+'/inc/'+sName;
			break;
		default:
			sFile = 'client/inc/'+sName;

			// JS preloader
			if (this.aStorage.library['client/inc/javascript']) return true;
	}

	// validate given name
	var aAllowed = [];
	if (this.bCheckNames && inArray(aAllowed,sFile)<0)
		throw new Error("cStorage.library() - disallowed filename: " + sName);

//	alert(sFile +' = '+this.aStorage.library[sFile]);

	//check cache
	if (this.aStorage.library[sFile]) return true;

	//retrieve Lib from server
	if (bASync){
        var ASrequest = new cHttpRequest();
            ASrequest.sURL = sFile + '.js';
			ASrequest.sendXML('',[this,'preloadLib',['library',sFile,sName],'Text']);

		return true;
	}

	this.request.sURL = sFile + '.js';
	this.request.sendXML();
	var str = this.request.responseText();
	if (str.length<1) return false;

	//append library into page
	pubEval(str);

	//save name to cache
    this.aStorage.library[sFile]='enabled';
	return true;
};

/**
 * Synchronous preload f selected objects in advance
 * (MUST be in synchro mode)
 **/
cStorage.prototype.preloadObj = function(){

	/*
	Login preloadne:
		obj_tabs
		obj_input
		obj_password
		obj_select
		obj_button
		
	removed for speedup
    'frm_contact','frm_event','frm_task'
	*/

    aObjects = ['obj_checkbox','obj_text',
    //Compose email
	'frm_compose','obj_attach_edit','obj_rich',
	//GW
	'obj_categories','obj_sharing','obj_timeinterval','obj_input_calendar','obj_time','obj_select_input',
    //Contact
    'obj_phone',
    //Event
	'obj_labeled_select_colors'
    //Task
	];
	
	//preload loop with anti freez (FFox losts focus)
	for (var i in aObjects){
		document.title = document.title;
		setTimeout(storage.object(aObjects[i]),0);
		document.title = document.title;
	}
};

/**
 * Asynchronous preload of selected templates
 **/
cStorage.prototype.preloadTpl = function(aResponseData,sName){
		/*
		Compose tpl
		    obj_popup
			frm_compose
			obj_attach_edit
			obj_rich
		*/
        if (aResponseData && sName)
			this.aStorage.template[sName] = aResponseData.Text || '';

        var aTemplates = ['obj_popup','frm_confirm',
		//Compose
		'frm_compose','obj_attach_edit','obj_rich','frm_addaddress',
		//GW
		'obj_categories','obj_labeled_select','obj_select_input',
		//Contact
		'frm_contact','frm_contact_tab1','obj_phone',
		//Event
		'frm_event','frm_event_tab1','obj_timeinterval','obj_input_calendar',
		//Task
		'frm_task','frm_task_tab1'];

		var no;
		if (!sName)
			no = 0;
		else
		if ((no = inArray(aTemplates,sName))<0)
			return;
		else
		    no++;
		    
		if (!aTemplates[no]) return;

		if (typeof this.aStorage.template[aTemplates[no]]=='undefined')
            setTimeout("storage.template('"+aTemplates[no]+"',true);",!sName?3000:500);
		else
		if (aTemplates[no+1])
		    this.preloadTpl('',aTemplates[no+1]);
};

/**
 * @brief:  load and reconstruct language array from server
 * @return: language array
 */
cStorage.prototype.language = function(sName){
	// validate given name
	var aAllowed = ['en'];
	if (this.bCheckNames && !inArray(aAllowed, sName))
		throw new Error("cStorage.language() - disallowed filename: " + sName);

	//check cache
	if (typeof this.aStorage.language["_ACTIVE_LANG"] != 'undefined' && this.aStorage.language["_ACTIVE_LANG"] == sName)
		return sName;

	//retrieve language from server
	if(!sName)
	    sName = 'en';
	    
	//retrieve language from server
	try{
		this.request.sURL = 'client/languages/'+sName+'/data.xml';
		this.request.sendXML();
		var lang = this.request.responseArray();
	}
	catch(e){
		var lang = null;
	}
	
	if (!Is.Array(lang)) {
		// try to load default language
		this.request.sURL = 'client/languages/en/data.xml';
		this.request.sendXML();
		lang = this.request.responseArray();
		if (!Is.Array(lang))
			throw new Error("cStorage.language() - bad language file syntax: " + sName);
	}

	//construct storage
	lang = lang['LANGUAGE'][0];
	var tmp = {};
	for (var i in lang){
		this.aStorage.language[i] = {};
		for (var j in lang[i][0]){
			if (j == 'VALUE') continue;
			
			var value = lang[i][0][j][0]['VALUE'];
			this.aStorage.language[i][j] = (Is.Defined(value)) ? value : '';
		}
	}

	return this.aStorage.language["_ACTIVE_LANG"] = sName;
};


/**
 * @brief :  load and template file from sever
 * @return: template as string
 * @date  : 4.7.2006 12:37:13
 */
cStorage.prototype.template = function(sName,bASync) {

	// validate given name
	var aAllowed = [];
	if (this.bCheckNames && inArray(aAllowed,sName)<0)
		throw new Error("cStorage.template() - disallowed filename: " + sName);

    var skin = GWOthers.getItem('LAYOUT_SETTINGS', 'skin');

	//check cache
	if (typeof this.aStorage.template[skin+'/'+sName]!='undefined')
		return this.aStorage.template[skin+'/'+sName];

	//retrieve language from server
	if (bASync){
        var ASrequest = new cHttpRequest();
            ASrequest.sURL = 'client/skins/'+skin+'/templates/'+sName+'.tpl';
			ASrequest.sendXML('',[this,'preloadTpl',[sName],'Text']);

		return true;
	}

	this.request.sURL = 'client/skins/'+skin+'/templates/'+sName+'.tpl';
	this.request.sendXML();
	var str = this.request.responseText();
	if (!str)
		throw new Error("cStorage.template() - blank template file: " + skin+'/'+sName);

	//save to cache
	return this.aStorage.template[skin+'/'+sName] = str;
};

/**
 * @brief:  load  page descriptor from sever
 * @return: page as array
 */
cStorage.prototype.object = function(sName) {

	//validate given name
	var aAllowed = [];
	if (this.bCheckNames && inArray(aAllowed,sName)<0)
		throw new Error("cStorage.object() - disallowed filename: " + sName);

	//check cache
	if (typeof this.aStorage.object[sName]!='undefined')
		return this.aStorage.object[sName];

	//retrieve object from server
	this.request.sURL = 'client/objects/'+sName+'.xml';
	this.request.sendXML();

	var aObject = this.request.responseArray();

	//check for <page> root and remove it
	try{
		aObject = aObject['OBJECT'][0];
	}
	catch(e){
		throw new Error("cStorage.object() - blank xml: " + sName);
	}

	//load CSS
	if (aObject['CSS'])
		for(var i in aObject['CSS'])
			this.css(aObject['CSS'][i]['VALUE']);

	//load JS
	if (aObject['LIBRARY'])
		for(var i in aObject['LIBRARY']){
            if (aObject['LIBRARY'][i]['ATTRIBUTES'] && aObject['LIBRARY'][i]['ATTRIBUTES']['PATH'])
            	this.library(aObject['LIBRARY'][i]['VALUE'],aObject['LIBRARY'][i]['ATTRIBUTES']['PATH']);
			else
				this.library(aObject['LIBRARY'][i]['VALUE']);
		}

	//save to cache
	return this.aStorage.object[sName] = aObject;
};

storage = new cStorage();

///////////////////////////

function getLang(str,aSubstitute,nobr){

	if (typeof str != 'string' || !str) return '';
    
	var out = '',a = str.toUpperCase().split('::');

	try{
		if (typeof a[1] == 'undefined')
			out = storage.aStorage.language[a[0]];
		else
			out = storage.aStorage.language[a[0]][a[1]];

		if (typeof out == 'string'){
			var parts = out.split('%s');
			out = parts.shift();
			for (var i in parts){
				if (Is.Array(aSubstitute))
					out += aSubstitute.shift();
				out += parts[i];
			}
		}

		return out;
	}
	catch(e){}

	return (nobr?str:'{'+str+'}');
};
