/**
 * @brief: WebmailIqAuth handling class
 **/
function wm_auth(){
	this.xmlns='auth';
};

wm_auth.inherit(wm_generic);

var _me=wm_auth.prototype;

/*
param:
	auth_array[username]
	auth_array[password] or auth_array[digest]
*/
_me.login = function(auth_array,sDataSet,sDataPath){
	if (!auth_array['username'] || (!auth_array['password'] && !auth_array['digest'])) return false;

	// encrypt password
	if (!auth_array.digest)
		auth_array.digest = this.digest(auth_array['username'],auth_array['password']);

	/* Logout prevous user from dataset */
	this.logout();

	/* prepare data for request */
	var request={"USERNAME":[{"VALUE":auth_array.username}],"DIGEST":[{"VALUE":auth_array.digest}],"METHOD":[{"VALUE":'RSA'}]};

	/* send data to server & set response handler */
	if(!sDataSet){
		var aData = this.create_iq(request,'','set');
		if (this.error) return false;
		aData = aData["IQ"][0]["ATTRIBUTES"]["SID"];
		dataSet.add('main',['sid'],aData);

		return aData;
	}
	else
		this.create_iq(request,[this,'response',['login',sDataSet,sDataPath]],'set');

	return true;
};


_me.logout = function(bIgnoreResponse){

	if (!dataSet.get("main",['sid'])) return true;

	//Login save state 3
	if (cookieManager.get('LoginState') == '3')
		cookieManager.set('LoginState','2');

	if (bIgnoreResponse) {
		this.create_iq(null,[this,'_void'],'set');
		return true;
	}
	else
		this.create_iq(null,'','set');

	dataSet.remove("main",['sid'],true);
	return true;
};

_me._forgot = function(email,csid,captcha){

	var aRequest = {"FORGOT" :[{"VALUE":email}],
					"CAPTCHA":[{'ATTRIBUTES':{'UID':csid},"VALUE":captcha}],
					"SUBJECT":[{"VALUE":GWOthers.getItem('FORGOT_SETTINGS','subject') || getLang('FORGOT_PASS::SUBJECT')}],
					"MESSAGE":[{"VALUE":GWOthers.getItem('FORGOT_SETTINGS','mail') || getLang('FORGOT_PASS::EMAIL')}]};

	var aResponse = (this.create_iq(aRequest,'','set')).IQ[0];

	if (aResponse.ATTRIBUTES.TYPE == 'result'){
		var emails = [];
		for(var i in aResponse.QUERY[0].EMAIL)
			emails.push(aResponse.QUERY[0].EMAIL[i].VALUE);

		return emails;
	}
	else
	if (aResponse.ATTRIBUTES.TYPE == 'error')
		return aResponse.ERROR[0].ATTRIBUTES.UID;
	else
		return false;
};

_me._void = function(){};

_me.digest = function(user,pass){
	// encrypt password
	var rsa = new RSAKey();
		rsa.setPublic(this.hashid({"username":user}), '10001');
	return rsa.encrypt(pass);
};

/**
 * @brief: retrieve HashID from server
 * @query: <iq><query xmlns="webmail:iq:auth"><username>admin</username><method>rsa</method></query></iq>
 **/
_me.hashid = function(auth_array)
{
	/* prepare data for request */
	var request={"USERNAME":[{"VALUE":auth_array['username']}],"METHOD":[{"VALUE":'RSA'}]};

	/* send data to server & get response */
	var response=this.create_iq(request);

	/* return HashId */
	return response["IQ"][0]["QUERY"][0]["HASHID"][0]["VALUE"];
};

_me.response=function(aData,sMethodName,sDataSet,sDataPath)
{
	if (sDataSet == 'main')
		sDataPath = ['sid'];
	else
		sDataPath = sDataPath || ['sid'];

	switch(sMethodName){
	case 'login':
		dataSet.add(sDataSet,sDataPath,aData['Array']["IQ"][0]["ATTRIBUTES"]["SID"]);
		break;
	case 'logout':
		dataSet.add(sDataSet,sDataPath,'');
	}
};

/////////////////////////
var auth = new wm_auth();