function wm_folders(){
	this.xmlns = 'folders';
};

wm_folders.inherit(wm_generic);
var _me = wm_folders.prototype;

//********************************************************************************
//Základní SET funkce pro pro přidávání a update folderu                    * OK *
//********************************************************************************
//Vstup: aFolderInfo ... asociativní pole klíčů:
//  povinné:   'aid':account_id, 'name':jméno folderu
//  nepovinné: 'fid':folder_id, 'type':typ folderu
//********************************************************************************

_me.add = function(aFolderInfo,sDataSet,aDataPath)
{
	//Máme definované account_id a jméno folderu?
	if (!aFolderInfo['aid'])
		return false;

	var sUpdDataSet = sDataSet || 'folders';

	var aRequest;

	//Budeme složku editovat či přidávat?
	if (aFolderInfo['fid']){
		if (!aFolderInfo['name'] && typeof aFolderInfo['channel'] == 'undefined')
	        return false;

		//Vytvoříme XML dotaz pro editaci folderu
		// <ACCOUNT UID='aid'><FOLDER UID='fid' ACTION='edit'><NAME>nové jméno folderu</NAME></FOLDER></ACCOUNT>
		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"edit"}}]}]};
	}
	else{
	    if (!aFolderInfo['name'])
	        return false;

		//V případě zadaného typu folderu ověříme, zdali je podporovaný, jinak ho nastavíme na 'mail'
		if (aFolderInfo['type']){
			if (inArray(['mail','contact','event','journal','note','task','file','m','r','c','e','j','n','t','f'],aFolderInfo['type'].toLowerCase()) < 0)
				return false;
				
			//create Virtual Account RSS if doesn't exists yet
			if (aFolderInfo['type'] == 'R' && !dataSet.get('accounts',[sPrimaryAccount+'_rss']))
				dataSet.add('accounts',[sPrimaryAccount+'_rss'],{DESCRIPTION:'RSS',TYPE:'rss'});
		}
		else
			aFolderInfo['type'] = 'mail';

		//Vytvoříme XML dotaz pro přidání folderu
		// <ACCOUNT UID='aid'><FOLDER ACTION='add'><NAME>jméno folderu</NAME><TYPE>typ folderu</TYPE></FOLDER></ACCOUNT>
		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"ACTION":"add"},"TYPE":[{"VALUE":aFolderInfo['type']}]}]}]};
	}

	if (aFolderInfo['name'])
        aRequest.ACCOUNT[0].FOLDER[0].NAME = [{"VALUE":aFolderInfo['name']}];

	if (typeof aFolderInfo['channel'] != 'undefined')
		aRequest.ACCOUNT[0].FOLDER[0].CHANNEL = [{"VALUE":aFolderInfo['channel']}];

	//Pracujeme synchronně či asynchronně?
	if (!sDataSet){
		var aResponse = this.create_iq(aRequest,'','set');

		try{
			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
				return true;
		}
		catch(e){}

		return false;
	}
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['add',sDataSet,aDataPath,aFolderInfo]],'set',aFolderInfo['aid']);
		return true;
	}
};

//********************************************************************************
//Základní GET funkce pro listování folderů                                 * OK *
//********************************************************************************
//Vstup: aFoldersInfo ... asociativní pole klíčů:
//  povinné:   'aid':account_id
//********************************************************************************

_me.list = function(aFoldersInfo,sDataSet,aDataPath,aHandler)
{
	//Máme definované account_id?
	if (!aFoldersInfo['aid'])
		return false;

	//Vytvoříme XML dotaz pro listování folderu v accountu
	// <ACCOUNT UID='aid'/>
	var aRequest={"ACCOUNT":[{"ATTRIBUTES":{"UID":aFoldersInfo['aid']}}]};

	//Pracujeme synchronně či asynchronně?
	if (!sDataSet){
		return this.parse(this.create_iq(aRequest));
	}
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['list',sDataSet,aDataPath,'',aHandler]],'',aFoldersInfo['aid']);
		return true;
	}
};

/**
 * funkce vraci Typ folderu
 * (nebo False)
 **/
_me.getType = function (aFolderInfo){
    var aid = aFolderInfo.aid || aFolderInfo[0];
    var fid = aFolderInfo.fid || aFolderInfo[1];
    
	return dataSet.get('folders',[aid,fid,'TYPE']) || false;
};

/**
 * funkce vraci Rights pro folder
 * (Prava na folder samotny)
 **/
_me.getRights = function (aFolderInfo,isRight){
	var aRights = {read:false,write:false,modify:false,remove:false};
	if(!aFolderInfo.aid || !aFolderInfo.fid) return isRight?false:aRights;

	var sRights = dataSet.get('folders',[aFolderInfo.aid,aFolderInfo.fid,'RIGHTS']);

	if (sRights){
		if (sRights.indexOf('r')>-1) aRights['read'] = true;
		if (sRights.indexOf('w')>-1) aRights['write'] = true;
		if (sRights.indexOf('m')>-1) aRights['modify'] = true;
		if (sRights.indexOf('d')>-1) aRights['remove'] = true;
	}

	if (isRight) return aRights[isRight] || false;
	return aRights;
};
/**
 * funkce vraci Access pro Items ve folderu
 **/
_me.getAccess = function (aFolderInfo,isRight){
	var aRights = {read:false,write:false,modify:false,remove:false};
	if(!aFolderInfo.aid || !aFolderInfo.fid) return isRight?false:aRights;

	var sRights = dataSet.get('folders',[aFolderInfo.aid,aFolderInfo.fid,'ACCESS']);

	if (sRights){
		if (sRights.indexOf('r')>-1) aRights['read'] = true;
		if (sRights.indexOf('w')>-1) aRights['write'] = true;
		if (sRights.indexOf('m')>-1) aRights['modify'] = true;
		if (sRights.indexOf('d')>-1) aRights['remove'] = true;
	}
	else
		return this.getRights(aFolderInfo,isRight);

	if (isRight)
		return aRights[isRight] || false;

	return aRights;
};

/**
 * funkce vraci opravdove jmeno systemoveho folderu v pripade ze nebyl
 * nalezen ale existuje jiny lisici se pouze v CaSe
 *  @param: sDataSet - optional
 **/
_me._getDefaultFolders = function(sDataSet){

    var aFolders = dataSet.get(sDataSet || 'folders');
	var aFolder = ['trash','drafts','sent','spam'];
	
    if (!aFolders) return aFolder;
	
	var sItem = '',test = {},out = {};
	storage.library('gw_others');

	for(var i in aFolder){

		sItem = GWOthers.getItem('DEFAULT_FOLDERS', aFolder[i]);
		sItem = sItem.substr(sItem.indexOf('/')+1);

		if (!aFolders[sPrimaryAccount][sItem])
			test[aFolder[i]] = [new RegExp('^'+sItem.quoteMeta()+'$', "i"),sPrimaryAccount+'/'+sItem];
		else
		    out[aFolder[i]] = sPrimaryAccount+'/'+sItem;
	}
	
	

	if (!count(test)) return out;

	//search for folders
	for (var itm in aFolders){
		for(var itm2 in test){
			if (itm.match(test[itm2][0])){
				out[itm2] = sPrimaryAccount+'/'+itm;
				test[itm2] = null;
				delete test[itm2];
			}
		}
	}
	
	//append not existing folders
	for(var itm2 in test)
        out[itm2] = test[itm2][1];

	//save to settings
	storage.library('gw_others');
	GWOthers.set('DEFAULT_FOLDERS',out,'storage');

	return out;
};

//********************************************************************************
//Základní SET funkce pro pro mazání folderu                                * OK *
//********************************************************************************
//Vstup: aFolderInfo ... asociativní pole klíčů:
//  povinné:   'aid':account_id, 'fid':folder_id
//********************************************************************************
_me.remove = function(aFolderInfo,sDataSet,aDataPath)
{
	//Máme definovaná account_id a folder_id?
	if (!aFolderInfo['aid'] || !aFolderInfo['fid'])
		return false;

	//Vytvoříme XML dotaz pro výmaz folderu
	// <ACCOUNT UID='aid'><FOLDER UID='fid' ACTION='delete'/></ACCOUNT>
	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"delete"}}]}]};
	

	//Pracujeme synchronně či asynchronně?
	if (!sDataSet){
		var aResponse = this.create_iq(aRequest,'','set');
		try{
			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
            	return true;
		}
		catch(e){}

		return false;
	}
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['remove',sDataSet,aDataPath,aFolderInfo]],'set',aFolderInfo['aid']);
		return true;
	}
};

//********************************************************************************
//Základní SET funkce pro pro vyprázdnění folderu                           * OK *
//********************************************************************************
//Vstup: aFolderInfo ... asociativní pole klíčů:
//  povinné:   'aid':account_id, 'fid':folder_id
//********************************************************************************

_me.empty = function(aFolderInfo,sDataSet,aDataPath,aDestination)
{
	//Máme definovaná account_id a folder_id?
	if (!aFolderInfo['aid'] || !aFolderInfo['fid'])
		return false;

	//Vytvoříme XML dotaz pro vyprázdnění folderu
	// <ACCOUNT UID='aid'><FOLDER UID='fid' ACTION='empty'/></ACCOUNT>
	var aRequest;
	if (Is.Defined(aDestination))
		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"empty"},"ACCOUNT":[{"VALUE":aDestination['aid']}],"FOLDER":[{"VALUE":aDestination['fid']}]}]}]};
	else
		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"empty"}}]}]};

	//Pracujeme synchronně či asynchronně?
	if (!sDataSet){
		var aResponse = this.create_iq(aRequest,'','set');

		try{
			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result'){
				dataSet.add('items',[aFolderInfo['aid'],aFolderInfo['fid']],{});
				return true;
			}
		}
		catch(e){}

		return false;
	}
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['empty',sDataSet,aDataPath,aFolderInfo]],'set',aFolderInfo['aid']+'/'+aFolderInfo['fid']);
		return true;
	}
};

/**
 *
 * @IQ Example: <folder uid="{uid}" action="edit"><subscribed>{0|1}</subscribed></folder>
 **/
_me.subscribe = function(aFolderInfo,sDataSet,aDataPath,bSync){
	var sDataSet = sDataSet || 'folders';
	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"edit"},"SUBSCRIBED":[{"VALUE":bSync?1:0}]}]}]};

	//AJAX...
	this.create_iq(aRequest,[this,'response',[bSync?'subscribe':'unsubscribe',sDataSet,aDataPath,aFolderInfo,bSync]],'set',aFolderInfo['aid']+'/'+aFolderInfo['fid']);
	return true;
};


_me.markItemsRead = function(aFolderInfo,sDataSet,aDataPath,bRead)
{
	//Máme definovaná account_id a folder_id?
	if (!aFolderInfo['aid'] || !aFolderInfo['fid'])
		return false;

	//Vytvoříme XML dotaz pro výmaz folderu
	// <ACCOUNT UID='aid'><FOLDER UID='fid' ACTION='delete'/></ACCOUNT>
	var sType = bRead ? 'markasread' : 'markasunread';
	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":sType}}]}]};
	aFolderInfo['bRead'] = bRead;
	
	//Pracujeme synchronně či asynchronně?
	if (!sDataSet){
		var aResponse = this.create_iq(aRequest,'','set');

		try{
			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
			return true;
		}
		catch(e){}

		return false;
	}
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['markread',sDataSet,aDataPath,aFolderInfo]],'set',aFolderInfo['aid']);
		return true;
	}
};

//********************************************************************************
//Pomocná funkce realizující asynchronní obsluhu odpovědi
//********************************************************************************

_me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHandler)
{
	var aXMLResponse = aResponse['Array'];
	var aIQAttribute = aXMLResponse['IQ'][0]['ATTRIBUTES'];

	//pri IMAP erroru
	try{
		if (aIQAttribute['TYPE'] == 'error'){

			var str, att;
			try{
				att = aXMLResponse.IQ[0].ERROR[0].ATTRIBUTES.UID;
				str = aXMLResponse.IQ[0].ERROR[0].VALUE;
			}
			catch(e){
				str = att = '';
			}

			if (att.toLowerCase() == 'imap_internal')
				alert (aIQAttribute['UID']+"\n"+str.unescapeHTML());
		}
	}
	catch(e){}

	switch(sMethodName){
	case 'unsubscribe':
	case 'subscribe':
		if (aIQAttribute['TYPE'] == 'result')
			dataSet.add('folders', [aFolderInfo['aid'],aFolderInfo['fid'],'SYNC'],sMethodName == 'subscribe'?1:0, true);
		break;
	case 'add':
		//pri erroru stahnout znova strom
		try{
			if (aIQAttribute['TYPE'] == 'error'){
				this.list({'aid':aIQAttribute['UID']},sDataSet,aDataPath);
				return true;
			}
		}
		catch(e){}

		//ASYNC DataSet change

		//edit
		if	(aFolderInfo['fid'] && ((aFolderInfo['name'] && aFolderInfo['fid'] != aFolderInfo['name']) || typeof aFolderInfo['channel'] != 'undefined')){
			var tmp, sName = aFolderInfo['name'] || aFolderInfo['fid'];
			var aDataSet = dataSet.get(sDataSet,[aFolderInfo['aid']]);

			for(var i in aDataSet){
				//folder
				if (i == aFolderInfo['fid']){
					tmp = aDataSet[i];

					if (typeof aFolderInfo['channel']!='undefined')
						tmp.CHANNEL = aFolderInfo['channel'];
						
					delete aDataSet[i];

					//tmp['NAME'] = aFolderInfo['name'];
					aDataSet[sName] = tmp;
				}
				else
				//subfolder
				if (i.indexOf(aFolderInfo['fid']+'/') === 0){
					tmp = aDataSet[i];
					delete aDataSet[i];

					//tmp['NAME'] = aFolderInfo['name'] + i.substr(aFolderInfo['fid'].length);
					aDataSet[sName + i.substr(aFolderInfo['fid'].length)] = tmp;
				}
			}

			//	bNoRefresh = true protoze pracujeme primo s datasetem, proto po add neni zmena dat (add je jen pro sychr)
			//	Notne vyvolat refresh manualne
			dataSet.add(sDataSet,[aFolderInfo['aid']],aDataSet,true);
			dataSet.update(sDataSet,[aFolderInfo['aid']]);
		}
		//add
		else
		if (!aFolderInfo['fid'] && aFolderInfo['name']){                                                                                                     //,NAME:aFolderInfo['name'],TOTAL:0
			out = {RIGHTS:'rwmd',TYPE:aFolderInfo['type']};
			if (typeof aFolderInfo['channel'] != 'undefined')
				out.CHANNEL = aFolderInfo['channel'];

			dataSet.add(sDataSet,[aFolderInfo['aid'],aFolderInfo['name']],out);
		}

		return false;

	case 'list':

		try{
			if (aIQAttribute['UID']){
				var aData = this.parse(aXMLResponse,true);
				dataSet.add(sDataSet, [aIQAttribute['UID']], aData[aIQAttribute['UID']]);

				if (typeof aHandler == 'object')
					executeCallbackFunction(aHandler);

				return true;
			}
		}
		catch(e){}

		return false;

	case 'remove':
		try{
			if (aIQAttribute['TYPE'] != 'result' && aIQAttribute['UID']){
				this.list({'aid':aIQAttribute['UID']},sDataSet,aDataPath);
				return true;
			}
		}
		catch(e){}

		//ASYNC DataSet change
		var blank = true;
		var bPerfm = false;
		var aDataSet = dataSet.get(sDataSet,[aFolderInfo['aid']]);

		for(var i in aDataSet){
			if (i == aFolderInfo['fid'] || i.indexOf(aFolderInfo['fid']+'/') === 0){
				bPerfm = true;
				delete aDataSet[i];
			}
			else
				blank = false;
		}

		//	bNoRefresh = true protoze pracujeme primo s datasetem, proto po add neni zmena dat (add je jen pro sychr)
		//	Notne vyvolat refresh manualne
		if (blank && aFolderInfo['aid'] == sPrimaryAccount+'_rss'){
			dataSet.remove('accounts',[aFolderInfo['aid']],true);
			dataSet.remove(sDataSet,[aFolderInfo['aid']]);
		}
		else
		if(bPerfm){
			dataSet.add(sDataSet,[aFolderInfo['aid']],aDataSet,true);
			dataSet.update(sDataSet,[aFolderInfo['aid']]);
		}

		return false;

	case 'empty':
		try{
			if (aIQAttribute['TYPE'] != 'result' && aIQAttribute['UID']) {
				var aFolder = Path.split(aIQAttribute['UID']);
				this.list({'aid':aFolder[0]},sDataSet,aDataPath);

				// je potreba znovu vylistovat otevreny folder?
				var aItems = dataSet.get('items');
				for(var sAccId in aItems)
					for(var sFolId in aItems[sAccId]);
						if (sAccId == aFolder[0] && sFolId == aFolder[1])
							WMItems.list({'aid':sAccId,'fid':sFolId,'values':items.default_values('M')},'items');

				return true;
			}
			else
			if (aIQAttribute['UID']){
				var aFolder = Path.split(aIQAttribute['UID']);

				// je potreba promazat Items?
				var aItems = dataSet.get('items');
				for(var sAccId in aItems)
					for(var sFolId in aItems[sAccId]);
						if (sAccId == aFolder[0] && sFolId == aFolder[1])
							dataSet.add('items',[aFolder[0],aFolder[1]],{});
			}
		}
		catch(e){}

		//ASYNC DataSet change
		if (dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'])>0)
			dataSet.add(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'],'0'); // ,true

		return false;


	case 'markread':
		if (aIQAttribute['TYPE'] == 'error')
			return true;

		// refresh folder list
		if (aFolderInfo.bRead){
			if (dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'])>0)
				dataSet.add(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'],'0');
		}
		else {
			var nCount = 0;
			if (Is.Defined(aXMLResponse['IQ'][0]['QUERY'][0]['RECENT'])) {
				nCount = parseInt(aXMLResponse['IQ'][0]['QUERY'][0]['RECENT']);
				if (!Is.Number(nCount))
					nCount = 0;
			}

			var i = dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT']);
				i = i>0?i:0;

			if (i != nCount)
				dataSet.add(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'],nCount.toString());
		}

		// je potreba aktualizovat Items?
		var aItems = dataSet.get('items');
		for(var sAccId in aItems)
			for(var sFolId in aItems[sAccId]);
				if (sAccId == aFolderInfo['aid'] && sFolId == aFolderInfo['fid'])
					gui.frm_main.main.list._serverSort();

		return true;
	}
};

/**
 * Pomocná funkce realizující abecední seřazení folderů
 *
 * @param:  aFolders - array of folders
 *          bRights  - apply rights (TRUE only if aFolders are fresh data from server!!)
 **/
_me.sort = function(aFolders,bRights)
{
	function sort(a,b){
		var sA;
		var sB;

		if (Is.String(a['TITLE']))
			sA = a['TITLE'].toLowerCase();
		else
			sA = a['TITLE'];

		if (Is.String(b['TITLE']))
			sB = b['TITLE'].toLowerCase();
		else
			sB = b['TITLE'];

		if (sA>sB) return 1;
		if (sA<sB) return -1;
		return 0
	};

	//Rozdělíme foldery mezi Inbox, IMAP a GW
	var aResult = {};
	var aResultFolFrame,aFolder,aFolderSplit;

	var aSortInbox, aSortOthers, aSortShared, aSortPublic, tmpPublic;

	var sTitle;
	var bSpamFolders = false;
	var sRights;

	//get common folders from langs
	var aCommon = getLang('COMMON_FOLDERS');

	//get groups
	storage.library('gw_groups');
	var groups = new gw_groups;
	var aGroupsProps = groups.get('storage');
	var aGroups = [];
	for(var n in aGroupsProps)
		if (aGroupsProps[n]['SUBSCRIBED'] == 'true')
			aGroups.push(aGroupsProps[n]['GRPTITLE']);

	for(var sAccId in aFolders){
		//Seřazovací pole nulujeme
		aSortInbox = [];
		aSortOthers = [];
		aSortShared = [];
        aSortPublic = [];
		tmpPublic = {};
		
		//Roztřídění folderů podle typu
		for(var sFolId in aFolders[sAccId]){

			aFolderSplit = sFolId.split('/');

			//INBOX
			if (aFolderSplit[0] == 'INBOX'){
				if (aFolderSplit[0] == sFolId)
					aFolders[sAccId][sFolId]['NAME'] = aCommon['INBOX'];

				aFolderSplit[0] = aCommon['INBOX'];
				sTitle = aFolderSplit.join('/');

				aSortInbox.push({'FOLDER':sFolId,'TITLE':sTitle});
			}
			else
			//COMMON FOLDERS
			if ((sTitle = aCommon[aFolderSplit[0].toUpperCase()]))
			{
				if (aFolderSplit[0] == sFolId){
					aFolders[sAccId][sFolId]['NAME'] = sTitle;
					if(bRights && (sRights = aFolders[sAccId][sFolId].RIGHTS)){
						aFolders[sAccId][sFolId].ACCESS = sRights;

						switch(sFolId.toUpperCase()){
						case 'TRASH':
						case 'SENT':
						case 'DRAFTS':
							aFolders[sAccId][sFolId].RIGHTS = sRights.replace(/[m]/gi,'');
							break;
						default:
							aFolders[sAccId][sFolId].RIGHTS = sRights.replace(/[md]/gi,'');
						}
					}
				}
				else
				if (aFolderSplit[0] == 'SPAM_QUEUE')
					aFolders[sAccId][sFolId]['NAME'] = aCommon[(aFolderSplit[0]+'-'+aFolderSplit[1]).toUpperCase()];

				aFolderSplit[0] = sTitle;
				sTitle = aFolderSplit.join('/');
				aSortOthers.push({'FOLDER':sFolId,'TITLE':sTitle});
			}
			//SHARED & PUBLIC & OTHER FOLDERS
			else{

				//SHARED & PUBLIC FOLDERS
				var fld = false;
				if (aFolders[sAccId][sFolId]['TYPE'] == 'A')
					fld = true;
				else
				for(var i in aGroups){
					if (sFolId == aGroups[i]){
						fld = true;
						delete aGroups[i];

						tmpPublic[sFolId] = true;
						break;
					}
				}

				if (aFolders[sAccId][aFolderSplit[0]] && aFolders[sAccId][aFolderSplit[0]].TYPE == 'A')
					aSortShared.push({'FOLDER':sFolId,'TITLE':sFolId});
				else
				if (tmpPublic[aFolderSplit[0]])
					aSortPublic.push({'FOLDER':sFolId,'TITLE':sFolId});
				else
					aSortOthers.push({'FOLDER':sFolId,'TITLE':sFolId});
					

				if (fld){
					if (aFolders[sAccId][sFolId+'/Events'] && aFolders[sAccId][sFolId+'/Events'].TYPE == 'E'){
						aFolders[sAccId][sFolId+'/Events'].NAME = aCommon.EVENTS;
						if (bRights && (sRights = aFolders[sAccId][sFolId+'/Events'].RIGHTS)){
							aFolders[sAccId][sFolId+'/Events'].ACCESS = sRights;
							aFolders[sAccId][sFolId+'/Events'].RIGHTS = sRights.replace(/[md]/gi,'');
						}
					}
					if (aFolders[sAccId][sFolId+'/Contacts'] && aFolders[sAccId][sFolId+'/Contacts'].TYPE == 'C'){
						aFolders[sAccId][sFolId+'/Contacts'].NAME = aCommon.CONTACTS;
						if (bRights && (sRights = aFolders[sAccId][sFolId+'/Contacts'].RIGHTS)){
							aFolders[sAccId][sFolId+'/Contacts'].ACCESS = sRights;
							aFolders[sAccId][sFolId+'/Contacts'].RIGHTS = sRights.replace(/[md]/gi,'');
						}
					}
					if (aFolders[sAccId][sFolId+'/Notes'] && aFolders[sAccId][sFolId+'/Notes'].TYPE == 'N'){
						aFolders[sAccId][sFolId+'/Notes'].NAME = aCommon.NOTES;
						if (bRights && (sRights = aFolders[sAccId][sFolId+'/Notes'].RIGHTS)){
							aFolders[sAccId][sFolId+'/Notes'].ACCESS = sRights;
							aFolders[sAccId][sFolId+'/Notes'].RIGHTS = sRights.replace(/[md]/gi,'');
						}
					}
					if (aFolders[sAccId][sFolId+'/Files'] && aFolders[sAccId][sFolId+'/Files'].TYPE == 'F'){
						aFolders[sAccId][sFolId+'/Files'].NAME = aCommon.FILES;
						if (bRights && (sRights = aFolders[sAccId][sFolId+'/Files'].RIGHTS)){
							aFolders[sAccId][sFolId+'/Files'].ACCESS = sRights;
							aFolders[sAccId][sFolId+'/Files'].RIGHTS = sRights.replace(/[md]/gi,'');
						}
					}
					if (aFolders[sAccId][sFolId+'/Tasks'] && aFolders[sAccId][sFolId+'/Tasks'].TYPE == 'T'){
						aFolders[sAccId][sFolId+'/Tasks'].NAME = aCommon.TASKS;
						if (bRights && (sRights = aFolders[sAccId][sFolId+'/Tasks'].RIGHTS)){
							aFolders[sAccId][sFolId+'/Tasks'].ACCESS = sRights;
							aFolders[sAccId][sFolId+'/Tasks'].RIGHTS = sRights.replace(/[md]/gi,'');
						}
					}
					if (aFolders[sAccId][sFolId+'/Journal'] && aFolders[sAccId][sFolId+'/Journal'].TYPE == 'J'){
						aFolders[sAccId][sFolId+'/Journal'].NAME = aCommon.JOURNAL;
						if (bRights && (sRights = aFolders[sAccId][sFolId+'/Journal'].RIGHTS)){
							aFolders[sAccId][sFolId+'/Journal'].ACCESS = sRights;
							aFolders[sAccId][sFolId+'/Journal'].RIGHTS = sRights.replace(/[md]/gi,'');
						}
					}
				}
			}
		}

		//Rozdělené foldery setřídíme
		aSortInbox.sort(sort);
		aSortOthers.sort(sort);
		aSortPublic.sort(sort);
		aSortShared.sort(sort);
		
		//Vytvoření seřazených folderů v rámci accountu
		aResultFolFrame = {};

		for(var n in aSortInbox){
			aFolder = aFolders[sAccId][aSortInbox[n]['FOLDER']];
			aResultFolFrame[aSortInbox[n]['FOLDER']] = {};
			for(var sItem in aFolder)
				aResultFolFrame[aSortInbox[n]['FOLDER']][sItem] = aFolder[sItem];
		}

		for(var n in aSortOthers){
			aFolder = aFolders[sAccId][aSortOthers[n]['FOLDER']];
			aResultFolFrame[aSortOthers[n]['FOLDER']] = {};
			for(var sItem in aFolder)
				aResultFolFrame[aSortOthers[n]['FOLDER']][sItem] = aFolder[sItem];
		}
		
		for(var n in aSortPublic){
			aFolder = aFolders[sAccId][aSortPublic[n]['FOLDER']];
			aResultFolFrame[aSortPublic[n]['FOLDER']] = {};
			for(var sItem in aFolder)
				aResultFolFrame[aSortPublic[n]['FOLDER']][sItem] = aFolder[sItem];
		}

		for(var n in aSortShared){
			aFolder = aFolders[sAccId][aSortShared[n]['FOLDER']];
			aResultFolFrame[aSortShared[n]['FOLDER']] = {};
			for(var sItem in aFolder)
				aResultFolFrame[aSortShared[n]['FOLDER']][sItem] = aFolder[sItem];
		}

		aResult[sAccId] = aResultFolFrame;
	}

	return aResult;
};

//********************************************************************************
//Pomocná funkce realizující převod z "XML" pole do Dataset struktur        * OK *
//********************************************************************************

_me.parse = function(aData,bRights)
{
	try
	{
		var aAccFrame = aData['IQ'][0]['QUERY'][0]['ACCOUNT'][0];
		var sAccId = aAccFrame['ATTRIBUTES']['UID'];
		var aFolFrame = aAccFrame['FOLDER'];
		var sFolId;
		var aResult = {};
		var aResultAccFrame = {};
		var aResultFolFrame;

		for(var nFolNum in aFolFrame){
			aResultFolFrame = {};

			for(var sItem in aFolFrame[nFolNum])
				if (sItem != 'ATTRIBUTES')
					aResultFolFrame[sItem] = aFolFrame[nFolNum][sItem][0]['VALUE'];

			sFolId = aFolFrame[nFolNum]['ATTRIBUTES']['UID'];
			aResultAccFrame[sFolId] = aResultFolFrame;
		}
		aResult[sAccId] = aResultAccFrame;

		return this.sort(aResult,bRights);
	}
	catch(e){
		return false;
	}
};

// TODO move this function somewhere else
_me.__emptyFolder = function(sAccId,sFolId,bMoveToTrash)
{
	var aTrashFolder;
	/*
	var nCount = parseInt(dataSet.get('folders',[sAccId,sFolId,'TOTAL']));
	if (!Is.Number(nCount) || nCount <= 0)
		return;
	*/
	if (bMoveToTrash) {
		var aFolData = clone(dataSet.get('folders', [sAccId,sFolId]), true);
		if (aFolData.TYPE == 'M'){
			storage.library('gw_others');
			aTrashFolder = Path.split(GWOthers.getItem('DEFAULT_FOLDERS', 'trash'));
			var sDestAccount = aTrashFolder[0];
			var sDestFolder = aTrashFolder[1];
			var aDestData = clone(dataSet.get('folders', [sDestAccount,sDestFolder]), true);

			if (Is.Defined(aDestData) && Is.Defined(aFolData)) {
				var iRec = parseInt(aFolData['RECENT'] || 0) + parseInt(aDestData['RECENT'] || 0);
				var iOld = parseInt(dataSet.get('folders', [sDestAccount,sDestFolder,'RECENT']) || 0);
				if (iRec != iOld)
					dataSet.add('folders', [sDestAccount,sDestFolder,'RECENT'], iRec.toString());
			}
			aTrashFolder = {'aid':sDestAccount,'fid':sDestFolder};
		}
	}

	if (dataSet.get('folders',[sAccId,sFolId,'RECENT'])>0)
		dataSet.add('folders',[sAccId,sFolId,'RECENT'],'0');

	//Remove mailview if in the same folder
	var aMView = dataSet.get('mailview');
	if (typeof aMView == 'object'){
		for(var mwa in aMView)
			for(var mwf in aMView[mwa]);

		if (mwa == sAccId && mwf == sFolId)
			dataSet.remove('mailview');
	}

	this.empty({'aid':sAccId,'fid':sFolId},'folders','',aTrashFolder);
};


///////////////////////////////
folders = new wm_folders();		// For bacward compatibility
var WMFolders = folders;		// TODO: create real static functions in wm_folders instead of simulating it.