function wm_freebusy()
{
  this.xmlns='freebusy';
}

wm_freebusy.inherit(wm_generic);
var _me = wm_freebusy.prototype;

/**
 * @brief   Gets informations from calendar (which times an user is FREE/BUSY).
 * @param[in]  aFreebusyInfo
 *    - email  Email of the user. 
 * @param[in]  sDataSet    Dataset name.
 * @param[in]  aDataPath   Array containing path ['folder','subfolder',...] dataset.folder.subfodler.
 * @return  Array containing these items:
 *    - TYPE      BUSY | TENTATIVE | FREE
 *    - START     Starting time in unix format.
 *    - FINISH    Final time in unix format.
 */  
_me.get = function(aFreebusyInfo,sDataSet,aDataPath,aHandler)
{
/*
*** IN ***
{iq}
<from>{julian date}</from>
<to>{julian date}</to>
<users>
<user>{email}</user>
<user>{email}</user>
...
</users>
{/iq}
*/
	var aRequest = {ACCOUNT:[{VALUE:sPrimaryAccount}]};
	if (aFreebusyInfo.from)
		aRequest.FROM = [{VALUE:aFreebusyInfo.from}];
	if (aFreebusyInfo.to)
		aRequest.TO = [{VALUE:aFreebusyInfo.to}];

	aRequest.USERS = [{USER:[]}];
	for(var i in aFreebusyInfo.users)
		aRequest.USERS[0].USER.push({VALUE:aFreebusyInfo.users[i]});

	//ClientTimeZone
    aRequest.CTZ = [{VALUE:(new Date).getTimezoneOffset()*-1}];

	// prepare data for request
	if (!sDataSet) {
		// synchronous
		return this.parse(this.create_iq(aRequest));
	}
	else {
		// asynchronous
		this.create_iq(aRequest,[this,'response',[aHandler,aFreebusyInfo.from,aFreebusyInfo.to]]);
		return true;
	}
}

/**
 * @brief   Callback function (from asynchronous call).
 * @param[in]  aData       Data returned from HTTP response.
 * @param[in]  sDataset    Dataset name.
 * @param[in]  aDataPath   Array containing path ['folder','subfolder',...] dataset.folder.subfodler.
 */ 
_me.response = function(aData,aHandler,iFrom,iTo){

	var aXMLResponse = aData['Array'];
	var aIQAttribute = aXMLResponse['IQ'][0]['ATTRIBUTES'];

	if (aIQAttribute['TYPE'] != 'result') return;
	
	var out = {};
    var email,itm,st,en,tmp;
	var user = aXMLResponse.IQ[0].QUERY[0].USER;

    for(var i = 0;i<user.length;i++){

		if (!user[i].ITEM) continue;
		
		email = user[i].ATTRIBUTES.EMAIL;

		for(var j = 0;j<user[i].ITEM.length;j++){
			itm = user[i].ITEM[j];

			st = iFrom && itm.STARTDATE[0].VALUE<iFrom?iFrom:itm.STARTDATE[0].VALUE;
			en = iTo && itm.ENDDATE[0].VALUE>iTo?iTo:itm.ENDDATE[0].VALUE;

            tmp = {};
            
			for(;st<=en;st++){
				if (!out[st]) out[st] = {};
				if (!out[st][email]) out[st][email] = [];

				for(var k in itm)
					tmp[k] = itm[k][0].VALUE;

                if (st<tmp.ENDDATE) tmp.ENDTIME = 1440;
                if (st>tmp.STARTDATE) tmp.STARTTIME = 0;

				tmp.STARTTIME = parseInt(tmp.STARTTIME);
                tmp.ENDTIME = parseInt(tmp.ENDTIME);

				out[st][email].push(tmp);
			}
		}
	}
	
	executeCallbackFunction(aHandler,out);
};

/**
 * @brief   Process data obtained from HTTP response and return them
 * in cultivated form.
 * @param[in]  aData   ['Array'] item from datas passed to callback function response()
 *    or returned from create_iq().
 * @see  response()
 * @see  create_iq()
 */ 
_me.parse = function(aData)
{

/*
<?xml version="1.0" encoding="utf-8" ?>
<iq type="result">
<query xmlns="webmail:iq:freebusy">
<user email="admin@icewarpdemo.com">
<item>
<type>BUSY</type>
<start>1203764400</start>
<finish>1203782400</finish>
</item>
<item>
<type>BUSY</type>
<start>1203850800</start>
<finish>1203868800</finish>
</item>
<item>
<type>BUSY</type>
<start>1203937200</start>
<finish>1203955200</finish>
</item>
<item>
.
.
.
</user>
</query>
</iq>
*/




   var aItems = aData['IQ'][0]['QUERY'][0]['ITEM'];
   
   $aResult = {};
   for (var nIndex in aItems) {
      $aResult[nIndex] = {};
      $aResult[nIndex]['TYPE'] = aItems[nIndex]['TYPE'][0]['VALUE'];
      $aResult[nIndex]['START'] = aItems[nIndex]['START'][0]['VALUE'];
      $aResult[nIndex]['FINISH'] = aItems[nIndex]['FINISH'][0]['VALUE'];
   }
   
   return $aResult;
}

//var freebusy = new wm_freebusy();