/**
 *  GENERIC class for all wm_<name> classes
 **/
function wm_generic(){
	this.xmlns; // All descendants needs this property set or set it as create_iq arg
	this.error; // error object
}
var _me = wm_generic.prototype;
	/**
	* @brief : IQ request builder function
	* @param :	aData		- xml content between <query> tags as an Array
	*			oResponse	- response object [object,method,[optional params,...]]
	*			sType		- Optional, GET or SET "type" attribute of <IQ> tag
	*			sId			- Optional, ID attribute of <IQ> tag
	*			sNs			- Optional (this.xmlns is defined in all descendants), xmlns attr. of <query> tag
	*
	* @return: Whole XML response in Array
	* @date  : 26.5.2006 16:36:53
	**/
_me.create_iq = function (aData,oResponse,sType,sId,sNs,bJSON){

	delete this.error;

	/* supported xmlns */
	var xmlns = {"auth":1,
			"accounts":1,
			"folders":1,
			"items":1,
			"freebusy":1,
			"spellchecker":1,
			"public":1,
			"private":1,
			"domain":1,
			"message":1,
			"import":1,
			"export":1
		};

	/* set xmlns from property */
	sNs = sNs || this.xmlns;
	if (!xmlns[sNs]){
		throw new Error('create_iq: unsupported xmlns "'+sNs+'"');
		//return (oReponse?false:{});
	}
	sNs = 'webmail:iq:'+sNs;

	/* prepare IQ object structure */
	var request = {"IQ":[{"ATTRIBUTES":{},"QUERY":[{"ATTRIBUTES":{}}]}]};

	/* append values into IQ */
	if ((sSID = dataSet.get('main',['sid'])))
		request['IQ'][0]['ATTRIBUTES']['SID']  = sSID;
	if (sId) request['IQ'][0]['ATTRIBUTES']['UID']   = sId;
		request['IQ'][0]['ATTRIBUTES']['TYPE'] = (sType!='set'?'get':'set');
		request['IQ'][0]['QUERY'][0]['ATTRIBUTES']['XMLNS'] = sNs;

	if (sType != 'set' || bJSON)
		request['IQ'][0]['ATTRIBUTES']['FORMAT'] = 'json';

	/* append data into query */
	if (aData && typeof aData == 'object')
		request['IQ'][0]['QUERY'][0] = arrConcat(request['IQ'][0]['QUERY'][0],aData);

	/* create httprequest instance */
	var http = new cHttpRequest();

	if (oResponse){
		http.sendArray(request,[this,'response_check',[oResponse]]);
		return true;
	}
	else{
		http.sendArray(request);
		
		var aOut = http.responseArray();

		// Error handler
		if (aOut['IQ'][0]['ATTRIBUTES']['TYPE']=="error"){
			var aErr = aOut['IQ'][0]['ERROR'][0];
			this.error = {};
			this.error["text"] = aErr['VALUE'];
			if (aErr['ATTRIBUTES'] && aErr['ATTRIBUTES']['UID']){
				this.error["id"]   = aErr['ATTRIBUTES']['UID'];
				this.error["lang"] = getLang("ERR_" + aErr['ATTRIBUTES']['ID']);
			}
		}

		return aOut;
	}
};

/**
 * Check for Server side low level errors
 * @Date: 2.11.2007 14:21:33
 **/
_me.response_check = function(aData,oResponse){
	var aXMLResponse = aData['Array'];
	var aIQAttribute = aXMLResponse['IQ'][0]['ATTRIBUTES'];

	if (aIQAttribute['TYPE'] == 'error'){
		try{
			switch(aXMLResponse.IQ[0].ERROR[0].ATTRIBUTES.UID){
			case 'session_expired':
			case 'session_no_user':
			
				dataSet.add('main',['sid'],'');

				if (gui && gui.frm_main)
					gui.frm_main.__logout();

				return;
			}
		}
		catch(er){}
	}

	executeCallbackFunction(oResponse, aData);
};

/**
 * @brief: HTTPrequest response handler
 * @note : this function has to be defined in all objects
 **/
_me.response = function(){};