<?php
/**
@file
@brief Script for downloading attachments.
*/
require_once('inc/get.php');
require_once('inc/user.php');
require_once('inc/mailparse.php');
require_once('inc/captcha.php');
require_once('inc/webmailiqitems.php');
function main() {
	if (($fullpath = Get()->fullpath) === null) {
		exit('Invalid fullpath');
	}
	if (($class = Get()->class) === null) {
		exit('Invalid class');
	}
	/* Get input variables. */
	if (($sid = Get()->sid) === null) {
		if($class!='captcha') {
			exit('Invalid sid');
		}
	}
	try {
		if($sid) {
			$user = User::load($sid);
		}
		$pathData = Tools::parseFullPath($fullpath,$class);
		switch($class) {
			// Send whole message / groupware item
			case 'itemtext':
			case 'itemhtml':
			case 'item':
				if(($pos = strpos($pathData['item'],'|')) !== false) {
					$pathData['part_id'] = substr($pathData['item'],$pos+1);
					$pathData['item'] = substr($pathData['item'],0,$pos);
					
				}
				$account_id = $pathData['account'];
				$folder_id = $pathData['folder'];
				$item_id = $pathData['item'];
				$part_id = $pathData['part_id'];
				$account = $user->getAccount($account_id);
				$folder = $account->getFolder($folder_id);
				$item = $folder->getItem($item_id);
				$aFilter['tag'] = 'date,subject,header_from,header_to,cc,bcc,html,text,attachments';
				if($class=='item'){
					if($part_id){
						$item->sendAttachment($part_id,$part_id,array(),$item->getSubjectFileName('.eml'));
					}else{
						$item->sendData();
					}
				}else{
					$aItem = WebmailIqItems::cnvMailItem($item,$aFilter,$part_id?$part_id:'1',false);
				
					//Tools::saveItem($aItem,)
					$aLanguage = Tools::loadLanguage();
					if($class=='itemtext'){
						$aItem = Tools::prepareItemToBeSaved($aItem);
						if(!$aItem['text'] && $aItem['html']){
							$aItem['text'] = Tools::removeHTML($aItem['html']);
						}
						$aItem['language'] = $aLanguage;//
						Tools::sendFileHeaders($item->getSubjectFileName('.txt'),false,'text/plain','UTF-8');
						echo template('inc/templates/saveastext.tpl',$aItem);
					}
					if($class=='itemhtml'){
						$aItem = Tools::prepareItemToBeSaved($aItem,'html');
						if(!$aItem['html'] && $aItem['text']){
							$aItem['html'] = Tools::text2html($aItem['text']);
						}
						$aItem['language'] = $aLanguage;//
						Tools::sendFileHeaders($item->getSubjectFileName('.html'),false,'text/html','UTF-8');
						echo template('inc/templates/saveashtml.tpl',$aItem);
					}
				}
				break;
			// Send message part / gw attachment
			case 'attachment':
				$account_id = $pathData['account'];
				$folder_id = $pathData['folder'];
				$item_id = $pathData['item'];
				$part_id = $pathData['part'];
				if(($pos = strpos($pathData['item'],'|')) !== false) {
					$start_part_id = substr($pathData['item'],$pos+1);
					$item_id = substr($pathData['item'],0,$pos);
				}
				$account = $user->getAccount($account_id);
				$folder = $account->getFolder($folder_id);
				$item = $folder->getItem($item_id);
				$item->sendAttachment($part_id,$start_part_id);
				break;
			// Export groupware
			case 'groupware':
				//Get account to be exported
				$sAccountID = $pathData['account'];
				$oAccount = $user->getAccount($sAccountID);
				//Does account have an groupware account attached ?
				if(!$oGWAccount = $oAccount->gwAccount) {
					throw new Exc('export_invalid_account',$sAccountID);
				}
				$sGWSessionID = $oGWAccount->gwAPI->OpenGroup('*');
				$sGroupWareXML = $oGWAccount->gwAPI->FunctionCall("ExportGroup",$sGWSessionID,$sAccountID);
				Tools::sendFileHeaders($sAccountID.'.xml',strlen($sGroupWareXML),'text/xml');
				echo $sGroupWareXML;
				break;
			case 'allattachments':
				$account_id = $pathData['account'];
				$folder_id = $pathData['folder'];
				$item_id = $pathData['item'];
				if(($pos = strpos($pathData['item'],'|')) !== false) {
					$start_part_id = substr($pathData['item'],$pos+1);
					$item_id = substr($pathData['item'],0,$pos);
				}else{
					$start_part_id = 1;
				}
				
				//dmp($pathData);die();
				$account = $user->getAccount($account_id);
				$folder = $account->getFolder($folder_id);
				$item = $folder->getItem($item_id);
				//Generate unique filename
				if ($item->wmclass=="M") {
					$fname = $item->getSubjectFileName();
				} else {
					$fname = date('YmdGis');
				}
				$fname.='.zip';
				$filename = $_SESSION['TEMPPATH'].$fname;
				// ZIP archive
				$zip = new ZipArchive();
				//Open zip archive
				if ($zip->open($filename, ZIPARCHIVE::CREATE)!==TRUE) {
					exit("cannot open <$filename>\n");
				}
				//Process item attachments
				//(GW does not check for attachments so hasAttachments property has not been set
				//so OR item class condition added)
				if ($item->hasAttachments || $item->wmclass=="GW") {
					if($item->wmclass!="GW") {
						$aAttachments = $item->getAllAttachmentFiles($start_part_id);
						
						foreach($aAttachments as $attachment){
							$zip->addFile($attachment["file"],$attachment["name"]);
						}
						
					}else{
						$aAttachments = $item->getAllAttachments();
						foreach($aAttachments as $attachment){
							$zip->addFromString($attachment["name"],$attachment["file_content"]);
						}
					}
				}
				//Clenup zip object
				$zip->close();
				
				//Send file to client
				Tools::sendFileHeaders($fname, filesize($filename), 'application/zip');
				readfile($filename);
				//Remove temporary file
				@unlink($filename);
				break;
			// Send recently uploaded attachment (stored in upload folder)
			case 'file':
				$folder_id = $pathData['folder'];
				$item_id = $pathData['item'];
				$userAttachments = $user->getAttachments();
				if (!isset($userAttachments[$folder_id][$item_id])) {
					exit('Attachment doesn\'t exist');
				}
				$attachment = $userAttachments[$folder_id][$item_id];
				$path = $attachment['file'];
				$size = filesize($path);
				$attachment['name'] = str_replace("+"," ",$attachment['name']);
				Tools::sendFileHeaders($attachment['name'], $size, $attachment['type']);
				$fp = fopen($path,'rb');
				while ($buffer = fread($fp,65536)) {
					echo $buffer;
				}
				fclose($fp);
				//WTF - Memory consumator
				//readfile($path);
				break;
			case 'exportcsv':
				$account_id = $pathData['account'];
				$folder_id = $pathData['folder'];
				$separator = $pathData['item'];
				//Get folder, account
				$oAccount = $user->getAccount($account_id);
				$oFolder = $oAccount->getFolder($folder_id);
				//Retrieve addresses
				$aAddresses = $oFolder->getContactList();
				//Generate temporary file
				$fileName = Tools::randomFilename();
				//Send headers
				Tools::sendFileHeaders("AddressBook_".date("Ymd-His").'.csv',0,'application/vnd.ms-excel');
				@$file = fopen($fileName,'wb+');
				//Stream addresses to file
				if($aAddresses) {
					$oFolder->saveAddressesToFile($file,$aAddresses,$separator);
					fseek($file,0);
					//Stream the output to the browser
					while(@$buffer = fread($file,65536)) {
						echo $buffer;
					}
				}
				@fclose($file);
				@unlink($file);
				break;
			case 'captcha':
				//Start the captcha session (it will be loaded in accounts::signup or forgot password)
				session_id($pathData['uid']);
				session_start();
				$oCaptcha = new Captcha;
				$sWord = $oCaptcha->getrandomword();
				$_SESSION['captcha']['word'] = $sWord;
				$_SESSION['captcha']['uid'] = $pathData['uid'];
				echo $oCaptcha->showrandomimage($sWord);
				break;
		}
	} catch (Exception $e) {
		exit($e->Message());
	}
}
main();
?>
