<?php

require_once('inc/system.php');

/**
  @class DB
  @brief Class for manipulating with the SQLite database.
  @author Martin Zlomek
  @revision Pavel Perna
  @date 08-02-2006
 */
class DB extends PDO {
	/** Transaction counter. */
	private $transaction;

	/** Rolback flag. */
	private $rollback;

	/**
	  @brief Create a connection to a database.
	  @param $database Database to connect to.
	  @param[out] $newDatabase True if a new database is created.
	 */
	public function __construct($location = false)
	{
	
	  // Text field for table structure
	  $this->sTextField = 'text';

	switch($_SESSION['DBTYPE']){
		case 'MYSQL':
		      $this->sDBType = 'TYPE = MyISAM';
		case 'MSSQL':
		case 'ODBC':

		      // Field type handling
		      if ($_SESSION['DBTYPE']=='ODBC') 
		      { 
		        $this->sAutoincrement = 'IDENTITY'; 
            $this->sTextField = 'varchar(255)';
		      } 
		      else 
		        $this->sAutoincrement = 'AUTO_INCREMENT';

          try{
            parent::__construct($_SESSION['DBCONN'],$_SESSION['DBUSER'],$_SESSION['DBPASS']);

            $stmt = $this->prepare("SELECT item_value FROM wm_metadata");
            $stmt->execute();

            $row = $stmt->fetch(PDO::FETCH_NUM);
           
            if (!is_array($row))  $newDatabase = true;
            else        $newDatabase = false;
       
          }catch (PDOException $e){
            //Try to create the database using mysql functions
            throw new Exc('db_auto_create',$e->getMessage());
          }
			break;
		case 'SQLITE':
    default:
		  $this->sAutoincrement = 'AUTOINCREMENT';
      $this->sDBType = '';
      $filename = str_replace("%WMUSERPATH%",$location,$_SESSION['PURECONN']);       

			$newDatabase = !is_file($filename);
			
      // Create database directory. 
			if ($newDatabase)
				if(!System::mkdir_r($location))	
          throw new Exc('db_create_sqlitefile',$filename);

		    $sConn = 'sqlite:'.$filename;

        parent::__construct($sConn);

        //Set pragmas
        $this->query('PRAGMA synchronous = 0;');
	      $this->query('PRAGMA temp_store = 2;');
	      
      break;
	 }


  $this->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
  $this->setAttribute(PDO::ATTR_STRINGIFY_FETCHES, false);
  $this->transaction = 0;
	
 
	 if ($newDatabase){
	 try{
	    $this->transaction();
			$this->query('CREATE TABLE folder (' /*)*/ .
						 'folder_id INTEGER PRIMARY KEY '.$this->sAutoincrement.', ' .
						 'parent_folder_id INTEGER, ' .
						 'account_id '.$this->sTextField.' NOT NULL, ' .
						 'name '.$this->sTextField.' NOT NULL, ' .
						 'rights INTEGER NOT NULL DEFAULT 0, '.
             'sync CHAR(1))'); 
      $this->query('CREATE TABLE item (' /*)*/ .
						 'item_id INTEGER PRIMARY KEY '.$this->sAutoincrement.', ' .
						 'folder_id INTEGER NOT NULL, ' .
						 'rid '.$this->sTextField.' NOT NULL, ' .
						 'size INTEGER NOT NULL, ' .
						 'date VARCHAR(32) NOT NULL, ' .
						 'header_from '.$this->sTextField.'  NOT NULL, ' .
						 'header_to '.$this->sTextField.'  NOT NULL, ' .
						 'subject '.$this->sTextField.' NOT NULL, ' .
						 'priority INTEGER NOT NULL DEFAULT 0, ' .
						 'flags INTEGER NOT NULL DEFAULT 0, ' .
						 'body TEXT, '. 
             'static_flags INTEGER NOT NULL DEFAULT 0, ' .
						 'smime_status INTEGER NOT NULL DEFAULT 0, ' .
						 'has_attachment CHAR(1) DEFAULT \'F\', ' .
						 'color CHAR(1) DEFAULT \'Z\', ' .
						 'completed_on VARCHAR(32)) '.$this->sDBType);
			$this->query('CREATE INDEX IDX_item_folder_id ON item (folder_id)');
			$this->query('CREATE INDEX IDX_item_date ON item (date)');

      //Create fulltext index on body for MySQL database	
      if($_SESSION['DBTYPE']=='MYSQL')
        $this->query('CREATE FULLTEXT INDEX IDX_item_body ON item (body(32000))');
			$this->query('CREATE TABLE wm_metadata (' .
						 'item_key VARCHAR(128), ' .
						 'item_value VARCHAR(128))');
			$stmt = $this->prepare('INSERT INTO wm_metadata (item_key, item_value) ' .
						 'VALUES (?, ?)');
			$stmt->execute(array('db_version', strval(WEBMAIL_DB_VERSION)));
			
	    $this->commit();
     	
		
			}catch(PDOException $e){
        throw new Exc('db_create_tables',$e->getMessage());
      }
    }else{
    	if($_SESSION['DBTYPE']=='MYSQL'){
				$this->query('SET character_set_connection = utf8;');
				$this->query('SET character_set_client = utf8;');
				$this->query('SET character_set_results = utf8;');
			}
		}
  }

  public function checkTables()
  {

    $stmt = $this->prepare("SELECT item_value FROM wm_metadata WHERE item_key = ? and item_value >= ?");
    if ($stmt) $stmt->execute(array(0=>'db_version',1=>WEBMAIL_DB_VERSION));

    if ($stmt)  $data = $stmt->fetch();
    if (!$data) return $this->alterTables();
    else return true;
  }
  
  private function saveQuery($sCommand)
  {
    try{
      $this->query($sCommand);
    }
    catch(Exception $e){
      
    }
  }
  
  private function alterTables()
  {
    $this->transaction();
    //FOLDER TABLE
    //Removed columns
    $this->saveQuery('ALTER TABLE folder DROP delimiter');
    //New columns
    $this->saveQuery('ALTER TABLE folder ADD sync CHAR(1)');
    $this->saveQuery('ALTER TABLE folder ADD rights INTEGER NOT NULL DEFAULT 0');
    //ITEM TABLE
    //Changed columns
    $this->saveQuery('ALTER TABLE item MODIFY header_from '.$this->sTextField);
    $this->saveQuery('ALTER TABLE item MODIFY header_to '.$this->sTextField);
    $this->saveQuery('ALTER TABLE item MODIFY date varchar(32)');
    $this->saveQuery('ALTER TABLE item MODIFY subject '.$this->sTextField);
    $this->saveQuery('ALTER TABLE item MODIFY rid '.$this->sTextField);
    //New columns 
    $this->saveQuery('ALTER TABLE item ADD sync CHAR(1)');
    $this->saveQuery('ALTER TABLE item ADD priority INTEGER NOT NULL DEFAULT 0');
    $this->saveQuery('ALTER TABLE item ADD static_flags INTEGER NOT NULL DEFAULT 0');
    $this->saveQuery('ALTER TABLE item ADD smime_status INTEGER NOT NULL DEFAULT 0');
    $this->saveQuery('ALTER TABLE item ADD has_attachment CHAR(1) DEFAULT \'F\'');
    $this->saveQuery('ALTER TABLE item ADD color CHAR(1) DEFAULT \'Z\'');
    $this->saveQuery('ALTER TABLE item ADD completed_on VARCHAR(32)');
    $this->saveQuery('ALTER TABLE item ADD body TEXT');
    //Update version
    try{
		  $stmt = $this->prepare('UPDATE wm_metadata SET item_value = ? WHERE item_key = ?');
		  $stmt->execute(array(strval(WEBMAIL_DB_VERSION), 'db_version'));
		//Or create metadata table on error(table not exist yet)
    }catch(Exception $e){
			$this->query('CREATE TABLE wm_metadata (' .
						 'item_key VARCHAR(128), ' .
						 'item_value VARCHAR(128))');
			$stmt = $this->prepare('INSERT INTO wm_metadata (item_key, item_value) ' .
						 'VALUES (?, ?)');
			$stmt->execute(array('db_version', strval(WEBMAIL_DB_VERSION)));      
    }
    $this->commit();
  }

	/**
	  @brief Begin a transaction.
	 */
	public function transaction()
	{
		if (!$this->transaction++) {
			parent::beginTransaction();
			$this->rollback = false;
		}
	}

	/**
	  @brief Commit a transaction.
	  @return True on success, false on failure.
	 */
	public function commit()
	{
		if (!--$this->transaction) {
			if ($this->rollback)
				parent::rollback();
			else
				parent::commit();
		}
	}

	/**
	  @brief Roll back a transaction.
	  @return True on success, false on failure.
	 */
	public function rollback()
	{
		if (!--$this->transaction)
			parent::rollBack();
		else
			$this->rollback = true;
	}

	/**
	  @brief Get the ID of the last inserted row.
	  @return The ID.
	 */
	public function getID()
	{
		// PDO lastInsertID support
		if ($_SESSION['DBTYPE']!='ODBC' && $_SESSION['DBTYPE']!='MYSQL') 
			return intval(parent::lastInsertId());

		// ODBC - MS SQL @@IDENTITY usage
		else
		{
	            $stmt = $this->prepare("SELECT @@IDENTITY AS ID");
        	    $stmt->execute();
	            $row = $stmt->fetch(PDO::FETCH_ASSOC);
		    return intval($row['ID']);
		}
	}

	/**
	  @brief Prepare a statement for execution.
	  @param $statement SQL statement.
	  @return PDOStatement object.
	 */
	public function prepare($statement)
	{
	  global $aError;
		try{
      $stmt = parent::prepare($statement);
    }
    catch(Exception $e){
          throw new Exc('db_statement_prepare',$e->getMessage());
    }
   // $stmt->setFetchMode(PDO::FETCH_ASSOC);

		return $stmt;
	}

	/**
	  @brief Execute a statement.
	  @param $statement SQL statement.
	  @return The result as an array of rows of SELECT statement,
	  or number of affected rows of INSERT, UPDATE or DELETE statements.
	 */
	public function query($statement)
	{
		switch (strtoupper(substr($statement, 0, 6))) {
		case 'SELECT':
		case 'CREATE':
		case 'DESCRI':
			return parent::query($statement);

		case 'INSERT':
		case 'UPDATE':
		case 'DELETE':
		case 'VACUUM':
		case 'PRAGMA':
		case 'ALTER ':
		case 'SET NA':
		case 'SET CH':
		case 'SHOW C':
			return parent::exec($statement);

		default:
			throw new Exc('db_invalid_statement', substr($statement, 0, 6));
		}
	}
}

?>
