<?php
require_once 'inc/cache/folder.php';

/**
  @class Folder
  @brief Class representing a generic folder.
  @author Martin Zlomek
  @date 09-02-2006
 */
abstract class Folder {
	/** List of folder rights. */
	/** { */
	const RIGHT_ADMIN  = 0x01; /**< Perform SETACL. */
	const RIGHT_CREATE = 0x02; /**< CREATE new sub-mailboxes in any
								 implementation-defined hierarchy. */
	const RIGHT_DELETE = 0x04; /**< STORE DELETED flag, perform EXPUNGE. */
	const RIGHT_INSERT = 0x08; /**< Perform APPEND, COPY into mailbox. */
	const RIGHT_LOOKUP = 0x10; /**< Mailbox is visible to LIST/LSUB commands. */
	const RIGHT_READ	= 0x20; /**< SELECT the mailbox, perform CHECK, FETCH,
								 PARTIAL, SEARCH, COPY from mailbox. */
	const RIGHT_SEEN	= 0x40; /**< STORE SEEN flag. */
	const RIGHT_WRITE  = 0x80; /**< STORE flags other than SEEN and DELETED. */
  const RIGHT_MAILBOX_DELETE = 0x100;
  const RIGHT_MAILBOX_WRITE = 0x200;
  const RIGHT_EXPUNGE = 0x400;
  const RIGHT_C = 0x800;
  const RIGHT_D = 0x1000;
  
	/** } */

	/** Name of the folder. */
	public $name;

	/** Rights to the folder. */
	public $rights;
	
	public $gw;

	/**
	  @brief Create a new folder object.
	  @param $name Name of the folder.
	  @param $rights Rights to the folder.
	 */
	protected function __construct($name, $rights)
	{
		$this->name	= $name;
		$this->rights = $rights;
  }

	/**
	  @brief Create a new folder.
	  @param $name Name of the folder.
	  @return Folder object.
	 */
	protected static function create($name)
	{
	}

	/**
	  @brief Delete the folder.
	 */
	abstract protected function delete();

	/**
	  @brief Rename the folder.
	  @param $newName New name of the folder.
	 */
	abstract protected function rename($newName);

	/**
	  @brief Get the list of folder items.
	  @param $filter SQL WHERE contition.
	  @param $orderBy SQL ORDER BY condition.
	  @param $offset SQL OFFSET statement.
	  @param $limit SQL LIMIT statement.
	  @return The list as an array of Item objects.
	 */
	protected function getItems($filter, $orderBy, $offset, $limit)
	{
	}

	/**
	  @brief Get an item.
	  @param $itemID ID of the item.
	  @return Item object.
	 */
	protected function getItem($itemID)
	{
	}

  abstract public function deleteItems($oItems = false);
  abstract public function moveItems($oFolder,$oItems = false);
  abstract public function copyItems($oFolder,$oItems = false);
  

	/**
	  @brief Get number of items.
	  @param $flags Flags of counted items.
	  @return The number of items.
	 */
	abstract protected function countItems($flags = 0);


	/**
	  @brief Delete an item.
	  @param $itemID ID of the item.
	 */
	final public function deleteItem($itemID,$cache = array())
	{
		$item = &$this->getItem($itemID,$cache);	
    if ($this->gw) $cache = false;
    $item->delete($cache);
	}
	
}

require_once('inc/gw/folder.php');
require_once('inc/cache/folder.php');

?>
