<?php
require_once 'inc/include.php';
require_once 'inc/account.php';
require_once 'inc/gw/gwapi.php';
require_once 'inc/gw/folder.php';
/**
  @class GroupWareAccount
  @brief Class representing groupware account.
  @author Pavel Perna
  @date 12-03-2006
 */
 class GroupWareAccount extends Account {
		public $gwAPI;				// Merak Groupware API object
		
		public $sGWSessionID;	// Group Session ID
		  
		public $bLogged;			// Boolean - indicates whether user is logged to the GW 
		
		/**
		  @brief Create a new account object.
		  @param $username GroupWareAccount logging username.
		  @param $password GroupWareAccount logging password.
		 */
		public function __construct($username, $password, &$account)
		{
			/* Initialize an API object */
			$this->gwAPI = new MerakGWAPI();

			//Reference to account
			$this->account = &$account;
			
      /* Logging information */
			$this->gwAPI->user = $username;
			$this->gwAPI->pass = $password;

			/* Login to GroupWare account */
			if (!$this->sGWSessionID = $this->gwAPI->Login()) 
			{ 
				 LogError::Error('Groupware initialization failed');
				 $this->bLogged = false;
				 return;
			}
			else{
				$this->bLogged = true;

      }
			$this->getFolders();
		}
		
		/**
		  @brief Re-initialize GW API after a connection to GW fails.
		  @return TRUE if user was logged to the GW, FALSE otherwise
		 */
		private function reloadGWAPI()
		{
			/* Login to GroupWare account */
			if (!$this->sGWSessionID = $this->gwAPI->Login())
					return false;

				//$this->gwAPI->cleanup();
				$this->bLogged = true;
			  unset($this->sGWSessionID);
				return true;
		}
		
		/**
		  @brief Get the list of folders.
		  @return The list as an array of Folder objects.
		 */	
		public function getFolders()
		{
		    //Default groupware folders
  			$aTypes = Array( 'E'=>'Events',
  							         'N'=>'Notes',
  							         'T'=>'Tasks',
  							         'C'=>'Contacts',
  							         'J'=>'Journal',
  							         'F'=>'Files');
  			$aDefaultTypes = array_flip($aTypes);
			  
        $this->folders["gw"] = array ();

        //bLogged is never true this object  re-created 
        //on folder request
			  if (/*!$this->bLogged ||*/ !$this->gwAPI->IsConnected())
				{
				  if (!$this->reloadGWAPI())
				  	return array();
			  }
 
			  /* groupSessionID to pass as reference */
			  if (!$this->sGWSessionID = $this->gwAPI->OpenGroup("*")) 
					return;//throw new Exc('Groupware OpenGroup failed');
					
			  /* Append folders froum dynamic group */
			  if (!$aFolders = $this->gwAPI->GetFolders($this->sGWSessionID)) 
					throw new Exc('folder_get_list',$this->sGWSessionID);	
        
        /* Get group access right list and store it in gwapi appenden in gwaccount */
        if (!$_SESSION['GW'][$this->gwAPI->user]['rights']){
            
          $rights = $this->gwAPI->FunctionCall("GetGroupAccessRightsList",$this->gwAPI->sessid);
          $rights = $this->gwAPI->ParseParamLine($rights);    
  
          foreach ($rights as $right)
               $_SESSION['GW'][$this->gwAPI->user]['rights'][$right["GRP_ID"]] = $right["GRPRIGHTS"];     
        }

        /* Process folders */
			  foreach ($aFolders as $folder){
			    // Restrictions 
          if(isset($_SESSION['RESTRICTIONS']) && isset($folder["FDRTYPE"]) && $_SESSION['RESTRICTIONS'][$folder["FDRTYPE"]]) continue;

          // set proper path for the ID of a groupware folder
					$sName = str_replace('\\', '/', $folder["FDR_ID"]);
	       	$sRights = '';
           
          // get folder rights 
          $aRights = Tools::decimal2binary($_SESSION['GW'][$this->gwAPI->user]['rights'][$folder["FDRGRP_ID"]]); 

          $sRights.=$aRights[1]?'w':'';
          $sRights.=$aRights[5]?'r':'';
          $sRights.=$aRights[4]?'m':'';
          $sRights.=$aRights[3]?'d':'';

          //Folder is subfolder              
          if (($pos = strpos($sName,"/"))!==false){
            $sParentFolderName = substr($sName,0,$pos);
            //Parent folder does not exist in both groupware nor in mail folders
            if (!isset($this->folders["gw"][$sParentFolderName])
            &&  !isset($this->account->folders["main"][$sParentFolderName]))
              $this->folders["gw"][$sParentFolderName] = new GroupWareFolder($this,$sParentFolderName,'A',$sRights);       
          }
          
          $this->folders["gw"][$sName] = new GroupWareFolder($this,$folder['FDR_ID'],$folder['FDRTYPE'],$sRights);
        }
			  return $this->folders["gw"];
		}
		
		/**
		  @brief Get the list of folders.
		  @return The list as an array of Folder objects.
		 */		  
		public function getFolder($folderID, &$type="")
		{
				 // set the only allowed type for groupware objects
				 $type="gw"; 

				 if (!isset($this->folders["gw"][$folderID]))
				      $this->getFolders();

				 if (!isset($this->folders["gw"][$folderID]))
				 	    throw new Exc('folder_does_not_exist', $folderID, true, false); //no error id
				 
				 return $this->folders["gw"][$folderID];			

		}
		
		/**
		 @brief Adds an folder object to the folder list.
		 @param $folder - result item from GWAPI->GetFolders($groupSessionID)
		*/		
		public function createFolder($folder,$type,$virtual = false)
		{
         //parent::createFolder($name);
				 return GroupWareFolder::create($this,$folder,$type);
		}
		
		public function sync(){
              unset($this->gwAPI->seconds);
    }
		
		/**
		 @brief Logout user (destroy groupware session)
		*/		 
		public function logout()
		{
			if ($this->gwAPI->IsConnected()) $this->gwAPI->Logout();
		}
 }
?>
