<?php
class GroupWareAddon
{
	private static $aAddonTypes =  array('recurrence'=>0,
	                      'reminder'=>1,
	                      'contact'=>2,
	                      'note'=>3,
	                      'location'=>4,
	                      'phone'=>5,
	                      'certificate'=>6,
                        'attachment'=>7);
	
	private static $aAddonTags =    array(0=>'recurrences',
	                        1=>'reminders',
	                        2=>'contacts',
	                        3=>'notes',
	                        4=>'locations',
	                        5=>'phones',
	                        6=>'certificates',
                          7=>'attachments');  
	                                   
	private static $aPrefixes     =  array(0=>'RCR',
	                        1=>'RMN',
	                        2=>'CNT',
	                        3=>'NOTNOTE',
	                        4=>'LCT',
	                        5=>'PHN',
	                        6=>'CRT',
                          7=>'');
	                                                                    
	private static $aGetFunctions =  array(0=>'GetEventRecurrence',
	                        1=>'geteventreminders',
	                        2=>'geteventcontacts',
	                        3=>'geteventnotetext',
	                        4=>'getcontactlocations',
	                        5=>'getcontactlocationphones',
	                        6=>'getcontactcertificate',
                          7=>'getattachmentlist');  
	
	private static $aSetFunctions =  array(0=>'addeventrecurrence',
	                        1=>'addeventreminder',
	                        2=>'addeventcontact',
	                        3=>'seteventnotetext',
	                        4=>'addcontactlocation',
	                        5=>'addcontactlocationphone',
	                        6=>'setcontactcertificate',
                          7=>'addattachment'); 
	                         
	private static $aRemoveFunctions =  array(0=>'deleteeventrecurrence',
	                        1=>'deleteeventreminders',
	                        2=>'deleteeventcontacts',
	                        3=>'seteventnotetext',
	                        4=>'deletecontactlocations',
	                        5=>'deletecontactlocationphones',
	                        6=>'setcontactcertificate',
                          7=>'deleteattachments');  
                        

                                                                       
      public function __construct(&$oItem,$sAddonType,$sParentID = false)
      {
            $this->sParentID = $sParentID;
            $this->sAddonType = $sAddonType;
            $this->AddonTypeID = self::$aAddonTypes[$sAddonType];

            $this->item = &$oItem; 
      }
      
      public function getData($param = null)
      {
            $oAccount = &$this->item->folder->account;
            $sFID = $this->item->openAccess();

            if ($this->sParentID) $itemID = $this->sParentID;
            else $itemID = $this->item->itemID;

            $result = $oAccount->gwAPI->FunctionCall(self::$aGetFunctions[$this->AddonTypeID],
                                                      $sFID,
                                                      $itemID,
																											$param);

            switch($this->AddonTypeID)
            {
                  //Note 
                  case 3:
                        if($result) return array(array('note_text' => $result));
                  break;
                  case 6:
                        if ($result) return array(array('certificate' => $result));
                  break;
                  //Location
                  case 4:
                       $result = $oAccount->gwAPI->ParseParamLine($result);
                       if (is_array($result)) {
                        foreach($result as $aLocation)
                        {
                            $sLctID = $aLocation['LCT_ID'];
                            
                            
                            $this->lowlevel['phones'][$sLctID] = new GroupWareAddon($this->item,'phone',$sLctID);
                        }
                        return $result;
                       }
                  break;
                  //Other
                  default:
                       $result = $oAccount->gwAPI->ParseParamLine($result);
                       if(isset($result[0])) return $result;
                  break;
                  
            }
                       
            return false;
      }
		
		/**
		 @brief Create a new addon
		 @param $oDOMDoc - document XML containter
		 @param $oParent - item element in document
    */					
		public function create($aData,$sParentID = false)
		{
          if (!$sParentID) $sParentID = $this->item->itemID;
          $oAccount = &$this->item->folder->account;

          
					//Get parameters for the Add function
					switch ($this->sAddonType)
					{
						case 'attachment':
							$this->processAttachments($aData);
							return;
						case 'note':
							$sParam = Tools::convertLFtoCRLF($aData['note_text']);
							break;
						
						case 'certificate':
							$sParam = $aData['certificate'];
							break;
							
						default:
							$sParam = $oAccount->gwAPI->CreateParamLine($aData);
							break;
					}

          $sFID = $this->item->openAccess();

          $result = $oAccount->gwAPI->FunctionCall(self::$aSetFunctions[$this->AddonTypeID],
                                                $sFID,
                                                $sParentID,
                                                $sParam);
                                                
          
          return $result;
    
    }
    

		/**
		 @brief Process attachments of an item
		 @param $aData Data obtained from XML
    */
		private function processAttachments($aData)
		{
			$attData = Tools::parseFullPath($aData['fullpath'],$aData['class']);
			//Process actions depending on type
			switch($aData['class'])
			{
				case 'file':
					$attachItems = $_SESSION['user']->getAttachments($attData['folder']);
					$attachItem = $attachItems[$attData['item']];
					$this->attachFile($attachItem['file'],
												$attachItem['name'],
												$attachItem['type'],
												'',
												'F');
					
					// cache the folder name for future deleting
					$this->aFolderList[$attData['folder']] = 1;
					break;
				case 'item':
  
					$this->attachItem($attData['account'],
												$attData['folder'],
												$attData['item'],
												$aData['description']);
					break;
				case 'attachment':
					$this->attachAttachment($attData['account'],
												$attData['folder'],
												$attData['item'],
												$attData['part'],
												$aData['description']);
					break;
				case 'itemlink':
					$this->attachItemLink($attData['account'],
												$attData['folder'],
												$attData['item'],
												$aData['description']);
					break;
				case 'image':
					$attachItems = $_SESSION['user']->getAttachments($attData['folder']);
					$attachItem = $attachItems[$attData['item']];
					$this->attachFile($attachItem['file'],
												$attachItem['name'],
												$attachItem['type'],
												'',
												'P');
					
					// cache the folder name for future deleting
					$this->aFolderList[$attData['folder']] = 1;
				break;
				case 'url':
					break;
			}
		}
		
		/**
		 @brief Delete addon specified by id
		 @param $sAID - string addon ID
    */					
		public function delete($sAID,$sParentID = false)
		{
          if (!$sParentID) $sParentID = $this->item->itemID;
          $oAccount = &$this->item->folder->account;

          $sFID  = $this->item->openAccess();
          
          switch($this->sAddonType)
          {   
              // in some cases the item is deleted if it is set to empty string:
							case 'note':
              case 'certificate':
              		$sAID = '';
              		break;
          }

          $result = $oAccount->gwAPI->FunctionCall(self::$aRemoveFunctions[$this->AddonTypeID],
                                                $sFID,
                                                $sParentID,
                                                $sAID);
                                                
          
          return $result;
    
    }   
    
		/**
		 @brief Edit addon specified by id
		 @param $sAID - str)ng addon ID
    */					
		public function edit($aData,$sAid,$sParentID = false)
		{
          if (!$sParentID) $sParentID = $this->item->itemID;
          $oAccount = &$this->item->folder->account;

          $sFID = $this->item->openAccess();

          $result = true;
          $aNewAddon = array();

          if ($aData) {
						$aData = $oAccount->gwAPI->CreateParamLine($aData);

            $result = $oAccount->gwAPI->FunctionCall(self::$aSetFunctions[$this->AddonTypeID],
                                                $sFID,
                                                $sParentID,
                                                $aData,
                                                $sAid);
					}
          
          return $result;
    
    }   
    
      
    public function process($aTreeItem)
    {
          //aFolderList stores all folders used to cache attachment addons
          $this->aFolderList = array();

          $aSentData = $this->getSentData($aTreeItem);   

	  // Distribution list
	  $dl = $this->sAddonType == 'location' && ($this->item->item['ITMCLASS']=='L');
	  $counter = 0;
       
          if ($aSentData) foreach($aSentData as $aData)
          {
          			$aData = Tools::unhtmlspecialchars_array($aData);
                //$aData['id'] = Tools::unhtmlspecialchars($aData['id']);
                switch($aData['action'])
                {
                      //Add new addon
                      case 'add':

	
			    // Distribution list add handling
			    if ($dl) 
			    {
                              if ($counter) $aData['data']['lcttype']='L';
                              $counter++;
			    }                                    
                            $addonID = $this->create($aData['data']);

                            //Create new phones 
                            if ($aData['phones']) {
                                  
                                  $oAddon = new GroupWareAddon($this->item,'phone',$addonID);
                                  foreach($aData['phones'] as $aPhone){
                                      if($aPhone['data']['phnnumber'])
                                        $oAddon->create($aPhone['data'],$addonID);
                                  }
                            }
                              
                      break;
                      
                      //Edit existing addon
                      case 'edit':
                          
                          $this->edit($aData['data'],$aData['id']);
                          
                          //Phones - add/edit/delete 
                          if ($aData['phones']) {
                                $oAddon = new GroupWareAddon($this->item,'phone',$aData['id']);
                                foreach($aData['phones'] as $aPhone)
                                {
                                    switch($aPhone['action'])
                                    {
                                      case 'add':
                                          $oAddon->create($aPhone['data'],$aData['id']);
                                      break;
                                      case 'edit':
                                          $oAddon->edit($aPhone['data'],$aPhone['id'],$aData['id']);
                                      break;
                                      case 'delete':
                                          $oAddon->delete($aPhone['id'],$aData['id']);
                                      break;
                                    }
                                }
                          }
                      break;
                      
                      //Delete existing addon
                      case 'delete':
                          $this->delete($aData['id']);
                      break;
                }

          }

          // Distribution list postprocessing
          if($dl) {

	    // Get contact locations with O type
            $aLocations = $this->getData('LCTTYPE=\'O\'');

	    // If none then get all (probably deleted the O item or blank)
 	    if (!$aLocations) $aLocations = $this->getData();

            $counter = 0;
            if($aLocations) foreach($aLocations as $location){

	      // Get correct type	      
              if (!$counter) $lcttype = 'O'; else $lcttype = 'L';

	      // Set if different
              if ($location['LCTTYPE'] <> $lcttype) 
	      {
		$location['LCTTYPE'] = $lcttype;
  	        $this->edit($location,$location['LCT_ID']);
	      }

              $counter++;
            }
          }
          
          // remove all temporary attachment folders
          $oUser = &$_SESSION['user'];
          foreach(array_keys($this->aFolderList) as $folder)
						$oUser->removeAttachments($folder);
    } 
    
		/**
		 @brief Load data from server
    */		    
    public function load()
    {
    
    }
    
		/**
		 @brief Get data sent to server
		 @param $oDOMDoc - document XML containter
		 @param $oParent - item element in document
    */					
		public function getSentData($aTreeItem)
		{

		      $result = array();

         $node = $aTreeItem['@childnodes'][self::$aAddonTags[$this->AddonTypeID]][0]['@childnodes'][$this->sAddonType];
         
         
         if($node)
         foreach($node as $key => $addon){
          $addondata = array();
          //ID is set - Edit/Delete
          if($sAddonID = $addon['@attributes']['uid']){
            //Edit - parameters are set
            if($addon['@childnodes']['values'])
              $result[$key]['action'] = 'edit';
            //Delete  
            else
              $result[$key]['action'] = 'delete';
              
            $result[$key]['id'] = $sAddonID;
          //ID is not set - Add  
          } else {
              $result[$key]['action'] = 'add';
          }
          
          //Locations special handling
          if($this->sAddonType=='location'){
            if($addon['@childnodes']['phones'])
              foreach($addon['@childnodes']['phones'][0]['@childnodes']['phone'] as $key2 => $phone){
                $phonedata = array();
                //ID is set  - edit/delete 
                if($pid = $phone['@attributes']['uid']){
                  $result[$key]['phones'][$key2]['id'] = $pid;
                  //ID & values => edit
                  if($phone['@childnodes']['values'])
                    $result[$key]['phones'][$key2]['action'] = 'edit';
                  //ID & !values => delete
                  else
                    $result[$key]['phones'][$key2]['action'] = 'delete';
                //No ID is set => add
                } else {
                    $result[$key]['phones'][$key2]['action'] = 'add';
                }
                
                //Get data to add or edit phone
                if ($result[$key]['phones'][$key2]['action']!='delete'){
                  foreach($phone['@childnodes']['values'][0]['@childnodes'] as $variable => $value)
                    $phonedata[$variable] = $value[0]['@value'];
                  $result[$key]['phones'][$key2]['data'] = $phonedata;
                }
              }
          }
        
          if ($addon['@childnodes']['values']){
            foreach($addon['@childnodes']['values'][0]['@childnodes'] as $variable => $value)
              $addondata[$variable] = $value[0]['@value'];
            $result[$key]['data'] = $addondata;
          }
                    
        }
  
	        return $result;	          
    }
    

    
		/**
		 @brief Get XML from server data
    */					
		public function getXML()
		{

		      $data = $this->getData();

		      $sReturn = '';
				if ($data) foreach ($data as $item)
		      {
		            //Attachments 
		            if ($this->sAddonType=='attachment')
                {
                  switch ($item['ATTTYPE'])
                  {
                    case 'F':
                      $item['ATTTYPE']='attachment';
                      break;
                    case 'I':
                      $item['ATTTYPE']='itemlink';
                      break;
                    case 'U':
                      $item['ATTTYPE']='url';
                      break;
                  }
                  $sID = htmlspecialchars($item['ATTNAME']);
                } else            
    		          $sID = $item[self::$aPrefixes[$this->AddonTypeID].'_ID'];
                  
                  $sStart  = '<'.$this->sAddonType.''.($sID?(' uid="'.$sID.'"'):('')).'>';
                  $sValue = Tools::makeXMLTags($item);
                  $sEnd = '</'.$this->sAddonType.'>';
                    
                  //Locations - phones
                  if ($this->sAddonType=='location'){
                          $sValue = '<values>'.$sValue.'</values>';
                          $phones = $this->lowlevel['phones'][$item['LCT_ID']]->getXML();
                          $sValue.=$phones;
                    
                  //All other
                  } else
                          $sValue = '<values>'.$sValue.'</values>';
                
                
                  $sReturn.=$sStart.$sValue.$sEnd;
          }
          
          if ($sReturn !== '')
          {
                $sReturn =  '<'.self::$aAddonTags[$this->AddonTypeID].'>'.
                            $sReturn.
                            '</'.self::$aAddonTags[$this->AddonTypeID].'>';
          }
            
          return $sReturn;            
    }    
    
 		public function getAttachment($name)
 		{
 		  $oFolder = &$this->item->folder;
 		  $oAccount = &$oFolder->account;
         
      $sFID = $this->item->openAccess();

      $result = $oAccount->gwAPI->FunctionCall("GetAttachment",
                                     $sFID,
                                     $this->item->itemID,
                                     $name);
                                     
      return $result;
     
    }
    
    public function getGroupAttachment($class,$group,$id)
    {
       $oFolder = &$this->item->folder; 
       $oAccount = &$oFolder->account;    
       
       $gid = $oAccount->gwAPI->OpenGroup($group);

       if ($class=='C')
       {
        $item = $oAccount->gwAPI->FunctionCall("getcontactinfo",$gid,$id);
       }
       else{
        $item = $oAccount->gwAPI->FunctionCall("geteventinfo",$gid,$id);
       }
       $item = $oAccount->gwAPI->ParseParamLine($item);
       $item = $item[0];
       $prefix = $item['EVNCLASS']=='C'?'ITM':'EVN';
       if ($item[$prefix.'FOLDER']=='')
       {
          switch($item[$prefix.'CLASS'])
          {
            case 'E':
              $fdr = 'Events';
              break;
            case 'T':
              $fdr = 'Tasks';
              break;
            case 'N':
              $fdr = 'Notes';
              break;    
            case 'C':
              $fdr = 'Contacts';
              break;             
            case 'J':
              $fdr = 'Journal';
              break;
            case 'F':
              $fdr = 'Files';
              break;

          }
       } else $fdr = $item[$prefix.'FOLDER'];

      // $owner = $oAccount->gwAPI->FunctionCall("getgroupowner",$group);
      $owner = $_SESSION['EMAIL'];
      
      $return['account'] = $owner;
      $return['folder'] =  $fdr;
      $return['item'] = $id;
      
      return $return; 
    } 
    
    public function addAttachment($param,$value,$objectID = false,$sFid = '')
    {
      $oFolder = &$this->item->folder; 
      $oAccount = &$oFolder->account;

      if (!$sFID) $sFID = $this->item->openAccess();
      if (!$objectID) $objectID = $this->item->itemID;

      //Create parameters line
      $param = $oAccount->gwAPI->CreateParamLine($param);
      if(!$oAccount->gwAPI->FunctionCall("AddAttachment",
                                      $sFID,
                                      $objectID,
                                      $param,
                                      $value))
        throw new Exc('attachment_add');
    }     
          
    public function attachFile($path,$filename,$mimeType,$fcontent='',$attType = 'F')
    {
      $oAccount = &$this->item->folder->account;
      
      if ($fcontent){
        $content = $fcontent;
      } else {
        if (!is_file($path))
          throw new Exc('attachment_file_not_found',$path);
      
        @$content = file_get_contents($path);
      }
      
      $now = time();
      $filename = basename($filename);
      $dot = strrpos($filename, '.');
      $md5 = '~' . substr(md5(mt_rand() . $now), 0, 4);
      if ($dot === false) {
				$attName = $filename . $md5;
			} else {
				$attName = substr($filename, 0, $dot) . $md5  . substr($filename, $dot);
			}
      
      $param['AttName'] = $attName;
      $param['AttType'] = $attType;
      $param['AttDesc'] = $filename;
      $param['AttSize'] = filesize($path);
      $param['AttTime'] = $now;
      $param['AttParams'] = Tools::createURL(array('mimetype' => $mimeType));

      $this->addAttachment($param,$content);
    }
    
    
 
    
    //TODO
    public function attachURL($url)
    {
    
    }

	public function attachItemLink($account,$folder,$item,$description)
	{
		$oAccount = &$this->item->folder->account;
		$attAccount = $_SESSION['user']->getAccount($account);
		$attFolder = $attAccount->getFolder($folder);
		$attItem = $attFolder->getItem($item);
		
		//GW item
		if ($attFolder->gw)
		{
			$sGID = ($attFolder->type=='C'?'ITM':'EVN').'GRP_ID';
			$cAID = $attFolder->type=='C'?'C':'E';
			$groupIndex = $attItem->item[$sGID];		
			$value = $cAID.':'.$groupIndex.':'.$attItem->itemID;
			
		//IMAP item ?
		} else {
			// not implemented yet
			throw new Exc('not_implemented');
		}
		
		$now = time();
		$param = array();
		$param['AttName'] = md5($attItem->itemID . $now);
		$param['AttType'] = 'I';
		$param['AttDesc'] = $description;
		$param['AttSize'] = 0;
		$param['AttTime'] = $now;

		$this->addAttachment($param,$value);
	}
		
	public function attachItem($account,$folder,$item,$description)
	{
		global $aError;
		
		$oAccount = &$this->item->folder->account;
		$attAccount = $_SESSION['user']->getAccount($account);
		$attFolder = $attAccount->getFolder($folder);
		$attItem = $attFolder->getItem($item);


		//GW item
		if ($attFolder->gw)
		{
			$value = $attItem->getVersitObject($info);
      $ext = $info['fileext'];
			$mimeType = $info['mimetype'];
			
		//IMAP item ?
		} else {
			$value = $attItem->getMessage($info);
			$ext = '.eml';
			$mimeType = 'message/rfc822';
		}

		// Remove ext if already present
	        if (eregi('\\' . $ext . '$', $description)) $ext='';

		$now = time();
		$param = array();
		$param['AttName'] = md5($attItem->itemID . $now);
		$param['AttType'] = 'F';
		$param['AttDesc'] = $description.$ext;
		$param['AttSize'] = strlen($value);
		$param['AttTime'] = $now;
		$param['AttParams'] = Tools::createURL(array('mimetype' => $mimeType));

		$this->addAttachment($param,$value); 
	}
	
	public function attachAttachment($account,$folder,$item,$partid,$description = false)
	{
		global $aError;
		
		$oAccount = &$this->item->folder->account;
		$attAccount = $_SESSION['user']->getAccount($account);
		$attFolder = $attAccount->getFolder($folder);
		$attItem = $attFolder->getItem($item);

    $value = $attItem->getAttachmentData($partid,$info);
    /* BUG 13453
		if($info['encoding']=='base64'){
      $value = base64_decode($value);
    }*/

    $now = time();
		$param = array();
		$param['AttName'] = $info['name'];
		$param['AttType'] = 'F';  
		$param['AttSize'] = strlen($value);
		$param['AttTime'] = $now;
		$param['AttParams'] = Tools::createURL(array('mimetype' => $info['mimetype']));
		
		if($description)
		  $param['AttDesc'] = $description;
		  
		$this->addAttachment($param,$value); 
	}
}
?>
