<?php

require_once 'inc/account.php';
require_once 'inc/folder.php';

require_once 'inc/include.php';
require_once 'inc/tools.php';
require_once 'inc/gw/gwapi.php';
require_once 'inc/gw/item.php';

/**
  @class GWFolder
  @brief Class representing groupware folder.
  @author Pavel Perna
  @date 03-06-2006
  @todo comments, some methods
 */
class GroupWareFolder extends Folder{
	  private static $aAliasTags = array('osd','oed');
    private static $contactQuery = '((LCTTYPE IS NULL) OR (ITMCLASS=\'C\' AND LCTTYPE=\'H\') OR (ITMCLASS=\'L\' AND LCTTYPE=\'O\'))';
 	
		public $folderID; // Internal ID of the folder. 
			
		public $name;	 // Name of the folder. 
		
		public $type;	 // Type of the folder. 
				
		public $folder;	// Storage struct(item of GWAPI->GetFolders result. 
	
		public $account; // Reference to the parent account. 

								
			
	  /**
		@brief Create a new folder object.
		@param $account parent account.
		@param $folder groupware folder parsed array .
		*/
		public function __construct(&$oAccount,$folder,$type,$rights = 'rwmd')
		{
	
		  // set proper path for the groupware folder
			$sName = str_replace('\\', '/', $folder);
			 
			$this->name= $sName;
		  $this->type = $type;
		  $this->folderID = $sName;	  
      $this->rights = $rights;

		  $this->account = &$oAccount;
		  $this->gw = true;
		  
		  //$this->getRights();

		}
		
		/**
		 @brief Loads a list of items for current folder.
		*/
		private function loadItems($aFilterTag)
		{
			$sFilter = $aFilterTag['sql'];
			$fields = $aFilterTag['tag'];

      $aAddons = $aFilterTag['gwAddons'];
			$attributes = array();
			if($aFilterTag['orderby']){
				if($_SESSION['COLLATE_ENABLE']){
					$aFilterTag['orderby'] = Tools::collateOrderBy($aFilterTag['orderby']);
				}
				$attributes[] = 'order='.$aFilterTag['orderby'];
			}
			if($aFilterTag['offset'])
				$attributes[] = 'POSITION='.$aFilterTag['offset'];
			if($aFilterTag['limit'])
				$attributes[] = 'limit='.$aFilterTag['limit'];
			$sAttributes = implode($attributes, '; ');
			$aItems = array();

			/* Open GW folder access and get folder session id($this->sFID) */	 
			$this->openAccess();

			if (!$sFilter) $sFilter = '';
			
			if (!$fields) {
			 if ($this->type=='C') 
				  $fields='ITM_ID';
				else
				  $fields='EVN_ID'; 
			} elseif ($this->type=='C') {
				// always get LCTTYPE attribute of message
				if (!strstr(strtoupper($fields), 'LCTTYPE'))
					$fields .= ',LCTTYPE';
			}
			
			
			//Contacts 
			if ($this->type=="C") {
			   			  
          $sFilter = ($sFilter?$sFilter." AND ":"").self::$contactQuery;

				  $eventsline = $this->account->gwAPI->FunctionCall('GetContactDetailList',
																	$this->sFID,
																	$sFilter,
																	$fields,
                                  $sAttributes);
     
				  $itemPrefix = "ITM";
      
      //Events
			} else {			
          //Remove tags that are used as alias
          $aFields = explode(",",$fields);

    			foreach($aFields as $item)
    				if (!in_array(trim($item),self::$aAliasTags)) 
              $array[] = trim($item);

    			$sIntervalFields = implode(",",$array);

          //Month & Week & Day View
				  if (isset($aFilterTag['interval']) && $aFilterTag['interval']){

            if (in_array('osd',$aFields))
              $sIntervalFields.=',EVNSTARTDATE as OSD';

            if (in_array('oed',$aFields))
              $sIntervalFields.=',EVNENDDATE as OED'; 

					  /* Call method ,select event class depending on folder type */
					  $eventsline = $this->account->gwAPI->FunctionCall('GetAllIntervalEvents', 
																		$this->sFID, 
																		$aFilterTag['interval'],																	 
																		$sFilter,
																		$sIntervalFields);
				  //List
          }else {
            $sFilter = ($sFilter?'('.$sFilter.') AND ':'') . "evnclass <> 'O'";
                   
					  $eventsline = $this->account->gwAPI->FunctionCall('GetEventList', 
																		$this->sFID, 
																		$sFilter,
																		$fields,
                                    $sAttributes);
																		
					  if($_SESSION['DEBUG'])log_buffer('GroupWareFolder->loadItems(list) : FolderID: '.$this->sFID.' Query:'.$sQuery.' Fields:'.$fields."\r\n",'DEBUG');
				  }										  						  
				  $itemPrefix = "EVN";
			}
          
      
			/* Parse result */
			$aEvents = $this->account->gwAPI->ParseParamLine($eventsline);

      //Reset seconds transition for GW TimeZone
      unset($this->account->gwAPI->seconds);
      
      /* Create list of GroupWare item objects */
			if ($aEvents) foreach($aEvents as $aEvent){
			  if (!isset($aEvent[$itemPrefix.'CLASS'])) $aEvent[$itemPrefix.'CLASS']=$this->type;
			  
        $aEvent['CTZ'] = $aFilterTag['timezone'];
			  $this->account->gwAPI->TimeZone($aEvent,'out'); 

			  if (isset($aFilterTag['interval']) && $aFilterTag['interval']){
				  
        	$aItems[] = new GroupWareItem($this,$aEvent,$aAddons,$fields);
			  
        }else{
                        $aItems[$aEvent[$itemPrefix."_ID"]] = new GroupWareItem($this,
                                                                    $aEvent,
                                                                    $aAddons,
                                                                    $fields);
      
        }
			}
			$this->count = count($aItems);
			return $aItems;
		}	
		/**
		 @brief Get all items for specified folder.
		 @return array with item's objects in folder
		*/	
		public function getItems($aFilterTag)
		{

		  $aItems = $this->loadItems($aFilterTag); 
		  return $aItems;
		}
	
	  private function prepareItems($aNoJoin,&$sAttributes,&$sFilter)
	  {
			    $aNoJoin = $this->account->gwAPI->FunctionCall('GetContactDetailList',
																	$this->sFID,
                                  $sFilter,
                                  'ITM_ID');

          $statement = '';

          if ($aNoJoin){
             $aNoJoin = $this->account->gwAPI->ParseParamLine($aNoJoin);
             foreach($aNoJoin as $item){
               if ($statement) $statement.=' OR ';
               $statement.="ITM_ID='".$item['ITM_ID']."'";
             }
             $sFilter = $statement;
          }  
    }
	
		/**
		 @brief Get item specified by id.
		 @return GroupWare item object
		*/		
		public function getItem($itemID,$bFlags = WITH_ADDONS)
    {
				/* Open folder access and get folder session id */	 
				$this->openAccess();

        $sType = isset($this->folder->type) ? $this->folder->type : $this->type;
        $prefix = $sType=='C'?'ITM':'EVN';

        $sTag = $sType=='C'?'Contact':'Event';

				/* Call method ,select event class depending on folder type */
				$aEvent = $this->account->gwAPI->FunctionCall('Get'.$sTag.'Info', 
															  $this->sFID,
															  $itemID);

				$aEvent = $this->account->gwAPI->ParseParamLine($aEvent);
				$this->account->gwAPI->TimeZone($aEvent[0],'out');
				//$this->closeAccess();
				if ($aEvent[0])
        { 
              if ($bFlags == NO_ADDONS) $bWithAddons = false;
              else $bWithAddons = true;
              return new GroupWareItem($this,$aEvent[0],array(),'*',$bWithAddons);
				}

				return false;
		}
    
    //Todo////////////////////////////////////////////////////////////
    public function deleteItems($oItems = false){
      foreach($oItems as $oItem)
        $oItem->delete();
    }
    
    public function moveItems($oFolder,$oItems = false){
      foreach($oItems as $oItem)
        $oItem->move($oFolder);      
    }
    
    public function copyItems($oFolder,$oItems = false){
      foreach($oItems as $oItem)
        $oItem->copy($oFolder); 
    }
  /////////////////////////////////////////////////////////////////////////
	  /**
	 	  @brief Get number of items.
	  	@param $flags Flags of counted items.
	  	@return The number of items.
		*/
		public function countItems($flags = 0,$positive = true,$filter = "")
		{
				// if a flag is required, return 0
				if ($flags)
					return 0;

        $this->openAccess();
				if ($this->type=="C"){
				
					$filter = ($filter?$filter." AND ":"").self::$contactQuery;
				  
           $count = $this->account->gwAPI->FunctionCall('GetContactDetailCount', $this->sFID, $filter/*'distinct'*/);

        }else
					$count = $this->account->gwAPI->FunctionCall('GetEventCount', $this->sFID, $filter);
	
				return $count;
		}

	
		/**
		 @brief Delete the folder.
		*/	 
		public function delete()
		{
		  global $aError;
		  
		  $this->openAccess();
		  
		  //Get parent account (Imap)
		  $oAccount = &$this->account->account;
      
		  //set proper path for the groupware folder
			$sName =str_replace('/', '\\', $this->name);

			 //Delete gw folder
			 if (!$oAccount->gwAccount->gwAPI->FunctionCall("DeleteFolder",$oAccount->gwAccount->sGWSessionID,$sName))
				throw new Exc('folder_delete',$sName);
			 
		  unset($oAccount->gwAccount->folders['gw'][$this->name]);
		  unset($oAccount->folders['gw'][$this->name]);
      if($_SESSION['DEBUG'])log_buffer("GWFolder $this->folderID deleted\n",'DEBUG');		  

      
      //Delete the IMAP virtual folder
      $type = 'main';
      try{
        $oFolder = $oAccount->getFolder($this->folderID,$type);
        $oFolder->delete();  
      }catch(Exc $e)
      {}
/*      if (!($oFolder===false) && isset($oAccount->folders['main'][$oFolder->name]))
      {
            
            $oFolder->delete();   
      }  */
    	//$this->closeAccess();
      
		}
		
		/**
		 @brief Create new folder.
		 @return GroupWareFolder object 
		*/	 
		public static function create(&$account,$folder,$type)
		{
				global $aError;
				
				//parent::create($folder);
				
				//Check GW connection
				if (!$account->gwAPI->IsConnected())
					  $account->gwAPI->Login();  
									
				//Get folder & group access
				if (!$account->sGWSessionID = $account->gwAPI->OpenGroup("*")) 
					  throw new Exc('group_open','*');															  
																
			  // set proper path for the groupware folder  - COMMENT on mysql
				$folder = str_replace('/', '\\', $folder);
			
        //$aFolders = explode('/',$folder);
        $aFolders = explode('\\',$folder);

        $aFolders = array_reverse($aFolders);
        
        //Get current groupware folders
        $aFoldersCurrent = $account->getFolders();
               
        //Find the nearest existing GW folder
        $iCount = count($aFolders);

        foreach($aFolders as $key => $fname)
        {
              $sName = '';
              
              //Compose folder name
              for ($i = 0; $i < ($iCount-$key); $i++)
                $sName .= $aFolders[$iCount-$i-1].(($i==$iCount-$key-1)?'':'\\');
              
              //Find the nearest groupware folder
              if (isset($aFoldersCurrent[/*!*/str_replace('\\', '/',/*!*/ $sName/*)*/)])){ 
                    $sParentFolder = $sName;
                    break;
              
              //Store folders to be created
              } else {
                    if ($sName==$folder)  $sNewFolder = $sName;
                    else                  $aFoldersCreate[] = $sName;   
              }
        }
        
        
        if($_SESSION['DEBUG'])log_buffer("GroupWareFolder->createParent GroupWare Folder:".$sParentFolder."\r",'DEBUG');
        
    
        //Create required folder tree
        $aFoldersCreate = @array_reverse($aFoldersCreate);
        
        if ($aFoldersCreate[0]) foreach($aFoldersCreate as $treefolder){
             // if($_SESSION['DEBUG'])log_buffer("GroupWareFolder->create() Folder to be created with type M: $treefolder\r",'DEBUG');        
              $result = $account->createFolder($treefolder,'M',true);
            //  if($_SESSION['DEBUG'])log_buffer("GroupWareFolder->create() Folder $treefolder create result:".var_export($result)."\r",'DEBUG');
        }

        
        if($_SESSION['DEBUG'])log_buffer("GroupWareFolder->create() Leaf folder to be created as GW with type $type: $sNewFolder\r",'DEBUG');

				//Create object in GroupWare
			  if (!$result = $account->gwAPI->FunctionCall("AddFolder",
												                    $account->sGWSessionID,
												                    $folder,
												                    $type))
			  {
					throw new Exc('folder_create',$folder);
			  }

			  //$this->closeAccess();
			  return new GroupWareFolder($account,$folder,$type);
		}
		
	
		/**
		 @brief Rename the folder.
		*/	  
		public function rename($newName)
		{		  
			  global $aError;
			  
				if($_SESSION['DEBUG'])log_buffer("GWFolder->rename",'DEBUG');
				if (!$newName)
					throw new Exc('folder_rename_empty', $this->name); 
				
					//Open folder access
			$this->openAccess();
			
        Tools::checkFolderMove($this->name,$newName);
				

		  	$newName = str_replace('/', '\\', $newName);
        $sName = str_replace('/', '\\', $this->name);
				
        if (!$this->account->gwAPI->FunctionCall("RenameFolder", 
						$this->account->sGWSessionID, 
						$sName, 
						$newName))
							  throw new Exc('folder_rename');
			 
       //$this->closeAccess();
			
		}

		/**
		 @brief add new GroupWare item
		*/
		public function createItem($aItem,$aTreeItem)
		{
			return GroupWareItem::create($this,$aItem,$aTreeItem); 
		}
		
		/**
		 @brief Get access to this folder
		*/		
		public function openAccess($bGroup = true)
		{
				global $aError;
				
				$oAccount = &$this->account;
			  
				//Check GW connection
				if (!$oAccount->gwAPI->IsConnected())
				  $oAccount->gwAPI->Login();  
				
        if (!$this->sGWSessionID)	
				//Get folder & group access
				if (!$oAccount->sGWSessionID = $oAccount->gwAPI->OpenGroup('*'))
					  throw new Exc('group_open'); 
		  
		  
				if(!$this->sFID = $oAccount->gwAPI->OpenFolder($oAccount->sGWSessionID,
																str_replace('/','\\',$this->folderID)))
					  throw new Exc('folder_open',str_replace('/','\\',$this->folderID));
		    
        return $this->sFID;		
    }
		
		public function closeAccess()
		{
    //  if ($this->sFID) $this->account->gwAPI->CloseFolder($this->sFID);
    //  if ($this->account->sGWSessionID) $this->account->gwAPI->CloseGroup($this->account->sGWSessionID);		

    }
  	public static function decodeRights($gwRights)
  	{
  		$rights = 0;
  		foreach (str_split($gwRights) as $right) {
  			switch ($right) {
  			case 'm': $rights |= Folder::RIGHT_MODIFY; break;
  			case 'd': $rights |= Folder::RIGHT_DELETE; break;
  			case 'r': $rights |= Folder::RIGHT_READ; break;
  			case 'w': $rights |= Folder::RIGHT_WRITE; break;
  			}
  		}
  
  		return $rights;
  	}
  	
  	public function importItem($type,$data)
  	{
			return GroupWareItem::import($this,$type,$data); 
    }
    
    
    public function getContactList($private = true,$short = 0) 
    {      
      if($this->type!='C')
        throw new Exc('export_folder_type',$this->type);
      
      $oGWAPI = &$this->account->gwAPI;
      $this->openAccess();
        
      $addresses = array();
  
      // server addressbook
      if($this->sFID)
      {
        if(!$private) $share="and (ITMSHARETYPE<>'P') and (ITMSHARETYPE Is Not NULL)"; else $share="";
  
        if ($short&1)
           $select = 'ITMCATEGORY,ITMCOMPANY,ITMCLASSIFYAS,ITMDEPARTMENT,LCTEMAIL1,LCTEMAIL2,LCTEMAIL3';
        else
           $select = 'ITMSHARETYPE,ITMCATEGORY,ITMTITLE,ITMFIRSTNAME,ITMMIDDLENAME,ITMSURNAME,ITMJOBTITLE,ITMCOMPANY,ITMCLASSIFYAS,ITMDESCRIPTION,ITMOFFICELOCATION,ITMGENDER,ITMPROFESSION,ITMDEPARTMENT,ITMNICKNAME,ITMBDATE,ITMSPOUSE,ITMANNIVERSARY,ITMINTERNETFREEBUSY,ITMASSISTANTNAME,ITMMANAGERNAME,LCT_ID,LCTTYPE,LCTWEBPAGE,LCTEMAIL1,LCTEMAIL2,LCTEMAIL3,LCTSTREET,LCTCITY,LCTSTATE,LCTCOUNTRY,LCTZIP,ITMCLASS';
       
        $result = $oGWAPI->ParseParamLine($oGWAPI->FunctionCall("GetContactDetailList",$this->sFID,'((ITMCLASS IS Null OR ITMCLASS=\'C\' OR ITMCLASS=\'\')'.($private?'':' AND ITMSHARETYPE=\'U\'').") $share","ITMGRP_ID,$select"));

         if(is_array($result))
        {
              foreach ($result as $key => $val)
              {
                  $address["NAME"]=$val[ITMCLASSIFYAS];
                  $address["ID"]=$val[ITM_ID];
                  if($short&1) {
                    if($val[LCTEMAIL1]) $address["EMAIL"][0]=$val[LCTEMAIL1];
                    if($val[LCTEMAIL2]) $address["EMAIL"][1]=$val[LCTEMAIL2];
                    if($val[LCTEMAIL3]) $address["EMAIL"][2]=$val[LCTEMAIL3];
                  }
                  $address["ORGANIZATION"]=$val[ITMCOMPANY];
                  $address["CATEGORY"]= $val[ITMCATEGORY];
                  $address["DEPARTMENT"] = $val[ITMDEPARTMENT];
  
                  if(!$short&1)
                  {
  
                       $address["NICK"] = $val[ITMNICKNAME];
  
                       if($val[LCT_ID]) $address["LOCATIONID"][]=$val[LCT_ID];
  
                       $address["STREET$val[LCTTYPE]"] = $val[LCTSTREET];
                       $address["CITY$val[LCTTYPE]"] = $val[LCTCITY];
                       $address["ZIP$val[LCTTYPE]"] = $val[LCTZIP];
                       $address["STATE$val[LCTTYPE]"] = $val[LCTSTATE];
                       $address["COUNTRY$val[LCTTYPE]"] = $val[LCTCOUNTRY];
  
                       $address['sharing']=$val[ITMSHARETYPE];
  
                       $address["BIRTH"] = $val[ITMBDATE];
                       $address["SEX"] =$val[ITMGENDER];
                       $address["NOTE"] =$val[ITMDESCRIPTION];
  
                       $address["TITLE"] = $val[ITMTITLE];
                       $address["NAME1"] = $val[ITMFIRSTNAME];
                       $address["NAME2"] = $val[ITMMIDDLENAME];
                       $address["NAME3"] =$val[ITMSURNAME];
                       $address["JOB"] = $val[ITMJOBTITLE];
                       $address["PROFESSION"] = $val[ITMPROFESSION];
                       $address["OFFICE"] = $val[ITMOFFICELOCATION];
                       $address["ASISTENT"] = $val[ITMASSISTANTNAME];
                       $address["MANAGER"] = $val[ITMMANAGERNAME];
  
                       $address["URL"] = $val[LCTWEBPAGE];
                       $address["CALENDARURL"] = $val[ITMINTERNETFREEBUSY];
                       $address["SPOUS"] = $val[ITMSPOUSE];
                       $address["ANNIVERSARY"] = $val[ITMANNIVERSARY];
  
                       $address["EMAIL"] =  $address["EMAIL"]?$address["EMAIL"]:$val[LCTEMAIL1];
                       $address["EMAIL2"] = $address["EMAIL2"]?$address["EMAIL2"]:$val[LCTEMAIL2];
                       $address["EMAIL3"] = $address["EMAIL3"]?$address["EMAIL3"]:$val[LCTEMAIL3];
                  }
                  

                    if ($address['LOCATIONID']) {
                       foreach($address['LOCATIONID'] as $vloc) {
                       
                          $phones=$oGWAPI->ParseParamLine($oGWAPI->FunctionCall("GetContactLocationPhones",$this->sFID,$vloc));

                          if($phones[0][PHNNUMBER]) {
                            $address["PHONE1"] = $phones[0][PHNNUMBER];
                            $address["PHONE1TYPE"] = $phones[0][PHNTYPE];
                            $address["PHONE1T"] = $phones[0][PHNDESCRIPTION];
                          }
                          if($phones[3][PHNNUMBER]) {
                            $address["PHONE4"] = $phones[3][PHNNUMBER];
                            $address["PHONE4TYPE"] = $phones[3][PHNTYPE];
                            $address["PHONE4T"] = $phones[3][PHNDESCRIPTION];
                          }
                          if($phones[1][PHNNUMBER]) {
                            $address["PHONE2"] = $phones[1][PHNNUMBER];
                            $address["PHONE2TYPE"] = $phones[1][PHNTYPE];
                            $address["PHONE2T"] = $phones[1][PHNDESCRIPTION];
                          }
                          if($phones[2][PHNNUMBER]) {
                            $address["PHONE3"] = $phones[2][PHNNUMBER];
                            $address["PHONE3TYPE"] = $phones[2][PHNTYPE];
                            $address["PHONE3T"] = $phones[2][PHNDESCRIPTION];
                          }
                       }
                    }
                     
                  if(!$result[$key+1][ITM_ID] || $result[$key+1][ITM_ID]!=$val[ITM_ID])
                  {
                    if(is_array($address["EMAIL"]) && count($address["EMAIL"])<2) $address["EMAIL"]=$address["EMAIL"][0];
                    elseif(is_array($address["EMAIL"])) $address["EMAIL"]=array_unique($address["EMAIL"]);
                     }
                    $addresses[$address["ID"]]= $address;
                    unset($address);
                    
                  }

              }
      }

      return $addresses;
    }
  public function saveAddressesToFile(&$file,$addresses,$separator = delimiterchar, $private = true)
  {
    
    //Sort address
    function ARcmp ($a, $b) {
      if ($a["ID"] == $b["ID"]) return 0;
      return ($a["ID"] > $b["ID"]) ? 1 : -1;
    }
    
    usort ($addresses, "ARcmp");

  /* TODO:Localized labels ? $oGlobal = Storage::getDefaults('layout_settings');
    $oDomain = Storage::getDomainDefaults('layout_settings',$_SESSION['DOMAIN']);
    $oUser = Storage::getUserData('layout_settings');
        
    $aSettings = WebmailIqPrivate::get($oGlobal,$oDomain,$oUser,2);
    
    $sLanguage = $aSettings['@childnodes']['item'][0]['@childnodes']['language'][0]['@value'];
    $sLanguage = $sLanguage?$sLanguage:'en';
    
    $oLanguage = simplexml_load_file('../client/languages/'.$sLanguage.'/data.xml');*/
 
    if($addresses){
    
      //Header
      $headerAddr = reset($addresses);
      unset($headerAddr["ID"]); 
      unset($headerAddr["LOCATIONID"]); 
      unset($headerAddr["sharing"]);
      $headerAddr = array_keys($headerAddr);
      fputcsv($file,$headerAddr,$separator);
      
      //Data
      foreach($addresses as $address)
      {
        unset($address["ID"]); 
        unset($address["LOCATIONID"]); 
        unset($address["sharing"]);
        
        fputcsv($file, $address, $separator);
      }
    }
    return true;
  }	
 }
?>
