<?

require_once('inc/include.php');
require_once('inc/account.php');
require_once('inc/gw/gwapi.php');
require_once('inc/gw/folder.php');
require_once('inc/gw/item.php');

class GroupWareManagement 
{
	
	public function __construct()
	{
		$user = &$_SESSION['user'];
		$this->account = &$user->getAccount($_SESSION['EMAIL'])->gwAccount;
	}
	
	/**
	 @brief Obtain public certificated related to a  Contacts item according to 
	 specified email address.
	 @param $email Email address of a contact
	 @return Public certificate for given email address (or empty string if cert 
	 doesn't exist)
	*/ 
	public function getEmailCertificate($email){
		$gwAPI = &$this->account->gwAPI;
		$contacts = $this->getContactIDsByEmail($email);
		foreach ($contacts as $id) {
			$cert = $gwAPI->FunctionCall('GetContactCertificate',
																			$this->account->sGWSessionID,
																			$id);
			if ($cert)
				return $cert;
		}
		return '';
	}
	
	/**
	 @brief Add email recipients to address book (unless they are there already).
	 @param $addresses Array of addresses in MailParse format - each entry contains
			subarray with entries [display] (contact name) and [address] => (contact 
			email address)
	*/ 
	public function addRecipientsToAddressBook($addresses) {
		$gwAPI = &$this->account->gwAPI;
		$email = $_SESSION['EMAIL'];
		$folder = false;
		
		foreach ($addresses as $item) {
			if ($gwAPI->FunctionCall('IsAddressBookEmail',$email,$item['address'],true))
				continue;
			
			// item doesn't exist yet, create it
			if (!$folder) {
				if (!$folder = $this->account->getFolder('Contacts'))
					throw new Exc('folder_open','Contacts');
				$sessionID = $folder->openAccess();
			}
			$params = $gwAPI->CreateParamLine(array('ITMCLASSIFYAS' => $item['display'], 'ITMSHARETYPE' => 'P', 'ITMCLASS' => 'C'));
			$id = $gwAPI->FunctionCall('AddContactInfo', $sessionID, $params);
			$params = $gwAPI->CreateParamLine(array('LCTEMAIL1' => $item['address'], 'LCTTYPE' => 'H'));
			$gwAPI->FunctionCall('AddContactLocation', $sessionID, $id, $params);
			
			// Finalize
			$gwAPI->FunctionCall('AddContactInfo', $sessionID, '', $id);
		}
	}
	
	/**
	 @brief Obtain IDs of Contacts according to specified email address
	 @param $email Email address of a contact
	 @return Array containing IDs of contact items with specified email.
	*/ 
	private function getContactIDsByEmail($email){
		$gwAPI = &$this->account->gwAPI;
		$where = '((ITMCLASS IS Null OR ITMCLASS=\'C\' OR ITMCLASS=\'\') AND (' .
							'LCTEMAIL1=' . self::quote($email) . ' OR ' .
							'LCTEMAIL2=' . self::quote($email) . ' OR ' .
							'LCTEMAIL3=' . self::quote($email) . '))';
		$contacts = $gwAPI->FunctionCall('GetContactDetailList',
																			$this->account->sGWSessionID,
																			$where,
																			'ITM_ID');
		$contacts = $gwAPI->ParseParamLine($contacts);

		$result = array();
		foreach ($contacts as $data)
			$result[] = $data['ITM_ID'];
		
		return array_unique($result);
	}
	
	/**
	 @brief Quote a string to be safe in a DB GW query
	 @param $string Sring to be quotted
	 @return Quotted string
	*/ 
	private static function quote($string) {
		return "'" . str_replace("'", "''", $string) . "'";
	}
	
}

?>
