<?php

require_once 'inc/account.php';
require_once 'inc/folder.php';
require_once 'inc/include.php';
require_once 'inc/gw/gwapi.php';
require_once 'inc/gw/addon.php';
require_once 'inc/imip.php';


/**
  @class GroupWareItem
  @brief Class representing groupware item.
  @author Pavel Perna
  @date 25-03-2006
 */
class GroupWareItem{

		public $sFields;	 //String container for XML fields
		
		public $itemID;  //Item's identification string
		
		private $itemPrefix; //Prefix of the item - ITM or EVN
		
		const FLAG_ORGANIZATOR = 0x01; /**< User is event's organizator */
		const FLAG_ATTENDEE = 0x02; /**< User is event's attendee */
		const FLAG_TRANSPARENT = 0x04; // Free time 
		const FLAG_TENTATIVE = 0x08; // Tentative time 
		const FLAG_OUTOFOFFICE = 0x10; // Out of office time 
		
	  /**
		@brief Initialization for new item object.
		@param $folder parent folder.
		@param $item groupware item parsed array.
		*/ 
		public function __construct(&$folder,
                                $item,
                                $aAddonSelect = array(),
                                $fields = '',
                                $bSingle = false)
		{
			$this->item = $item;

			$this->folder = &$folder;
			$this->fields = $fields;

			$this->setPrefix();
			$this->bSingle = $bSingle;
			//Lower case vs. upper case problem(protocol vs. API) to get item ID
			$this->itemID = $this->item[$this->itemPrefix."_ID"];
			if (!$this->itemID){
				$this->itemID = $this->item[strtolower($this->itemPrefix."_ID")];
			}
			//Item type
			$this->itemType = $this->folder->type;
			$this->openAccess();
			//FIXME : Parse XML in WebmailIqItems->cnvItems
			$sAddonsXML = '';
			if ($bSingle){
				$this->getAddons();
				if ($this->aAddons) foreach($this->aAddons as $key => $addon){
					$sAddonsXML.=$addon->getXML();
				}
			}
			$this->sFields = $this->composeXML($sAddonsXML);
			$this->wmclass = 'GW';
		}
		
		/**
		@brief Get the prefix for itemdata in parsed data array.
		@param item - if item (gw parsed item array) is set, set 
					  the item property as well
		@sets an itemPrefix property =("ITM"|"EVN")
		*/		
	/*	private function convertTime($prefix,&$item)
		{
		  $date = strtoupper($prefix.'date');
			if (isset($item[$date])) 
		  {
					$time = strtoupper($prefix.'time');
		  } else {
          $date = strtolower($prefix.'date');
          $time = strtolower($prefix.'time');
      }

      //Transform server date+time to time
			if (isset($item[$date])){
          $item[$time] = MerakGWAPI::calendar2unixTime($item[$date], $item[$time]);
          unset($item[$date]);     
      }
      
      //Transform time to date+time
      else if (isset($item[$time]))
      {
          $unixTime = $item[$time];
          $item[$time] = MerakGWAPI::unix2calendarTime($unixTime);
      		$item[$date] = MerakGWAPI::unix2calendarDate($unixTime);
      }
     
    }*/
	  /**
		@brief Get the prefix for itemdata in parsed data array.
		@param item - if item (gw parsed item array) is set, set 
					  the item property as well
		@sets an itemPrefix property =("ITM"|"EVN")
		*/		
		private function setPrefix($item = null)
		{
			 /* Check and set for the item */
			 if ($item) $this->item = $item;
			 
			 /* Get the itemPrefix */
			 if (isset($this->item["ITMCLASS"]) && ($this->item["ITMCLASS"]=='C' || $this->item["ITMCLASS"]=='L')
          || isset($this->item["itmclass"]) && ($this->item["itmclass"]=='C' || $this->item["itmclass"]=='L')) 
			 {
				  $sPrefix = 'ITM';
			 } else{
				  $sPrefix = 'EVN';	
			 }	
			 /* Set the itemPrefix */
			 $this->itemPrefix = $sPrefix;
		} 

		public function getAddons()
		{     
		      if (!$this->aAddons) 
		      {
  		      $oAddon = array();
  					switch($this->itemType)
  		      {
                  case 'C':
                        $oAddon['location'] = new GroupWareAddon($this,'location');
                        $oAddon['certificate'] = new GroupWareAddon($this,'certificate');
                  break;
                  // group of contacts:
                  case 'L':
                        $oAddon['location'] = new GroupWareAddon($this,'location');
                  break;
                  case 'E':
                        $oAddon['recurrence'] = new GroupWareAddon($this,'recurrence');
                        $oAddon['reminder'] = new GroupWareAddon($this,'reminder');
                        $oAddon['contact'] = new GroupWareAddon($this,'contact');
                        $oAddon['note'] = new GroupWareAddon($this,'note');
                  break;
                  case 'T':
                        $oAddon['recurrence'] = new GroupWareAddon($this,'recurrence');
                        $oAddon['reminder'] = new GroupWareAddon($this,'reminder');
                        $oAddon['contact'] = new GroupWareAddon($this,'contact');
                        $oAddon['note'] = new GroupWareAddon($this,'note');
                  break;
                  case 'N':
                        $oAddon['note'] = new GroupWareAddon($this,'note');
                  break;
                  case 'J':
                        $oAddon['contact'] = new GroupWareAddon($this,'contact');
                  break;                      
            }
            $oAddon['attachment'] = new GroupWareAddon($this,'attachment');
            return $this->aAddons = $oAddon;
        } else return $this->aAddons;
    }
		
	//	public static function timeTagFix($aItem,$case = CASE_UPPER){
		
    //  $aItem = array_change_key_case($aItem,$case);
      //$aTimeTags = array_change_key_case(array('evnstartdate','evnstarttime','evnenddate','evnendtime'),$case);
     // foreach($aTimeTags as $tag)
    //    if ($aItem[$tag]==0) unset($aItem[$tag]);
   //   return $aItem;
    
   // }
		
		
		public static function import(&$folder,$sType,$sData)
		{
			global $aError;
	
			/* Open folder access */
			$sFID = $folder->openAccess();
      $sFuncName = $folder->type=='C'?'Contact':'Event';
      
      /* Add blank item*/
		/*	if(!$sIID = $folder->account->gwAPI->FunctionCall("Add".$sFuncName."Info", $sFID))
				throw new Exc('i_add_import_failed_blank_item','i_add_import_failed_blank_item');*/
      
			switch($sType){
        case 'vcalendar':
        case 'sif':
			   if(!$sIID = $folder->account->gwAPI->FunctionCall("AddVCalendar",
                                                    $sFID,
                                                    $sData,
                                                    '',
                                                    $sType))
        				throw new Exc('import_vcalendar');          
        break;
        case 'vcard':
        case 'ldif':
         /* Add vCard/LDIF event*/
			   if(!$sIID = $folder->account->gwAPI->FunctionCall("AddVCard",
                                                    $sFID,
                                                    $sData,
                                                    '',
                                                    $sType))
        				throw new Exc('import_vcard');
        break;
      }
      $oItem = $folder->getItem($sIID);
  
      return $oItem;
			
    }	
	 /**
		@brief Create a new item object.
		@param $folder parent folder.
		@param $item groupware item parsed array.
		@return GroupWareItem
		*/		
		public static function create(&$folder,$aItem,$aTreeItem)
		{
			global $aError;
	
			/* Open folder access */
			$sFID = $folder->openAccess();
					
			if(isset($aItem['evnflags']) 
			&& self::isOrganizator($aItem['evnflags']) ){
        		$sOrganizer = $folder->account->gwAPI->FunctionCall(
					"GetGroupOwnerEmail",
					$folder->account->sGWSessionID
				);
        		$aItem['evnorganizer'] = $sOrganizer;
      		}
			/* Get item array */
			//$aItem = Tools::makeArrayFromXML(/*$oIq->*/$oDOMValues);
			$aItem = array_change_key_case($aItem,CASE_LOWER);
      //$aItem = self::timeTagFix($aItem,CASE_LOWER);
      
      /* Fill out important values */
			$sPrefix = $folder->type=='C'?'itm':'evn';
			$sFuncName = $folder->type=='C'?'Contact':'Event';
			

			
			if (!isset($aItem[$sPrefix.'class'])) $aItem[$sPrefix.'class'] = $folder->type;
		//	if (!isset($aItem[$sPrefix.'folder'])) 
     //       $aItem[$sPrefix.'folder'] = $folder->folderID;
			
/*			// Do not convert time for contacts
			if ($folder->type!='C')
			{
      			self::convertTime('evnstart',$aItem);
            self::convertTime('evnend',$aItem);
      }
*/
      
      
      
      $folder->account->gwAPI->TimeZone($aItem,'in');

			/* Create item's param line */
			$sParameters = $folder->account->gwAPI->CreateParamLine($aItem);
      
			
      /* Add event	*/
			if(!$sEvnId = $folder->account->gwAPI->FunctionCall("Add".$sFuncName."Info", 
																//$folder->account->sGWSessionID, 
															  $sFID,
                              	$sParameters))
				throw new Exc('item_create',$sParameters);

				
			$aItem[$sPrefix.'_id'] = $sEvnId;

      $item = new GroupWareItem($folder,$aItem);
      
      //Initialize addons
		  $item->getAddons();

		  //Process addons
      if($item->aAddons) foreach($item->aAddons as $addon)
            $addon->process($aTreeItem);
            
			// Finalize
			$folder->account->gwAPI->FunctionCall("Add".$sFuncName."Info",$sFID,'',$sEvnId);

			return $item;
			
    }	
		
	static public function isOrganizator($flags)
	{
      return  $flags & GroupWareItem::FLAG_ORGANIZATOR;
    }
    
    static public function isAttendee($flags)
	{
      return $flags & GroupWareItem::FLAG_ATTENDEE;
    }
		
	 /**
	  @brief Delete an item
	  */
		public function delete($datestamp = false,$following = false)
		{
			$bException = false;
			try{
				if($this->item['EVNFLAGS']){
					$bOrganizator = $this->isOrganizator($this->item['EVNFLAGS']);
					//Cancel all attendees and send the cancel versit
					if($this->isOrganizator($this->item['EVNFLAGS'])){
						$this->cancel();
						//Continue deleting action
						//Decline this event and send info to organizator
					} else if($this->isAttendee($this->item['EVNFLAGS'])){
						return $this->decline();
					}
				}
			}catch(Exc $imipException){
				$bException = true;
			}
			//Account to work with
			$oAccount = &$this->folder->account;
			//Get folder access
			$this->openAccess();
			$sClass = $this->item['EVNCLASS'];
			$sRcrID = $this->item['EVNRCR_ID'];
			$oAccount->gwAPI->TimeZone($this->item,'in');
			$oldItem = $this->item;
			if ($oldItem['EVNSTARTDATE']!=$this->item['EVNSTARTDATE']){
				$datestamp-= ($oldItem['EVNSTARTDATE']-$this->item['EVNSTARTDATE']);
			}
			//Delete event in specified day of reccurence
			if ($datestamp){
				//Delete also following
				if ($following){
					$addon = new GroupWareAddon($this,'recurrence');
					$newAddon = $addon->getData();
					$newAddon = $newAddon[0];
					$newAddon['RCRENDDATE'] = $datestamp-1;
					$result = $addon->edit($newAddon,$newAddon['RCR_ID']);
					return $this->removeExceptions($newAddon['RCR_ID'],true,$datestamp-1) && $result;
					//Delete only specified occurrance(make exception)
				} else if ($oItem->item['EVNCLASS']!='O')
					$this->updateMasterObject($oAccount);
					return $this->makeException($datestamp);
				}
				//Occurrance
				if ($sClass=='O') {
					$this->updateMasterObject($oAccount,false,true);
					if (!$result = $oAccount->gwAPI->FunctionCall(
						'DeleteEvent',
						$this->sFID,
						$this->itemID,
						strval(true)
					)){
						throw new Exc('item_delete',$this->itemID);
					}
				}
				//Clasic delete
				else{
					$sFuncName = $this->folder->type=='C'?'Contact':'Event';
					if (!$datestamp && $this->folder->type=='E'){
						$result = $this->removeExceptions($this->item['EVNRCR_ID'],true) && $result;
					}
					//Delete Event
					$result = $oAccount->gwAPI->FunctionCall(
						'Delete'.$sFuncName,
						$this->sFID,
						$this->itemID
					) && $result;
				}
				if($bException){
					throw new $imipException;   
				}
				return $result;
		}
		
		public function removeExceptions($sRcrID,$occurrances = false,$datestamp = false)
    {
      $oAccount = $this->folder->account;

      //Retrieve exceptions - in order to delete all O class events
      $exceptions = $oAccount->gwAPI->FunctionCall('GetEventException',
                                                 $oAccount->sGWSessionID,
                                                 $sRcrID);                                                    
      $exceptions = $oAccount->gwAPI->ParseParamLine($exceptions);
      //Delete exceptions and occurrances 
      $result = true;
      if($exceptions) foreach($exceptions as $exception){ 
	if (($datestamp < $exception['EXPDATE'])
          || !$datestamp){
	          //Delete possible occurrances                                                            
        if ($exception['EXPEVNID'] && $occurrances)
	{
		if (!$result = $oAccount->gwAPI->FunctionCall('DeleteEvent',
                                                  $this->sFID,
                                                  $exception['EXPEVNID'],
                                                  strval(true)))
			throw new Exc('item_delete',$exception['EXPEVNID']);    
	}
        $result = $oAccount->gwAPI->FunctionCall('DeleteEventException',
                                                  $this->sFID,
                                                  $exception['EXP_ID']); 

                        
        }
      } 
    }    
		
		public function moveExceptions($newItem,$occurrances = false,$datestamp = false)
    {
      $oAccount = $this->folder->account;     

      //Retrieve exceptions - in order to delete all O class events
      $exceptions = $oAccount->gwAPI->FunctionCall('GetEventException',
                                                 $oAccount->sGWSessionID,
                                                 $oldRcrID);                                                    
      $exceptions = $oAccount->gwAPI->ParseParamLine($exceptions); 
  
      //Delete exceptions and occurrances 
      $result = true;
      if ($exceptions) foreach($exceptions as $exception){ 
        if (($datestamp < $exception['EXPDATE'])
          || $datestamp == false){
        $result = $oAccount->gwAPI->FunctionCall('DeleteEventException',
                                                  $this->sFID,
                                                  $exception['EXP_ID']) && $result;     
        
        //Delete possible occurrances                                                            
        if ($exception['EXPEVNID'] && $occurrances)
        $result = $oAccount->gwAPI->FunctionCall('DeleteEvent',
                                                  $this->sFID,
                                                  $exception['EXPEVNID'],
                                                  strval(true)) && $result;                             
        }
      } 
    } 		
	
	 /**
	  @brief Edit an item (depending on oDOMItem)
	  */
		public function edit($aItem,$aTreeItem,$datestamp = false,$following = false)
		{

         if($this->isOrganizator($this->item['EVNFLAGS']) 
		 || $this->isOrganizator($aItem['evnflags'])) {
          $sOrganizer = $this->folder->account->gwAPI->FunctionCall(
		  	"GetGroupOwnerEmail",
			$this->folder->account->sGWSessionID
		  );
          $aItem['evnorganizer'] = $sOrganizer;

          //Init attendees
          $this->getAddons();

          //Get current and changed list
          $aCurrentList = $this->aAddons['contact']->getData();
          $aChangedList = &$aTreeItem['@childnodes']['contacts'][0]['@childnodes']['contact'];
          
          $bInvitation = true;

        }

        
        //Get folder access
				$sFID = $this->openAccess();

        //Retrieve objects to work with
				$oAccount = &$this->folder->account;		
        
        //Signalize the changes in Event(when only attendees are edited)
        $bChanged = false;
        
        if($bInvitation){
          if($property) foreach($aItem as $property => $value)
            if($property!='ctz')
              if($this->item[strtoupper($property)]!=$aItem[$property])
                $bChanged = true;
        }
        
        $sClass = $this->item['EVNCLASS'];
        $sRcrID = $this->item['EVNRCR_ID'];
        $oldItem = $this->item;
        		    
		    //Init function name root
        $sFuncName = $this->folder->type=='C'?'Contact':'Event';
        
        //Transform DOMValues to an array
      //  if (/*$oIq->*/$oDOMValues){ 
      //    $aItem = Tools::makeArrayFromXML(/*$oIq->*/$oDOMValues,true);
        $aItem = array_change_key_case($aItem,CASE_UPPER);
         // $aItem = self::timeTagFix($aItem,CASE_UPPER);
      //  }
        
        $oAccount->gwAPI->TimeZone($aItem,'in');  

        if ($oldItem['EVNSTARTDATE']!=$this->item['EVNSTARTDATE']){
          $datestamp-= ($oldItem['EVNSTARTDATE']-$this->item['EVNSTARTDATE']);
        }
                    
        //Edit only selected occurrance
        if ($datestamp)
          //Edit only selected instance
          if (!$following){
            if ($this->item['EVNCLASS']!='O' ){
	      $sItem = 'evntitle='.$this->item['EVNTITLE'];
	      //Edit  
        if (!$oAccount->gwAPI->FunctionCall("Add".$sFuncName."Info",
                                              $sFID,
                                              $sItem,
                                             $this->itemID))
                throw new Exc('item_edit',$this->itemID);

        $result = $this->makeOccurrance($datestamp,$aItem,$aTreeItem);

        //Finalize
	      $oAccount->gwAPI->FunctionCall("Add".$sFuncName."Info",$sFID,'',$this->itemID);

	      return $result;                 
        }
          //Edit all following
          }else{        
            $oldItem = this; 

            //Get reccurrence  
            $addon = new GroupWareAddon($this,'recurrence');
            $oldAddon = $addon->getData();
            $oldAddon = $oldAddon[0];
            
            $oldEnd = $oldAddon['RCRENDDATE'];
            $oldStart = $oldItem['EVNSTARTDATE'];
            
            $newAddon = $oldAddon;
            
            $newAddon['RCRENDDATE'] = $datestamp-1;
            
            

            //Create new item
            $newItem = $this->item;
                    
                      
            //Old addon change
            $result = $addon->edit($newAddon,$newAddon['RCR_ID']);
            
            if($aItem) foreach($aItem as $itemProperty => $itemValue)
                $newItem[$itemProperty] = $itemValue;
            
            $newItem['EVNSTARTDATE'] = $datestamp;
            if($aItem['EVNENDTIME']!=-1)
              $newItem['EVNENDDATE'] = $datestamp;

            unset($newItem['EVN_ID']);
            unset($newItem['EVNRCR_ID']);
            unset($newItem['EXPDATE']);
            unset($newItem['EXPFOLLOWING']);
            
            self::removeEditTags($newItem,CASE_UPPER);

            $newItem = $oAccount->gwAPI->CreateParamLine($newItem);
          	
            if(!$sEvnId = $oAccount->gwAPI->FunctionCall("Add".$sFuncName."Info", 
            									  $sFID,
                              	$newItem))
              throw new Exc('item_edit',$newItem); 
              
            $newItem = $this->folder->getItem($sEvnId,NO_ADDONS);
            
            if ($oldStart==$datestamp)
              $this->folder->deleteItems(array(0=>$this));
            //Create new reccurrence  

            
            if ($oldAddon['RCRCOUNT'])
              $oldAddon['RCRCOUNT']=$oldAddon['RCRCOUNT']-($datestamp-$oldStart);
              
            $addon = new GroupWareAddon($newItem,'recurrence');
            $newRcrID = $addon->create($oldAddon);
                        
            //Copy following exceptions to new Event
            $result = $this->moveExceptions($newItem,true,$datestamp) && $result;
            
            //Finalize
            $oAccount->gwAPI->FunctionCall("Add".$sFuncName."Info",$sFID,'',$sEvnId);
            
            return $result;
          }

        //Occurrance
        if ($sClass=='O') {
		      $this->updateMasterObject($oAccount);
          //Edit occurrance
	         unset($aItem['EXPDATE']);
	          $sItem = $oAccount->gwAPI->CreateParamLine($aItem);
            if (!$oAccount->gwAPI->FunctionCall("Add".$sFuncName."Info",
                                              $oAccount->sGWSessionID,
                                              $sItem,
                                              $this->itemID))
                throw new Exc('item_edit',$this->itemID);                        

          //Process addons 
          $this->getAddons();
          if($this->aAddons) foreach($this->aAddons as $aAddon)
                $aAddon->process($aTreeItem);               
              
        //Classic event
        } else {

            //Posunuti evnenddate
            if (!$datestamp && isset($aItem['EVNSTARTDATE']) && isset($aItem['EVNENDDATE'])){

              $addon = new GroupWareAddon($this,'recurrence');
              $aAddon = $addon->getData();
	      
              if ($aAddon)
              {
                $aAddon = $aAddon[0];
                $oldEnd = $aAddon['RCRENDDATE'];
		
		            $diff = $aItem['EVNSTARTDATE'] - $this->item['EVNSTARTDATE'];
              
                if ($aAddon['RCRENDDATE']){
			             $aAddon['RCRENDDATE'] += $diff; 
			             $result = $addon->edit($aAddon,$aAddon['RCR_ID']);
		            }
              }
            }

    $sItem = $oAccount->gwAPI->CreateParamLine($aItem);
		
		if (!$oAccount->gwAPI->FunctionCall("Add".$sFuncName."Info",
                                              $sFID,
                                              $sItem,
                                              $this->itemID))
                throw new Exc('item_edit',$this->itemID);
 
	       //Process addons 
          $this->getAddons();
          
          if($this->aAddons) foreach($this->aAddons as $aAddon)
                $aAddon->process($aTreeItem);
                
    //Finalize
		$oAccount->gwAPI->FunctionCall("Add".$sFuncName."Info",$sFID,'',$this->itemID);

      }

       //Do imip actions(send emails about changes)
        if($this->isOrganizator($this->item['EVNFLAGS']) 
	   || $this->isOrganizator($aItem['EVNFLAGS'])){

          //Index by uid
          if($aCurrentList) foreach($aCurrentList as $aAtt)
            $aCurrentID[$aAtt['CNT_ID']] = $aAtt;

          //Cache attendees
          $this->aAttendees = $aCurrentID;
          //Detect actions
          if($aChangedList) 
            foreach($aChangedList as $key => $aAttendee){
              if($uid = $aAttendee['@attributes']['uid']){
                
                if(isset($aAttendee['@childnodes']['values'])){
                  $sEmail = $aAttendee['@childnodes']['values'][0]['@childnodes']['cntemail'][0]['@value'];
                  $aAction['action'] = 'change';
                  $aAction['email'] = $sEmail;

                  if($this->aAttendees[$uid]['CNTEMAIL'] != $aAction['email']){
                    $aAction['action'] = 'invite';

                    if($aCurrentList) foreach($aCurrentList as $itm)
                      if($itm['CNTEMAIL']==$this->aAttendees[$uid]['CNTEMAIL'])
                        $aDelete[] = $itm;                    

                    $this->aAttendees[$uid]['CNTEMAIL'] = $aAction['email'];

                  }
                  $aActions[] = $aAction;
                  
                } else {
                  //Cancel
                  $aDelete[] = array_change_key_case($aCurrentID[$uid],CASE_UPPER);
                  unset($aCurrentID[$uid]);
                }
                
              //Invitation
              } else {
                $aAtt = array();
                //$aTreeItem['@childnodes']['contacts'][0]['@childnodes']['contact'][$key]['@childnodes']['values'][0]['@childnodes']['cntrsvp'][0]['@value'] = 1;
                $cursor = &$aTreeItem['@childnodes']['contacts'][0]['@childnodes']['contact'][$key]['@childnodes']['values'][0]['@childnodes'];
                if($cursor) foreach($cursor as $key2 => $val){
                  $aAtt[strtoupper($key2)] = $val[0]['@value'];
                }
                $aAtt['CNTRSVP'] = 1;

                $this->aAttendees[] = $aAtt;
                $sEmail = $aAttendee['@childnodes']['values'][0]['@childnodes']['cntemail'][0]['@value'];
                $aAction['action'] = 'invite';
                $aAction['email'] = $sEmail;                
                $aActions[] = $aAction;
              }
            }
            
            if ($aCurrentID) foreach($aCurrentID as $aCurrent){

              $aAction['action'] = ($aItem['EVNFLAGS'] && ($aItem['EVNFLAGS']!=$this->item['EVNFLAGS']))?'invite':'change';
              $aAction['email'] = $aCurrent['CNTEMAIL'];
              
              $this->aAttendees[$aCurrent['CNT_ID']]['CNTRSVP'] = 1;
              $aActions[] = $aAction;
            }

            $aAttachment = false;
     

            
            //Invite and send updates
             if ($aActions){
              foreach($aActions as $aAction){
                switch($aAction['action']){
                  case 'change':
                  case 'invite':
                    if(isset($aItem['EVNTITLE']))
                      $this->item['EVNTITLE'] = $aItem['EVNTITLE'];

                      $oInvitation = $this->createInvitation();
                      $aAttachment = $this->createInvitationAttachment($oInvitation);

                    WebmailIqItems::iMIP_Email($this,$aAction['action'],$aAttachment,false,$aAction['email']);
                  break;
                  case 'cancel':
                    $oInvitation = $this->createInvitation();
                    $aAttachment = $this->createInvitationAttachment($oInvitation);
                    
                    WebmailIqItems::iMIP_Email($this,$aAction['action'],$aAttachment,false,$aAction['email']);
                  break;
                }
              }
            }

            //Send cancel
            if($aDelete)
              $this->cancel($aDelete);
        }
        
    return $bChanged;
}
    
	public function updateMasterObject($account,$deleteException = false,$occuranceID = false)
	{
		$eventsline = $account->gwAPI->FunctionCall('GetEventList', 
									 $this->sFID,"evnrcr_id='".$this->item['EVNRCR_ID']."' AND evnclass='E'",'evntitle');
		$eventsline = $account->gwAPI->ParseParamLine($eventsline); 									
		$event = $eventsline[0];		  

		//Update parent event
		$sItem = 'evntitle='.$event['EVNTITLE'];

		if (!$account->gwAPI->FunctionCall("AddEventInfo",
                                              $this->sFID,
					                                    $sItem,
                                              $event['EVN_ID']))
                throw new Exc('item_edit',$event['EVN_ID']);

		if ($deleteException || $occuranceID){
			//Retrieve exceptions - in order to delete all O class events
			$exceptions = $account->gwAPI->FunctionCall('GetEventException',
                                                 $account->sGWSessionID,
                                                 $this->item['EVNRCR_ID']);                                                    
			$exceptions = $account->gwAPI->ParseParamLine($exceptions); 		

			if ($exceptions) foreach($exceptions as $exc)
				if ($exc['EXPRCR_ID']==$this->item['EVNRCR_ID']){
					$toDelete = $exc['EXP_ID'];
        

        //If occurance ID passed , clear occurrance ID
				if($occuranceID && $this->itemID == $exc['EXPEVNID']){
				  $sParam = $account->gwAPI->CreateParamLine(array('EXPEVNID'=>''));
				  if (!$account->gwAPI->FunctionCall('AddEventException',
                                                 $account->sGWSessionID,
                                                 $exc['EXPRCR_ID'],
                                                 $sParam,
                                                 $exc['EXP_ID']))	
            throw new Exc("item_occurance_delete",$exc['EXP_ID']); 
      } else if($deleteException){                          			  
        //if deleteException param passed delete exceptions
				if (!$account->gwAPI->FunctionCall('DeleteEventException',
                                                 $account->sGWSessionID,
                                                 $toDelete))
						throw new Exc("item_exception_delete",$toDelete);                        
        }                         
      }     
		}
		// Finalize
		$account->gwAPI->FunctionCall("AddEventInfo",$this->sFID,'',$event['EVN_ID']);
          	
	}
    
    
	 /**
		@brief Move an item to another folder.
		@param $destination Destination folder object.
	 */
		public function move($destination)
		{
			global $aError;
			
			if ($this->copy($destination)){
       $this->delete();
      }
			
			// This code could not work for other accounts, public folders, shared... 
			/*
      $folder = $this->folder;
			$account = $folder->account;
			
      $folder->openAccess();
			$destination->openAccess();
			$aItem = array();
			
			// Fill out important values 
			$sPrefix = $folder->type=='C'?'ITM':'EVN';
			$sFuncName = $folder->type=='C'?'Contact':'Event';
			
			$aItem[$sPrefix.'Folder'] = str_replace("/","\\",$destination->folderID);
			
			// Create item's param line 
			$sParameters = $account->gwAPI->CreateParamLine($aItem);
			
			// Add event	
			if(!$sEvnId = $account->gwAPI->FunctionCall("Add".$sFuncName."Info", 
																$destination->sFID, 
																$sParameters,
                                $this->itemID));
			{	
				throw new Exc('item_move');
			}

			$this->folder = $destination;
		  */
    }
		
	 /**
		@brief Copy an item to another folder.
		@param $destination Destination folder object.
	 */
		public function copy($destination)
		{
			global $aError;

			$folder = $this->folder;
			$account = $folder->account;
      
			/* Open folder access */
			$folder->openAccess();
			$destination->openAccess();
			
			/* Fill out important values */
			$sPrefix = $folder->type=='C'?'ITM':'EVN';
			$sFuncName = $folder->type=='C'?'Contact':'Event';
			
			/* Get original ITEM data */
			if(!$sItemData = $account->gwAPI->FunctionCall("Get".$sFuncName."Info", 
																$folder->sFID, 
																$this->itemID))
			{	

				throw new Exc('item_invalid_id',$this->itemID);
			}
			
			/* Change the folder in the new ITEM */
			$aItem = $account->gwAPI->ParseParamLine($sItemData);
      
			if (count($aItem) !== 1)
				throw new Exc('item_copy_multiple');
				
			$aItem = $aItem[0];
			unset($aItem[$sPrefix.'FOLDER']);// = $destination->folderID;
			unset($aItem[$sPrefix.'_ID']);
			
			/* store the new ITEM */
			$sItemData = $account->gwAPI->CreateParamLine($aItem);
			
			/* Add new ITEM	*/
			if(!$sEvnId = $account->gwAPI->FunctionCall("Add".$sFuncName."Info", 
																$destination->sFID, 
																$sItemData))
			{	
				throw new Exc('item_copy');
			}
			
			$result = true;
      /* copy all addons connected with the original item */
			if ($folder->type=='C')
			{
				$this->copyGWAddonData('GetContactCertificate', 'SetContactCertificate', $account, $this->itemID, $sEvnId,$folder,$destination);
				$aLctID = $this->copyGWAddonData('GetContactLocations', 'AddContactLocation', $account, $this->itemID, $sEvnId,$folder,$destination,'LCT_ID');
				if($aLctID) foreach ($aLctID as $sLctID)
					$this->copyGWAddonData('GetContactLocationPhones', 'AddContactLocationPhone', $account, $sLctID['old'], $sLctID['new'],$folder,$destination);
            
			} else {
				$getFunctions = array('GetEventReminders', 'GetEventRecurrence', 'GetEventContacts', 'GetEventNoteText');
				$setFunctions = array('AddEventReminder', 'AddEventRecurrence', 'AddEventContact', 'SetEventNoteText');
				foreach ($getFunctions as $key => $function)
					$this->copyGWAddonData($function, $setFunctions[$key], $account, $this->itemID, $sEvnId,$folder,$destination);
			}
			
			/* Copy item attachments */
			$attachment = new GroupWareAddon($this,'attachment');
			$attData = $attachment->getData();
			if ($attData)
				foreach ($attData as $item) {
					$data = $attachment->getAttachment($item['ATTNAME']);
					$attachment->addAttachment($item, $data, $sEvnId, $destination->sFID);
				}
			
      /* Add new ITEM	- finalize*/
		  $account->gwAPI->FunctionCall("Add".$sFuncName."Info",$destination->sFID, '',$sEvnId);
      		
      return $result;
    }
		
		/**
			@brief Copy GW addon data from original item to the new one
			@param $getFunction Name of the GW API function that is called to obtain original item data
			@param $setFunction Name of the GW API function that is called to store new item data
			@param $account Account object representing current active account
			@param $oldItemID ID of the original item
			@param $newItemID ID of the new item
			@param $sReturnItem If set, an entry specified by this string of the original item will be returned in the result array
			@return Array containing the IDs of newly added GW addon data, optionally contains an entry (ususally the ID) of original GW addon data 
	 */
		private function copyGWAddonData($getFunction, $setFunction, $oAccount, $oldItemID, $newItemID,$source,$dest, $sReturnItem = '') {
			$aResult = array();
      
      
			if ($sGetData = $oAccount->gwAPI->FunctionCall($getFunction, 
																$source->sFID, 
																$oldItemID))
			{	
			  //API mismatch this fix is needed
			  if($getFunction == 'GetEventNoteText')
			   $aData[0] = $sGetData;
			  else
				 $aData = $oAccount->gwAPI->ParseParamLine($sGetData);

        if($aData) foreach ($aData as $data) 
				{
				  //API mismatch this fix is needed
				  if($getFunction == 'GetEventNoteText')
				    $sNewData = $sGetData;
          else 
					  $sNewData = $oAccount->gwAPI->CreateParamLine($data);
						
					if(!$sNewID = $oAccount->gwAPI->FunctionCall($setFunction, 
																$dest->sFID, 
																$newItemID,
																$sNewData))
					   throw new Exc('addon_copy');
					
					$aItem = &$aResult[];
					$aItem['new'] = $sNewID;
					if ($sReturnItem)
						$aItem['old'] = $data[$sReturnItem];
				}
			}
			
			return $aResult;
		}
		
		
		/*brief Make an exception in event recurrence
		 *
		 *
		 *
		 */
      public function makeException($datestamp, $occurrance_id = false)
      {
        $this->openAccess();
        $folder = $this->folder;
			  $oAccount = $folder->account;

			  $result = $oAccount->gwAPI->FunctionCall("GetEventRecurrence",
                                                $oAccount->sGWSessionID,
                                                $this->itemID);
        $result = $oAccount->gwAPI->ParseParamLine($result);
     
        $sEvnRcrID = $result[0]['RCR_ID'];                                             

			  $parameters['expDate'] = $datestamp;
			  if ($occurrance_id)
          $parameters['expEvnID'] = $occurrance_id;

			  $parameters = $oAccount->gwAPI->CreateParamLine($parameters);

        return $oAccount->gwAPI->FunctionCall("AddEventException",
                                      $oAccount->sGWSessionID, 
                                      $sEvnRcrID, 
                                      $parameters);
      }   
             		 
		/*brief Make an occurrance - this means taht webmail 
		 *will create an event with type O linked to the event
		 *reccurrence 
		 *@param $datestamp - specifies the exception date
		 *@param $parameters[optional] - if there are some changes in 
		 *exceptions this array is used to specify new occurrance parameters     		 
		 */
      public function makeOccurrance($datestamp,$parameters = array(),$aTreeItem)
      {
        $this->openAccess();

        $folder = $this->folder;
	      $oAccount = $folder->account;
               
        $occurrance = $this->item;
        
        if ($occurrance){
		$occurrance = array_change_key_case($occurrance,CASE_LOWER);
		unset($occurrance['evn_id']);
		unset($occurrance['evngrp_id']);
		unset($occurrance['evn_created']);
		$sRcrID = $occurrance['evnrcr_id'];
		unset($occurrance['evnrcr_id']);
	}
        if ($parameters){
          $parameters = array_change_key_case($parameters,CASE_LOWER);
          unset($parameters['expdate']);
	  
          $itm = array_change_key_case($this->item,CASE_LOWER);
	if($parameters) foreach($parameters as $key => $val)
            if ($val!='') $occurrance[$key] = $val;
        }
        self::removeEditTags($occurrance);
        
        $occurrance['evnclass'] = 'O'; //Set event class to occurrance
        $sOccurrance = $oAccount->gwAPI->CreateParamLine($occurrance);
          
	  //Update parent event
	  $exceptions = $oAccount->gwAPI->FunctionCall('GetEventException',
                                                     $oAccount->sGWSessionID,
                                                    $this->item['EVNRCR_ID']);     
					     
          $exceptions = $oAccount->gwAPI->ParseParamLine($exceptions); 
	  if ($exceptions) foreach($exceptions as $exc){
		if ($exc['EXPEVNID']==$this->itemID)
			$sEvnRcrID = $exc['EXPRCR_ID'];
	  }
		
		//Update parent object
		$sItem = 'evntitle='.$this->item['EVNTITLE'];
		if (!$oAccount->gwAPI->FunctionCall("Add".$sFuncName."Info",
                                              $oAccount->sGWSessionID,
					                                    $sItem,
                                              $sEvnRcrID))
                throw new Exc('item_edit',$sEvnRcrID);
	
	      //Create an occurrance event
        if(!$sEvnId = $oAccount->gwAPI->FunctionCall("AddEventInfo", 
						    $this->sFID,
                $sOccurrance,
						    '',
						    $sRcrID))
          throw new Exc('item_occurrance_create'); 
  
        //Create an exception in reccurrence
        if (!$this->makeException($datestamp, $sEvnId))
          throw new Exc('item_exception_create');
          
	       //Process addons 
          $this->getAddons();
          
          if($this->aAddons) foreach($this->aAddons as $aAddon)
                $aAddon->process($aTreeItem);          

    }
    
    
      
      
  	private static function removeTimeTags(&$array,$case = CASE_LOWER)
  	{
  	  $unset = array(0=>'evnstarttime',1=>'evnendtime',2=>'evnstartdate',3=>'evnenddate');
  	  
      $array = array_change_key_case($array,$case);
      
      foreach($unset as $u)
        if ($case == CASE_UPPER)
          unset($array[strtoupper($u)]);
        elseif ($case == CASE_LOWER)
          unset($array[strtolower($u)]);
      
    }	
        
  	private static function removeEditTags(&$array,$case = CASE_LOWER)
  	{
  	  $unset = array(0=>'evn_editcounter',1=>'evnt_created',2=>'evn_modified',3=>'evnfolder');
      
      $array = array_change_key_case($array,$case);
      
      foreach($unset as $u)
        if ($case == CASE_UPPER)
          unset($array[strtoupper($u)]);
        elseif ($case == CASE_LOWER)
          unset($array[strtolower($u)]);
          
           
    }	
		/**
		 @brief Compose string fields from collected data
		 @param aAddons - Addons retrieved by getAddons() function
		*/		
		private function composeXML($sAddonsXML)
		{		

			  $sReturn = '<values>'.Tools::makeXMLTags($this->item,$this->fields).'</values>'.$sAddonsXML;

			  return $sReturn;
		}

		/**
		 @brief Get access to this folder
		*/		
		public function openAccess()
		{
				$oAccount = &$this->folder->account;
			  
				//Check GW connection
				if (!$oAccount->gwAPI->IsConnected())
					  $oAccount->gwAPI->Login();  
					  
				if (!$this->sGWSessionID)					
				//Get folder & group access
				if (!$oAccount->sGWSessionID = $oAccount->gwAPI->OpenGroup('*')) 
					  throw new Exc('group_open'); 
					  
				if(!$this->sFID = $oAccount->gwAPI->OpenFolder($oAccount->sGWSessionID,
																                       str_replace('/','\\',$this->folder->folderID)))
					  throw new Exc('folder_open');
		    return $this->sFID;
    }
    
		public function closeAccess()
		{
      //if ($this->sFID) $this->folder->account->gwAPI->CloseFolder($this->sFID);
      //if ($this->folder->account->sGWSessionID) $this->folder->account->gwAPI->CloseGroup($this->folder->account->sGWSessionID);		

    }	
	/**
	  @brief Send ITEM attachment data to output.(Tonda)
	  @param $partID ID of the attachment part
	 */
	/**
	  @brief Send ITEM data to output.
	 */
	public function sendData()
	{
		$data = $this->getVersitObject($info);
		$size = strlen($data);
		$fileName = 'item'.$info['fileext'];
		$mimeType = $info['mimetype'];
		
		Tools::sendFileHeaders($fileName, $size, $mimeType);
		echo $data;
	}
	
	public function sendAttachment($partID)
	{
		$attachment = new GroupWareAddon($this,'attachment');
		$att = $attachment->getData($partID);
		// FIXME: delete this FOREACH once Merak mailserver is fixed

		if($att) foreach ($att as $item) {
			if ($att['ATTNAME'] == $partID) {
				$item['ATTPARAMS'] = Tools::createURL(array('mimetype' => 'application/octet-stream'));
				$att = array(0 => $item);
				break;
			}
		}
		
		// parse attachment info
		if (!$att)
			throw new Exc('attachment_get',$partID);
			
    $att = reset($att);
    
    $result = $attachment->getAttachment($partID);
		
    // parse attachment info
		$params = Tools::parseURL($att['ATTPARAMS']);
		$mimeType = $params['mimetype'];
		$size = strlen($result);
		$fileName = $att['ATTDESC'];
		
		Tools::sendFileHeaders($fileName, $size, $mimeType);
		echo $result;
	}
	

  

  /**
	  @brief Get versit object (vCalendar or vCard) related to the item.
	  @param[out] $info Info about versit object
	  @return Data of versit object related to the item
	 */
	public function getVersitObject(&$info = array()) 
	{
		/* Open folder access */
		$folder = $this->folder;
		$account = $folder->account;
		$folder->openAccess();
		$info = array();
		
		switch ($this->itemType)
		{
			case 'C':
				$function = 'GetvCard';
				$info['fileext'] = VCARD_EXT;
				$info['mimetype'] = 'text/x-vcard';
			break;
			
			default:
				$function = 'GetvCalendar';
				$info['fileext'] = VCALENDAR_EXT;
				$info['mimetype'] = 'text/x-vcalendar';
			break;
		}

		/* Get versit Data	*/
		return $account->gwAPI->FunctionCall($function, 
															$account->sGWSessionID, 
															$this->itemID);
	  //$folder->closeAccess();
		
	}
	
	public function getAttachment($partID)
	{

		$attachment = new GroupWareAddon($this,'attachment');
		$aAttachments = $attachment->getData($partID);

    //Create result with name and content
		if($aAttachments) foreach ($aAttachments as $key => $aAttachment) {
		  
      $aResult['file_content'] = $attachment->getAttachment($aAttachment['ATTNAME']);
		  $aResult['name'] = $aAttachment['ATTDESC'];
		  $aResult['param'] = Tools::parseURL($aAttachment['ATTPARAMS']);
    }

		if (!$aResult)
			throw new Exc('attachment_get',$partID);
		
		return $aResult;    
  }    	

	public function getAllAttachments()
	{	
		$attachment = new GroupWareAddon($this,'attachment');
		$aAttachments = $attachment->getData();

    //Create result with name and content
		if ($aAttachments) foreach ($aAttachments as $key => $aAttachment) {
		  
      $aResult[$key]['file_content'] = $attachment->getAttachment($aAttachment['ATTNAME']);
		  $aResult[$key]['name'] = $aAttachment['ATTDESC'];
		  $aResult[$key]['param'] = Tools::parseURL($aAttachment['ATTPARAMS']);
    }

		if (!$aResult)
			throw new Exc('attachment_get',$partID);
		
		return $aResult;    
  }
// Universal functions for retrieving event/task/notes/journal... content or its attachment to be used
// in webmailiqmessage.php as well as in download.php same functions have to be in cache/item.php

	public function getItemData(&$info = array())
	{
		$data = $this->getVersitObject($vinfo);

    $title=$this->item['ITMCLASSIFYAS']?$this->item['ITMCLASSIFYAS']:$this->item['EVNTITLE'];
    if (!$title) $title = 'item';

    $info['size'] = strlen($data);
		$info['name'] = $title.$vinfo['fileext'];
		$info['mimetype'] = $vinfo['mimetype'];
		$info['encoding'] = 'base64';

		return $data;
	}
	
	public function getAttachmentData($partID,&$info = array())
	{
		$attachment = $this->getAttachment($partID);

		$info['name'] = $attachment['name'];
		$info['mimetype'] = $attachment['param']['mimetype'];
		$info['encoding'] = 'base64';
		
		return base64_encode($attachment['file_content']);
	}	
	
	public function getAttachmentDataFile($partID,&$info = array())
	{
		$attachment = $this->getAttachment($partID);
		$fName = Tools::randomFilename();
		
    file_put_contents($fName,$attachment);
   
    $info['name'] = $attachment['name'];
		$info['mimetype'] = $attachment['param']['mimetype'];
		$info['encoding'] = 'base64';
		
		return $fName;
	}	
	
	/************************************************************************
	 * iMIP Invitations                                                     *
	 ************************************************************************/   	
  
  public function createInvitation($bAttendeeInit = false)
  {
    //Account, folder and item specific variables
    $oGWAccount = &$this->folder->account;
    $sFolder = $this->folder->name;
    $sEvnID = $this->itemID;

    //Create an invitation
    $iMIP = iMIP::load($oGWAccount);
    $oInvitation = $iMIP->createInvitation($sFolder, $sEvnID,$bAttendeeInit);

    return $oInvitation;
  }
  
  public function createInvitationAttachment($oInvitation)
  {
    //Account, folder and item specific variables
    $oGWAccount = &$this->folder->account;
     //Create an invitation
    $iMIP = iMIP::load($oGWAccount);   
    //Create an attachment with versit object (text/calendar)
    $aAttachment = $iMIP->createAttachment($oInvitation->sInvitation);
    $sXML = $iMIP->convertVersit($oInvitation->sInvitation);
    $aAttachment['xml'] = $sXML;  
    
    return $aAttachment;
  }

  //Done by organizer
  //Cancels the event and sent the information to attendee list
  public function cancel($aAttendees = array())//INVM_ATTENDEE
  {
    //Load an invitation
    $iMIP = iMIP::load($this->folder->account);
    $oInvitation = $iMIP->loadInvitationID($this->folder->name, $this->itemID);

    //Cancel the event(decline if ATTENDEE) or
    //Cancel the list of attendees
    if(!$aAttendees){
      $oAttendees = $this->aAddons['contact'];
      $aAttendees = $oAttendees->getData();
      $this->aAttendees = $aAttendees;
    }
    
    if($aAttendees) foreach($aAttendees as $aAttendee)
      $aAttendeeID[] = $aAttendee['CNTEMAIL'];


    $sData = $oInvitation->cancel($aAttendeeID);
    

    $sXML = $iMIP->convertVersit($sData);

    //Create an attachment with versit object (text/calendar)
    $aAttachment = $iMIP->createAttachment($sData);
    $aAttachment['imip_xmlstr'] = $sXML;
    
    if($aAttendees) foreach($aAttendees as $aAttendee)
      WebmailIqItems::iMIP_Email($this,'cancel',$aAttachment,false,$aAttendee['CNTEMAIL']);

    
    return $aAttachment;
  }  
  
  //Done by attendee
  //Cancels the event and sent the information to organizer
  public function decline()
  {
    //Load an invitation
    $iMIP = iMIP::load($this->folder->account);
    $oInvitation = $iMIP->loadInvitationID($this->folder->name, $this->itemID);
    
    //Cancel the event(decline if ATTENDEE) or
    //Cancel the list of attendees
    $sData = $oInvitation->decline();
    $sXML = $iMIP->convertVersit($sData);
    
    //Create an attachment with versit object (text/calendar)
    $aAttachment = $iMIP->createAttachment($sData);
    $aAttachment['imip_xmlstr'] = $sXML;
    
    WebmailIqItems::iMIP_Email($this,'decline',$aAttachment);
    
    return $aAttachment;
  }  
}
