<?php

require_once('inc/cache/account.php');
require_once('inc/imap/folder.php');
require_once('inc/imap/imap.php');

/**
  @class IMAPAccount
  @brief Class representing an IMAP account.
  @author Martin Zlomek
  @date 13-02-2006
 */
class IMAPAccount extends CacheAccount {
	/**
	  @brief Create a new IMAPAccount object.
	  @param $user Parent user object of the account.
	  @param $accountID ID of the account.
	  @param $server Name of the server where the account is located.
	  @param $port Port where connect to on the server.
	  @param $username Account logging username.
	  @param $password Account logging password.
	 */
	public function __construct($user, $accountID,
								$server, $port, $username, $password, $primary = false,$properties = false)
	{
	  $this->folderClassName = 'IMAPFolder';
		parent::__construct($user, $accountID,
							$server, $port, $username, $password, $primary,'IMAP',$properties);
		
		

	}

	/**
	  @brief Create a new folder.
	  @param $name Name of the folder.
	 */
	public function createFolder($name,$type='')
	{
		// Check for folder existence
		parent::createFolder($name);
		// Create imap folder object 
		$this->folders['main'][$name] = IMAPFolder::create($this, $name);
	}

	/**
	  @brief Synchronize the folder cache with the IMAP server.
	 */
	public function sync()
	{
		try{
			$imap = IMAP::instance($this);
			$folders = $imap->getMailboxes($this->accountID);
			foreach ($folders as $folder){
				$folder->rights = IMAPFolder::decodeRights($folder->rights);
			}
			parent::sync($folders);
		}catch(Exc $e){
			
		}catch(Exception $e){
			
		}
		return true;
	}


	public function test()
	{
      $imap = IMAP::instance($this);
      
      return true;
  }
}
?>
