<?php
require_once('inc/user.php');

//Owner mode
define('INVM_OWNER',0x001);
define('INVM_ATTENDEE',0x002);
//Load type
define('INVL_ID',0x001);
define('INVL_VERSIT',0x002);
/**********************************************
 *@Class  iMIP
 *@Brief  Handles iMIP invitations
 *        Creates requests or process an
 *        invitation ,works with versit objects
 *        (convert to XML available)      
 *@Author Pavel Perna
 *@Rev    2007-12-19 
 **********************************************/
   
class iMIP
{
  static $instance;   //Here an object instances will be stored


  /***************************************************
   *@Method   __construct
   *@Brief    iMIP class initialization
   *@Scope    Private   
   *@Param    $oAccount   GroupWareAccount  GroupWare object to be used
   ***************************************************/              
  private function __construct(&$oAccount)
  {
    $this->oAccount = &$oAccount;

  }


/////////////
// PUBLIC  //
/////////////

  /*********************************************************************
   *@Method   load
   *@Brief    load an iMIP class, if an instance already
   *          exists, return it , or create new one   
   *@Scope    Public     
   *@Param    $oAccount   GoupWareAccount   GroupWare object to be used
   *@Result               iMIP              an iMIP class instance   
   **********************************************************************/         
  public static function load($oAccount)
  {
    if(!isset(self::$instance[$oAccount->accountID]))
      self::$instance[$oAccount->accountID] = new iMIP($oAccount);
      
    return self::$instance[$oAccount->accountID];
  }
  
  
  
  /*****************************************************
   *@Method   createInvitation   
   *@Brief    Calls processVCalendar function from
   *          GroupWare API     
   *@Scope    Private   
   *@Param    $sFolder      string      Folder name to work with 
   *@Param    $sEvnID       string      ID of event we want invitation for 
   *@Result   $oInvitation  Invitation  Object or error(0)  
   *****************************************************/         
  public function createInvitation($sFolder,$sEvnID,$bAttendeeInit = false)
  {
      //Open access to folder
    $this->initAttendees($sFolder,$sEvnID);
    $sData = $this->processVCalendar($sFolder,$sEvnID,'','REQUEST');
    
    $oInvitation = new Invitation($this,INVM_OWNER,$sData,$sFolder);    
    return $oInvitation;
  }
  
  
  /*****************************************************
   *@Method   loadInvitationID   
   *@Brief    Loads an invitation from folder and id
   *          Calls processVCalendar function from
   *          GroupWare API     
   *@Scope    Private   
   *@Param    $sFolder      string      Folder name to work with 
   *@Param    $sEvnID       string      ID of event we want invitation for 
   *@Result   $oInvitation  Invitation  Object or error(0)  
   *****************************************************/         
  public function loadInvitationID($sFolder,$sEvnID)
  {
    $oInvitation = new Invitation($this,INVM_OWNER,'',$sFolder,$sEvnID);  
    $oInvitation->mLoad = INVL_ID;  
    return $oInvitation;
  }
  
    
  /*****************************************************
   *@Method   loadInvitation   
   *@Brief    Loads an invitation request from versit
   *          object in string ( attached in invitation
   *          email )       
   *@Scope    Private   
   *@Param    $sData      string      Versit object string
   *@Result   $oInvitation  Invitation  Object or error(0)  
   *****************************************************/       
  public function loadInvitation($sData)
  {
    $oInvitation =  new Invitation($this,INVM_ATTENDEE,$sData); 
    $oInvitation->mLoad = INVL_VERSIT;  
    return $oInvitation; 
  }

 /*****************************************************
   *@Method   convertVersit   
   *@Brief    Transfoms Versit from String to 
   *          some other format(XML)              
   *@Scope    Public   
   *@Param    $sVersit    string    Versit object string 
   *@xResult  $xResult    XMLstring Versit object xml  
   *****************************************************/     
  public function convertVersit($sVersit,$sFormat = 'XML')
  {
    if(!$sVersit)
      throw new Exc('imip_versit_empty');

    if(!$xResult = $this->oAccount->gwAPI->FunctionCall("ConvertVersit",
                                                    $this->oAccount->gwAPI->sessid,
                                                    $sVersit,
                                                    $sFormat))
      throw new Exc('imip_versit_convert',$xResult);
                                                    
    return $xResult;
  }

 /*****************************************************
   *@Method   createAttachment   
   *@Brief    Creates an attachment in user object
   *          that will be used when sending invitation/reply email                
   *@Scope    Public   
   *@Param    $sVersit    string    Versit object string 
   *@Param    $sName      string    Name of the attachment
   *@Result   $aAttachment['fullpath']  string    the name of the uploaded folder
   *@Result   $aAttachment['classname'] string    the name of the uploaded item          
   *****************************************************/     
  public function createAttachment($sVersit,$sName='',$sMethod='REQUEST')
  {
    $oUser = &$this->oAccount->account->user;
    
     //Generate attachment folder name and item name (to be used in fullpath)
		$sFolderID = "vCalendar";
		$sItemID = date('Y-m-d-') .Tools::my_uniqid();
    
    $oUser->addStringAttachment($sVersit,
                                ($sName?$sName:'Invitation') .' - '. ($oUser->countAttachments($folderID) + 1) . imipext,
                                'text/calendar',
                                $sFolderID,
                                $sItemID,
                                $sMethod);

    $aAttachment['fullpath'] = $sFolderID.'/'.$sItemID;
    $aAttachment['classname'] = 'file';
    $aAttachment['versit'] = $sVersit;
    return $aAttachment;
  }
  
   /*****************************************************
   *@Method   processVCalendar   
   *@Brief    Calls processVCalendar function from
   *          GroupWare API     
   *@Scope    Public   
   *@Param    $sFSID      string  Folder session id to work with 
   *@Param    $sEvnID     string  Event ID 
   *@Param    $sData      string  Versit object     
   *@Result   $sResult    result  Versit object(string) or error(0)  
   *****************************************************/         
  public function processVCalendar($sFolder='',$sEvnID='',$sData='',$sType = 'REQUEST')
  {
    switch($sFolder){
      case 'group':
        $sSID = $this->oAccount->gwAPI->OpenGroup();
      break;
      default:
        if(!$sFolder)
          throw new Exc('imip_missing_folder');
        
        $sSID = $this->openAccess($sFolder);
      break;
    }

    return $this->oAccount->gwAPI->FunctionCall("ProcessVCalendar",$sSID,$sEvnID,$sData,$sType);
  }
    
/////////////
// PRIVATE //
/////////////
    
  /*****************************************************
   *@Method   initAttendees   
   *@Brief    Set the init values for attendees of the
   *          specified event in the desired folder,
   *          befrore creating an invitation           
   *@Scope    Private   
   *@Param    $sFolder    string  Folder name to work with 
   *@Param    $sEvnID     string  ID of event we want invitation for 
   *@Result   $sResult    BOOL    True if success else throws an exc  
   *****************************************************/         
  private function initAttendees($sFolder,$sEvnID)
  {
    //Get folder
    if(!$oFolder = &$this->oAccount->getFolder($sFolder))
      throw new Exc('folder_invalid_id',$sFolder);
    
    //Get event
    if(!$oItem = &$oFolder->getItem($sEvnID))
      throw new Exc('item_invalid_id',$sEvnID);
    

    
    //Get attendees
    $oAttendees = $oItem->aAddons['contact'];
    $aAttendees = $oAttendees->getData();
    
    //Set RSVP to 1
    $aData = array();
    if($aAttendees) foreach($aAttendees as $aAttendee){
      
      $aData['CNTRSVP'] = 1;
      $oAttendees->edit($aData,$aAttendee['CNT_ID']);  
    }
    
    return true;
  }


  
  /*****************************************************
   *@Method   openAccess   
   *@Brief    Opens an access to the specified folder
   *          in order to work with its items        
   *@Scope    Public   
   *@Param    $sFolder    string  Folder id to work with 
   *@Result   $sResult    string  GroupWare folder session id  
   *****************************************************/           
	private function openAccess($sFolder)
	{
			$oAccount = &$this->oAccount;
			$sFolder = str_replace('/','\\',$sFolder);
      		  
			//Check GW connection
			if (!$oAccount->gwAPI->IsConnected())
				  $oAccount->gwAPI->Login();  
				  
			if (!$this->sGWSessionID)					
			//Get folder & group access
			if (!$oAccount->sGWSessionID = $oAccount->gwAPI->OpenGroup('*')) 
				  throw new Exc('group_open'); 

			if(!$this->sFID = $oAccount->gwAPI->OpenFolder($oAccount->sGWSessionID,
															                       $sFolder))
				  throw new Exc('folder_open');
				  
	    return $this->sFID;
  }
}

/**********************************************
 *@Class  Invitation 
 *@Brief  Represents an invitation for
 *        some event  
 *@Author Pavel Perna
 *@Rev    2007-12-19
 **********************************************/
class Invitation
{

  /***************************************************
   *@Method   __construct
   *@Brief    iMIP class initialization
   *@Scope    Public   
   *@Param    $iMIP   iMIP    iMIP class (parent)
   ***************************************************/              
  public function __construct(&$oIMIP,
                                $cMode,
                                $sData = '',
                                $sFolder = '',
                                $sEvnID = '')
  {
    $this->cMode = $cMode;
    $this->oIMIP = &$oIMIP;
    $this->sInvitation = $sData;
    $this->sFolder = $sFolder;
    $this->sEvnID = $sEvnID;
  }  
  
  public function setFolder($sFolder)
  {
    $this->sFolder = $sFolder;
  }
  
  public function setEvent($sEvnId)
  {
    $this->sEvnId = $sEvnId;
  }
  
  
  public function accept($sFolder)
  {
    $sResult = $this->oIMIP->processVCalendar($sFolder,'',$this->sInvitation,'REPLY=ACCEPTED'); //Accept
    return $sResult;
  }
  
  //Decline the invitation or event(if invitation loaded by id)
  public function decline()
  {
    
    switch($this->mLoad){
      //Decline invitation (done over the versit sent as invitation)
      case INVL_VERSIT:
        $sResult = $this->oIMIP->processVCalendar('group','',$this->sInvitation,'REPLY=DECLINED'); // Decline
        
        break;
      //Decline invitation (done when attendee deletes the event from his calendar)
      case INVL_ID:
        $sResult = $this->oIMIP->processVCalendar($this->sFolder,$this->sEvnID,'','REPLY=DECLINED'); // Decline
          
        break;
    }

    return $sResult;
  
  }

  //Done over existing item when organizer removes 
  //some attendees or when he deletes the invited event
  public function cancel($aAttendeeID=array())
  {

    if ($aAttendeeID)
      $sAttendeeID = implode(",",$aAttendeeID); 
    
    $sResult = $this->oIMIP->processVCalendar($this->sFolder,$this->sEvnID,$this->sInvitation,'CANCEL='.$sAttendeeID);
    return $sResult;
  
  }
  
  public function process()
  {
    $sResult = $this->oIMIP->processVCalendar('group','',$this->sInvitation,'');
    return $sResult;
  
  }


}

?>
