<?php

require_once 'api/account.php';
require_once 'api/api.php';
require_once 'api/domain.php';
require_once 'api/remoteaccount.php';
require_once 'security.php';
require_once 'tools.php';
require_once 'xmlrequest.php';

//VERSION
define('WEBMAIL_DB_VERSION', 4);

// LOGIN
define('heshexpire', 60);	// sec
define('sessionexpire', 240); // min

//DATABASE
define('DB_MYSQL',0x0);
define('DB_SQLITE',0x1);
define('WM_DB_TYPE',DB_SQLITE);
define('DB_QUERY_CHUNK',1);
define('DB_BODY_LIMIT',32000);
define('DB_MAX_IDS_IN_QUERY',1000);

//SYNCHRONIZATION
define('MAX_CHUNK_SIZE',1000);
define('CHUNK_SIZE',1000);
define('UNAVAILABLE_FILE', 'unavailable.tmp');
define('MAX_EXECUTION_TIME', 960);

//LOGS
define('HARD_LOG',0);
define('EXCEPTION_LOG',1);

// Paths
if (!($path = get_cfg_var("icewarp_server_path"))) $path = '../../../';

$api = new MerakAPI;
$_SESSION['LOGPATH'] = $api->GetProperty('C_System_Storage_Dir_LogPath');
$_SESSION['SERVER_ID'] = $api->GetProperty('C_System_Storage_LB_ServerID');

define('WM_SERVER_PATH', $path);
define('WM_CONFIGPATH', $api->GetProperty('C_ConfigPath').'_webmail/');
define('WM_HOSTSPATH', WM_CONFIGPATH.'_hosts/');

if(file_exists(WM_CONFIGPATH.'server.xml')){
	$xml = simplexml_load_file(WM_CONFIGPATH.'server.xml');
	if(intval($xml->item->logs)==1){
		$_SESSION['LOGS'] = 1;
	}
}

//Folders
define('SETTINGS_FOLDER','~webmail/');
define('SETTINGS_FOLDER_NO_SLASH','~webmail');
define('UPLOAD_FOLDER','~webmail/~upload/');
define('RSA_CONFIGPATH','data/');
define('sessiondir', './sessions/');
define('loggingdir', /*WM_SERVER_PATH.'logs/*/'webmail/');

//File Extensions
define('storeageext', '.xml');
define('filterext', '.xml');
define('loggingext', '.log');
define('sessionext', '.ses');
define('dataext', '.dat');
define('templateext', '.tpl');
define('messageext', '.msg');
define('imipext','.ics');
define('localpopmessageext', '.tmp');


//Files
define('accountsfile', 'accounts.xml');
define('rssfile', 'rss.xml');
define('SETTINGS_FILE','settings');
define('COOKIE_FILE','cookies');
define('LOGIN_FILE','login');
define('CERTIFICATE_FILE','cert.pem');
define('GLOBAL_FILE','server');
define('SPELLCHECKER_FILE','spellchecker');
define('GLOBAL_SETTINGS_FILE',WM_CONFIGPATH.GLOBAL_FILE.storeageext);
define('GLOBAL_SETTINGS_DEFAULT','./inc/templates/globalsettings.tpl');
define('SPELLCHECKER_SETTINGS_FILE',WM_CONFIGPATH.SPELLCHECKER_FILE.storeageext);
define('SPELLCHECKER_SETTINGS_DEFAULT','./inc/templates/spellchecker.tpl');

// OTHER
define('restriction_suffix','_restriction');
define('allowdadmin', 1);	 // domain admin can edit global setttings like full admin

//Timezone
define('SERVER_TIMEZONE',100);

//RSS
define('RSS_ENABLED',true);

//URL locations
define('DOWNLOAD_PHP','server/download.php');
define('WEBMAIL_PHP','server/webmail.php');

//Useful constants
define('CR',"\r");
define('LF',"\n");
define('CRLF',"\r\n");
define('UNIX1900',2415019);
define('DAY_LENGTH',86400);
define('WEEK_LENGTH',604800);
define('WORK_WEEK_LENGTH',432000);
define('WITH_ADDONS',0x0);
define('NO_ADDONS',0x1);
define('delimiterchar',',');

//cVard and vCalendar
define('VCARD_EXT','.vcf');
define('VCALENDAR_EXT','.ics');

// PHP5 FUNCTIONS
/*
	@Brief: Logging class
	@		Class for handling exceptions , it handles critical and non-critical errors
	@Author Pavel Perna
 */



 
class LogError {
	/**
	  @Brief : Logs an error to log file
	  @param $file - file to log an error
	  $param $message an error message to log
	 */
	public static function Error($message,$type = "Warning"){
		//Get file name
		$date = date("Ymd");

		$file = $_SESSION['LOGPATH'].loggingdir.$_SESSION['SERVER_ID'].$date;
    $file = self::checkRotationName($file);
    
		//Gets an error message
		list($msec, $time) = explode(" ", microtime());
		$buffer = 'ERROR    '.date('H:i:s.',$time).substr($msec,2,6)."\r\n".$type.": ".$message."\r\n";

		//Opens log file for appending or creates a new one
		if (!file_exists($_SESSION['LOGPATH'].loggingdir)) 
      $_SESSION['LOGPATH'].loggingdir;
    if (!file_exists($file)) $fp = @fopen($file,"w+");
		else $fp = fopen($file,"a+");
		//Write the message
		@fwrite($fp,$buffer);
		//Close the file
		@fclose($fp);
		return true;
	}
	
	public static function checkRotationName($file){
    $i = "";		

    
   	if($_SESSION['LOG_ROTATION']){
      $strings[''] = $strings[0] = "-00";
      for($j = 1;$j<10;$j++)
      $strings[$j] = "-0".$j;
      
      while(@filesize($file.($strings[$i]?$strings[$i]:("-".$i)).loggingext)>$_SESSION['LOG_ROTATION'])
        $i++;
    }
    $file = $file.($strings[$i]?$strings[$i]:(($i?"-":'').$i)).loggingext;
      
    return $file;    
  }
	
	
	public static function CommunicationLog($message,$file = false){
		//Get file name
		$date = date("Ymd");

    if(!$file){
		  $file = $_SESSION['LOGPATH'].loggingdir.$_SESSION['SERVER_ID'].$date;
		  $file = self::checkRotationName($file);
    }
    //Gets an error message
		$buffer = $message;
    if (!file_exists($_SESSION['LOGPATH'].loggingdir)) 
        if(!System::mkdir_r($_SESSION['LOGPATH'].loggingdir))
          throw new Exc('logdir_create',$_SESSION['LOGPATH'].loggingdir);

		//Opens log file for appending or creates a new one
		if (!file_exists($file)) $fp = @fopen($file,"w+");
		else $fp = @fopen($file,"a+");
		
		
		
		//Write the message
		@fwrite($fp,$buffer);
		//Close the file
		@fclose($fp);
		return true;
	}
		
}

/*
	@Brief: Exception class
	Class for handling exceptions , it handles critical and non-critical errors
	@Author Pavel Perna
 */
class Exc extends Exception {

	/**
	  @Brief : constructor
	  @params:
	  msg /STRING: Description of error
	  code /INT: Error code
	 */
	 
	public $message;
   
	public function __construct($code = "default",$msg = '',$showLocation = false,$logError = true,$sView = false){
    
    $this->WMCode = $code;
    
		parent::__construct($msg,intval($code));
    $this->wmcode = $code;
		$this->sView = $sView;
    $this->showLocation = $showLocation;
		//Log error
		if (($logError && EXCEPTION_LOG && $_SESSION['LOGS']))
			LogError::Error($this->Message(),"Error"); 

	}

	/**
	  @Brief : Generates an error message
	 */
	public function Message()
	{

		$ret = htmlspecialchars($this->message);
  //  if ($this->showLocation) 
    $ret.=" in file:".$this->file." on line:".$this->line." found in function <b>".$this->function."</b> and class <b>".$this->class."</b>\r\n";
		return $ret;
	}
	
	public function XHTMLMessage($oLanguage = false,$sSkin = false)
	{
		global $_iqdata,$sInput;
		

		$template['eid'] = strval($this->wmcode);
   
   switch(strval($this->wmcode)){
    case 'login_invalid':
      $langid = 'invalid_login';
      break;
    case 'default':
      $langid = strval($this->wmcode); 
      break;
   }

   if($oLanguage){
     $code = $oLanguage->get('error',$langid);
     if($code)
      $this->wmcode = $code; 
   }
    //if ($this->showLocation) 
    //$template['message'].=" in file:".$this->file." on line:".$this->line;
      
		$template['id'] = $_iqdata['id'];
		$template['message'] = nl2br($this->message);
    $template['sid'] = $_iqdata['sid'];

    $template['source'] = $sInput;

		
		return trim(template('./interface/skin/'.$sSkin.'/template/error.tpl',$template));
	}
  	
	public function XMLResponse()
	{
		global $_iqdata,$sInput;
		

		$template['message'] = htmlspecialchars($this->message);
   
    //if ($this->showLocation) 
    //$template['message'].=" in file:".$this->file." on line:".$this->line;
      
		$template['id'] = $_iqdata['id'];
		$template['eid'] = strval($this->wmcode);
    $template['sid'] = $_iqdata['sid'];

    $template['source'] = htmlspecialchars($sInput);


		return trim(template('./inc/templates/xmlrequest_error.tpl',$template));
	}

}
/*
//Var_dump on DOMElement
function dump_r($o,$ommit = array())
{
  if ($o->childNodes)
  {
	  for ($i = 0; $i <  $o->childNodes->length; $i++)
	  {
			if (!$o->childNodes->item($i)->tagName && $o->nodeName){
			  $key = $o->childNodes->item($i)->tagName?$o->childNodes->item($i)->tagName:$o->nodeName;
        if (!in_array($key,$ommit))
          $r[$key] = $o->childNodes->item($i)->nodeValue;
			}
      if ($o->childNodes->item($i)->childNodes){
         if (!in_array($o->nodeName,$ommit))
            $r[$o->nodeName][$i] = dump_r($o->childNodes->item($i));
			}
			
	  }
	  return $r;
  }
  else 
  {
    if (is_array($o)) foreach ($o as $key => $val)
      if (in_array($key,$ommit)) unset($o[$key]);
	return $o;
  }
}*/
function dmp($o,$ommit = array(),$print = true)
{
	print_r($o);
//	$r = dump_r($o,$ommit);
/*	if ($print ||(!is_array($o) && !is_object($o)){
		print_r($o);
	}else{
		foreach($o as $key=>$val){
			if(!in_array($o,$key) && !isset($o->$key))
				dmp($o,$ommit);
		}
	}
*/
}

//Output buffering
function log_buffer($buffer, $type, $attributes = array(),$file = false)
{
      
			static $start,$msec,$id,$sid;
      global $_iqdata;
			switch (strtoupper($type)) {
			  case 'DEBUG':
			    //Do not log debug logs
			    return;
				case 'RESPONSE':
				case 'ERROR':
					list($mesec, $end) = explode(" ", microtime());
					if ($start && $msec)
					 $duration = $end - $start + $mesec - $msec;
          $CommunicationLog='-------------------------------------------------------------------------------------'."\r\n";
					$CommunicationLog.=strtoupper($type).' '.date('H:i:s.',$end).substr($mesec,2,6).', IQ ID: '.$id.', SID: '.$sid.' Duration: '.$duration."\r\n";
					break;
				
				case 'REQUEST':
								  
					$id = $attributes['id'];
					$sid = $attributes['sid'];
					
					list($msec, $start) = explode(" ", microtime());
					if (!$id) $id = rand();
					$CommunicationLog="\r\n====================================================================================\r\n";
					$CommunicationLog.='REQUEST  '.date('H:i:s.',$start).substr($msec,2,6).', IQ ID: '.$id.', SID: '.$sid."\r\n";
					break;
			}

      if (!@$_SESSION['LOGS'] && !HARD_LOG)
      	return;

      if ($buffer) $CommunicationLog.=$buffer."\r\n";

      if ($attributes) $_iqdata = $attributes;

      LogError::CommunicationLog($CommunicationLog, $file);

}

/*class Singleton 
{
	protected static $instance;
	protected function __construct()
	{
	
	}
	public function instance($sClassName)
	{
		if(!isset(self::$instance[$sClassName])){
			self::$instance[$sClassName] = new $sClassName();
		}
		return self::$instance[$sClassName];
	}
}
*/

//	 function __autoload($className) {
//		  require_once(dirname($_SERVER['SCRIPT_FILENAME'])."/inc/$className.php");
//	 }

?>
