<?php
require_once ('inc/api/account.php');
require_once ('inc/api/api.php');
require_once ('inc/tools.php');
require_once ('inc/system.php');
require_once ('inc/search.php');
/**
 * @class LocalPOP
 * @brief Class for accessing LocalPOP accounts.
 * @author Pavel Perna
 * @date 17-04-2007
 */
class LocalPOP
{
	private static $instance;

	public static function instance($account)
	{
		$email = $account->accountID;
		if (!isset(self::$instance[$email]))
		{
			self::$instance[$email] = new LocalPOP($email);
		}
		return self::$instance[$email];
	}

	/**
	 * @brief Create an instance of localpop object.
	 */
	private function __construct($email)
	{
		$this->email = $email;
		$this->mAccount = new MerakAccount();
		$this->mAccount->Open($email);
		$this->delimiter = '/';
		$this->mailboxPath = $this->getMailboxPath();
	}

	/***********/
	/* Mailbox */
	/***********/
	public function getMailboxPath()
	{
		$userpath = $this->mAccount->getProperty("u_fullmailboxpath");
		return $userpath;
		//return $_SESSION['MAILPATH'].$userpath;
	}
	/**
	 * @brief Get the list of mailboxes.
	 * @param $ownerID ID of owner account
	 * @return The list as an array of objects.
	 */
	public function getMailboxes()
	{
		$mailboxes = array();
		/*if (realpath($this->mailboxPath."INBOX")!=$this->mailboxPath."INBOX"){
		$itm->name = "INBOX";
		$_SESSION['INBOX_REMAP'] = true;
		$_SESSION['INBOX_FOLDER'] = basename(realpath($this->mailboxPath."INBOX"));
		}else{
		$_SESSION['INBOX_REMAP'] = false;
		$itm->name = "INBOX";
		}
		$itm->delimiter = $this->delimiter;
		$itm->rights = 255;
		$mailboxes[] = $itm;*/
		$api = new MerakAPI();
		$mailboxes = array_merge($mailboxes, $this->mailboxDirectories($this->mailboxPath, $api));
		return $mailboxes;
	}

	private function mailboxDirectories($path, $api)
	{
		$mailboxes = array();
		$mailboxes = icewarp_get_folder_list($path, 1);
		$mailboxes = Tools::explode_j(CRLF, $mailboxes);
		$_SESSION['SPAM_REMAP'] = false;
		$_SESSION['INBOX_REMAP'] = false;
		$inboxExists = false;
		$return = array();
		foreach ($mailboxes as $nr => $m)
		{
			$itm = new stdClass();
			$itm->name = substr(str_replace('\\', '/', $m), strlen($path), strlen($m) - strlen($path) - 1);
			$itm->rights = 255;
			$itm->delimiter = $this->delimiter;
			//~spam Folder remapped to Spam
			if (substr($itm->name, 0, 5) == '~spam' && $_SESSION['SPAM_FOLDER'])
			{
				$_SESSION['SPAM_REMAP'] = true;
			}
			if (strtolower(substr($itm->name, 0, 5)) == 'inbox')
			{
				$_SESSION['INBOX_FOLDER'] = substr($itm->name, 0, 5);
				$_SESSION['INBOX_REMAP'] = true;

				if (strtolower($itm->name) == 'inbox')
				{
					$inboxExists = true;
				}
			}
			//skip spam folder
			if (strtolower($itm->name == 'spam'))
			{
				continue;
			}
			$itm->name = $this->decode($itm->name);
			if ($itm->name[0]!= '~' 
			&& strpos($itm->name, '~quarantine') === false)
			{
				$return[] = $itm;
			}
		}
		//If no virtual inbox subfolder exists and inbox does not exist, manually add it
		if (!$inboxExists)
		{
			$itm = new stdClass();
			$itm->name = 'INBOX';
			$itm->rights = 255;
			$itm->delimiter = $this->delimiter;
			$return[] = $itm;
		}
		return $return;
	}


	/**
	 * @brief Create a new mailbox.
	 * @param $mailbox Name of the mailbox.
	 */
	public function createMailbox($mailbox)
	{
		$mailbox = $this->encode($mailbox, $this->delimiter);
		//Check for existence
		if (file_exists($this->mailboxPath . $mailbox))
			throw new Exc('folder_already_exists', $mailbox);
		//Secure path on mailbox - ../ etc.
		if (Tools::securePath($mailbox) === false)
			throw new Exc('folder_name_bad_char', $mailbox);
		//Create mailbox
		$result = System::mkdir_r($this->mailboxPath . $mailbox);
		return $result;
	}

	/**
	 * @brief Delete a mailbox.
	 * @param $mailbox Name of the mailbox.
	 */
	public function deleteMailbox($mailbox)
	{
		$mailbox = $this->encode($mailbox, $this->delimiter);
		if (System::rmdir($this->mailboxPath . $mailbox) === false)
			throw new Exc('folder_delete', $mailbox);
	}

	/**
	 * @brief Rename a mailbox.
	 * @param $oldMailbox Name of the mailbox.
	 * @param $newMailbox New name of the mailbox.
	 */
	public function renameMailbox($oldMailbox, $newMailbox)
	{
		$oldMailbox = $this->encode($oldMailbox);
		$newMailbox = $this->encode($newMailbox);
		if (!is_dir($this->mailboxPath . $oldMailbox))
			throw new Exc('folder_does_not_exist', $oldMailbox);
		//&& ($newMailbox==$oldMailbox) is used to hack folder case sensitivity(rename test->Test)
		if (is_dir($this->mailboxPath . $newMailbox) && ($newMailbox == $oldMailbox))
			throw new Exc('folder_already_exists', $newMailbox);
		if (Tools::securePath($newMailbox) === false)
			throw new Exc('folder_name_bad_char', $newMailbox);
		if (rename($this->mailboxPath . $oldMailbox, $this->mailboxPath . $newMailbox)
			=== false)
			throw new Exc('folder_rename');
	}

	/**
	 * @brief Empty an IMAP mailbox.
	 * @param $mailbox Name of the mailbox.
	 */
	public function emptyMailbox($mailbox)
	{
		$mailbox = $this->encode($mailbox);
		$mailbox = $this->getFolderPath($mailbox);
		$content = glob($mailbox . '*' . localpopmessageext);
		if ($content)
			foreach ($content as $item)
				if (!is_dir($item))
					unlink($item);
	}

	public function getFolderPath($mailbox)
	{
		if (strtolower($mailbox) == 'inbox')
			return $this->mailboxPath;
		else
		{
			$mailbox = self::encode($mailbox);
			return $this->mailboxPath . $mailbox . $this->delimiter;
		}
	}

	/********/
	/* Item */
	/********/

	public function getItemPath($mailbox, $uid)
	{
		return $this->getFolderPath($mailbox) . $uid . localpopmessageext;
	}

	public function getItem($mailbox, $uid)
	{
		$itm = $this->getItemDetails($mailbox, $uid);

		return $itm;
	}


	/**
	 * @brief Get the list of items of the mailbox.
	 * @param $mailbox Name of the mailbox.
	 * @return The list as an array of objects.
	 */
	public function getItems($mailbox, $sequence = "1:*")
	{
		$mailbox = $this->encode($mailbox, $this->delimiter);
		if (!$this->syncList[$mailbox])
		{
			$mbox = $this->getFolderPath($mailbox);
			$itms = glob($mbox . '*' . localpopmessageext);

			$list = array();
			if ($itms)
				foreach ($itms as $path => $item)
				{
					eregi('([a-z0-9-]{0,})' . localpopmessageext, $item, $regs);
					$itm = $this->getItemDetails($mailbox, $regs[1]);
					$list[$itm->uid] = $itm;
					unset($itm);
				}
			$this->syncList[$mailbox] = $list;
		}
		if ($sequence != "1:*")
		{
			$range = explode(":", $sequence);
			$range[0] = $range[0] - 1;
			//Possible fix
			if ($range[1] == '*')
				$range[1] = count($this->syncList[$mailbox]);
			$list = array_slice($this->syncList[$mailbox], $range[0], $range[1] - $range[0]);
		}
		return $list;
	}

	//This function is used as alternative to imap msgnumber
	public function msgNumber($uid, $items)
	{
		$number = 0;
		foreach ($items as $msg)
		{
			$number++;
			if (eregi_replace('e', '', $msg->uid) == eregi_replace('e', '', $uid))
			{
				return $number;
			}
		}
	}

	/**
	 * @brief Get the list of items of the mailbox.
	 * @param $mailbox Name of the mailbox.
	 * @return The list as an array of objects.
	 */
	public function getItemsID($mailbox, &$handle = false)
	{
		//$mailbox = $this->encode($mailbox);
		$mbox = $this->getFolderPath($mailbox);

		$itms = glob($mbox . '*' . localpopmessageext);

		$list = array();
		if ($itms)
			foreach ($itms as $item)
			{
				eregi('([a-z0-9-]{0,})' . localpopmessageext, $item, $regs);
				$itm->uid = $regs[1];
				$list[$itm->uid] = $itm;
				unset($itm);
			}
		return $list;
	}

	public function drop()
	{
		unset($this->syncList[$mailbox]);
	}

	public function countItems($mailbox, &$itemsID = false)
	{
		$itemsID = $this->getItemsID($mailbox);
		return count($itemsID);
	}
	/***********/
	/* Message */
	/***********/

	/**
	 * @brief Get a message.
	 * @param $mailbox Name of the mailbox.
	 * @param $uid UID of the message.
	 * @return Raw data of the message.
	 */
	public function getMessage($mailbox, $uid)
	{
		$mailbox = $this->encode($mailbox);
		//Get items in mailbox
		$path = $this->getItemPath($mailbox, $uid);
		throw new Exc('a');
		//Read message(specified by UID) file
		/*@*/
		$message = str_replace("\n.", "\n", file_get_contents($path));

		return $message;
	}


	/**
	 * @brief Create a new message in a mailbox.
	 * @param $mailbox The mailbox.
	 * @param $message Raw data of the message.
	 * @return The message item.
	 */
	public function createMessage($mailbox, $message, $file, &$uid = false)
	{
		$mailbox = $this->encode($mailbox, $this->delimiter);
		$mbox = $this->getFolderPath($mailbox);

		//Generate unique filename
		$uid = false;
		while (!$uid || file_exists($path))
		{
			$uid = self::generateMessageName();
			$path = $this->getItemPath($mailbox, $uid);
		}
		if ($file === false)
		{
			//Write the message data to the file - obsolete not in use now
			@file_put_contents($path, str_replace("\n.", "\n..", $message));
		} else
		{
			copy($file, $path);
		}


		return $this->getMessageItem($message, $uid, 0, $staticFlags, $file);
	}

	/**
	 * @brief Generate message item for future usage by other functions.
	 * @param $message Raw data of the message.
	 * @param $uid UID of the message.
	 * @param $flags Flags of the message.
	 * @param $staticFlags Static flags of the item.
	 * @return The message item.
	 */
	private function getMessageItem($message, $uid, $flags = 0, $staticFlags = array
		(), $file = false)
	{
		$headers = "";

		if ($file === false)
		{
			$separator = strpos($message, "\r\n\r\n");

			if ($separator === false)
				$headers = $message;
			else
				$headers = substr($message, 0, $separator + strlen("\r\n\r\n"));

			//if (!is_object($this->parser))
			if ($headers)
			{
				$parser = &new MailParse(false, false, false, false, '1', $headers);
				$msgHeaders = $parser->getHeaders();
			}
			$item->size = strlen($message);
		} else
		{
			$parser = &new MailParse($file);
			$msgHeaders = $parser->getHeaders();
			$item->size = filesize($file);
		}
		$item->uid = $uid;
		$item->flags = $flags;
		$item->color = 'Z';
		$item->date = isset($msgHeaders['date']) ? $msgHeaders['date'] : '';
		//Read the date from the file creation time
	    if(!$item->date || $item->date == -1){
	      @$item->date = filectime($file);
	    }
		if(!is_int($item->date)){
			$item->date = strtotime($item->date);
		}
		$item->from =  isset($msgHeaders['from']) ? $msgHeaders['from'] : '';
		$item->to =  isset($msgHeaders['to']) ? $msgHeaders['to'] : '' ;
		$item->subject = isset($msgHeaders['subject']) ? $msgHeaders['subject'] : '';
		$item->priority = isset($msgHeaders['priority']) ? intval($msgHeaders['priority']) :Item::NORMAL_PRIORITY;
		$item->staticFlags = isset($staticFlags['flags']) ? $staticFlags['flags'] : 0;
		$item->sMimeStatus = isset($staticFlags['smime_status']) ? $staticFlags['smime_status'] :0;
		$item->hasAttachments = isset($staticFlags['has_attachment']) ? $staticFlags['has_attachment'] : false;

		return $item;
	}

	/**
	 * @brief Get details of specified message of the mailbox.
	 * @param $mailbox Name of the mailbox.
	 * @param $uid UID of the message.
	 * @return The object containing detials of a message.
	 */
	public function getItemDetails($mailbox, $uid)
	{
		//$message = $this->getMessage($mailbox, $uid);
		//return $this->getMessageItem($message, $uid);

		$file = $this->getItemPath($mailbox, $uid);
		return $this->getMessageItem(false, $uid, 0, array(), $file);
	}

	/**
	 * @brief Generates name for the message file
	 * @return The message name.
	 */
	private static function generateMessageName()
	{
		//Date and time : YYYYMMDDhhmmmss
		$prefix = date('YmdHis');

		//Suffix : RRRR - random
		$suffix = intval(mt_rand(0, 9999));

		if ($suffix < 10)
			$suffix = '000' . $suffix;
		else
			if ($suffix < 100)
				$suffix = '00' . $suffix;
			else
				if ($suffix < 1000)
					$suffix = '0' . $suffix;

		return $prefix . $suffix;
	}

	/**
	 * @brief Delete a message.
	 * @param $mailbox Name of the mailbox.
	 * @param $uid UID of the message.
	 */
	public function deleteMessage($mailbox, $uid)
	{
		global $aError;

		$mailbox = $this->encode($mailbox, $this->delimiter);

		$path = $this->getItemPath($mailbox, $uid);

		if (!unlink($path))
			throw new Exc('item_delete', $path);
	}

	/**
	 * @brief Move a message to another mailbox.
	 * @param $srcMailbox Name of the source mailbox.
	 * @param $uid UID of the message.
	 * @param $dstMailbox Name of the destination mailbox.
	 * @param $copy If TRUE, the message will be copied instead of moved
	 * @return New UID of the message.
	 */
	public function moveMessage($srcMailbox, $uid, $dstMailbox, $copy = false)
	{

		//$srcMailbox = str_replace("\\","/",$srcMailbox);
		//$dstMailbox = str_replace("\\","/",$dstMailbox);

		$srcFolder = $this->getFolderPath($srcMailbox);
		$dstFolder = $this->getFolderPath($dstMailbox);

		if (!is_dir($srcFolder) || !is_dir($dstFolder))
			throw new Exc('folder_does_not_exist', $srcFolder . "," . $dstFolder);

		$nuid = $this->generateMessageName();

		$flags = explode("-", $uid);
		if ($flags[1])
		{
			$flags = $flags[1];
			$nuid .= '-' . $flags;
		}

		if (!$copy)
			$nuid = $uid;

		$srcPath = $this->getItemPath($srcMailbox, $uid);
		$dstPath = $this->getItemPath($dstMailbox, $nuid);

		if ($copy)
		{
			if (copy($srcPath, $dstPath) === false)
				throw new Exc('folder_copy', $srcPath . '=>' . $dstPath);
		} else
		{
			if (rename($srcPath, $dstPath) === false)
				throw new Exc('folder_rename', $srcPath . '=>' . $dstPath);
		}
		return $nuid;
	}

	/**
	 * @brief Delete items from the LocalPOP folder
	 * @param $oFolder Name of the source mailbox.
	 * @param $to Name of the destination mailbox.
	 */
	public function deleteItems($oFolder, $oItems = false)
	{
		if ($oItems)
			foreach ($oItems as $oItem)
				$this->deleteMessage($oFolder->name, $oItem->rid);
			else
			{

				$this->emptyMailbox($oFolder->name);
			}
	}
	/**
	 * @brief Move items from the LocalPOP folder to another one.
	 * @param $source Name of the source mailbox.
	 * @param $dest Name of the destination mailbox.
	 * @param 	 
	 * @param $copy - boolean flag for copying instead of moving    
	 */
	public function moveItems($oSource, $oDest, $oItems = false, $bCopy = false)
	{
		//Process copy/move bettween different accounts
		if ($oSource->account->accountID != $oDest->account->accountID)
		{

			//Check target type
			if ($oDest->itemClassName == 'POP3Item')
				throw new Exc($bCopy ? 'item_copy' : 'item_move');

			//Check target rights
			if (!($oDest->rights & Folder::RIGHT_INSERT || $oDest->rights & Folder::RIGHT_D))
				throw new Exc($bCopy ? 'item_copy_rights' : 'item_move_rights');

			if ($oItems === false)
				$oItems = $oSource->getItems();

			foreach ($oItems as $item)
			{
				$fileName = $item->getMessageFile();

				if (!file_exists($fileName))
					$item->autoCreateMessage($fileName);

				$newID = $oDest->createItem(false, $fileName, true);

				if ($oDest->itemClassName == 'IMAPItem')
				{
					$strFlags = IMAPItem::encodeFlags($item->flags);
					//FIXME ? If this is not done the new message is not SEEN...
					$strFlags = str_replace("\\Flagged", "", $strFlags);
					$targetIMAP = IMAP::instance($oDest->account);

					$targetIMAP->setFlags($oDest->name, $newID, $strFlags);
				}

				$aReturn[$item->rid] = $newID;
			}

			if (!$bCopy)
				$this->deleteItems($oSource, $oItems);
			//Process copy/move in the same account
		} else
		{
			if ($oItems)
				foreach ($oItems as $oItem)
					$aReturn[$oItem->rid] = $this->moveMessage($oSource->name, $oItem->rid, $oDest->name, $bCopy);
				else
				{
					$oItems = $this->getItemsID($oSource->name);
					foreach ($oItems as $oItem)
						$aReturn[$oItem->uid] = $this->moveMessage($oSource->name, $oItem->uid, $oDest->name, $bCopy);
				}
		}
		return $aReturn;
	}

	public static function encode($mailbox)
	{
		$mailbox = mb_convert_encoding($mailbox, 'UTF7-IMAP', 'UTF-8');
		if ($_SESSION['SPAM_REMAP'] && preg_match("/(^spam$|^spam\/)/i", $mailbox, $matches))
		{
			$mailbox = preg_replace('#spam#i', "~spam", $mailbox, 1);
		}
		if ($_SESSION['INBOX_REMAP'])
			$mailbox = preg_replace('#INBOX#i', $_SESSION['INBOX_FOLDER'], $mailbox, 1);
		return $mailbox;
	}

	public static function decode($mailbox)
	{
		$mailbox = mb_convert_encoding($mailbox, 'UTF-8', 'UTF7-IMAP');
		if ($_SESSION['SPAM_REMAP'] && preg_match("/(^~spam$|(~spam\/))/i", $mailbox, $matches))
		{
			$mailbox = preg_replace('#~spam#i', "Spam", $mailbox, 1);
		}
		if ($_SESSION['INBOX_REMAP'])
		{
			$mailbox = preg_replace('#' . $_SESSION['INBOX_FOLDER'] . '#', 'INBOX', $mailbox,
				1);
		}
		return $mailbox;
	}

	public function search($oFolder, $sPhrase, $aItems = false)
	{
		$mailbox = self::encode($oFolder->name);
		$sPath = $this->getFolderPath($mailbox);

		$search = Search::instance($sPath);
		$result = $search->search($sPhrase, 'tmp', false, $aItems);
		return $result;
	}

	public function getBody($mailbox, $itemID)
	{
		$sMailboxPath = $this->getFolderPath($this->encode($mailbox));
		return icewarp_get_message_content($sMailboxPath . '\/' . $itemID .localpopmessageext, '', DB_BODY_LIMIT, 0);
	}

	/**
	 * @brief Get a message.
	 * @param $mailbox Name of the mailbox.
	 * @param $uid UID of the message.
	 * @return Raw data of the message.
	 */
	public function saveMessageToFile($mailbox, $uid, $filename)
	{
		$dir = dirname($filename);

		if (!is_dir($dir))
			System::mkdir_r($dir);

		$source = $this->getItemPath($mailbox, $uid);

		$fs = fopen($source, 'rb');
		$fd = fopen($filename, 'wb+');

		$last = '';
		while ($line = fgets($fs))
		{
			if ($line[0] == '.' && $line[1] == '.')
				$line = substr($line, 1);

			fwrite($fd, $line);
		}

		fclose($fs);
		fclose($fd);

		return true;
	}
}

?>
