<?php
require_once ('./inc/storage/storage.php');
require_once ('./inc/api/api.php');
require_once ('./inc/tools.php');
/**
 * @brief		Class for handling webmailIqAuth namespace
 * @revision		01.03.2007
 * @author		Pavel Perna
 */
abstract class PrivateAndPublic extends XMLRequestInterface
{
	private $aPaths  = Array(
				'public'=>'xml/public/',
				'private'=>'xml/private/'
		);
		
	protected static $aNonMergingResources = array(0=>'shared_accounts');
	protected static $aGWResources = array(0=>'shared_accounts',1=>'gw_groups');
	protected static $aPublicNamespace = array('login_settings'=>1,
                                             'layout_settings'=>1,
                                             'restrictions'=>1,
                                             'forgot_settings'=>1,
                                             'languages'=>1,
                                             'signup_domains'=>1,
                                             'skins'=>1);
	/**
	* @brief	Private
	* @return  TRUE/throws exception
	*/
	protected function checkInputXML(){
		global $aError;
		if ($this->aAttrs['type']=='set' && !$this->aAttrs['sid'])
			throw new Exc('settings_access_denied');

      //Check if user have rights to change settings
      foreach($this->oResources as $oResource)
       $result[] = $oResource->tagName;

      if ((@$_SESSION['ACCOUNT']==0 || @$_SESSION['ACCOUNT']==2) && !$this->aAttrs['sid'])			  
        foreach($result as $res)
          if (!PrivateAndPublic::$aPublicNamespace[$res])
            throw new Exc('settings_access_denied');

		// Prefix private or public 
		$this->prefix = $this->sPP.":";
		
		if (!is_object($this->oResources->item(0)))
			throw new Exc('settings_missing_tag','resource');	
			
		switch(@$_SESSION['ACCOUNT']){
			case 1:
				$this->rights = 'admin';
				//Work with domain settings , while being logged as admin
				if ($oDOMDomain = $this->oDOMDoc->getNode($this->prefix."domain",$this->oDOMQuery)){
					$this->rights = 'domain';

					$this->domain = $oDOMDomain->nodeValue;

					$this->adminAsDomain = true;
				}
				break;
			case 2:
				$this->rights = 'domain';
				$this->domain = $_SESSION['DOMAIN'];
				break;
			case 0:
			default:
				$this->rights = 'user'; 
				
        $host=$_SERVER['HTTP_HOST'];
				if ($pos=strpos($host,":")) $host=substr($host,0,$pos);
				
				if (!@$_SESSION['DOMAIN']){
				  $filename = WM_HOSTSPATH.$host.dataext;
				    
        if (file_exists($filename))
			    @$this->domain = trim(file_get_contents($filename));
        else
          $this->domain = $host;
            
        } else
          $this->domain = @$_SESSION['DOMAIN'];

				break;
		}

	}

	protected function exeInputXML()
	{
		$aData['data'] = '';
		//Flag whether to write to settings file or not
		$fWrite = false; 

		if ($this->aAttrs['sid']){ 
		    @$oUser = &$_SESSION['user'];
		    $oAccount = &$oUser->getAccount(@$_SESSION['EMAIL']);
		    $oGWAccount = &$oAccount->gwAccount; 
		    
		    //Open group
		    if (!$oGWAccount->gwAPI->IsConnected())
		      $oGWAccount->sGWSessionID = $oGWAccount->gwAPI->OpenGroup("*");
		    
        if ($this->aAttrs['type']=='set' && $this->sPP=='public'){
            $default = Storage::getDefaults();
            $domain = Storage::getDomainDefaults(false,$this->domain);
            
            if ($this->adminAsDomain)
              $rights = '2';
            else 
              $rights = @$_SESSION['ACCOUNT']; 
            $return = $this->get($default,$domain,$user,$rights);

        }
        else
            $return = Storage::getUserData();  
		}
      
                			  
		switch ($this->aAttrs['type']) {
			case 'get': 
				$this->sTemplateFile = 'webmailiqprivateandpublic_get';

				  if(!file_exists(GLOBAL_SETTINGS_FILE)){
				    $basedir = dirname(GLOBAL_SETTINGS_FILE);

            if(!is_dir($basedir))
				      System::mkdir_r($basedir);
				      
            $fp = fopen(GLOBAL_SETTINGS_FILE,"w+");
            fwrite($fp,file_get_contents(GLOBAL_SETTINGS_DEFAULT));
            fclose($fp);
          } 
        $error = false;
				foreach($this->oResources as $oResource){
					$data = array();
					$sResourceName = $oResource->tagName;
					$sResourceNames[] = $sResourceName;
					$result = array();
					$result['resourcename'] = $sResourceName;
					switch($sResourceName){
						          //Well-known resources
						case 'gw_groups':
							if (!$oGWAccount->bLogged){ 
                $error = true;
							  break;
              }
              $result['items']['num'] = Storage::getGwGroups($oGWAccount);
							if ($this->sPP=='public') $result['resourcename'] = $sResourceName;
							$aData['data'] .= trim(template('inc/templates/groups.tpl',Tools::htmlspecialchars_array($result)));
							break;
						case 'gw_friends':
							if (!$oGWAccount->bLogged){ 
                $error = true;
							  break;
              }
              $result['items']['num'] = Storage::getGwFriends($oGWAccount);
							if ($this->sPP=='public') $result['resourcename'] = $sResourceName;
							$aData['data'] .= trim(template('inc/templates/friends.tpl',Tools::htmlspecialchars_array($result)));
							break;
						case 'gw_mygroup':
							if (!$oGWAccount->bLogged){ 
                $error = true;
							  break;
              }
              $aSubscribed = Storage::getMyGroups($oGWAccount);                       
							$result['items']['num'][0] = @reset($aSubscribed); 
							if ($this->sPP=='public') $result = array();
							$aData['data'] .= trim(template('inc/templates/groups.tpl',Tools::htmlspecialchars_array($result)));
							break;
						case 'gw_owner':
							if (!$oGWAccount->bLogged){ 
                $error = true;
							  break;
              }
              $result['items']['num'][0] = Storage::getOwnerInfo($oGWAccount);
							if ($this->sPP=='public') $result['resourcename'] = $sResourceName;
							$aData['data'] .= trim(template('inc/templates/owner.tpl',Tools::htmlspecialchars_array($result)));
							break;
						case 'shared_accounts':
							if (!$oGWAccount->bLogged){ 
                $error = true;
							  break;
              }
							if ($this->sPP!='public')
							  $aData['data'] .= Storage::getSharedAccounts($oGWAccount);
							else
							  $aData['data'] .= trim(template('inc/templates/shared_accounts.tpl',array())); 
						break;
						case 'autoresponder':
						case 'forwarder':
						case 'filter_rules':
						case 'antispam':
							$result['items']['num'][0] = Storage::getUserProperties($sResourceName);
							if ($this->sPP=='public') $result = array();
							$aData['data'] .= trim(template('inc/templates/account_properties.tpl',$result));
							break;
						case 'holidays':
						  if (!$oGWAccount->bLogged){ 
                $error = true;
							  break;
              }
						  $result['holidays']['num'] = Storage::getAvailableHolidays($oGWAccount);
							if ($this->sPP=='public') $result['resourcename'] = $sResourceName;
							$aData['data'] .= trim(template('inc/templates/holidays.tpl',Tools::htmlspecialchars_array($result)));
					  break;
						case 'languages':
							$result['items']['num'][0]['language']['num'] = Storage::getAvailableLanguages();
							if ($this->sPP=='public') $result['resourcename'] = $sResourceName;
							$aData['data'] .= trim(template('inc/templates/languages.tpl',Tools::htmlspecialchars_array($result)));
							break;
						case 'skins':
							$result = Storage::getAvailableSkins();
							$aData['data'] .= trim($result);
							break;
						case 'certificate':
							$result['items']['num'][0] = Storage::getCertificates(true);
							if ($this->sPP=='public') $result['resourcename'] = $sResourceName;
							$aData['data'] .= trim(template('inc/templates/certificate.tpl',Tools::htmlspecialchars_array($result)));
							break;
						case 'cookie_settings':
							$cookie_settings = Storage::getUserData($sResourceName);
							if ($this->sPP=='public') $cookie_settings = '';
							$cookie_settings = /*Tools::htmlspecialchars_array(*/$cookie_settings/*)*/;
							$data = Tools::makeXMLStringFromTree($cookie_settings,'cookie_settings');
							$aData['data'] .= $data; 
							break;
						case 'domains_settings':
							$domains['items']['num'] = Storage::getAvailableDomains($sResourceName);
							$domains['resourcename'] = 'domains_settings';
							$data = trim(template('inc/templates/available_domains.tpl',Tools::htmlspecialchars_array($domains)));
							$aData['data'] .= $data; 
							break;
						case 'signup_domains':
							$domains['items']['num'] = Storage::getSignupDomains($sResourceName);
							$domains['resourcename'] = 'signup_domains';
							$data = trim(template('inc/templates/available_domains.tpl',Tools::htmlspecialchars_array($domains)));
							$aData['data'] .= $data; 
						break;
						case 'global_settings':
              $global_settings = Storage::getGlobalSettings();
						  $aData['data'] .= $global_settings;
            break;
						case 'login_settings':
						  $login_settings = Storage::getLoginSettings();
              $aData['data'] .= Tools::makeXMLStringFromTree($login_settings,$sResourceName); 
						break;
						case 'spellchecker_languages':
						  $aData['data'] .= Storage::getSpellcheckerLanguages($sResourceName);
						break;
						case 'sip_calls':
						  $sip_calls['sip_calls']['num'] = Storage::getSIPCallsHistory();

							$sip_calls['resourcename'] = 'sip_calls';
							$data = trim(template('inc/templates/sip_calls.tpl',Tools::htmlspecialchars_array($sip_calls)));
						  $aData['data'] .= $data;
						break;
						case 'settings':
						//Custom resources
						default:    
							$user = Storage::getUserData($sResourceName); 
							$domain = Storage::getDomainDefaults($sResourceName,$this->domain);
							$default = Storage::getDefaults($sResourceName);

							if ($this->adminAsDomain)
								$rights = 2;
							else
								$rights = @$_SESSION['ACCOUNT'];
								
							$data = $this->get($default,$domain,$user,$rights);

              if($sResourceName=='imip_settings' || $sResourceName=='mail_settings_default'){
                self::setDefaults($data,$sResourceName);
              }


              $aData['data'] .= Tools::makeXMLStringFromTree($data,$sResourceName);                                
							break; 
					}
				}		 
				if ($error) LogError::error('Groupware initialization failed');
				$this->aData = $aData;
				break;

			case 'set':	
				//Process changes
				foreach($this->oResources as $oResource){
					$sResourceName = $oResource->tagName;
					$sResourceNames[] = $sResourceName;
					$aActions = $this->getSentData($sResourceName);
					switch($sResourceName){
						//Well-known resources
						case 'gw_groups':
							Storage::processGroups($oGWAccount,$aActions);
							break;
						case 'gw_mygroup':
						  $group = $aActions[0]['data'];
						  unset($group['rights']);
							Storage::editGroup($oGWAccount,$group,$group['grp_id']);
							break;
						case 'gw_friends':
							Storage::processFriends($oGWAccount,$aActions);
							break;
						case 'gw_owner':  
							Storage::setOwnerInfo($oGWAccount,$aActions[0]['data']);
							break;
						case 'autoresponder':
						case 'forwarder':
						case 'filter_rules':
						case 'antispam':
							Storage::setUserProperties($sResourceName,$aActions[0]['data']);
							break;
						case 'shared_accounts':
							Storage::setSharedAccounts($oGWAccount,$aActions);
							break;
						case 'certificate':
							// store passphrase to default settings XML
							$fWrite = true;
							$settings[$sResourceName] = Storage::setCertificates($aActions[0]['data']);
							break;
						case 'cookie_settings':
							//$fWrite = true;
							Storage::setUserDataStr(Tools::makeXMLStringFromTree($this->set($sResourceName,$aActions),'cookie_settings'),$sResourceName);
							break;
						//Custom resources
						case 'domains_settings':
							Storage::removeAvailableDomains($aActions);
							break;
						case 'global_settings':
						  if($this->rights!='admin' || $this->sPP!='public') 
                throw new Exc('settings_access_denied');
						  
              if($aActions[0]['dataTree']['@childnodes']['database']){
						    $val = Tools::unhtmlspecialchars($aActions[0]['dataTree']['@childnodes']['database'][0]['@value']);
                $aActions[0]['data']['database'] = $val;
                $aActions[0]['dataTree']['@childnodes']['database'][0]['@value'] = $val;
              }

							$data = $this->set($sResourceName,$aActions);

							Storage::setGlobalSettings(Tools::makeXMLStringFromTree($data,$sResourceName));
							break;
						case 'spellchecker_languages':
						  break;
						case 'holidays':
						  Storage::subscribeHolidays($oGWAccount,$aActions);
            break;
            case 'personalities':
              $fWrite = true;
              $settings[$sResourceName]=$this->set($sResourceName,$aActions,false,false,true);
						break;
            case 'imip_settings':

              foreach($aActions[0]['data'] as $skey => $sval)
                $aActions[0]['data'][$skey] = htmlspecialchars($sval);
              
              	$fWrite = true;
						    $settings[$sResourceName]=$this->set($sResourceName,$aActions,$this->adminAsDomain,$this->domain);

            break;
						case 'login_settings':
						  //Do not set logging_type (set in global settings)
						  unset($aActions[0]['data']['logging_type']);
						  unset($aActions[0]['dataTree']['@childnodes']['logging_type']);

            case 'settings':
						default:
						      $fWrite = true;
						      $settings[$sResourceName]=$this->set($sResourceName,$aActions,$this->adminAsDomain,$this->domain);
						      break;
					}                    
				}

       /* if (($sNewCharset = $settings['mail_settings_default']['@childnodes']['item'][0]['@childnodes']['charset'][0]['@value'])
        && $this->sPP == 'private')
          $_SESSION['DEFAULTCHARSET'] = $sNewCharset;*/

        $this->updateSettings($settings);

				//Write data to settings file
				if (is_array($settings) && $fWrite){
					foreach($settings as $key => $value)
						$return['@childnodes'][$key][0] = $value;

					$return = Tools::makeXMLStringFromTree($return,'settings');    
					switch($this->rights){
						case 'user':
							Storage::setUserDataStr($return,$sResourceName);
							break;
						case 'domain':
							if ($this->sPP=='public')
								Storage::setDomainDefaultsStr($return,$sResourceName,$this->domain);
							else
								Storage::setUserDataStr($return,$sResourceName);
							break;
						case 'admin':
							if ($this->sPP=='public')
								Storage::setDefaultsStr($return,$sResourceName); 
							else
              	Storage::setUserDataStr($return,$sResourceName);
              break;
					}  						    
				}		
		 		//Do we have to close the group ?
		    $fCloseGroup = false;
        if ($sResourceNames) foreach($sResourceNames as $sName)
    			if (in_array($sName,self::$aGWResources))
    				$fCloseGroup = true;
    				
    		//Close group if logged user
    			if (($this->aAttrs['sid'] && $fCloseGroup) || $this->adminAsDomain){
             $oGWAccount->gwAPI->CloseGroup($oGWAccount->sGWSessionID);       			
			       $oGWAccount->gwAPI->cleanup();
          }       

      break;
		}
	}
		
	public function updateSettings($aData)
  {
    $aUpdate = array(array('DEFAULTCHARSET','mail_settings_default','charset'),
                    array('IMIP_SUBJECT_INVITE','imip_settings','imip_subject_invite'),
                    array('IMIP_SUBJECT_UPDATE','imip_settings','imip_subject_update'),
                    array('IMIP_SUBJECT_CANCEL','imip_settings','imip_subject_cancel'),
                    array('IMIP_SUBJECT_ACCEPT','imip_settings','imip_subject_accept'),
                    array('IMIP_SUBJECT_DECLINE','imip_settings','imip_subject_decline'));
                    
    foreach($aUpdate as $aUp){
      if($aData[$aUp[1]]['@childnodes']['item'][0]['@childnodes'][$aUp[2]][0]['@value']
        && (!$aUp[3] || $this->sPP == $aUp[3]) ){
        @$_SESSION[$aUp[0]] = $aData[$aUp[1]]['@childnodes']['item'][0]['@childnodes'][$aUp[2]][0]['@value'];
      }
      if(isset($aData[$aUp[1]]['@childnodes']['item'][0]['@childnodes'][$aUp[2]][0])
          && !isset($aData[$aUp[1]]['@childnodes']['item'][0]['@childnodes'][$aUp[2]][0]['@value']))
        @$_SESSION[$aUp[0]] = '';
    }
              
    @$_SESSION['user']->imipDefaults();
 
    return true;
  }	
		
	public function getSentData($sResourceName)
	{
		$result = array();
		$oDOMResource = $this->oDOMDoc->query($this->sPP.':resources/'.$this->sPP.':'.$sResourceName.'/'.$this->sPP.':item',$this->oDOMQuery);
		foreach($oDOMResource as $oDOMItem){

			$aItem['data'] = Tools::makeArrayFromXML($oDOMItem);
			$aItem['dataTree'] = Tools::makeTreeFromXML($oDOMItem); 
			//Id is set - work with existing item
			if ($aItem['uid'] = $oDOMItem->getAttribute("uid")){
				//There is some content => EDIT
				if ($oDOMItem->hasChildNodes())
					$aItem['action'] = 'edit';
				else
					$aItem['action'] = 'delete';
			} else {
				$aItem['action'] = 'add';
			} 
			$aResult[] = $aItem;
		}
		return $aResult;	          
    }
    
	public static function removeAttrs($data, $account)
	{

		if($data['@childnodes']['item']) foreach($data['@childnodes']['item'] as $itemID => $itemVal)
			foreach($itemVal['@childnodes'] as $key => $val){
				if($rights = $val[0]['@attributes'][Storage::$rightType[$account]])
					$itm['@attributes']['access'] = $rights;

				if ($itm['@attributes']['access']!='none')
					$itm['@value'] = $val[0]['@value'];
	
				$data['@childnodes']['item'][0]['@childnodes'][$key][0] = $itm;
			}
		
		return $data;
	}
    
  private static function setDefaults(&$aData,$sResourceName)
  {
    $aUpdate = array(array('DEFAULTCHARSET','mail_settings_default','charset'),
                    array('IMIP_SUBJECT_INVITE','imip_settings','imip_subject_invite'),
                    array('IMIP_SUBJECT_UPDATE','imip_settings','imip_subject_update'),
                    array('IMIP_SUBJECT_CANCEL','imip_settings','imip_subject_cancel'),
                    array('IMIP_SUBJECT_ACCEPT','imip_settings','imip_subject_accept'),
                    array('IMIP_SUBJECT_DECLINE','imip_settings','imip_subject_decline'));
    
    foreach($aUpdate as $aUp){
      if(!$aData['@childnodes']['item'][0]['@childnodes'][$aUp[2]][0]['@value'] && $sResourceName==$aUp[1]){
        $aData['@childnodes']['item'][0]['@childnodes'][$aUp[2]][0]['@value'] = @$_SESSION[$aUp[0]];
      }
    }      
    
    return true;
    
  }   

	abstract public static function set($sResourceName,$aActions);
	abstract public static function get($default,$domain,$user,$rights = false);
}


?>
