<?php

  class RSSFolder extends CacheFolder{
  
    public function __construct(&$account, $folderID, $name){
      parent::__construct(&$account, $folderID, $name, 'rwmd', false);
      $this->itemClassName = 'RSSItem';
      $this->type = 'R';
      
      $this->channel = $this->getChannelURL($name,&$this->channelName);
    }
    
	/**
	  @brief Create a new folder.
	  @param $account Parent account object of the folder.
	  @param $name Name of the folder.
	  @return RSSFolder object.
	 */
	public static function create($account, $name, $channel)
	{
    /* Create the folder in the RSS setting file. */
		$rss = RSS::instance($account);
		$rss->createFolder($name, $account->accountID, $channel);

		/* Create the folder in the folder cache. */
		$folderID = parent::create($account, $name, 255);

		return new RSSFolder($account, $folderID, $name);
	}    
	
	/**
	  @brief Get channel
	  @param $name Name of the folder.
	  @return RSSFolder object.
	 */
	public function getChannelURL($name,&$sChannelName)
	{
    /* Get the folder URL from the RSS setting file. */
		$rss = RSS::instance($account);

		return $rss->getChannelURL($name,$sChannelName);
	}  
	
	public function getItems($aFilterTag = array()){
		$filter = $aFilterTag['sql'];
		$orderBy = $aFilterTag['orderby'];
		$offset = $aFilterTag['offset'];
		$limit = $aFilterTag['limit'];
		$fields = $aFilterTag['tag'];
    $search = $aFilterTag['fulltext'];

		// always get RID attribute of message
		if ($fields && !strstr($fields, 'rid'))
			$fields .= ',rid';
		return parent::getItems('RSSItem', $filter, $orderBy, $offset, $limit,$fields,$search);
  }

	/**
	  @brief Get an item.
	  @param $itemID ID of the item.
	  @return IMAPItem object.
	 */
	public function getItem($itemID, $cache = array())
	{

	  if (!$cache['cache']) $cache = false;
		return parent::getItem('RSSItem', $itemID, $cache);
	}
	
	public function markItems($flag)
	{
    parent::markItems($flag);

  }
  
  public function unmarkItems($flag)
  {
    parent::unmarkItems($flag);   
  }
  
  public function edit($sName,$sChannel,$bChangeChannel = false)
  {
    $account = &$this->account;
    $rss = RSS::instance($account);
    
    $rss->editFolder($this->name,$sName,$sChannel,$bChangeChannel);
    
    if($sName)
      $this->name = $sName;

    if($sChannel)
      $this->channel = $sChannel;
      
    $this->account->aSyncedFolders[$this->name] = false;
      
    if(!$sName) $sName = $this->name;  
    
    parent::rename($sName);
    
    return true;
  }
  
	public function delete()
  {
    $account = &$this->account;
    $rss = RSS::instance($account);

    $rss->deleteFolder($this->name);

    parent::delete(); 
    
    unset($_SESSION['user']->aAccounts[$account->accountID]->folders["main"][$this->name]);

    //Delete the rss file
    if(count($account->folders['main'])==0){
      @unlink($_SESSION['USERDIR'].SETTINGS_FOLDER . rssfile);
      unset($_SESSION['user']->aAccounts[$account->accountID]);
    }
    
    return true;
  }
  
	public function deleteItems($oItems = false)
  {
    if(!$oItems) $oItems = $this->getItems(); 
    if($oItems){ 
      foreach($oItems as $oItem)
        $oItem->delete();
      return true;
    }
  }
  
  /**
	  @brief Synchronize the item cache with the IMAP server.
	 */
	public function sync()
	{
    if($this->channel){
  		$account = $this->account;
      
      //Create instances of rss and cache classes
  		$rss = RSS::instance($account);
      $aItemsID = $rss->getItemsID($this);
  
      parent::sync($this->itemClassName,$aItemsID,$this);
	  }
  }
      
}

?>
