<?php
require_once ('inc/cache/item.php');

class RSSItem extends CacheItem
{
	/**
	  @brief Create a new IMAP item object.
	  @param $folder Parent folder object of the item.
	  @param $itemID ID of the item.
	  @param $rid Remote ID of the item.
	  @param $size Size of the item.
	  @param $date Date of the item.
	  @param $flags Flags of the item.
	  @param $from From header of the item.
	  @param $to To header of the item.
	  @param $subject Subject of the item.
	  @param $staticFlags Static flags of item (cannot be set by webmail client).
	  @param $priority Priority of the item.
	  @param $color Color of the item.
	  @param $sMimeStatus S-Mime status of the item (item is signed/verified).
	  @param $hasAttachments Item has attachments.
	 */
	public function __construct($folder, $itemID, $rid,$size, $date, $flags, 
								$from, $to, $subject, $staticFlags, $priority, $color,
								$sMimeStatus, $hasAttachments, $sBody = '')
	{
		
  		parent::__construct($folder, $itemID, $rid, $size, $date, $flags, 
							$from, $to, $subject, $staticFlags, $priority, $color,
							$sMimeStatus, $hasAttachments);	
      $this->wmclass = 'R';
      $this->body = $sBody;
  }

	/**
	  @brief Decode an item from IMAP format.
	  @param[in,out] $item The IMAP item.
	  @param $staticFlags Value of the static flags
	  */
	public static function decode(&$item)
	{
	 /*$item->from = MailParse::decodeMimeHeader($item->from);
		$item->to = MailParse::decodeMimeHeader($item->to);
		$item->subject = MailParse::decodeMimeHeader($item->subject);*/

    $item->color   = 'Z';
 
		$item->priority  = isset($item->priority) ? intval($item->priority) : Item::NORMAL_PRIORITY;
		$item->staticFlags = isset($staticFlags['flags']) ? $staticFlags['flags'] : 0;
		
    $item->size = 0;
    $item->flags = 0;
    $item->sMimeStatus = 0;
    $item->date = strtotime($item->date);
		if(!$item->date) $item->date = time();

  } 
	
	/**
	  @brief Delete the item.
	 */
	public function delete($cache = array())
	{
		$folder  = &$this->folder;
		$account = &$folder->account;

		/* Delete the item from the IMAP server. */
    //$rss = RSS::instance($account);
		//$rss->deleteMessage($folder->name, $this->rid);

		/* Delete the item from the item cache. */
		parent::setFlags(Item::FLAG_DELETED,'flags');
	}
	
  /**
	  @brief Set specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	public function markAsRead()
	{
	   parent::setFlags(32,'flags');
  }
  
  /**
	  @brief Set specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	public function markAsUnread()
	{
	   parent::setFlags(0,'flags');
  }
    	
	/**
	  @brief Set specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	public function setFlags($flags, $field = 'flags')
	{
	 
		/* Set flags on the cached item. */
		parent::setFlags($flags, $field);
  }
	/**
	  @brief Unset specified flags on the item.
	  @param $flags The flags.
	  @param $field Identificator of field where flags are stored.
	 */
	protected function unsetFlags($flags, $field = 'flags')
	{
		/* Unset flags on the cached item. */
		parent::unsetFlags($flags, $field);
	}
	
	/**
	  @brief Get the message of the item.
	  @return The message.
	 */
	public function autoCreateMessage($filename)
	{
		$folder  = $this->folder;
		$account = $folder->account;

    /* Check cached message. */
		if (parent::getMessage() === false) {

			/* Get IMAP message. */
			$rss = RSS::instance($account);
			
			$dir = dirname($filename);
			
			if(!is_dir($dir))
			 System::mkdir_r($dir);

      //Cache the message
			$rss->saveMessageToFile($folder->name, $this->rid, $filename);

      $fp = fopen($filename,'ab+');
      
      fseek($fp,-4,SEEK_END);
      $end = fread($fp,4);
      
      //No CRLF at the end of message
      if ($end!=CRLF.CRLF)
        fwrite($fp,CRLF.CRLF);
      
      fclose($fp);

			$this->sync();
		} else {
			// mark message as read (unleass it is read already)
			$this->setFlags(Item::FLAG_SEEN);
		}
		return true;
	}
	
  public function getHTML(&$aItem)
  {
      $aHTML['title'] = $this->subject;
      $aHTML['href'] = $this->to;
      $aHTML['description'] = Tools::cleanHTML($this->body,$base);
    
      $aURL = parse_url($this->to);
      $aItem['base'] = $aURL['host'];

      @$sHTML = htmlspecialchars(template('inc/templates/rss.tpl',$aHTML));

      return $sHTML;
  } 

}

?>
