<?php
require_once('inc/xml.php');
require_once('inc/tools.php');

class RSS
{
    private static $instance;
    
	/**
	  @brief Get instance of RSS class
	  For each account there is an only one instance of this class allowed.
	  @return Instance of IMAP class
	 */
	public static function instance($account) 
  {
    $index = $account->accountID;
		if (!isset(self::$instance[$index])) {
        $rss = __CLASS__;
        self::$instance[$index] = new $rss();
    }

    return self::$instance[$index];
  }
  
  private function __construct($sEncoding = "UTF-8"){
    $this->sEncoding = $sEncoding;

    $this->rssDoc = XML::open($_SESSION['USERDIR'].SETTINGS_FOLDER . rssfile);

  } 
  
  public function createFolder($sName,$sType,$sChannel){

  	$this->rssDoc = XML::open($_SESSION['USERDIR'].SETTINGS_FOLDER . rssfile);
    if (!is_object($this->rssDoc)) {
			$this->rssDoc =  new XML($_SESSION['USERDIR'].SETTINGS_FOLDER . rssfile);
			$rssNode = new DOMElement('rss');
			$this->rssDoc->appendChild($rssNode);
		} else
			$rssNode = $this->rssDoc->getNode('/rss');	
		
		//$sChannel = htmlspecialchars($sChannel);
		
    if(@$this->rssDoc->query("/rss/folder[@name='".$sName."'"))
      throw new Exc('folder_already_exists',$sName);
     
    if($sChannel){    
    $sChannel = str_replace(array('rss://','feed://'),'http://',$sChannel);
        
      if(!@$fp = fopen($sChannel,'rb'))
          throw new Exc('rss_invalid_channel',$sChannel);
    }
      
    /* Append the folder to the configuration file. */
		$folderNode = $rssNode->appendChild(new DOMElement('folder'));
		$folderNode->setAttribute('name',$sName);
		$folderNode->nodeValue = htmlspecialchars($sChannel);
    
    $this->rssDoc->save();
  }
  
  public function editFolder($sOldName,$sName,&$sChannel,$bChannelChange = false){

    $this->rssDoc = XML::open($_SESSION['USERDIR'].SETTINGS_FOLDER . rssfile);
//  	if (!is_object($this->rssDoc)) 
//			$this->rssDoc = XML::open($_SESSION['USERDIR'].SETTINGS_FOLDER . rssfile);

    //Edited node
    foreach($this->rssDoc->query("/rss/folder[@name='".$sOldName."']") as $channelNode)
      $result = $channelNode;

    if(!$result)
      throw new Exc('folder_does_not_exist',$sOldName);
  
    Tools::checkFolderMove($sOldName,$sName);
    
    // Edit the folder in the configuration file. 
		if($sName){
      $result->setAttribute('name',$sName);
    
      // Edit Sub-nodes
      foreach($this->rssDoc->query("/rss/folder") as $channelNode)
      {
    
        if($sFolder = $channelNode->getAttribute('name')){
          if(strpos($sFolder,$sOldName.'/')!==false){
              $name = preg_replace("#$sOldName#",$sName,$sFolder,1);
              $channelNode->setAttribute('name',$name);
          }
        } 
      }
    }

    //Change channel
    if($bChannelChange){

		  if($sChannel){
        $sChannel = str_replace(array('rss://','feed://'),'http://',$sChannel);
      }

      if(@$fp = fopen($sChannel,"rb")) @fclose($fp);
        else if($sChannel) throw new Exc('rss_invalid_channel',$sChannel);

      $result->nodeValue = htmlspecialchars($sChannel);
    }

    $this->rssDoc->save();
  }

  public function deleteFolder($sName){
    $file = $_SESSION['USERDIR'].SETTINGS_FOLDER . rssfile;
    if (!is_object($this->rssDoc)) {
			$this->rssDoc = XML::open($file);
			$rssNode = $this->rssDoc->appendChild(new DOMElement('rss'));
		} else
			$rssNode = $this->rssDoc->getNode('/rss');
    
    /* Remove the folder from the configuration file. */    
    foreach($this->rssDoc->query("/rss/folder") as $channelNode){
      $sFolderName = $channelNode->getAttribute("name");
      if(strpos($sFolderName,$sName.'/')!==false || $sFolderName == $sName){
        $rssNode->removeChild($channelNode);
        $result = true;
      }
    }

    if(!$result)
      throw new Exc('folder_does_not_exist',$sName);
  
    $this->rssDoc->save();

  }
      
  public function getChannels()
  {
    $oUser = &$_SESSION['user'];  	
    $rssDoc =  XML::open($_SESSION['USERDIR'].SETTINGS_FOLDER . rssfile);
    if (!is_object($rssDoc)) {
      return array();
    }
    foreach ($rssDoc->query('/rss/folder') as $channelNode){
      $channel = false;
      $channel->name = $channelNode->getAttribute("name");
      $channel->channel = $channelNode->nodeValue;
      $channel->rights = 255;     
      $aResult[] = $channel;
    }
    
    return $aResult;
  } 
   
  
  public function getChannelURL($name)
  {
    $rssDoc =  $this->rssDoc;
    
    if (!is_object($rssDoc)) {
      return false;
    } 

    foreach($rssDoc->query("/rss/folder[@name='".$name."']") as $channelNode)
      return $channelNode->nodeValue;
    
    return false;
  } 
  
  public function getItemsID(&$folder)
  {
		$aResult  = array();
    if(!@$fp = fopen($folder->channel,'r'))
      throw new Exc('rss_invalid_channel',$folder->channel);

    while($buffer = @fread($fp,65535))
      $xml.=$buffer;
    @fclose($fp);

    $test = @simplexml_load_string($xml);

    if(!$test)
      $test = Tools::fixXML($xml);

    
    if(!$test)
      throw new Exc('rss_invalid_channel',$folder->channel);

    if($test){

      //Parse RSS
      if($test->channel){
          
          $folder->channelName = strval($test->channel->title);
          
          if($test->channel->item) 
            $itmPointer = &$test->channel->item;
          if($test->item)
            $itmPointer = &$test->item;
            
          
          
          foreach($itmPointer as $item){
            $itm = false;

            if($item->guid)
              $itm->uid = strval($item->guid);
            else
              $itm->uid = md5(trim(strval($item->link)) . trim(strval($item->title)));
               
						$itm->subject = str_replace("&nbsp;"," ",Tools::unhtmlspecialchars(strval($item->title)));
            $itm->subject = Tools::decodeASCII($itm->subject);
	
						$itm->from = htmlspecialchars(strval($folder->channelName));
            $itm->date = strval($item->pubDate);
            $itm->to = strval($item->link);
            $itm->body = strval($item->description);
            

						
            $aResult[$itm->uid] = $itm;
          }

      } else {

          $folder->channelName = strval($test->title);
          $url = str_replace($test->link["href"],"",$folder->channel);

          foreach($test->entry as $item){
            $itm = false;

            if($item->id){
              $itm->uid = strval($item->id);
              $sTo = $itm->uid;
            }else
              $itm->uid = md5(trim(strval($item->link)) . trim(strval($item->title)));
          
            $itm->subject = Tools::unhtmlspecialchars(strval($item->title));
            $itm->subject = Tools::decodeASCII($itm->subject);
            
            $itm->from = htmlspecialchars($folder->channelName);
            $itm->date = strval($item->updated);
            
            foreach($item->link as $link)
              $aLinks[strtolower(strval($link["rel"]))] = strval($link["href"]);
            
            if($aLinks['alternate'])
              $itm->to = $aLinks['alternate'];
            else if($aLinks['self'])
              $itm->to = strval($aLinks['self']);

            if(strpos($itm->to,'http://')===false){
              if($aLinks['via']){
                if(strpos($aLinks['via'],'http://')!==false)
                  $itm->to = $aLinks['via'];
                else
                  $itm->to = $url.$aLinks['alternate'];
                
              }else 
                $itm->to = $sTo;
            }


            $itm->body = Tools::unhtmlspecialchars(strval($item->content->asXML()));
            if(!$itm->body)
              $itm->body = Tools::unhtmlspecialchars(strval($item->summary));
            
            $aResult[$itm->uid] = $itm;
          
          }
          
                            
      }
      
      
      
    }

    return $aResult;
  } 


  
  public function drop(){}
}

?>
