<?

require_once('inc/tools.php');

/**
 * @brief		Class for s/mime message handling
 */
class SMime
{

  //public static function processMessage($message, $recipient, $sign, $encrypt)
	public static function processMessage(&$fp,&$filename, $recipient, $sign, $encrypt)
	{
		//$plainfile = self::convertMessageToPlainfile($message);
		//$headerarray = self::getOpenSSLheaderarray($filename);
		$plainfile = $filename;
		$signHeaders = self::getOpenSSLheaderarray($filename);
		$encryptHeaders = self::getOpenSSLheaderarray($filename,'encrypt');
		
		
		
		// Sign with the sender's private key
		if ($sign)
		{
			$cert = self::getUserCertificate();
			$encodedfile = self::randomFilename();
						
			if ($cert) {
				$crt = $cert['data'];
				$passphrase = $cert['passphrase'];
				$pubkey = substr($crt,0,strpos($crt,'-----END CERTIFICATE-----')+strlen('-----END CERTIFICATE-----'));
				$privkey = str_replace($pubkey,'',$crt);
				
				
				if ($passphrase) 
					$pkey = openssl_get_privatekey($privkey,$passphrase);
				else 
					$pkey = openssl_get_privatekey($privkey,'');
				if ($pkey)
					if (openssl_pkcs7_sign($plainfile, $encodedfile, $pubkey, $pkey, $signHeaders))
					{
						$fp = null;
						@unlink($plainfile);
						chmod($encodedfile,0755);
						$plainfile = $encodedfile;
					} 
					else 
						unlink($encodedfile);
			}
		}

		// Encrypt with the recipients public key
		if ($encrypt)
		{
			$encodedfile = self::randomFilename();
			$key = self::getRecipientCertificate($recipient);
			$pkey = openssl_get_publickey( $key );
			if ($pkey) 
				if (openssl_pkcs7_encrypt($plainfile, $encodedfile, $key, $encryptHeaders))
				{
					$fp = null;
					unlink($plainfile);
					chmod($encodedfile,0755);
					$plainfile = $encodedfile;
				} 
				else 
					unlink($encodedfile);
		}
    	$filename = $plainfile;
	}

	public static function decodeMessage(&$certmsg)
	{
		// Save To File
		$encodedfile = self::randomFilename();
		$plainfile = self::randomFilename();

		if (@$fd = fopen($encodedfile, "wb"))
		{
			fwrite($fd, $certmsg, strlen($certmsg));
			@fclose($fd);
		}

		// Get Private Key
		$cert = self::getUserCertificate();
		$crt = $cert['data'];
		$pubkey = substr($crt,0,strpos($crt,'-----END CERTIFICATE-----')+strlen('-----END CERTIFICATE-----'));
		$privkey = str_replace($pubkey,'',$crt);
		$passphrase = $cert['passphrase'];
		
		//Decrypt
		$pkey = openssl_get_privatekey( $privkey, $passphrase );
		if (openssl_pkcs7_decrypt( $encodedfile, $plainfile, $pubkey, $pkey )) {}
		@unlink($encodedfile);
		if (file_exists($plainfile)) {

			$result = false;
			if (filesize($plainfile) > 0) {

				// Load file back
				@$certmsg = file_get_contents($plainfile);
				$certmsg = Tools::cr2crlf($certmsg);
				$certmsg = Tools::dotConvert($certmsg);
				if (substr($certmsg,strlen($certmsg)-4,4)!=CRLF.CRLF)
          			$certmsg.=CRLF.CRLF;
          		$pom = $certmsg;
			//	self::decodeMessage($pom);
		//		print_r($pom);die();
				$result = true;
			}

			// Delete file
			@unlink($plainfile);
			return $result;
		}
		
		return false;
	}
	
	public static function verify(&$filename)
  {
    /*$filename = self::randomFilename();
    if (!$message || !@file_put_contents($filename, $message))
    	return false;*/
    if (openssl_pkcs7_verify($filename, PKCS7_NOVERIFY) === true) 
			$result = true;
		else
			$result = false;

    return true;
  } 

	public static function getCertificateInfo($key = '')
	{
		$result = '';

		// Get user's certificate
		if (!$key)
		{
			$cert = self::getUserCertificate();
			$key = $cert['data'];
		}

		if ($key)
		{
			 // Get key info
			@$array = openssl_x509_parse($key);

			// Parse cert array
			if (is_array($array))
			{
				$result .= $array['name'] . "\r\n\r\n";

				$result .= $array['subject']['CN'] . "\r\n";
				$result .= $array['subject']['emailAddress'] . "\r\n\r\n";

				$result .= $array['issuer']['C'] . "\r\n";
				$result .= $array['issuer']['ST'] . "\r\n";
				$result .= $array['issuer']['L'] . "\r\n";
				$result .= $array['issuer']['O'] . "\r\n";
				$result .= $array['issuer']['OU'] . "\r\n";
				$result .= $array['issuer']['CN'] . "\r\n";
				$result .= $array['issuer']['emailAddress'] . "\r\n\r\n";

				$result .= $array['validFrom'] . "\r\n";
				$result .= $array['validTo'] . "\r\n\r\n";
			}
		}

		return trim($result);
	}

	private static function getRecipientCertificate($email)
	{
		require_once('inc/gw/gwmanage.php');
		
		$gwMGMT = new GroupWareManagement();
		return $gwMGMT->getEmailCertificate($email);
	}

	private static function getUserCertificate()
	{
		require_once('inc/storage/storage.php');
		
		return Storage::getCertificates();
	}
	
	private static function randomFilename() {
		return Tools::randomFilename();
	}

	private static function convertMessageToPlainfile($message)
	{
		$filename = self::randomFilename();
		$message = str_replace("\x0D\x0A.", "\x0D\x0A", $message);

		if (@$file = fopen($filename, "wb"))
		{
			fwrite($file, $message, strlen($message));
			fclose($file);
		}
		else
			return false;

		return $filename;
	}

	private static function getOpenSSLheaderarray($filename,$type = 'sign')
	{
		$parser = &new MailParse($filename);
		$headers = $parser->getHeaders(true);
		if($type == 'encrypt'){
			$sslHeaders = array("Subject", "From", "Reply-To", "To", "Date", 
													"Return-Receipt", "Message-ID");
			$result = array();
			
			// add all well known headers
			foreach ($sslHeaders as $item) {
				if (isset($headers[strtolower($item)]))
					$result[$item] = $headers[strtolower($item)];
			}
			
			// add all X-* headers
			foreach ($headers as $key => $value) {
				if (substr($key, 0, 2) == 'x-')
					$result['X-'.ucwords(substr($key, 2))] = $value;
			}
		}else{
			$result = $headers;	
		}
		return $result;
	}

}
?>
