<?php
require_once('inc/crypt/rsa.php');
require_once('inc/xmlrequest.php');
require_once('inc/include.php');
require_once('inc/forgot.php');
require_once('inc/captcha.php');
/**
* @brief	Class for handling webmailIqAuth namespace
*/
class WebmailIqAuth extends XMLRequestInterface
{
	public $aSupportedMethods = Array(
		'rsa' => 1,
		'plain' => 1
	);
	/**
	* @brief	Public: Constructor
	* @param	aInputXML		Array that contains subrequest
	*		aAttrs			Array that contains IQ attributes
	*/
	public function __construct($oDOMQuery,&$oDOMDoc,&$attrs)
	{
		$this->oDOMQuery = $oDOMQuery;
		$this->oDOMDoc = &$oDOMDoc;
		$this->aAttrs = &$attrs;
		$this->checkInputXML();
		$this->exeInputXML();
	}
	/**
	* @brief	Private - Checks if the input XML is valid
	* @return	TRUE/FALSE and throws exception
	*/
	private function checkInputXML()
	{
		global $aError;
		// METHOD id mandatory for all
		if (!$this->oDOMMethod = $this->oDOMDoc->getNode("auth:method",$this->oDOMQuery)) {
			$this->sMethod = 'plain';
		} else {
			$this->sMethod = strtolower($this->oDOMDoc->getNodeValue("auth:method",$this->oDOMQuery));
		}
		if (!isset($this->aSupportedMethods[$this->sMethod])) {
			$this->sMethod = 'plain';
		}
		switch ($this->aAttrs["type"]) {
			case 'get':
				break;
			case 'set':
				//Forgot Password handling
				if ($oDOMForgot = $this->oDOMDoc->getNode("auth:forgot",$this->oDOMQuery)) {
					$this->forgot = true;
					//Mail to be used to identify user's account inforation
					$this->forgotEmail = $this->oDOMDoc->getNodeValue("auth:forgot",$this->oDOMQuery);
					//Subject for message to be sent with forgotten password
					if ($this->forgotSubject = $this->oDOMDoc->getNode("auth:subject",$this->oDOMQuery)) {
						$this->forgotSubject = $this->oDOMDoc->getNodeValue("auth:subject",$this->oDOMQuery);
					} else {
						$this->forgotSubject = false;
					}
					//Body of the message
					if ($this->forgotMessage = $this->oDOMDoc->getNode("auth:message",$this->oDOMQuery)) {
						$this->forgotMessage = $this->oDOMDoc->getNodeValue("auth:message",$this->oDOMQuery);
					} else {
						$this->forgotMessage = false;
					}
					return true;
				}
				$aMandatoryTags = Array();
				if (!$this->aAttrs['sid']){
					$aMandatoryTags = Array('username','digest');
					if ($this->sMethod == 'rsa') {
						$aMandatoryTags = Array('username','digest','method');
					}
					foreach ($aMandatoryTags as $tag) {
						if (!$DOMnode = $this->oDOMDoc->getNode("auth:".$tag,$this->oDOMQuery)) {
							throw new Exc('auth_missing_tag',$tag);
						}
					}
				}
			break;
		}
		return true;
	}
	/**
	* @brief	Private: Do operations dependent on 'query xmlns'
	* @return	XML subanswer (string)
	*/
	private function exeInputXML()
	{
		global $aError;
		switch ($this->aAttrs['type']) {
			case 'get':
				$this->sTemplateFile = 'webmailiqauth_get';
				if ($this->sMethod == 'rsa') {	
					$this->aData['method'] = "RSA";
					self::checkServerKeys();
					@$this->aData['hashId'] = file_get_contents(WM_CONFIGPATH.'public.key');
				} else {
					$this->aData['method'] = "PLAIN";
				}
				return;
			case 'set':
			//Forgot password
			if ($this->forgot){
				$this->sTemplateFile = 'webmailiqauth_set';
				if($oDOMCaptcha = $this->oDOMDoc->getNode("auth:captcha",$this->oDOMQuery)){
					//Get word & id
					$sWord = $this->oDOMDoc->getNodeValue("auth:captcha",$this->oDOMQuery);
					$sUID = $oDOMCaptcha->getAttribute('uid');
				}
				$oCaptcha = new Captcha();
				$oCaptcha->check($sUID,$sWord);

				//Use forgot class
				$oForgot = new Forgot($this->forgotEmail);
				if ($this->forgotSubject) {
					$oForgot->setSubject($this->forgotSubject);
				}
				if ($this->forgotMessage) {
					$oForgot->setMessage($this->forgotMessage);
				}

				try {
					$return = $oForgot->send();
					$this->aData['email']['num'] = $return; 
				} catch(Exc $e) { 
					throw new $e;
				}

				return;
			}
											
			// Logout
			if ($this->aAttrs['sid']) {
				// clear cache
				require_once('inc/cache/cache.php');
				$cache = Cache::instance($_SESSION['user']);
				$cache->cleanup();
				$_SESSION['user']->Logout();
				$this->aAttrs['sid'] = '';
				return;
			}
			// Login
			//Get mandatory tags
			$userName = $this->oDOMDoc->getNodeValue("auth:username",$this->oDOMQuery);
			$digest = $this->oDOMDoc->getNodeValue("auth:digest",$this->oDOMQuery);
			//If RSA method is set, get hash string
			if ($this->sMethod == 'rsa')
			{
				$hashID = $this->oDOMDoc->getNodeValue("auth:hashid",$this->oDOMQuery);
				self::checkServerKeys();
				@$privateData = file_get_contents(WM_CONFIGPATH.'private.key');
				$privateKey = openssl_pkey_get_private($privateData);
				openssl_private_decrypt(self::hexStringToBinary($digest), $password, $privateKey);
				$this->aAttrs['sid'] = User::login($userName, $password, $hashID, null);
			} else {
				$this->aAttrs['sid'] = User::login($userName, $digest, $hashID, $this->sMethod);
			}
			return;
		}
	}
	private static function createPublicKey($privatePEM)
	{
		// store private key and modulus into public.key
		$keyPair = Crypt_RSA_KeyPair::fromPEMString($privatePEM);
		$publicKey = $keyPair->getPublicKey();
		@file_put_contents(WM_CONFIGPATH.'public.key', self::binaryToHexString($publicKey->getModulus()));
	}
	public static function checkServerKeys() 
	{
		// does private key exist?
		if (!is_file(WM_CONFIGPATH.'private.key')) {
			// create new one
			$configArgs = array('config' => realpath(RSA_CONFIGPATH.'openssl.cnf'));
			$privateKey = openssl_pkey_new($configArgs);
			if (openssl_pkey_export($privateKey, $privateKeyString, null, $configArgs) === false) {
				throw new Exc('auth_export_key_private');
			}
			if (!file_exists(WM_CONFIGPATH)) {
				System::mkdir_r(WM_CONFIGPATH);
			}
			@file_put_contents(WM_CONFIGPATH.'private.key', $privateKeyString);
			openssl_pkey_free($privateKey);
			self::createPublicKey($privateKeyString);
		} else if (!is_file(WM_CONFIGPATH.'public.key')) {
			@self::createPublicKey(file_get_contents(WM_CONFIGPATH.'private.key'));
		}
	}
	private static function binaryToHexString($binary)
	{
		$binary = strrev($binary);
		$valueToHexChar = array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f');
		$binaryLength = strlen($binary);
		for ($i = 0; $i < $binaryLength; $i++) {
			$result .= $valueToHexChar[ord($binary[$i]) >> 4];
			$result .= $valueToHexChar[ord($binary[$i]) & 0x0F];
		}
		return ltrim($result, '0');
	}
	public static function hexStringToBinary($hexString) {
		$result = '';
		$hexStringLength = strlen($hexString);
		if ($hexStringLength % 2) {
			$result = chr(self::hexCharValue($hexString[0]));
			$i = 1;
		} else {
			$result = '';
			$i = 0;
		}
		for (; $i < $hexStringLength; $i += 2) {
			$result .= chr(self::hexCharValue($hexString[$i])*16 + self::hexCharValue($hexString[$i+1]));
		}
		return $result;
	}
	public static function hexCharValue($hexChar) {
		if ($hexChar == '') return 0;
		if ($hexChar <= '9') {
			if ($hexChar >= '0') {
				return ord($hexChar) - ord('0');
			} else {
				throw new Exc('auth_invalid_char');
			} 
		} else if ($hexChar <= 'F') {
			if ($hexChar >= 'A') {
				return ord($hexChar) - ord('A') + 10;
			} else {
				throw new Exc('auth_invalid_char');
			}
		} else if ($hexChar <= 'f') {
			if ($hexChar >= 'a') {
				return ord($hexChar) - ord('a') + 10;
			} else {
				throw new Exc('auth_invalid_char');
			}
		} else {
			throw new Exc('auth_invalid_char');
		}
	}
}
?>