<?php

require_once('inc/system.php');

/**
 * @brief		Class for handling webmailIqAccounts namespace
 */
class WebmailIqFolders extends XMLRequestInterface{
	
  public $aActions = array('edit'=>1, 'delete'=>1, 'add'=>1, 'sync'=>1, 'empty'=>1,'markasread'=>1,'markasunread'=>1,'search'=>1);
	
	private $oDOMAccount; //DOM Element representing account element in XML request
	
	private $oDOMFolder;  //DOM Element representing folder element in XML request
	
	private $sAID;	 //String ID of account element
	
	private $sFID;	 //String ID of folder element
	
	private $sFAction; //String folder action
	
	  /**
		* @brief Public: Constructor
		* @param $oDOMQuery A DOMElement filled with query element
		* @param $oDOMDoc	A DOMDocument of the request
		* @param $attrs  An Array with parent attributes
		*/
		public function __construct($oDOMQuery,&$oDOMDoc,&$attrs)
		{
			$this->oDOMQuery = $oDOMQuery;
			$this->oDOMDoc = &$oDOMDoc;
			$this->aAttrs = &$attrs;

				// Retrieve list of user's accounts 
		  	$this->aAccounts = &$_SESSION['user']->getAccounts();
		  	
			$this->checkInputXML();
		  	$this->exeInputXML();

	  	}
	  /**
		* @brief	Private - Checks if the input XML is valid
		* @return  TRUE/FALSE and throws exception
		*/
	  	private function checkInputXML(){
	  	global $aError;
	  	  
		    // Check if there is an account element in query 
	  	  if (!$this->oDOMAccount = $this->oDOMDoc->getNode("folders:account",$this->oDOMQuery))
	  		     throw new Exc('folder_missing_tag','account');
	  	 
	  	  // Check if an account id is set 
	  		if (!$this->sAID = $this->oDOMAccount->getAttribute("uid"))
	  				 throw new Exc('folder_missing_account_id');


	  		// Conditions for iq type='set' 
	  		if ($this->aAttrs['type']=='set'){
	  		
      	  		// Check for folder element existence 
      	  		if (!$this->oDOMFolder = $this->oDOMDoc->getNode("folders:folder",
                                                               $this->oDOMAccount))
                    throw new Exc('folder_missing_tag','folder');
      	  				
      			  // Check for folder action attribute existence 
      		  	if (!$this->sFAction = $this->oDOMFolder->getAttribute("action"))
      	  			    throw new Exc('folder_missing_tag','action');  
      			  		
      	  				 
      	  		// Check for folder  type element existence 
      		  	if (!($this->sFType = $this->oDOMDoc->getNodeValue("folders:type",
                                                                 $this->oDOMFolder))
              && !$this->sFAction=='add')
      	  					throw new Exc('folder_missing_tag','type');
      		  
      	  		// Check if there is an account with specified id 
      	  		if (!$this->aAccounts[$this->sAID]){
      	  		     $rssAccount = false;
      
      	  	    	 if($this->aAttrs['type']=='set')
      	  	    	  if($this->sFType = $this->oDOMDoc->getNodeValue("folders:type",
                                                                       $this->oDOMFolder))
                      if($this->sFType=='R'){
                          $rssAccount = new RSSAccount($_SESSION['user'],$this->sAID);
                          $rssAccount->description = 'RSS';
      		                $_SESSION['user']->aAccounts[$this->sAID] = $rssAccount;
                      }
                   if(!$rssAccount)
      	  				   throw new Exc('folder_invalid_account_id',$this->sAID);
      	  		}  
                 		  
      			  // Check for folder  id attribute existence 
      		  	if ((!$this->sFID = $this->oDOMFolder->getAttribute("uid")) 
                  && ($this->sFAction!='add'))
      	  					throw new Exc('folder_missing_id');

        			// Check if folder action attribute is correct 
        			if (!isset($this->aActions[$this->sFAction]))
        	  					throw new Exc('folder_invalid_action',$this->sFAction);
        	  }else{
      	  		// Check if there is an account with specified id 
      	  		if (!$this->aAccounts[$this->sAID])
      	  		 throw new Exc('folder_invalid_account_id',$this->sAID);         
            }
	  	}

	  /**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/
		 private function exeInputXML(){
				global $aError;

				// Get account to worh with 
				$oUser = &$_SESSION['user'];
				$oAccount = &$oUser->getAccount($this->sAID);	 

				// Process actions depending on set/get
				switch ($this->aAttrs['type']) {
						case 'get':
                  //Get folders
									$folders = $oAccount->getFolders();

									$this->sTemplateFile = 'webmailiqfolders_get';
									$this->aData['aid']  = $this->sAID;
									$this->aData['folder']['num'] = $this->cnvFolders($folders);
									break;
						case 'set':
									
                  // Get the type of the folder. 
                  if ($this->sFAction == 'add') {
									     $sFolderType = strtoupper($this->sFType{0});
									} else {
									     $folders = $oAccount->getFolders();
                       if (!isset($folders[$this->sFID]))
											    throw new Exc('folder_does_not_exist',$this->sFID);

                       $sFolderType = $folders[$this->sFID]->type;
									}
									
									// Do some actions in case the folder is from groupware ,rss or mail 	
									switch ($sFolderType) 
									{
									  // MAIL folder
										case 'M': 
											//do nothing
											break;
											
									  //Quarantine folder
										case 'Q':
										case 'QL':
										//RSS folder
										case 'R':
										  break;
										  
										case 'C': // CONTACT folder
										case 'E': // EVENTS folder
										case 'T': // TASKS folder
										case 'N': // NOTES folder
										case 'J': // Journal folder
                    case 'F': // FILES folder
											$oAccount = &$oAccount->gwAccount;
											if (!is_object($oAccount))
                        throw new Exc('gw_init_error');
											$this->gwFolder = true;
											break;
													
										default:
											throw new Exc('folder_invalid_type',$sFolderType);
									}
									 

									switch ($this->sFAction){
									    // Synchronize folder
											case 'sync':
											       
														$oFolder = &$oAccount->getFolder($this->sFID);
														if (isset($oFolder->gw) && $oFolder->gw)
															throw new Exc('folder_sync',$this->sFID);
														$oFolder->sync($this); 
														break;
											// Delete existing folder
											case 'delete':
														// Retrieve folder to work with
														$oFolder = &$oAccount->getFolder($this->sFID);

														if($_SESSION['DEBUG'])log_buffer("WebmailIqFolders->Delete : Deleting folder".$oFolder->type."=".$oFolder->name."\r",'DEBUG');
														$oFolder->delete();
														
														if ($sFolderType == 'M')
														{
															//Synchronize account to uncache this new folder
															$oAccount->sync();
														}
														break;
											// Delete all items from a folder
											case 'empty':
														// Only email folders (except from INBOX) can be emptied
														if ($sFolderType != 'M' 
                                && $sFolderType != 'R' 
                                && $sFolderType != 'Q' 
                                && $sFolderType != 'QL')
															throw new Exc('folder_empty_non_mail');
															
														// TODO: support for moving to different account
														$sDestinationAccount = $this->oDOMDoc->getNodeValue("folders:account",$this->oDOMFolder);
														$sDestinationFolder = $this->oDOMDoc->getNodeValue("folders:folder",$this->oDOMFolder);
														
														// INBOX cannot be deleted
														/*if ($this->sFID == 'INBOX' && !$sDestinationFolder)
															throw new Exc($aError['f_empty_non_mail']);*/

                            //Retrieve folder to work with
														$oFolder = &$oAccount->getFolder($this->sFID);

                            //Copy items to folder
														if ($sDestinationFolder && $sFolderType == 'M') {
												  
														  $oDestinationAccount = &$oUser->getAccount($sDestinationAccount);
															$oDestination = &$oDestinationAccount->getFolderWithAutoCreate($sDestinationFolder,'main');
                              
                              $oFolder->moveItems($oDestination);
                              
														}else{
                            	$oFolder->deleteItems();                        	
														}
                            break;
                      case 'markasread':
                      case 'markasunread':
                            $flag = Item::FLAG_SEEN;
														
                            if ($sFolderType == 'M' || $sFolderType == 'R')
														{
                              //Retrieve folder to work with
														  $oFolder = &$oAccount->getFolder($this->sFID);
														
														  if ($this->sFAction=='markasread')
                                $oFolder->markItems($flag);
                              else 
                                $oFolder->unmarkItems($flag);
                            }else
                              throw new Exc($aError['folder_mark_non_mail']);
                            
                      break;								
											// Add folder
											case 'add':
														// Get folder name
														$sFolderName = $this->oDOMDoc->getNodeValue("folders:name",
																								                        $this->oDOMFolder);
													  
													  if(!Tools::checkFolderName($sFolderName))
													   throw new Exc('folder_name_bad_char',$sFolderName);
                            
                            $sChannelURL = trim($this->oDOMDoc->getNodeValue("folders:channel",
																								                        $this->oDOMFolder));
														if (!$sFolderName) throw new Exc('folder_missing_name');
														
														
														//Create folder
														$oAccount->createFolder($sFolderName,$sFolderType,$sChannelURL);
																					//Synchronize account to cache this new folder
														if ($sFolderType == 'M' || $sFolderType == 'R'){
															$oAccount->sync();
														}
														break;
											//Edit folder
											case 'edit':
											
														//Retrieve folder to work with
														$type = 'main';
								
														$oFolder = &$oAccount->getFolder($this->sFID,$type);

														 /* Get old and new name*/
														 $sFolderName = $oFolder->name;
														 $sNewFolderName = $this->oDOMDoc->getNodeValue("folders:name",
                                                                            $this->oDOMFolder);
                            
                             if($sNewFolderName && !Tools::checkFolderName($sNewFolderName))
													     throw new Exc('folder_name_bad_char',$sNewFolderName);
                                                                            
                          
														 if ($sFolderType == 'R'){
  													   if($newFolderChannelNode = $this->oDOMDoc->getNode("folders:channel",
  													                                                     $this->oDOMFolder)){
  													     $sNewFolderChannel = trim($this->oDOMDoc->getNodeValue("folders:channel",
  													                                                     $this->oDOMFolder));
  													     $changeChannel = true;
                               }
  													   else
  													     $changeChannel = false;

                               $oFolder->edit($sNewFolderName,$sNewFolderChannel,$changeChannel);                             
                               
                               // Synchronize account to cache this new folder 
    													 $oAccount->sync();
                               
                             }else{
                                try{                                  
                                    ////* Check existence *///////////////////////////////////
                                    //GW folder - check mail folder
                                    if($oAccount->account)
                                      $oMailFolder = $oAccount->account->getFolder($sNewFolderName);
                                    
                                    if($oMailFolder)
                                      throw new Exc('folder_already_exists',$sNewFolderName);
                                    
                                    //Mail folder - check for GW folder ( or GW folder check GW)
                                    $oFolder = $oAccount->getFolder($sNewFolderName);
                                    
                                    if($oFolder)
                                      throw new Exc('folder_already_exists',$sNewFolderName);                              
      
                                  }catch(Exc $e){
                                    if($e->wmcode == 'folder_already_exists'){
                                      throw new Exc('folder_already_exists',$sNewFolderName);
                                    }
                                  }    
                                                                                          
  														 // Check new folder name 
														  Security::checkpath($sNewFolderName);
                              
                                // Rename folder 
  														 if ($sNewFolderName && ($sFolderName!=$sNewFolderName))	{	
                                  $oFolder->rename($sNewFolderName);
  														    
                                 // Rename GW folder as well,in order to keep 
                                 // synchronization between IMAP and GW database   
    														 if ($sFolderType == 'M'){
    														    
    																try {
    																  if ($oAccount->gwAccount){
                                       $oGWFolder = $oAccount->gwAccount->getFolder($sFolderName);
    																	 $oGWFolder->rename($sNewFolderName); 
    																  }
                                    } catch (Exc $e) {
    																			// folder does not exist in GW, continue
    															  }
    																// Synchronize account to cache this new folder 
    																$oAccount->sync();
                                       
                                  }else{
                                       $type='main';
                                        
                                       try{
                                        $oIMAPFolder = $oAccount->account->getFolder($sFolderName,
                                                                                    $type);
                                        $oIMAPFolder->rename($sNewFolderName); 
                                       }catch (Exc $e)
                                       {}                                                                    
                                 }
                              }
                            }
                             
                             if($sSubscription = $this->oDOMDoc->getNode("folders:subscribed",
                                                                            $this->oDOMFolder)){
                                $sSubscription = $this->oDOMDoc->getNodeValue("folders:subscribed", $this->oDOMFolder);
                                
                                $oFolder->setSubscription($sSubscription);
                             }
														 break;

									}
									
									break;
				}
		 }
		/**
		 *  @brief Converts folder array into template compatible data array
		 *  @param $folders Array of folder objects
		 *  @return folders array in the form for template system
		 */		 
		 public static function cnvFolders($folders,$interface = false)
		 {  

				if (!is_array($folders)) return;
				$aItems = array();
				foreach($folders as $k=>$v){
				  $aItm = array();
          $sItemRights = '';
          // Process Mail folder 
				  if (!isset($v->gw) || !$v->gw) {
				      //  echo $v->name;
				        $aItm['rights'] = self::rightsToString($v->rights,$sItemRights);		      

                if($sItemRights){
                  $aItm['access'] = $aItm['rights'];
                  $aItm['rights'] = $sItemRights;
                }
      					
                if(strtolower($v->name)=='inbox'){
                  $aItm['access'] = $aItm['rights'];
                  $aItm['rights'] = 'rw';
                }
                
                if($v->type=='R'){
                  $aItm['access'] = 'rmd';
                  $aItm['rights'] = 'rwmd';
                  $aItm['channel'] = $v->channel;
                }
                                
					      $aItm['recent']  = $v->countItems(Item::FLAG_SEEN, false); 
                $aItm['subscribed']	  = $v->sync?true:false; 
          
          // Process GW folder 
          } else {
                $aItm['rights'] = $v->rights;
      					$aItm['recent']  = '0'; 
         }
          // Process both  
					$aItm['name']	  = $v->name;
					if(!$interface){
						$aItm['id']		= $v->name;
					//NON - AJAX interfaces helper
					}else{
						if($v->localizedName){
							$aItm['localizedName']	  = $v->localizedName;
						}
						$aItm['id'] = $v->id;
						switch($v->type){
							case 'C':
								$sView = 'contact_list';
							break;
							case 'M':
								$sView = 'mail_list';
							break;
						}
						$aItm['view'] = $sView;
					}
					$aItm['type']	  =  $v->type;
					//$aItm['total']	 = $v->countItems();
					$aItems[] = $aItm;
				}
				return $aItems;
			 
		 }
		 
		 public static function rightsToString($rights,&$sItemRightsOut)
		 {

        $sItemRights=$sFolderRights='';

      //FOLDERS = swapped names
        $sItemRights .=  ($rights & Folder::RIGHT_READ || $rights & Folder::RIGHT_ADMIN || $rights & Folder::RIGHT_D) ?'r':'';
        $sItemRights .=  ($rights & Folder::RIGHT_INSERT || $rights & Folder::RIGHT_ADMIN || $rights & Folder::RIGHT_D) ?'w':'';
        $sItemRights .=  ( $rights& Folder::RIGHT_WRITE || $rights & Folder::RIGHT_ADMIN || $rights & Folder::RIGHT_D ) ?'m':'';
        $sItemRights .=  ($rights & Folder::RIGHT_DELETE || $rights & Folder::RIGHT_ADMIN || $rights & Folder::RIGHT_D)?'d':'';
        
        //ITEMS
        $sFolderRights .=  ($rights & Folder::RIGHT_LOOKUP || $rights & Folder::RIGHT_ADMIN || $rights & Folder::RIGHT_D) ?'r':'';
        $sFolderRights .=  ($rights & Folder::RIGHT_MAILBOX_WRITE || $rights & Folder::RIGHT_ADMIN || $rights & Folder::RIGHT_D) ?'w':'';
        $sFolderRights .=  ($rights & Folder::RIGHT_WRITE || $rights & Folder::RIGHT_ADMIN || $rights & Folder::RIGHT_D) ?'m':'';
        $sFolderRights .=  ($rights & Folder::RIGHT_DELETE || $rights & Folder::RIGHT_ADMIN || $rights & Folder::RIGHT_D ) ?'d':'';

        
        if($sFolderRights!=$sItemRights)
          $sItemRightsOut = $sItemRights;
		//    echo '='.$sFolderRights."\r\n";        
   /*     echo $name.":";
        echo $rights."-";
        echo $sItemRights.'=';
        echo $sFolderRights."\r\n";

        $sRights .=  ($rights & Folder::RIGHT_READ) ?'r':'';
        $sRights .=  ($rights & Folder::RIGHT_WRITE) ?'w':'';
        $sRights .=  ($rights & Folder::RIGHT_DELETE) ?'d':'';
        $sRights .=  (($rights & Folder::RIGHT_INSERT) || ($rights & Folder::RIGHT_WRITE) || ($rights & Folder::RIGHT_CREATE) )?'m':''; 
        */

        return $sFolderRights;
     }
 }
?>
