<?php
require_once 'error.php';
/**
* @brief  class for handling webmailIqItems namespace
@author Pavel Perna
@date	04.04.2006
*/
class WebmailIqFreeBusy extends XMLRequestInterface {
	/**
	* @brief	Public: Constructor
	* @param	$oQuery A DOMElement filled with query element
	* @param	$oDoc	A DOMDocument of the request
	* @param	$attrs  An Array with parent attributes
	*/
	public function __construct(&$oDOMQuery,&$oDOMDoc,&$attrs)
	{
		$this->oDOMQuery = &$oDOMQuery;
		$this->oDOMDoc = &$oDOMDoc;
		$this->aAttrs = &$attrs;
		$this->checkInputXML();
		$this->exeInputXML();
	}
	/**
	* @brief	Private - Checks if the input XML is valid
	* @return	TRUE/FALSE and throws exception
	*/
	private function checkInputXML()
	{
		global $aError;
		// GroupWare API object
		$this->oGroupWareAPI = new MerakGWAPI();
		if (!$this->oDOMUsers = $this->oDOMDoc->getNode("freebusy:users",$this->oDOMQuery)) {
			throw new Exc('freebusy_missing_tag','user');
		}
		if (!$this->sFrom = $this->oDOMDoc->getNodeValue("freebusy:from",$this->oDOMQuery)) {
			$this->sFrom = $this->oGroupWareAPI->unix2calendarDate(time());
		}
		if (!$this->sTo= $this->oDOMDoc->getNodeValue("freebusy:to",$this->oDOMQuery)) {
			$this->sTo = $this->sFrom + 30;
		}
		if (!$this->sCTZ = $this->oDOMDoc->getNodeValue("freebusy:ctz",$this->oDOMQuery)) {
			$this->sCTZ = 0;
		}
		if(!$this->sAccountID = $this->oDOMDoc->getNodeValue("freebusy:account",$this->oDOMQuery)) {
			throw new Exc('freebusy_missing_tag','account');
		}
	}
	/**
	* @brief	Private: Do operations dependent on 'query xmlns'
	* @return	XML subanswer (string)
	*/
	private function exeInputXML()
	{
		$oUser = &$_SESSION['user'];
		switch($this->aAttrs["type"]) {
			case "get":
				$this->sTemplateFile = 'webmailiqfreebusy_get';
				$i = 0;
				foreach($this->oDOMDoc->query('/iq/freebusy:query/freebusy:users/freebusy:user') as $item){
					$sEmail = strval($item->nodeValue);
					// Get FreeBusy depending on specified email
					$sVersit = $this->oGroupWareAPI->FunctionCall("GetVFreeBusy",
					$sEmail,
					$this->sFrom . '-' . $this->sTo);
					if(!$sVersit) {
						$oAccount = $oUser->getAccount($this->sAccountID);
						$oFolder = $oAccount->getFolder('Contacts');
						$aFilter['tag'] = 'itminternetfreebusy,lctemail1,lctemail2,lctemail3';
						$aFilter['sql'] = "(itminternetfreebusy NOT NULL AND (lctemail1 = '".$sEmail."' OR lctemail2 = '".$sEmail."' OR lctemail3 = '".$sEmail."'))";
						$oItems = $oFolder->getItems($aFilter);
						$oItem = reset($oItems);
						$sAddress = $oItem->item['ITMINTERNETFREEBUSY'];
						$sVersit = @file_get_contents($sAddress);
					}
					// Parse versit data into return array for template
					$this->aData['users']['num'][$i] = $this->parseVersitObject($sVersit,$sEmail);
					$this->aData['users']['num'][$i++]['userid'] = $sEmail;
				}
				break;
		}
	}
	/**
	* @brief	Private: Do operations dependent on 'query xmlns'
	* @return	XML subanswer (string)
	*/
	private function parseVersitObject($sVersit,$sEmail)
	{
		$aLines = explode("\r\n",$sVersit);
		// Get all freebusy items
		foreach ($aLines as $sLine){
			$aLine = explode(":",$sLine);
			$aLineType = explode(";",$aLine[0]);
			if ($aLineType[0]=="FREEBUSY"){
				$aBusyType = explode("=",$aLineType[1]);
				$sBusyType = $aBusyType[1];
				$aTime = explode("/",$aLine[1]);
				$aItem["type"] = $sBusyType;
				$aItem["user"] = $sEmail;
				$aItem["start"] = $aTime[0];
				$aItem["finish"] = $aTime[1];
				//From and to time
				for ($i = 0; $i < 2; $i++)
				{
					$sYear[$i] =  substr($aTime[$i],0,4);
					$sMonth[$i] =  substr($aTime[$i],4,2);
					$sDay[$i] =  substr($aTime[$i],6,2);
					$sHours[$i] = $sMinutes[$i] = $sSeconds[$i] = 0;
					if (strpos($aTime[$i],"T")!=-1) {
						$sHours[$i] = substr($aTime[$i],9,2);
						$sMinutes[$i] = substr($aTime[$i],11,2);
						$sSeconds[$i] = substr($aTime[$i],13,2);
						$sUnix[$i] = mktime($sHours[$i],
						$sMinutes[$i],
						$sSeconds[$i],
						$sMonth[$i],
						$sDay[$i],
						$sYear[$i])+($this->sCTZ*60);
					}
				}
				$aItem["startdate"] = $this->oGroupWareAPI->unix2calendarDate($sUnix[0]);
				$aItem["starttime"] = $this->oGroupWareAPI->unix2calendarTime($sUnix[0]);
				$aItem["enddate"] = $this->oGroupWareAPI->unix2calendarDate($sUnix[1]);
				$aItem["endtime"] = $this->oGroupWareAPI->unix2calendarTime($sUnix[1]);
				$aItems["items"]["num"][] = $aItem;
			}
		}
		return $aItems;
	}
}
?>
