<?php
require_once 'error.php';
require_once 'mailparse.php';
require_once 'webmailiqfolders.php';
require_once 'webmailiqmessage.php';
require_once 'mail.php';
/**
* @brief class for handling webmailIqItems namespace
* @author Pavel Perna
* @revision 06.11.2006
*/
class WebmailIqItems extends XMLRequestInterface {
	// Array with allowed actions
	private $aActions = array(
		'edit'=>1,
		'delete'=>1,
		'add'=>1,
		'attachments'=>1,
		'move'=>1,
		'copy'=>1,
		'blacklist'=>1,
		'whitelist'=>1,
		'deliver'=>1,
		'redirect'=>1,
		'search'=>1,
		'accept'=>1,
		'decline'=>1,
		'process'=>1
	);
	/**
	* @brief Public: Constructor
	* @param $oQuery A DOMElement filled with query element
	* @param $oDoc A DOMDocument of the request
	* @param $attrs An Array with parent attributes
	*/
	public function __construct($oDOMQuery,&$oDOMDoc,&$attrs)
	{
		$this->oDOMQuery = $oDOMQuery;
		$this->oDOMDoc = &$oDOMDoc;
		$this->aAttrs = &$attrs;
		//Get all or only one item(s)?
		$this->bGetSingleItem = false;
		$this->checkInputXML();
		$this->exeInputXML();
	}
	/**
	* @brief Private - Checks if the input XML is valid
	* @return TRUE/FALSE and throws exception
	*/
	private function checkInputXML()
	{
		global $aError;
		$this->oUser = $oUser = &$_SESSION['user'];
		// Check if there is an account element in query
		if (!$this->oDOMAccount = $this->oDOMDoc->getNode("items:account", $this->oDOMQuery)) {
			throw new Exc('item_missing_tag','account');
		}
		// Check if an account id is set
		if (!$this->sAID = $this->oDOMAccount->getAttribute("uid")) {
			throw new Exc('item_missing_account_id');
		}
		// Check if there is a account with specified id
		if (!($this->oAccount = &$oUser->getAccount($this->sAID))) {
			throw new Exc('item_invalid_account_id');
		}
		// Check for folder element existence
		if (!$this->oDOMFolder = $this->oDOMDoc->getNode("items:folder", $this->oDOMAccount)) {
			throw new Exc('item_missing_tag','folder');
		}
		// Check for folder id attribute existence */
		if (!$this->sFID = $this->oDOMFolder->getAttribute("uid")) {
			throw new Exc('item_missing_folder_id');
		}
		/* Check if there is a folder with specified id */
		try{
			$this->oFolder = $this->oAccount->getFolder($this->sFID);
		} catch(Exc $e){
			//Try to autocreate gw folder (FIX for Files folder)
			$this->oFolder = $this->oAccount->getFolderWithAutoCreate($this->sFID,'F');
		}
		if (
			!$this->oFolder
		&&
			$this->oFolder->type!='M'
		&&
			$this->oFolder->type!='R'
		&&
			$this->oAccount->gwAccount->bLogged == false
		) {
			throw new Exc('gw_init_error');
		}
		/*GW restrictions*/
		if (!$this->oFolder && $_SESSION['RESTRICTIONS'][$this->sFID[0]]) {
			throw new Exc('gw_init_error');
		}
		//Check for delete files older than ....
		if ($this->sCleanupDays = $this->oDOMDoc->getNodeValue("items:cleanup",$this->oDOMFolder)) {
		}
		//Store unsubscribed folders 1st synchronization till next account->refresh/sync
		if (
			!$this->oFolder->gw
		&&
			!$this->oFolder->sync
		&&
			!$this->oAccount->aSyncedFolders[$this->oFolder->folderID]
		) {
			$this->oAccount->aSyncedFolders[$this->oFolder->folderID] = true;
			if ($this->oFolder) {
				$this->oFolder->sync();
			}
		}
		//Parse items
		$this->oDOMItems = array();
		$this->sAttId = false;
		$this->oDOMItems = array();
		foreach($this->oDOMDoc->query('/iq/items:query/items:account/items:folder/items:item') as $item) {
			$this->oDOMItems[] = $item;
		}
		//Check single or multiple item(s) get
		if (isset($this->oDOMItems[0]) && $oDOMItem = $this->oDOMItems[0]) {
			if ($this->sIID = $oDOMItem->getAttribute("uid")) {
				if (($pos = strpos($this->sIID,'|'))!==false) {
					$this->sPartID = substr($this->sIID,$pos+1);
					$this->sIID = substr($this->sIID,0,$pos);
					$this->aData['partid'] = $this->sPartID;
				}
				$this->sPartID = $this->sPartID?$this->sPartID:1;
				$this->bGetSingleItem = true;
				if ($this->sAttId = $oDOMItem->getAttribute("atid")) {
					$this->bGetAttach = true;
				}
				if (!is_object($this->oFolder->getItem($this->sIID,NO_ADDONS))) {
					throw new Exc('item_invalid_item_id',$this->sIID);
				}
			}
		}
		if ($this->aAttrs['type']=='set') {
			/* Check if there is an item element */
			if (!$this->oDOMItem = $this->oDOMItems[0]) {
				throw new Exc('item_missing_tag','item');
			}
			/* Check if there is an item action specified*/
			if (!$this->sIAction = $this->oDOMItem->getAttribute("action")) {
				throw new Exc('item_missing_tag','action');
			}
			if ($this->sTargetAID = $this->oDOMDoc->getNodeValue("items:account",$this->oDOMItem)) {
				//POP3Class FIX - if move to folder in the same account(move to trash action),
				//delete messages
				if ($this->oFolder->itemClassName=="POP3Item"){
					if (
						$this->sIAction=='move'
					&&
						$this->sAID == $this->sTargetAID
					)
					$this->sIAction = 'delete';
				}
			}
			/* Check if actual action is correct*/
			if (!isset($this->aActions[$this->sIAction])) {
				throw new Exc('item_invalid_action',$this->sIAction);
			}
			//Do some check before specific actions
			switch($this->sIAction) {
				case 'add':
				case 'edit':
				case 'delete':
					/* Check if there is an item values specified*/
					if (!$this->oDOMValues = $this->oDOMDoc->getNode("items:values",$this->oDOMItem)) {
						//LogError::Error($aError['missing values']);
						if (!($this->sIAction=='delete')) {
							throw new Exc('item_missing_tag','values');
						}
					}
					if ($pom = $this->oDOMDoc->getNode("items:folder",$this->oDOMValues)) {
						$this->sSpamFolder = $pom;
					}
					break;
				case 'edit':
				case 'delete':
				case 'move':
				case 'copy':
				case 'attachments':
					if (!$this->sIID = $this->oDOMItem->getAttribute("uid")) {
						throw new Exc('item_missing_item_id');
					}
					break;
				case 'accept':
					if (!$this->sAcceptFolder = $this->oDOMDoc->getNodeValue("items:folder",$this->oDOMItem)) {
						throw new Exc('item_missing_tag','folder');
					}
				case 'decline':
					if (!$this->sAttachmentID = $this->oDOMDoc->getNodeValue("items:partid",$this->oDOMItem)) {
						throw new Exc('item_missing_tag','partid');
					}
					break;
			}
		}
	}
	/**
	* @brief Private: Do operations dependent on 'query xmlns'
	* @return XML subanswer (string)
	*/
	private function exeInputXML()
	{
		// GET/SET
		switch ($this->aAttrs['type']) {
			case 'get':
				
				$iItemCount = -1;
				//Process folder cleanup
				if ($this->sCleanupDays) {
					$this->oFolder->deleteOlder($this->sCleanupDays);
				}
				$this->sTemplateFile = 'webmailiqitems_get';
				$aFilterTag = $this->filterAndTags();
				if ($sPhrase = $aFilterTag['fulltext']) {
					$aItems = $this->oFolder->search($sPhrase,$aFilterTag);
					$iItemCount = count($aItems);
				} else {
					//Handle single item get vs. multiple item get
					if ($this->bGetSingleItem) {
						$oItem = $this->oFolder->getItem($this->sIID);
						if ($oItem->wmclass=="M" || $oItem->wmclass=="R") {
							$oItem->markAsRead();
						}
						//Attachments
						if ($this->bGetAttach) {
							$attachment = new GroupWareAddon($oItem,'attachment');
							$value = $attachment->getAttachment($this->sAttId);
							$value = explode(':',$value);
							$value = $attachment->getGroupAttachment($value[0],$value[1],$value[2]);
							$this->sAID = $value['account'];
							$this->sFID = $value['folder'];
							$this->sIID = $value['item'];
							$oAccount = $_SESSION['user']->getAccount($this->sAID);
							$oFolder = $oAccount->getFolder($this->sFID);
							$oItem = $oFolder->getItem($this->sIID);
						}
						$aItems = array(0 => $oItem);
					} else {
						$aItems = $this->oFolder->getItems($aFilterTag);
						
					}
				}
				//Prepare data for template - default is -1 ( count is done over the cache ), while
				// fulltext searching the count(result) is used to count items..
				$this->aData['items']['num'] = $this->cnvItems($aItems,$aFilterTag,$this->bGetSingleItem,$this->sPartID,$this->oAccount,$this->oFolder);
				$this->aData["values"] = !(isset($this->oFolder->gw) && $this->oFolder->gw);
				$this->aData['aid'] = htmlspecialchars($this->sAID);
				$this->aData['fid'] = htmlspecialchars($this->sFID);
				$this->aData['counter'] = ($iItemCount!=-1)?$iItemCount:($this->bGetSingleItem?1:$this->oFolder->countItems(0,true,$aFilterTag['sql'],$aFilterTag['fulltext']));
				
				// Process Mail folder
				if (
					!isset($this->oFolder->gw)
				||
					!$this->oFolder->gw
				) {
					$this->aData['recent'] = $this->oFolder->countItems(Item::FLAG_SEEN,false,'',$aFilterTag['fulltext']);
					// Process GW folder
				} else {
					$this->aData['recent'] = '0';
				}
				$offset = intval($aFilterTag['offset']);
				if (
					$this->aData['counter']>0
				&&
					$offset<$this->aData['counter']
				&&
					count($this->aData['items']['num']) == 0
				) {
					$this->aData['counter'] = 0;
				}
				$this->aData['offset'] = $offset;
				//dmp($this->aData);die();
				break;
			case 'set':
				//Create instances of Cache, API and optionaly IMAP objects
				//to speed up transactions with many items
				$cache['cache'] = Cache::instance($this->oAccount->user);
				//$cache['cache']->transaction();
				$cache['api'] = new MerakAPI;
				if (strtolower(get_class($this->oFolder))=='imapfolder') {
					$account = $this->oAccount;
					$cache['imap'] = IMAP::instance($account);
				} else {
					$cache['imap'] = false;
				}
				
				//Do actions
				switch ($this->sIAction) {
					//Delete an item
					case 'delete':
						foreach($this->oDOMItems as $oDOMItem) {
							$sUID = $oDOMItem->getAttribute('uid');
							// Groupware
							// Check wheter we delete all events in the serie or the single one
							if ($oDOMExpdate = $this->oDOMDoc->getNode("items:expdate",$this->oDOMValues)) {
								$sExpdate = $oDOMExpdate->nodeValue;
								//Handle additional data for reccuring events
								if ($this->oDOMFollowing = $this->oDOMDoc->getNode("items:expfollowing",$this->oDOMValues)) {
									$bFollowing = $this->oDOMFollowing->nodeValue=='true'?true:false;
								} else {
									$bFollowing = false;
								}
								//Delete GW item on specified day
								$oItem = $this->oFolder->getItem($sUID);
								$oItem->delete($sExpdate,$bFollowing);
								//E - mails
								//Delete emails or non-repeating gw items
							} else {
								$oItem = $this->oFolder->getItem($sUID,$cache);
								$aItemsToDelete[] = $oItem;
							}
						}
						//Multiple item delete - optiomalization
						if ($aItemsToDelete) {
							$this->oFolder->deleteItems($aItemsToDelete);
						}
						break;
					//Add new item
					case 'add':
						if ($this->oFolder->gw || $this->oFolder->type=='Q' || $this->oFolder->type=='QL') {
							//$param = $this;
							//Cannot create messages using IQ:Items ( use IQ:Message)
						} else {
							throw new Exc('item_create');
						}
						$aItem = Tools::makeArrayFromXML($this->oDOMValues);
						$aTreeItem = Tools::makeTreeFromXML($this->oDOMItem);
						$oItem = $this->oFolder->createItem($aItem,$aTreeItem);
						//Events - iMIP functions - return an invitation object
						// - check EVN_FLAGS first ?
						if ($this->oFolder->type=='E' || $this->oFolder->type=='J') {
							$oUser = &$_SESSION['user'];
							//Send e-mail
							if ($oItem->isOrganizator($oItem->item['EVNFLAGS']) 
							|| $oItem->isOrganizator($aItem['EVNFLAGS'])) {
								$this->iMIP_Email($oItem,'invite');
							}
						}
						break;
					//Edit existing item
					case 'edit':
						//Set the template file
						$this->sTemplateFile = 'webmailiqitems_set';
						foreach($this->oDOMItems as $oDOMItem) {
							//Email
							if (!isset($this->oFolder->gw) || !$this->oFolder->gw) {
								//EMail - it is allowed to edit message flags and color only
								if ($oFlag = $this->oDOMDoc->getNode("items:values/items:flags", $oDOMItem)) {
									$sFlags = $oFlag->nodeValue;
									$oItem = $this->oFolder->getItem($oDOMItem->getAttribute('uid'));
									$oItem->setFullFlags($sFlags);
								} elseif ($oColor = $this->oDOMDoc->getNode("items:values/items:color", $oDOMItem)) {
									$sColor = $oColor->nodeValue;
									$oItem = $this->oFolder->getItem($oDOMItem->getAttribute('uid'));
									$oItem->changeColor($sColor);
									$oItem->setFullFlags( $oItem->flags & ~ITEM::FLAG_FLAGGED );
								} else {
									throw new Exc('item_edit');
								}
								//GroupWare //GW items
							} else {
								//$aFilterTag = $this->filterAndTags();
								$this->oDOMValues = $this->oDOMDoc->getNode("items:values",$oDOMItem);
								$oItem = $this->oFolder->getItem($oDOMItem->getAttribute('uid'),NO_ADDONS);
								//Handle additional data for reccuring events
								if ($this->oDOMDate = $this->oDOMDoc->getNode("items:expdate",$this->oDOMValues)) {
									$sExpdate = $this->oDOMDate->nodeValue;
								} else {
									$sExpdate = false;
								}
								if ($this->oDOMFollowing = $this->oDOMDoc->getNode("items:expfollowing",$this->oDOMValues)) {
									$bFollowing = $this->oDOMFollowing->nodeValue=='true'?true:false;
								} else {
									$bFollowing = false;
								}
								$aItem = Tools::makeArrayFromXML($this->oDOMValues,true);
								$aTreeItem = Tools::makeTreeFromXML($this->oDOMItem);
								$oItem->edit($aItem,$aTreeItem,$sExpdate,$bFollowing);
							}
						}
						break;
					//Copy & move
					case 'move':
					case 'copy':
						foreach($this->oDOMItems as $oDOMItem) {
							$sUID = $oDOMItem->getAttribute('uid');
							if (!$sFolderName = $this->oDOMDoc->getNodeValue("items:folder",$oDOMItem)) {
								throw new Exc('item_copy_destination');
							}
							// TODO: support moving to different account
							$sAccountName = $this->oDOMDoc->getNodeValue("items:account",$oDOMItem);
							$oDestAccount = $this->oUser->getAccount($sAccountName);
							if ($this->oFolder->gw) {
								$sFolderName = str_replace("/","\\",$sFolderName);
							}
							// compare types of folder
							if ($this->oFolder->type=='M') {
								$sType = 'main';
							} else {
								$sType = 'gw';
							}
							$destination = $oDestAccount->getFolderWithAutoCreate($sFolderName,$sType);
							if ($this->oFolder->type != $destination->type) {
								throw new Exc('item_copy_destination_type');
							}
							$oItem = $this->oFolder->getItem($sUID,$cache);
							$aItemsToMove[] = $oItem;
						}
						if ($aItemsToMove) {
							if ($this->sIAction == 'move') {
								$this->oFolder->moveItems($destination,$aItemsToMove, $cache);
							} else {
								$this->oFolder->copyItems($destination,$aItemsToMove, $cache);
							}
						}
						break;
					// Quarantime actions
					case 'deliver':
						if ($this->oFolder->type=='Q') {
							foreach($this->oDOMItems as $oDOMItem) {
								$this->oFolder->deliverItem($oDOMItem->getAttribute('uid'));
							}
						}
						break;
					case 'whitelist':
						if (
							$this->oFolder->type=='Q'
						||
							$this->oFolder->type=='QL'
						) {
							foreach($this->oDOMItems as $oDOMItem){
								if ($this->oFolder->type=='Q') {
									$deliver = true;
								} else {
									$deliver = false;
								}
								$this->oFolder->whitelistItem($oDOMItem->getAttribute('uid'),false,$deliver);
							}
						}
						break;
					case 'blacklist':
						if (
							$this->oFolder->type=='Q'
						||
							$this->oFolder->type=='QL'
						) {
							foreach($this->oDOMItems as $oDOMItem) {
								$this->oFolder->blacklistItem($oDOMItem->getAttribute('uid'));
							}
						}
						break;
					case 'redirect':
						foreach($this->oDOMItems as $oDOMItem) {
							$oItem = $this->oFolder->getItem($oDOMItem->getAttribute('uid'));
							$message = $oItem->getMessage();
							//No CRLF at the end of message
							if (substr($message,strlen($message)-2,2)!=CRLF) {
								$message.=CRLF.CRLF;
							}
							$message = explode(CRLF.CRLF,$message,2);
							$mail = new Mail();
							$mail->SetFrom($_SESSION['EMAIL']);
							foreach($this->oDOMDoc->query('items:to',$oDOMItem) as $to) {
								$mail->AddTo($to->nodeValue);
							}
							//??????????????
							foreach($this->oDOMDoc->query('items:distrib',$oDOMItem) as $distrib) {
								foreach($this->oDOMDoc->query('items:account',$distrib) as $account) {
									foreach($this->oDOMDoc->query('items:folder',$account) as $folder) {
										WebmailIQMessage::processDistributedList('items',$_SESSION['user'],$this->oDOMDoc,$account->getAttribute('uid'),$folder->getAttribute('uid'),$folder,$mail); 
									}
								}
							}
							$mail->Send($message[0],$message[1]);
						}
						break;
					case 'search':
						$sPhrase = $this->oDOMDoc->getNodeValue("items:fultext",$this->oDOMFolder);
						$oFolder = &$oAccount->getFolder($this->sFID);
						$oFolder->search($sPhrase);
						break;
						//iMIP Invitations
					//Accept - done over Mail items
					case 'accept':
						$oItem = $this->oFolder->getItem($this->sIID);
						$aAttachment = $oItem->acceptInvitation($this->sAttachmentID,$this->sAcceptFolder);
						//Send e-mail
						$this->iMIP_Email($oItem,'accept',$aAttachment);
						break;
					//Decline - done over Mail items
					case 'decline':
						$oItem = $this->oFolder->getItem($this->sIID);
						$aAttachment = $oItem->declineInvitation($this->sAttachmentID);
						//Send e-mail
						$this->iMIP_Email($oItem,'decline',$aAttachment,$this->sAttachmentID);
						break;
					default:
						break;
				}
				break;
		}
	}
	/**
	* @brief Checks and returns filter and selected tags
	and returns them as array
	@return Filter and Tag array
	*/
	private function filterAndTags(){
		global $oDOMItem,$oDOMDoc;
		$oDOMDoc = $this->oDOMDoc;
		// init the $aFilterTags array
		$aFilterTags = array();
		$aFilterTags['offset'] = 0;
		$aFilterTags['limit'] = '';
		$aFilterTags['sql'] = '';
		$aFilterTags['interval'] = '';
		$aFilterTags['tag'] = '';
		$aFilterTags['orderby'] = '';
		$aFilterTags['gwAddons'] = array();
		/* Check if there is an item and filter element */
		if ($this->oDOMItem = $this->oDOMDoc->getNode('items:item',$this->oDOMFolder)) {
			//Filter
			if ($oDOMFilter = $this->oDOMDoc->getNode('items:filter',$this->oDOMItem)){
				//Limit?
				if ($oDOMLimit = $this->oDOMDoc->getNode("items:limit",$oDOMFilter)) {
					$aFilterTags['limit'] = $this->oDOMDoc->getNodeValue('items:limit',$oDOMFilter);
				}
				if ($oDOMLimit = $this->oDOMDoc->getNode("items:offset",$oDOMFilter)) {
					$this->aData['offset'] = $aFilterTags['offset'] = $this->oDOMDoc->getNodeValue('items:offset',$oDOMFilter);
				}
				if ($oDOMOrderBy = $this->oDOMDoc->getNode("items:order_by",$oDOMFilter)) {
					$aFilterTags['orderby'] = $this->oDOMDoc->getNodeValue('items:order_by',$oDOMFilter);
				}
				//FIXME : security chceck ?
				if ($oSQL = $this->oDOMDoc->getNode('items:sql',$oDOMFilter)) {
					$aFilterTags['sql'] = $this->oDOMDoc->getNodeValue("items:sql",$oDOMFilter);
				}
				if ($oDOMInterval = $this->oDOMDoc->getNode('items:interval',$oDOMFilter)) {
					$aFilterTags['interval'] = $this->oDOMDoc->getNodeValue("items:interval",$oDOMFilter);
				}
				if ($oDOMFulltext = $this->oDOMDoc->getNode('items:fulltext',$oDOMFilter)) {
					$aFilterTags['fulltext'] = $this->oDOMDoc->getNodeValue("items:fulltext",$oDOMFilter);
				}
			}
			//If item id is specified , return only this item
			if ($sIID = $this->oDOMItem->getAttribute('uid')) {
				if (strlen($aFilterTags['sql'])) {
					$aFilterTags['sql'] = '(' . $aFilterTags['sql'] . ') AND ';
				}
				$aFilterTags['sql'] .= "item_id='".$sIID."'";
			}
			//Tags
			if ($oDOMValues = $this->oDOMDoc->getNode('items:values',$this->oDOMItem)){
				$tagvalues = Tools::makeArrayFromXML($oDOMValues,true);
				$tags = @array_keys(Tools::makeArrayFromXML($oDOMValues,true));
				foreach ($tags as $key=>$tag){
					// rename some tags - their value is different in the protocol than in the server
					switch (strtolower($tag)) {
						case 'from':
							$tags[$key] = 'header_from';
							break;
						case 'to':
							$tags[$key] = 'header_to';
							break;
						case 'ctz':
							unset($tags[$key]);
							$aFilterTags['timezone'] = $tagvalues[strtolower($tag)];
							break;
						// case 'all_headers':
						// $tags[$key] = '*';
						// break;
						default:
							// do nothing
							break;
					}
				}
				//GW FIX
				$aFilterTags['tag']= @implode(',',$tags);
			}
			//XML parser trouble fix
			if ($aFilterTags['tag']=="#text") {
				$aFilterTags['tag']=false;
			}
			if (isset($this->oFolder->gw) && $this->oFolder->gw) {
				$aAddons = array(
					'recurrence'=>'recurrence',
					'reminders'=>'reminder',
					'contacts'=>'contact',
					'note'=>'note',
					'locations'=>'location',
				);
				
				foreach ($aAddons as $key => $addon) {
					if ($subnode = $this->oDOMDoc->getNode("items:".$key,$this->oDOMItem)) {
						$aFilterTags['gwAddons'][] = $addon;
					}
				}
				//If item id is specified , return only this item
				if ($sIID) {
					$sPrefix = isset($this->type) && $this->type=='C' ? 'ITM' : 'EVN';
					$aFilterTags['sql'] = $sPrefix."_ID='".$sIID."'";
				}
			}
			$oDOMItem = $this->oDOMItem;
		}
		return $aFilterTags;
	}
	/**
	* @brief Converts folder array into template compatible data array
	* @param $folders Array of folder objects
	* @return folders array in the form for template system
	*/
	public static function cnvItems($items,$aFilterTag = array(),$bSingle = false,$partID = 1,&$account = false,&$folder = false)
	{
		if (!is_array($items)) {
			return;
		}
		
		$rssDoc = false;
		foreach ($items as $oItem) {
			$aItem = array();
			$aItem["id"] = $oItem->itemID;
			switch($oItem->wmclass) {
				//Mail
				case 'M':
					$aItem = self::cnvMailItem($oItem,$aFilterTag,$partID,true);
					$aItem["mail"] = 1;
					break;
				case 'R':
					$aItem = self::cnvRSSItem($oItem,$aFilterTag,$partID,$account,$folder,$rssDoc);
					$aItem["mail"] = 1;
					break;
				//GroupWare
				case 'GW':
				/*Fixme : do not store sFields in GroupWareItem, call XML transformation in this method */
					$aItem["sFields"] = $oItem->sFields;
					break;
				//Quarantine
				case 'Q':
					$aItem["owner"] = (string) $oItem->owner;
					$aItem["sender"] = (string) $oItem->sender;
					$oldAFilter = $aFilterTag;
					$array = explode(",",$aFilterTag['tag']);
					foreach($array as $item) {
						$array2[trim($item)] = 1;
					}
					$aFilterTag['tag'] = $array2;
					if ($aFilterTag['tag']['header_to']) {
						$aItem["to"] = $aItem["owner"];
					}
					if ($aFilterTag['tag']['header_from']) {
						$aItem["from"] = $aItem["sender"];
					}
					$aItem["subject"] = (string) $oItem->subject;
					$aItem["domain"] = (string) $oItem->domain;
					$aItem["createdat"] = (string) $oItem->createdAt;
					$aItem["createdon"] = (string) $oItem->createdOn;
					$value = jdtounix($oItem->createdOn) + $oItem->createdAt -$_SESSION['TIMEZONE'];
					$aItem["date"] = (string) $value;
					$aItem['id'] = base64_encode($aItem['id']);
					//if ($_iqdata['format']=='xml')
					if ($bSingle && $oItem->folder->type=='Q') {
						$aItem = array_merge(self::cnvMailItem($oItem,$oldAFilter),$aItem);
						$aItem["mail"] = 1;
						$aItem["quarantine"] = 0;
						// dmp($aItem);die();
					}else{
						$aItem = Tools::htmlspecialchars_array($aItem);  
						$aItem["quarantine"] = 1;
						$aItem["mail"] = 0;
					}
					break;
			}
			
			$aItems[] = $aItem;
			unset($aItem);
		}
		return $aItems;
	}

	static public function cnvMailItem($oItem,$aFilterTag = array(),$partID = 1,$htmlSpecial = true)
	{
		global $_iqdata;
		// Default message tags
		if ($aFilterTag['tag']=='*') {
			$aFilterTag['tag'] = 'item_id, rid, size, date, flags, priority, static_flags, header_from, header_to, subject, html, text, color, bcc, cc, smime_status, has_attachment, attachments, confirm_addr';
		}
		$array = explode(",",$aFilterTag['tag']);
		foreach($array as $item) {
			$array2[trim($item)] = 1;
		}
		$aFilterTag['tag'] = $array2;
		$aItems = array();

		//Do we have to parse a message?
		if (
			isset($aFilterTag['tag']['html'])
		||
			isset($aFilterTag['tag']['attachments'])
		||
			isset($aFilterTag['tag']['text'])
		||
			isset($aFilterTag['tag']['cc'])
		||
			isset($aFilterTag['tag']['bcc'])
		||
			isset($aFilterTag['tag']['confirm_addr'])
		||
			isset($aFilterTag['tag']['all_headers']) 
		) {
			$message = $oItem->parseMessage(true,$partID,$newMessage);
			//If we parsing message for the firts time , return an imip invitation info
			/* if ($newMessage && $message['imip'] && isset($aFilterTag['tag']['imip'])){
				//Load imip object
				$iMIP = iMIP::load($oItem->folder->account->gwAccount);
				//Versit in XML format
				$aItem["imip"] = (string) $iMIP->convertVersit($message['imip']);
				dmp($message['imip']);die();
			}*/
			if (isset($aFilterTag['tag']['reply_to'])) {
				$aItem["reply_to"] = (string) ($message['headers']['reply-to']);
			}
			if (isset($aFilterTag['tag']['all_headers'])) {
				$aItem["all_headers"] = $message['headers_plain'];
			}
			//TEXT body
			if (isset($aFilterTag['tag']['text']) && $aFilterTag['tag']['text']) {
				if (isset($message['plain_body'])) {
					$sBodyToIndex = $aItem["text"] = rtrim($message['plain_body']);
				} else {
					$aItem["text"] = '';
				}
			}
			//HTML body
			
			if (isset($aFilterTag['tag']['html']) && $aFilterTag['tag']['html']) {
				if ($message['html_body']) {
					$value = Tools::cleanHTML($message['html_body'],$base);
					
					//Check if we do not overwriting the plain text body by empty html
					if ($_SESSION['FULLTEXT'] && $value) {
						$t = time();
						$m = microtime();
						$sBodyToIndex  = icewarp_get_message_content($oItem->getMessageFile(),'',DB_BODY_LIMIT,0);
					}
					if ($base) {
						$aItem["base"] = $base;
					}
				} else {
					if ($message['plain_body']){
						$value = Tools::text2html(trim($message['plain_body']), true);
						$sBodyToIndex = $value;
					} else {
						$value = '';
					}
				}
				$aItem["html"] = $value;
			}

			
			if ($_SESSION['FULLTEXT'] && $sBodyToIndex){
				if (strtolower($oItem->wmclass) == 'm') {
					$oItem->updateItem(array('body'=>substr($sBodyToIndex,0,32000)));
				}
			}
			//Attachments
			if (
				isset($aFilterTag['tag']['attachments']) 
			&& 
				$aFilterTag['tag']['attachments'] 
			&& 
				$message['attachments']
			) {
				foreach($message['attachments'] as $attachment){
					$attachment['size'] = (int) $attachment['size'];
					$aItem['attachments']['num'][] = $attachment;
				}
			}
			//Carbon copies
			if (
				isset($aFilterTag['tag']['cc']) 
			&& 
				$aFilterTag['tag']['cc'] 
			&& 
				$message['headers']['cc']
			) {
				$aItem['cc'] = (string) ($message['headers']['cc']);
			}
			//Blind carbon copies
			if (
				isset($aFilterTag['tag']['bcc']) 
			&& 
				$aFilterTag['tag']['bcc'] 
			&& 
				$message['headers']['bcc']
			) {
				$aItem['bcc'] = (string) ($message['headers']['bcc']);
			}
			//X-Confirm-Reading-To
			//Disposition-notification-to
			if (
				isset($aFilterTag['tag']['confirm_addr'])
			&& 
				$aFilterTag['tag']['confirm_addr']
			&& 
				(
					$message['headers']['custom-headers']['x-confirm-reading-to']
				|| 
					$message['headers']['custom-headers']['disposition-notification-to']
				)
			) {
				$aItem['confirm_addr'] = (string) ($message['headers']['custom-headers']['x-confirm-reading-to']);
				if (!$aItem['confirm_addr']) {
					$aItem['confirm_addr'] = (string) ($message['headers']['custom-headers']['disposition-notification-to']);
				}
			}
			if (isset($message['headers']['subject'])) {
				$oItem->subject = $message['headers']['subject'];
			}
			if (isset($message['headers']['from'])) {
				$oItem->from =  $message['headers']['from']; 
			}
			if (isset($message['headers']['to'])) {
				$oItem->to =  $message['headers']['to'];
			}
			if (isset($message['headers']['size'])) {
				$oItem->size =  $message['headers']['size'];
			}
			if (isset($message['headers']['date'])) { 
				$oItem->formattedDate = $message['headers']['date'];
			}
		}
		if (isset($aFilterTag['tag']['static_flags'])) {
			$aItem["static_flags"] = (string) $oItem->staticFlags;$aItem["has_st_flags"] = true;
		}
		if (isset($aFilterTag['tag']['smime_status'])) {
			$aItem["smime_status"] = (string) $oItem->sMimeStatus;$aItem["has_smime"] = true;
		}
		if (isset($aFilterTag['tag']['color'])) {
			$aItem["color"] = (string) $oItem->color;$aItem["has_color"] = true;
		}
		if (isset($aFilterTag['tag']['has_attachment'])) {
			$aItem["has_attachment"] = $oItem->hasAttachments ? 'true' : 'false';
		}
		if (isset($aFilterTag['tag']['flags'])) {
			$aItem["flags"] = (string) $oItem->flags;$aItem["has_flags"] = true;
		}
		if (isset($aFilterTag['tag']['size'])) {
			$aItem["size"] = (string) $oItem->size;
		}
		if (isset($aFilterTag['tag']['reply_to'])) {
			$aItem["reply_to"] = (string) ($message['headers']['reply-to']);
		}
		if (isset($aFilterTag['tag']['priority'])) {
			$aItem["priority"] = (string) $oItem->priority;
		}
		if (isset($aFilterTag['tag']['date'])) {
			$aItem["timestamp"] = $oItem->date;
			if(!$oItem->formattedDate){
				$oItem->formattedDate = $oItem->date;
			}
			$aItem["date"] = (string) $oItem->formattedDate;
		}
		if (isset($aFilterTag['tag']['subject'])) {
			$aItem["subject"] = (string) ($oItem->subject); $aItem["has_subject"] = true;
		}
		if (isset($aFilterTag['tag']['header_from'])) {
			$aItem["from"] = (string) iconv_substr($oItem->from,0,2000,'UTF-8');
		}
		if (isset($aFilterTag['tag']['header_to'])) {
			if ($oItem->to) {
				$aItem["to"] = (string) ($oItem->to);
			}
		}
		$aItem['id']=$oItem->itemID;
		if($htmlSpecial){
			$aItem = Tools::htmlspecialchars_array($aItem);
		}
		return $aItem;
	}
	private static function cnvRSSItem($oItem,$aFilterTag = array(),$partID = 1,&$account = false,&$folder = false,&$rssDoc = false)
	{
		global $_iqdata;
		// Default message tags
		if ($aFilterTag['tag']=='*') {
			$aFilterTag['tag'] = 'item_id, rid, size, date, flags, priority, static_flags, header_from, header_to, subject, html, text, color, bcc, cc, smime_status, has_attachment, attachments, confirm_addr';
		}
		$array = explode(",",$aFilterTag['tag']);
		foreach($array as $item) {
			$array2[trim($item)] = 1;
		}
		$aFilterTag['tag'] = $array2;
		$aItems = array();
		//Do we have to parse a message?
		if (
			isset($aFilterTag['tag']['html']) 
		|| 
			isset($aFilterTag['tag']['text'])
		) {
			$rss = RSS::instance($account);
			//TEXT body
			if (isset($aFilterTag['tag']['text']) && $aFilterTag['tag']['text']) {
				$sBodyToIndex = $aItem["text"] = $oItem->getText();
			}
			//HTML body
			if (isset($aFilterTag['tag']['html']) && $aFilterTag['tag']['html']) {
				$aItem["html"] = $oItem->getHTML($aItem);
				if (!$sBodyToIndex) {
					$sBodyToIndex = $aItem["html"];
				}
			}
			//if ($_SESSION['FULLTEXT'] && $sBodyToIndex){
				// $oItem->updateItem(array('body'=>substr($sBodyToIndex,0,32000)));
			//}
		}
		if (isset($aFilterTag['tag']['static_flags'])) {
			$aItem["static_flags"] = (string) $oItem->staticFlags;$aItem["has_st_flags"] = true;
		}
		if (isset($aFilterTag['tag']['smime_status'])) {
			$aItem["smime_status"] = (string) $oItem->sMimeStatus;$aItem["has_smime"] = true;
		}
		if (isset($aFilterTag['tag']['color'])) {
			$aItem["color"] = (string) $oItem->color;$aItem["has_color"] = true;
		}
		if (isset($aFilterTag['tag']['has_attachment'])) {
			$aItem["has_attachment"] = $oItem->hasAttachments ? 'true' : 'false';
		}
		if (isset($aFilterTag['tag']['flags'])) {
			$aItem["flags"] = (string) $oItem->flags;$aItem["has_flags"] = true;
		}
		if (isset($aFilterTag['tag']['size'])) {
			$aItem["size"] = (string) $oItem->size;
		}
		if (isset($aFilterTag['tag']['reply_to'])) {
			$aItem["reply_to"] = (string) ($message['headers']['reply-to']);
		}
		if (isset($aFilterTag['tag']['priority'])) {
			$aItem["priority"] = (string) $oItem->priority;
		}
		if (isset($aFilterTag['tag']['date'])) {
			$aItem["date"] = (string) $oItem->date;
		}
		if (isset($aFilterTag['tag']['subject'])) {
			$aItem["subject"] = (string) htmlspecialchars($oItem->subject); $aItem["has_subject"] = true;
		}
		if (isset($aFilterTag['tag']['header_from'])) {
			$aItem["from"] = (string) ($oItem->from);
		}
		// if (isset($aFilterTag['tag']['header_to'])) if ($oItem->to) $aItem["to"] = (string) ($oItem->to);
		$aItem['id'] = $oItem->itemID;
		return $aItem;
	}
	public static function iMIP_Email($oItem,$sAction = 'invite',$aAttachment = array(),$aAttachmentID = false,$sSingleAttendee = false)
	{
		$oAccount = (strtolower($oItem->wmclass) == 'gw')?$oItem->folder->account->account:$oItem->folder->account;
		if (!$aAttachment){
			//Create invitation
			$oInvitation = $oItem->createInvitation(true);
			//Create attachment
			$aAttachment = $oItem->createInvitationAttachment($oInvitation);
		}
		//Create invitation e-mail
		$oMail = new Mail('UTF-8',true);
		$oMail->setFrom($oAccount->accountID,$oAccount->fullname);
		switch($sAction){
			case 'change':
			case 'invite':
				if (!$oItem->aAttendees){
					//Add to from all attendees with RSVP = 1
					$oAttendees = $oItem->aAddons['contact'];
					$aAttendees = $oAttendees->getData();
				} else {
					$aAttendees = $oItem->aAttendees;
				}
				//Invitation for attendees
				$bSend = false;
				if ($aAttendees) foreach($aAttendees as $aAttendee) {
					if ($aAttendee['CNTRSVP']) {
						//in attendee calendar ?
						if (($sSingleAttendee===false) || ($aAttendee['CNTEMAIL']==$sSingleAttendee)) {
							$oMail->addTo($aAttendee['CNTEMAIL'],$aAttendee['CNTCONTACTNAME']);
							$bSend = true;
						}
					}
				}
				//Event title
				$sTitle = ($oItem->item['EVNTITLE']?$oItem->item['EVNTITLE']:'Event');
				$sMethod = 'REQUEST';
				$aSubjData['TITLE'] = $sTitle;
				$aSubjData['ORGANIZER'] = ($oAccount->fullname?$oAccount->fullname:$oAccount->accountID);
				if ($sAction=='change') {
					$sSubject = self::fixSubject($_SESSION['IMIP_SUBJECT_UPDATE'],$aSubjData);
				} else {
					$sSubject = self::fixSubject($_SESSION['IMIP_SUBJECT_INVITE'],$aSubjData);
				}
				break;
			case 'accept':
				//Parse info from XML
				$xml = simplexml_load_string($aAttachment['xml']);
				$oPointer = self::getIMIPTag($xml);
				$sAddress = str_replace("mailto:","",strval($oPointer->ORGANIZER));
				$bSend = true;
				//Reply to organizer
				$oMail->addTo($sAddress,$sName);
				$sTitle = strval($oPointer->SUMMARY);
				$sMethod = 'REPLY';
				$aSubjData['TITLE'] = $sTitle;
				$aSubjData['ATTENDEE'] = ($oAccount->fullname?$oAccount->fullname:$oAccount->accountID);
				$sSubject = self::fixSubject($_SESSION['IMIP_SUBJECT_ACCEPT'],$aSubjData);
				break;
			case 'decline':
				if (strtolower($oItem->wmclass)=='gw'){
					$xml = simplexml_load_string($aAttachment['imip_xmlstr']);
				} else {
					$aInfo = $oItem->parseMessage();
					foreach($aInfo['attachments'] as $aAttach) {
						if ($aAttach['part_id']==$aAttachmentID) {
							$xml = $aAttach['imip_xml'];
						}
					}
				}
				$oPointer = self::getIMIPTag($xml);
				$sAddress = str_replace("mailto:","",strval($oPointer->ORGANIZER));
				$bSend = true;
				//Reply to organizer
				$oMail->addTo($sAddress);
				$sTitle = strval($oPointer->SUMMARY);
				$sMethod = 'REQUEST';
				$aSubjData['TITLE'] = $sTitle;
				$aSubjData['ATTENDEE'] = ($oAccount->fullname?$oAccount->fullname:$oAccount->accountID);
				$sSubject = self::fixSubject($_SESSION['IMIP_SUBJECT_DECLINE'],$aSubjData);
				break;
			case 'cancel':
				$bSend = false;
				if ($sSingleAttendee) {
					$oMail->addTo($sSingleAttendee);
					$bSend = true;
				} else {
					if (!$oItem->aAttendees){
						//Add to from all attendees with RSVP = 1
						$oAttendees = $oItem->aAddons['contact'];
						$aAttendees = $oAttendees->getData();
					} else {
						$aAttendees = $oItem->aAttendees;
					}
					if ($aAttendees) { 
						foreach($aAttendees as $aAttendee) {
							if (($sSingleAttendee===false) || ($aAttendee['CNTEMAIL']==$sSingleAttendee)){
								$oMail->addTo($aAttendee['CNTEMAIL'],$aAttendee['CNTCONTACTNAME']);
								$bSend = true;
							}
						}
					}
				}
				$sTitle = ($oItem->item['EVNTITLE']?$oItem->item['EVNTITLE']:'Event');
				$sMethod = 'REQUEST';
				$aSubjData['TITLE'] = $sTitle;
				$aSubjData['ORGANIZER'] = ($oAccount->fullname?$oAccount->fullname:$oAccount->accountID);
				$sSubject = self::fixSubject($_SESSION['IMIP_SUBJECT_CANCEL'],$aSubjData);
				$oMail->setPriority('high');
				break;
		}
		//Set the invitation subject
		$oMail->setSubject($sSubject);
		//Set the invitation body
		//$oMail->setBody("Hello,\nYou have been invited to the following event(s):".$sTitle."\n\nLet me know about your presence...");
		//Add the invitation attachment
		$fullPath = $aAttachment['fullpath'];
		//Get file and folder
		$lastSlash = strrpos($fullPath,"/");
		$item = substr($fullPath,$lastSlash+1);
		$folder = substr($fullPath,0,$lastSlash);
		$aAttachments = $oAccount->user->getAttachments($folder);
		$aAttachData = $aAttachments[$item];
		$oMail->addAttachment($aAttachData['file'],'text/calendar',$sTitle.imipext,$sMethod);
		try{
			if ($bSend) {
				$oMail->send();
			}
		} catch(Exc $e) {
			if ($sAction!='cancel') {
				throw $e;
			}
		}
	}
	private static function getIMIPTag($xml)
	{
		if ($xml->VEVENT) {
			return $xml->VEVENT;
		}
		if ($xml->VTODO) {
			return $xml->VTODO;
		}
		if ($xml->VJOURNAL) {
			return $xml->VJOURNAL;
		}
	}
	private static function fixSubject($sSubject,$aData)
	{
		$sSubject = str_replace('%ORGANIZER%',$aData['ORGANIZER'],$sSubject);
		$sSubject = str_replace('%TITLE%',$aData['TITLE'],$sSubject);
		$sSubject = str_replace('%ATTENDEE%',$aData['ATTENDEE'],$sSubject);
		return Tools::unhtmlspecialchars($sSubject);
	}
}
?>