<?php
/**
 * @brief		Class for handling webmailIqMessage namespace
 * @author Pavel Perna
 * @date 17-04-2007
 */
require_once('inc/mail.php');
require_once('inc/include.php');

class WebmailIqMessage extends XMLRequestInterface
{
	/**
		* @brief Public: Constructor
		* @param $oQuery A DOMElement filled with query element
		* @param $oDoc	A DOMDocument of the request
		* @param $attrs  An Array with parent attributes
		*/
		public function __construct($oDOMQuery,&$oDOMDoc,&$attrs)
		{
			$this->oDOMQuery = $oDOMQuery;
			$this->oDOMDoc = &$oDOMDoc;
			$this->aAttrs = &$attrs;
			$this->addrList = array();
			$this->checkInputXML();
			$this->exeInputXML();
		}
	/**
		* @brief	Private - Checks if the input XML is valid
		* @return  TRUE/FALSE and throws exception
		*/
		private function checkInputXML()
		{
			global $aError;
			
			if ($this->oDOMMessage = $this->oDOMDoc->getNode('message:message', $this->oDOMQuery))
			{
				$this->queryType = 'regular';
				$result = $this->checkMessageXML();
			// message request can be for dialing as well
			} else if ($this->oDOMMessage = $this->oDOMDoc->getNode('message:dial', $this->oDOMQuery)) {
				$this->queryType = 'phonecall';
				$result = $this->checkDialXML();
			} else
				throw new Exc('message_missing_tag','message');
			
			
		}
		
		private function checkDialXML()
		{
			global $aError;
			
			if (!$oDOMAction = $this->oDOMDoc->getNode('message:phone', $this->oDOMMessage))
				throw new Exc('message_missing_tag','phone');
			
			$this->sPhoneNumber = $this->oDOMDoc->getNodeValue('message:phone', $this->oDOMMessage);

			return true;
		}
			
		private function checkMessageXML()
    {
				global $aError;
				$this->bAllowEmptyReceipient = false;
				if ($this->aAttrs['type']=='get'){

                    $this->sAID = $this->oDOMMessage->getAttribute('account');  
                    $this->sFID = $this->oDOMMessage->getAttribute('folder');
                    $this->sIID = $this->oDOMMessage->getAttribute('uid');
              
                    if (!isset($this->sAID))
                          throw new Exc('message_missing_account_id');
                    if (!isset($this->sFID))
                          throw new Exc('message_missing_folder_id');
                    if (!isset($this->sIID))
                          throw new Exc('message_missing_item_id');  
                    
                    $this->aData['mid'] = $this->sIID;        
       }
        
        
        if ($this->aAttrs['type']=='set')
        {
        	//Get action and do some things depending on it
	        if (!$oDOMAction = $this->oDOMDoc->getNode('message:action', $this->oDOMMessage))
	        		throw new Exc('message_missing_tag','action');
              
					//Send as email ?
          if ($this->sSEmail = $this->oDOMDoc->getNode('message:send_as_email', $oDOMAction))
                $this->sSEmail = $this->oDOMDoc->getNodeValue('message:send_as_email',
                                                            $oDOMAction)=='true'?true:false;   
          
          //Send as Instant Messaging ? 
          if ($this->sSIM = $this->oDOMDoc->getNode('message:send_as_im', $oDOMAction))
                $this->sSIM = $this->oDOMDoc->getNodeValue('message:send_as_im',
                                                        $oDOMAction)=='true'?true:false;    
          
          //Keep attachements ? 
          if ($this->sKA = $this->oDOMDoc->getNode('message:keep_attachments', $oDOMAction))
                $this->sKA = $this->oDOMDoc->getNodeValue('message:keep_attachments',
                                                           $oDOMAction)=='true'?true:false;                                         

          //Send message with HTML attachment ? 
          if ($this->bHTML = $this->oDOMDoc->getNode('message:html_body', $oDOMAction))
                $this->bHTML = $this->oDOMDoc->getNodeValue('message:html_body',
                                                           $oDOMAction)=='true'?true:false;                                         

          
          //Add recipient automatically to address book ? 
          if ($this->bAutoABook = $this->oDOMDoc->getNode('message:auto_addressbook', $oDOMAction))
                $this->bAutoABook = $this->oDOMDoc->getNodeValue('message:auto_addressbook',
                                                           $oDOMAction)=='true'?true:false;
          
          //Save to folder ?
          if ($oDOMSave = $this->oDOMDoc->getNode('message:save_to_folder', $oDOMAction))
                $this->sSave = true;
          else  
								$this->sSave = false;

          //Encrypt message ? 
          if ($this->sEncrypt = $this->oDOMDoc->getNode('message:encrypt', $oDOMAction))
                $this->sEncrypt = $this->oDOMDoc->getNodeValue('message:encrypt',
                                                           $oDOMAction)=='true'?true:false; 

          //Sign the message ? 
          if ($this->sSign = $this->oDOMDoc->getNode('message:sign', $oDOMAction))
                $this->sSign = $this->oDOMDoc->getNodeValue('message:sign',
                                                           $oDOMAction)=='true'?true:false; 
          //SMTP relay
          if ($this->bRelay = $this->oDOMDoc->getNode('message:smtp_relay', $oDOMAction))
                $this->bRelay = $this->oDOMDoc->getNodeValue('message:smtp_relay',
                                                           $oDOMAction)=='true'?true:false;

					//Has any action been specified?
					if (!$this->sSEmail && !$this->sSIM && !$this->sSave)
							throw new Exc('message_invalid_action');  	  
		  		
				// Get email properties
				$this->sCharset = $this->oDOMDoc->getNodeValue('message:charset', $this->oDOMMessage);
				$this->sSubject = $this->oDOMDoc->getNodeValue('message:subject', $this->oDOMMessage);
    		$this->sBody = $this->oDOMDoc->getNodeValue('message:body', $this->oDOMMessage);
    		$this->sTo = $this->oDOMDoc->getNodeValue('message:to', $this->oDOMMessage);
        $this->sCC = $this->oDOMDoc->getNodeValue('message:cc', $this->oDOMMessage);
    		$this->sBCC = $this->oDOMDoc->getNodeValue('message:bcc', $this->oDOMMessage);


				$this->oDistrib = $this->oDOMDoc->getNode('message:distrib', $this->oDOMMessage); 
	
	
				if (!$this->sFrom = $this->oDOMDoc->getNodeValue('message:from', $this->oDOMMessage))
		  			  throw new Exc('message_missing_tag','from');
		  		
	  		  // Has any recipient been specified?
				if ($this->sSEmail && !$this->sTo && !$this->sCC && !$this->sBCC && !$this->oDistrib)
	  	  			throw new Exc('message_missing_tag','recipient(to,cc,bcc..)');
	  	  	
				if ($this->sSIM && !$this->sTo)
	  	  			throw new Exc('message_missing_tag','to');
	  	
	  
			  $this->sCharset = strtoupper($this->sCharset);
			  if($this->sCharset=='GB2312')
			     $this->sCharset = 'GBK';


	  	  	if ($this->oDOMAttachments = $this->oDOMDoc->getNode('message:attachments', $this->oDOMMessage))
	  	  	      $this->sAttachments = true;
	        else
	              $this->sAttachments = false;
	        
	          
          //If save to folder
          if ($this->sSave){
                    		
                if(!$this->sTo){
            		  $this->bAllowEmptyReceipient = true;
            		}
            		
                //Account to save message
                if (!$this->sSaveAccount = $this->oDOMDoc->getNode('message:account', $oDOMSave))
                	throw new Exc('message_missing_tag','account');
                $this->sSaveAccount = $this->oDOMDoc->getNodeValue('message:account', $oDOMSave);
                
                //Folder to save message
                if (!$this->sSaveFolder = $this->oDOMDoc->getNode('message:folder', $oDOMSave))
                  throw new Exc('message_missing_tag','folder');
								$this->sSaveFolder = $this->oDOMDoc->getNodeValue('message:folder', $oDOMSave);
                
                //Item to save message
                if ($this->sSaveItem = $this->oDOMDoc->getNode('message:item', $oDOMSave)){
                  $this->sSaveItem = $this->oDOMDoc->getNodeValue('message:item', $oDOMSave);
               	}
          }
	                             
        }
        return true;
	  	}
	  /**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/
		private function exeInputXML()
		{
			$oUser = &$_SESSION['user'];
			$this->session_user = $oUser;

			switch($this->aAttrs['type'])
			{
				//Retrieve message to be edited
				case 'get':
					if ($this->queryType !== 'regular')
						throw new Exc('message_get_not_allowed');
					$this->parseMessage();
				break;
				case 'set':
					switch ($this->queryType)
					{
						case 'regular':
							$this->sendMessage();
						break;
					
						case 'phonecall':
							$this->dialPhone();
						break;
					}
				break;
			}
		}
		
		/* Parse message and store
		 * the data in it                    OBSOLETE !!!!
		 */		 
		private function parseMessage()
		{
			$oUser = &$this->session_user;
			$oCache = Cache::instance($oUser);
			
			//Template path
			$this->sTemplateFile = 'webmailiqmessage_get';
			
			//Get correct item
			$oAccount = $oUser->getAccount($this->sAID);
			$oFolder = $oAccount->getFolder($this->sFID); 	        
			$oItem = $oFolder->getItem($this->sIID);
			
			//get message structure
			$aMessage = $oItem->parseMessage();
			
			//Fill in template data
			$this->aData['body'] = $aMessage['plain_body']; 
			
			if ($aMessage['attachments'][0]) 
				$this->aData['attachments']['num'] = $aMessage['attachments'];
			
			if ($aMessage['cc']) $this->aData['cc'] = $aMessage['cc'];
			if ($aMessage['bcc']) $this->aData['bcc'] = $aMessage['bcc'];
			if ($aMessage['reply_to']) $this->aData['reply_to'] = $aMessage['reply_to'];
			if ($aMessage['html_body']) $this->aData['html'] = $aMessage['html_body']; 

			$this->aData['to'] = $oItem->to;
			$this->aData['from'] = $oItem->from;
			$this->aData['subject'] = $oItem->subject;
		}
		
		
		/* Sends a message
		*/
		private function sendMessage()
		{
			$oUser = &$this->session_user;
			$this->sTemplateFile = 'webmailiqmessage_set';
			$oMail = &new Mail($this->sCharset,$this->bRelay,$this->bAllowEmptyReceipient);
			//Process from address
			$aFrom = MailParse::parseAddresses($this->sFrom);
			$oMail->setFrom($aFrom[0]['address'],$aFrom[0]['display'],true);
			//aFolderList stores all folders used to cache attachments
			$this->aFolderList = array();
			//Get to addresses and add them to the message
			if ($this->sTo)
			{
				$aToArray = MailParse::parseAddresses($this->sTo);
				
				foreach($aToArray as $key => $aToItm){
				  //Add @primarydomain to email addresses without it 
				  
          if(strpos($aToItm['address'],'@')===false && $this->sSEmail){
          	if($aToArray[$key]['address'])
            	$aToArray[$key]['address'] .= '@'.$_SESSION['PRIMARY_DOMAIN'];
          }
          $oMail->addTo($aToArray[$key]['address'],$aToItm['display']);
        }
				
				
				$this->addrList = array_merge($this->addrList, $aToArray);
			}

			//Set the message subject
			if ($this->sSubject)
				$oMail->setSubject($this->sSubject);
			
			//Set carbon copy
			if ($this->sCC)
			{
				$aCc = MailParse::parseAddresses($this->sCC);
				foreach($aCc as $key => $aCcItm){
					//Add @primarydomain to email addresses without it 
          if(strpos($aCcItm['address'],'@')===false && $this->sSEmail){
          	if($aCc[$key]['address'])
            	$aCc[$key]['address'] .= '@'.$_SESSION['PRIMARY_DOMAIN'];
          }
					$oMail->addCC($aCc[$key]['address'],$aCcItm['display']);
				}
				$this->addrList = array_merge($this->addrList, $aCc);
			}
			
			//Browse distributed lists - accounts
			foreach($this->oDOMDoc->query('message:account',$this->oDistrib) as $dAccount)
			{
				$sAID = $dAccount->getAttribute('uid'); 
				//Folders
				foreach($this->oDOMDoc->query('message:folder',$dAccount) as $dFolder)
				{
					$sFID = $dFolder->getAttribute('uid');
					self::processDistributedList('message',$oUser,$this->oDOMDoc,$sAID,$sFID,$dFolder,$oMail);
				}
			}
			
			//Set blind carbon copy
			if ($this->sBCC)
			{
				$aBcc = MailParse::parseAddresses($this->sBCC);
				foreach ($aBcc as  $key => $aBccItm){
					//Add @primarydomain to email addresses without it
					if (strpos($aBccItm['address'], '@') === false && $this->sSEmail)
					{
						if ($aBcc[$key]['address'])
							$aBcc[$key]['address'] .= '@' . $_SESSION['PRIMARY_DOMAIN'];
					}
					$oMail->addBCC($aBcc[$key]['address'], $aBccItm['display']);
				}
				$this->addrList = array_merge($this->addrList, $aBcc);
				$this->aBcc = $aBcc;
			}
			
			//X-Headers
			if ($oDOMXHeader = $this->oDOMDoc->getNode('message:custom_headers',$this->oDOMMessage))
			{
				foreach($this->oDOMDoc->query('//message:header',$oDOMXHeader) as $oDOMXHeaderNode)
				{
					$aHeaderArr = Tools::makeArrayFromXML($oDOMXHeaderNode);
					$oMail->addHeader($aHeaderArr['#text']);
				}
			}
			
			//Body handling
			if (isset($this->sBody)) {
				$plainBody = Tools::removeHTML($this->sBody);
				if ($this->bHTML) {
					$this->processEmbeddedAttachments($oMail);

					$oMail->SetBody($plainBody,$this->sBody);
				} else {
					$oMail->SetBody($plainBody);
				}
			}


			//Attachements
			if ($this->sAttachments)
				$this->processAttachments($oMail);
				
			$this->execFinalMessageAction($oMail);
  
		}
		
		/*Process an distributed list in
		 * @param $namespace - "items|message"
		 * @param $oUser - user object 
		 * @param $oDOMDoc - DOM document handler
		 * @param $accountID - Account containg Dlist
		 * @param $folderID - Folder containing Dlist
		 * $oDOMFdr - content of the <folder> element
		 * $oMail - output parameter (adds an recipients to the $oMail)
		 */
                                        		 
		public static function processDistributedList($namespace,$oUser,$oDOMDoc,$accountID,$folderID,$oDOMFdr,&$oMail)
		{
    
      $oAccount = $oUser->getAccount($accountID);
      $oFolder = $oAccount->getFolder($folderID);
      
      $aDest = array(0=>'to',1=>'cc',2=>'bcc');
      
      foreach ($aDest as $dest){
        foreach($oDOMDoc->query($namespace.':'.$dest,$oDOMFdr) as $dst)
        {
          //Distributed list name
          $value = $dst->nodeValue;
          
          //Retrieve first d. list from database
          $aFilter['sql'] = "itmtitle LIKE '".$value."' AND itmclass = 'L'";
          $aFilter['limit'] = "0,1";
          $contacts = $oFolder->getItems($aFilter);
          $oDL = reset($contacts);
          
          
          $sDLID = $oDL->item['ITM_ID'];
          $oDL = $oFolder->getItem($sDLID);
          
          $aAddons = $oDL->getAddons();
          $aLocations = $aAddons['location']->getData();
          
          foreach($aLocations as $location)
          {
            $email = $location['LCTEMAIL1'];
            $methodName = 'add'.$dest;
            
            if (strtolower($email)!='undefined')
              $oMail->$methodName($email);
          }  
        }   
      }
    }
    
    /**
     * Obtain links to embedded attachments from email body
     */		     
		private function processEmbeddedAttachments(&$oMail)
    {

    
			// search all links to DOWNLOAD_PHP with current session idd			
			$downloadUrl = 'http'.($_SERVER['HTTPS']=='ON'?'s':'').'://'.$_SERVER['HTTP_HOST'].str_replace(WEBMAIL_PHP,DOWNLOAD_PHP,$_SERVER['SCRIPT_NAME']);
      $url = 'src="' . $downloadUrl . '?sid=' . encodeURL($this->aAttrs['sid']);
			   
			$start = 0;
			$this->aEmbeddedAttachments = array();

			while (($start = strpos($this->sBody, $url, $start)) !== false) {
				// move the start to the quotation (") starting character
				$start += strlen('src="');
				$end = strpos($this->sBody, '"', $start);
				if ($end === false)
					break;
				$cid = strval(Tools::my_uniqid(rand(), true));
				$start2 = $start + strlen($downloadUrl) + 1;

        // &amp; -> &
				$sUrl = str_replace("&amp;","&",substr($this->sBody, $start2, $end - $start2));
				$data = Tools::parseURL($sUrl);

				if ($data['class'] && $data['fullpath']) {
				  $this->sBody = substr_replace($this->sBody, 'cid:'.$cid, $start, $end - $start);
				  $this->addAttachment($oMail, 'embedded', $data['class'], $data['fullpath'], $cid);
			  }
			}
		}
		
		/* Function to handle email attachments
		 */
		private function processAttachments(&$oMail)
		{
			foreach($this->oDOMDoc->query('message:attachment',$this->oDOMAttachments) as $oDOMAttach)
			{
				if (!$oDOMAttValues = $this->oDOMDoc->getNode("message:values",$oDOMAttach))
					throw new Exc('message_missing_tag','values');

				$class = $this->oDOMDoc->getNodeValue("message:class",$oDOMAttValues);
				$fullpath = $this->oDOMDoc->getNodeValue("message:fullpath",$oDOMAttValues); 
				$type = $this->oDOMDoc->getNodeValue("message:type",$oDOMAttValues);

				if (!$class || !$fullpath || !$type)
					throw new Exc('message_missing_tag','type,class,fullpath');

				$this->addAttachment($oMail, $type, $class, $fullpath);
			}
		}
		
		/* Add attachment to the mail object
		 * @param $oMail - mail object
		 * @param $namespace - "normal|embedded"
		 * @param $class - "item|attachment|file"
		 * @param $fullpath - Fullpath to the attachment data
		 * @param $cid - CID (required when setting up embedded attachment)
		 */
		private function addAttachment($oMail, $type, $class, $fullpath, $cid = '')
		{
			$oUser = &$this->session_user;
			$aAttachments = $oUser->getAttachments();
			$attData = Tools::parseFullPath($fullpath,$class);

      $api = new MerakAPI();


			switch($class)
			{
				//Attach whole message
				case 'item':
					static $attachCount;
			
					$sAID = $attData['account'];
					$sFID = $attData['folder'];
					$sIID = $attData['item'];
				
					$oAccount = &$oUser->getAccount($sAID);
					$oFolder = $oAccount->getFolder($sFID);
					$oItem = $oFolder->getItem($sIID);
					
					$file = $oItem->getItemData($info);          
      

					switch ($type) {
						case 'normal':

							$oMail->AddStringAttachment($file, 
													$info['name'],
													$info['encoding'], 
													$info['mimetype'],
													$info['encoding']=='base64'?true:false);
						break;
						default:
							throw new Exc('message_attachment_type',$type);
					}
				break;
				
				//Attach message part
				case 'gwattachment':
				  //$info['encoding'] = 'base64';
				case 'attachment':		

					$oAccount = $oUser->getAccount($attData['account']);
					$oFolder = $oAccount->getFolder($attData['folder']);
					$oItem = $oFolder->getItem($attData['item']);


					$file = $oItem->getAttachmentDataFile($attData['part'],$info);

					//Base64 encode binary data for sending


          if($info['encoding']=='uu'){
            $info['encoding'] = 'base64';
            $part = base64_encode($part);

          }

					switch ($type) {
						case 'normal':
					/*		$oMail->AddAttachment($file, 
													$info['name'],
													$info['encoding'], 
													$info['mimetype'],
													true);*/

							$oMail->addAttachment($file, 
													$info['mimetype'],
													$info['name'],
													$info['encoding'],
                          false);
						break;
						case 'embedded':
             			  $oMail->AddStringAttachment(file_get_contents($file),
                          $info['name'],
                          $info['encoding'],
													$info['mimetype'],
													true,
													$cid);
													
							/*$oMail->AddAttachment($file,
                          $info['name'],
													$info['encoding'], 
													$info['mimetype'],
													false,
													$cid);*/
						break;
						default:
							throw new Exc('message_attachment_type',$type);
					}
				break; // case 'attachment'
			
				//Attach recently uploaded attachment
				case 'file':

					if (!isset($aAttachments[$attData['folder']][$attData['item']]))
						throw new Exc('message_attachment_existence');

					$aAttachData = $aAttachments[$attData['folder']][$attData['item']];
					Tools::overrideContentType($aAttachData['name'],$aAttachData['type']);
					if (!in_array($attData['folder'],$this->aFolderList))
						$this->aFolderList[] = $attData['folder'];



					switch ($type) {
						case 'normal':
							$oMail->addAttachment($aAttachData['file'],
												$aAttachData['type'],
												$aAttachData['name']);
						break;
						case 'embedded':
							$oMail->addEmbededAttachment($aAttachData['file'],
													$aAttachData['type'],
													$aAttachData['cid'],
													$aAttachData['name']);
						break;
						default:
							throw new Exc('message_attachment_type',$type);
					}										  
				break; // case 'file'
			}

		}

		
		private function addRecipientsToAddressBook()
		{
			require_once('inc/gw/gwmanage.php');
			$gw = new GroupWareManagement();
			$gw->addRecipientsToAddressBook($this->addrList);
		}
		
		
		private function execFinalMessageAction(&$oMail)
		{
			$oUser = &$this->session_user;
		  //Send the message
			if ($this->sSEmail) {
				if ($this->sEncrypt)
					$oMail->encrypt();
				if ($this->sSign)
					$oMail->sign();
				
				if (!$this->sSave)
					$this->sTemplateFile = 'default';
        
        $filename = Tools::randomFileName();

				if (!($msg = $oMail->send(false,false,$this->sCharset,$filename))) {
					throw new Exc($msg);
				}
					
				if ($this->bAutoABook)
					$this->addRecipientsToAddressBook(); 
			}
				
			//Save message to the temporary location
			if ($this->sSave){

			
				//Get folder
				$oAccount = $oUser->getAccount($this->sSaveAccount);
				if (!is_object($oAccount))
					throw new Exc('message_invalid_account',$this->sSaveAccount);
					
				$oFolder = $oAccount->getFolderWithAutoCreate($this->sSaveFolder,'main');

				if (!is_object($oFolder))
					throw new Exc('message_invalid_folder',$this->sSaveFolder);

				
				// store BCC headers for further usage
				if ($this->sBCC){
					$bccCount = 0;
					foreach($this->aBcc as $bccAddress){
						$aFixedBCC[] = '"'.$bccAddress['display'].'" <'.$bccAddress['address'].'>';
					}
					$oMail->addHeader('Bcc: ' . implode(',',$aFixedBCC));
				}
	
			  	$filename = $oMail->getMessageTempFile($this->sCharset);

        		
				//Save to folder
				if ($this->sSaveItem) {
					//Item already exists?
          			try{
						  $oItem = $oFolder->getItem($this->sSaveItem);
						  $oItem->edit(false,$filename);
              $oItem->setFlags(Item::FLAG_SEEN);
          //Create new item
					}catch (Exc $e){
						echo file_get_contents($filename);
            $oItem->itemID = $oFolder->createItem(false,$filename);
						$oItem->setFlags(Item::FLAG_SEEN);
					}
					$this->aData['itemid'] = $oItem->itemID;
				}
				else{
          $this->aData['itemid'] = $oFolder->createItem(false,$filename);
					
          $oItem = $oFolder->getItem($this->aData['itemid']);
					$oItem->markAsRead();
				}
			}
			//Remove attachments folder if keep_attachments flag is set to false
			if (!$this->sKA && $this->aFolderList){
				foreach($this->aFolderList as $folder)
					$oUser->removeAttachments($folder);
			  @unlink($filename);
      }

		}
		
		private function dialPhone()
		{
			// get primary GW account
			$oUser = &$this->session_user;
			$oAccount = &$oUser->getAccount($_SESSION['EMAIL']);
			$oGWAccount = &$oAccount->gwAccount;
			//print_r($this->sPhoneNumber);die();
			$oGWAccount->gwAPI->FunctionCall('DialSIPNumber',$oGWAccount->gwAPI->sessid,$this->sPhoneNumber);
		}
		
}

?>