<?php
require_once('inc/xmlrequest.php');
require_once('inc/spellchecker.php');
/**
@class	WebmailIqSpellChecker
@brief	Class for handling webmail:iq:spellchecker queries.
@author	Martin Zlomek
@date	10-04-2006
*/
class WebmailIqSpellChecker extends XMLRequestInterface {
	/**
	* @brief Public: Constructor
	* @param	$oQuery	A DOMElement filled with query element
	* @param	$oDoc	A DOMDocument of the request
	* @param	$attrs	An Array with parent attributes
	*/
	public function __construct($oQuery, $oDoc, &$attrs)
	{
		$this->oQuery = $oQuery;
		$this->oDoc = $oDoc;
		$this->aAttrs = &$attrs;
		$this->checkInputXML();
		$this->exeInputXML();
	}
	private function checkInputXML()
	{
		if (!$this->aAttrs['sid']) {
			throw new Exv('access_denied','access_denied');
		}
	}
	/**
	@brief Handle the query.
	*/
	private function exeInputXML()
	{
		$customDict = $_SESSION['user']->getDir() . 'custom.dic';
		switch ($this->aAttrs['type']) {
			case 'get':
				if ($checkNode = $this->oDoc->getNode('spellchecker:check', $this->oQuery)) {
					if (!($id = $checkNode->getAttribute('uid')))
					{
						if ($id==0) {
							$id = "0";
						} else {
							throw new Exc('spellchecker_missing_check_id');
						}
					}
					if (!($textNode = $this->oDoc->getNode('spellchecker:text', $checkNode))) {
						throw new Exc('spellchecker_missing_tag','text');
					}
					$suggestNode = $this->oDoc->getNode('spellchecker:suggest', $checkNode);
					$spellChecker = new SpellChecker($id, $customDict);
					$misspelled = $spellChecker->check($textNode->nodeValue, $suggestNode != null);
					$this->aData['DICTIONARY_ID'] = XML::encode($id);
					$this->aData['SUGGEST'] = $suggestNode != null;
					foreach ($misspelled as $word => $suggests) {
						$word = array('WORD' => XML::encode($word));
						if ($suggestNode) {
							foreach ($suggests as &$suggest) {
								$suggest = array('WORD' => XML::encode($suggest));
							}
							unset($suggest);
							$word['SUGGESTS'] = array('num' => $suggests);
						}
						$this->aData['WORDS']['num'][] = $word;
					}
					$this->sTemplateFile = 'webmailiqspellchecker_get_check';
				} else if ($dictionaryNode = $this->oDoc->getNode('spellchecker:dictionary', $this->oQuery)) {
					if (!($id = $dictionaryNode->getAttribute('uid'))) {
						throw new Exc('spellchecker_missing_dictionary_id');
					}
					$spellChecker = new SpellChecker($id, $customDict);
					$this->aData['DICTIONARY_ID'] = XML::encode($id);
					foreach ($this->oDoc->query('spellchecker:suggest', $dictionaryNode) as $suggestNode) {
						if (!($word = $suggestNode->getAttribute('word'))) {
							throw new Exc('spellchecker_missing_word');
						}
						$suggests = $spellChecker->suggests($word);
						foreach ($suggests as &$suggest) {
							$suggest = array('WORD' => XML::encode($suggest));
						}
						unset($suggest);
						$this->aData['WORDS']['num'][] = array(
							'WORD' => XML::encode($word),
							'SUGGESTS' => array(
								'num' => $suggests
							)
						);
					}
					$this->sTemplateFile = 'webmailiqspellchecker_get_suggest';
				} else {
					throw new Exc('spellchecker_missing_tag','dictionary,check');
				}
				break;
			case 'set':
				if (!($dictionaryNode = $this->oDoc->getNode('spellchecker:dictionary', $this->oQuery))) {
					throw new Exc('spellchecker_missing_tag','dictionary');
				}
				if (!($id = $dictionaryNode->getAttribute('uid'))) {
					throw new Exc('spellchecker_missing_dictionary_id');
				}
				$spellChecker = new SpellChecker($id, $customDict);
				foreach ($this->oDoc->query('spellchecker:add', $dictionaryNode) as $addNode) {
					$spellchecker->add($addNode->nodeValue);
				}
				$this->aData['DICTIONARY_ID'] = XML::encode($id);
				$this->sTemplateFile = 'webmailiqspellchecker_set';
				break;
			default:
				break;
		}
	}
}
?>
