<?php
require_once('inc/webmailiqfolders.php');

function fdrCompare($a,$b)
{
	return strcasecmp($a['localizedName'],$b['localizedName']);
}


class FolderExtension
{
	public function definedFolderList(&$oAccount)
	{
		$aMapped = array('drafts','trash','sent','spam');
		//Inbox
		$oInbox = $oAccount->getFolder('INBOX');
		if(!$oInbox->localizedName){
			$oInbox->localizedName = self::localizeFolder($oInbox->name);
		}
		$aPreffered[] = $oInbox;
		
		//Get mapped folders
		foreach($aMapped as $sFolder){
			if($oMappedFolder = self::getMappedFolder($sFolder)){
				$aFolders[] = $oMappedFolder;
			}
		}
		//Try because groupware can be disabled
		try{
		//Contacts
		$aFolders[] = $oAccount->getFolder('Contacts');
		}catch(Exc $e){
			//Ignore
		}
		
		//Localize names
		foreach($aFolders as $oFolder){
			$oFolder->id = rawurlencode($oFolder->name);
			if(!$oFolder->localizedName){
				$oFolder->localizedName = self::localizeFolder($oFolder->name);
			}
			$aOther[] = $oFolder;
		}
		
		//Convert objects to array
		$aFdrPreffered = WebmailIqFolders::cnvFolders($aPreffered,true);
		$aFdrOther = WebmailIqFolders::cnvFolders($aOther,true);

		//Sort folders 
		uasort($aFdrPreffered,"fdrCompare");
		uasort($aFdrOther,"fdrCompare");
		
		if ($aFdrOther) {
			$aFdr['num'] = array_merge($aFdrPreffered,$aFdrOther); 
		}else{
			$aFdr['num'] = $aFdrOther;
		}

		return $aFdr;
	}
	//List of folders
	public function folderList(&$oAccount)
	{

		$aPreffered = $aFolders = array();

		if($oAccount){
			$oFolders = $oAccount->getFolders();
			if($oFolders) foreach($oFolders as $oFolder){
	
				//Filter folder types
				if ($oFolder->type!='M' && $oFolder->type!='C'){
					continue;
				}
				if(
					strtolower($oFolder->name)=='inbox'
				 ){
					$oFolder->id = rawurlencode($oFolder->name);
					if(!$oFolder->localizedName){
						$oFolder->localizedName = self::localizeFolder($oFolder->name);
					}
					$aPreffered[] = $oFolder;
					continue;
				}
				if(
					strtolower($oFolder->name)=='drafts'
				 	|| strtolower($oFolder->name)=='trash'
				 	|| strtolower($oFolder->name)=='sent'
				 	|| strtolower($oFolder->name)=='contacts'
				){
					$oFolder->id = rawurlencode($oFolder->name);
					if(!$oFolder->localizedName){
						$oFolder->localizedName = self::localizeFolder($oFolder->name);
					}
					$aFolders[] = $oFolder;
				}
			}

			//Convert objects to array
			$aPrefferedConverted = WebmailIqFolders::cnvFolders($aPreffered,true);
			$aFoldersConverted = WebmailIqFolders::cnvFolders($aFolders,true);
		
			//Sort folders 
			uasort($aPrefferedConverted,"fdrCompare");
			uasort($aFoldersConverted,"fdrCompare");
			//Place preffered to the top
			if ($aPrefferedConverted) {
				$aFdr['num'] = array_merge($aPrefferedConverted,$aFoldersConverted); 
			}else{
				$aFdr['num'] = $aFoldersConverted;
			}
		}
		return $aFdr;
	}
	//Localize names  
	public function localizeFolder($sName){

		$oLanguage = &Singleton::instance('WMLanguage');
		$sIndex = $sName;
		$aIndex = explode('/',$sIndex,2);
		if (is_array($aIndex)) {
			$sIndex = strtolower($aIndex[0]);
		} else {
			$sIndex = strtolower($sIndex);
		}

		if($oLanguage->get('common_folders',$sIndex)) {
			$sName = $oLanguage->get('common_folders',$sIndex);
			if ($aIndex[1]) {
				$sName.='/'.$aIndex[1];
			}
		}
		
		return $sName;
	}
	public function getMappedFolder($sType,$autoCreate = false)
	{
		$oUser = &$_SESSION['user'];
		$oWMSettings = &Singleton::instance('WMSettings');
		//Get folders from settings
		$aDefaultFolders = $oWMSettings->getPrivate('default_folders');
		$sDefaultFolder = $aDefaultFolders['@childnodes']['item'][0]
				['@childnodes'][strtolower($sType)][0]['@value'];
		if($sDefaultFolder){
			$aDefaultFolderInfo = explode('/',$sDefaultFolder,2);
			$sAccount = $aDefaultFolderInfo[0];
			$oAccount = $oUser->getAccount($sAccount);
			$sFolder = $aDefaultFolderInfo[1];
			$oFolder = $oAccount->getFolderWithAutoCreate($sFolder);
		}else{
			//Primary account
			$sAccount = $_SESSION['EMAIL'];
			$oAccount = $oUser->getAccount($sAccount);
			try{
				$sFolder = ucfirst(strtolower($sType));
				
				$oFolder = $oAccount->getFolder($sFolder);
			}catch(Exception $e){
				try{
					$sFolder = strtolower($sType);
					$oFolder = $oAccount->getFolder($sFolder);
				}catch(Exception $e){
					$sFolder = ucfirst(strtolower($sType));
					if($autoCreate){
						$oFolder = $oAccount->getFolderWithAutoCreate($sFolder);
					}else{
						return false;
					}
				}
			}
		}
		return $oFolder;
	}
}
?>
