<?php

define('UI_SKIN_DEFAULT','pda');
define('UI_VIEW_DEFAULT',false);
define('OBJ_JS_EXT','.js');
define('OBJ_CSS_EXT','.css');
define('OBJ_TPL_EXT','.tpl');
/*
 *Introduction 
 *============
 *In this file are stored basic Graphic user interface
 *classes, abstracts and interfaces
 *
 *Graphic user interface is used to
 *easily maintain the look and behaivour of
 *web application.It does not involve getting
 *or setting the data, just the way they are
 *displayed to the client.*/ 


class WMUserInterface
{
	private $sSkin;
	private $sView;
	private $aJavascript;
	private $aStylesheet;
	private $aError;
	
	
	public function __construct()
	{
		//Defaults
		$this->setSkin(UI_SKIN_DEFAULT);
		$this->setView(UI_VIEW_DEFAULT);
	}
	
	public function setSkin($sSkin)
	{
		if(WMUserInterface::checkPath($sSkin,'skin_invalid')){
			$this->sSkin = $sSkin;
		}
		return true;
	}
	
	public function getSkin()
	{
		return $this->sSkin;
	}
	
	public function setView($sView)
	{
		if(WMUserInterface::checkPath($sView,'view_invalid')){
			$this->sView = $sView;
		}
		$this->sView = $sView;
	}
	
	public function getView()
	{
		return $this->sView;
	}
	
	public function draw()
	{
		try {
			$oXML = $this->getDescriptor();
		} catch (Exception $drawException) {
			throw new Exception('Descriptor retrieval failed!');
		}
		$sBody = $this->getBody($oXML);

		return $sBody;
	}

	static public function &safeInstance(&$aConstruct,$sLibrary = false)
	{
		
		WMUserInterface::checkPath($aConstruct['objID'],'class_invalid');
		$sClass = $aConstruct['objID'];
		$sClassLibrary = './interface/inc/obj/'.($sLibrary?$sLibrary:$sClass).'.php';
		if(file_exists($sClassLibrary)){
			require_once($sClassLibrary);//FIXME:move to singleton class
			// fixme: tady to uz prostym odchytem exceptiony a poslanim dal nejde :((
			$oResult = &Singleton::instance('obj_'.$sClass,$aConstruct);
		} else{
			$oResult = false;
		}

		return $oResult;
	}
	public function addJavascript($sFile)
	{
		$this->aJavascript[$sFile]['src'] = ROOT_SERVER_DIR.$this->getPath('javascript').$sFile.OBJ_JS_EXT;
	}
	
	public function addStylesheet($sFile)
	{
		$this->aStylesheet[$sFile]['src'] = ROOT_SERVER_DIR.$this->getPath('stylesheet').$sFile.OBJ_CSS_EXT;
	}	
	public function getJavascript()
	{
		return $this->aJavascript;
	}
	public function getStylesheet()
	{
		return $this->aStylesheet;
	}
		
	private function getDescriptor()
	{
		try {
			$sPath = $this->getFile('view');
		} catch (Exception $descriptorException) {
			throw new Exception('File descriptor load failed!');
		}
		if(!file_exists($sPath)){
			throw new Exc('invalid_descriptor_path',$sPath);
		}
		$oXML = simplexml_load_file($sPath);
		return $oXML;
	}
	
	private function getBody(&$oXML)
	{
		$sBody = '';
		$aBody = array();
		//Fill constructor data
		$aConstruct['objID'] = 'html';
		$aConstruct['xml'] = &$oXML;
		//Create html object ( main container )
		$oObjClass = &self::safeInstance($aConstruct);
		$oObjClass->create($aConstruct);
		
		//Set title
		if($_SESSION['user']){
			$sFullname = $_SESSION['user']->aAccounts[$_SESSION['EMAIL']]->fullname;
			$sEmail = '&lt;'.$_SESSION['EMAIL'].'&gt;';
			$oObjClass->setTemplateData('title',$sFullname.' '.$sEmail);
		}
		if($this->aError){
			$oObjClass->setTemplateData('error',$this->aError);
		}
		$sBody .= $oObjClass->getBody($oXML);
		return $sBody;
	}
	
	//FIXME : regular expr
	static private function checkPath($sPath,$sErrorID = 'path_invalid')
	{
		if(strpos($sPath,'..')!==false){
			throw new Exc($sErrorID,$sPath);
		}
		if(strpos($sPath,'\\')!==false){
			throw new Exc($sErrorID,$sPath);
		}
		if(strpos($sPath,':/')!==false){
			throw new Exc($sErrorID,$sPath);
		}
		return true;
	}
	
	//Returns path for javascripts,templates,css and skin
	protected function getPathByParam($sSkin,$sView,$sType)
	{
		switch($sType){
			case 'skin':
				if(!$sSkin){
					throw new Exc('skin_not_set');
				}
				return 'interface/skin/'.$sSkin.'/';//.$this->sView.'.xml';
				
			case 'template':
				$sFolder = 'template';
			break;
			case 'stylesheet':
				$sFolder = 'css';
			break;
			case 'javascript':
				$sFolder = 'javascript';
			break;
		}
		
		return $this->getPath('skin').$sFolder.'/';
	}
	
	public function getLanguageLabel($sLabel)
	{
		$aLabel = explode('|',$sLabel);
		$oWMLanguage = Singleton::instance('WMLanguage');
		$sResult = $oWMLanguage->get($aLabel[0],$aLabel[1]);
		if(!$sResult){
			$sResult = $sLabel;
		}
		return $sResult;
	}
	//Returns full file name
	public function getFileByParam($sSkin,$sView,$sType,$sObjID = false)
	{
		$sDefault = 'default';
		switch($sType){
			case 'view':
				if(!$sSkin){
					throw new Exc('skin_not_set');
				}
				if(!$sView){
					throw new Exc('view_not_set');
				}
				$sSkinPath = $this->getPath('skin');
				return $sSkinPath.$this->sView.'.xml';
			break;
			case 'javascript':
				$sExtension = OBJ_JS_EXT;
			break;
			case 'template':
				$sExtension = OBJ_TPL_EXT;
			break;
			case 'stylesheet':
				$sExtension = OBJ_CSS_EXT;
			break;
			
		}
		$sPath = $this->getPath($sType);
		if(!file_exists($sPath.$sObjID.$sExtension)){
			return $sPath.$sDefault.$sExtension;
		}
		return $sPath.$sObjID.$sExtension;
	}
	
	public function getPath($sType,$sObjID = false)
	{
		return $this->getPathByParam($this->sSkin,$this->sView,$sType,$sObjID);
	}
	public function getFile($sType,$sObjID = false)
	{
		try {
			$someFile = $this->getFileByParam($this->sSkin,$this->sView,$sType,$sObjID);
		} catch (Exception $fileException) {
			throw new Exception('File loading failed!');
		}
		return $someFile;
	}
	public function setError($sError)
	{
		$this->aError['num'][] = array('error'=>$sError);
	}
	
	
	public function __clone()
	{
		throw new Exc('singleton_clone');
	}
}



interface UIObjectInterface
{
	function getBody(&$oXML);
	function create($sID);	//Appends new object of the same type
	function open($sID);
	function getProperty($sProperty);
	function setProperty($sProperty,$value);
	function getTemplateData($sProperty);
	function setTemplateData($sProperty,$value);
	
}

abstract class UIObject implements UIObjectInterface
{
	protected $aData;	//Store data for all widgets of the same type
	protected $pData;	//Points to the data of current widget(must be Opened)
	protected $sObjID;	//Object id
	protected $sTemplateFile;	//Template file
	protected $sJavascriptFile;	//Javascript file
	protected $sCSSFile;	//CSS file
	
	public function __construct($aConstruct)
	{
		$this->objID = $aConstruct['objID'];
		$this->aData = array();
		$this->sTemplateFile = $this->getFile('template');
		$this->sJavascriptFile = $this->getFile('javascript');
		$this->sStylesheetFile = $this->getFile('stylesheet');
		
	}
	
	public function create($aConstruct)
	{
		$oWMUserInterface = &Singleton::instance('WMUserInterface');
		$oWMLanguage = &Singleton::instance('WMLanguage');

		$this->aData[$aConstruct['id']] = new stdClass();
		$this->pData = &$this->aData[$aConstruct['id']];
		$this->setProperty('id',$aConstruct['id']);
		$this->setProperty('xml',$aConstruct['xml']);
		$this->setTemplateData('lang',$oWMLanguage->data());
		$this->setTemplateData('script',Tools::getServerURL());
		$this->setTemplateData('path_skin','../interface/skin/'.$oWMUserInterface->getSkin().'/');

		if($aConstruct['xml']['template']){
			$this->sTemplateFile = $this->getPath('template').strval($aConstruct['xml']['template']).'.tpl';
		}

		if($aConstruct['xml']->config->javascript->src){
			foreach($aConstruct['xml']->config->javascript->src as $javascript){
				$oWMUserInterface->addJavascript(strval($javascript));
			}
		}
		if($aConstruct['xml']->config->stylesheet->src){
			foreach($aConstruct['xml']->config->stylesheet->src as $stylesheet){
				$oWMUserInterface->addStylesheet(strval($stylesheet));
			}
		}
	}
	
	public function open($sID)
	{
		if(!@$this->aData[$aData['uid']]){
			throw new Exc('uiobject_invalid_id'.$sID);
		}
		$this->pData = &$this->aData[$sID];
	}
	
	public function getProperty($sProperty)
	{
		if(!isset($this->pData->$sProperty))
			throw new Exception('uiobject_invalid_property'.$sProperty);
		return $this->pData->$sProperty;
	}
	
	public function setProperty($sProperty,$value)
	{
		$this->pData->$sProperty = $value;
	}
	
	public function getTemplateData($sIndex)
	{
		return @$this->pData->aTemplate[$sIndex];
	}
	
	public function setTemplateData($sIndex,$value)
	{
		if(is_array($value) && is_array($this->pData->aTemplate[$sIndex])){
			$this->pData->aTemplate[$sIndex] = array_merge($this->pData->aTemplate[$sIndex],$value);
		}else{
			$this->pData->aTemplate[$sIndex] = $value;
		}
	}
	public function getLanguageData($sIndex)
	{
		return @$this->pData->aTemplate['lang'][$sIndex];
	}
	
	public function setLanguageData($sIndex,$value)
	{
		$this->pData->aTemplate['lang'][$sIndex] = $value;
	}
	
	public function getFile($sType,$sObjID = false)
	{
		$sObjID = $sObjID?$sObjID:$this->objID;
		return Singleton::instance('WMUserInterface')->getFile($sType,$this->objID);
	}
	public function getPath($sType)
	{
		return Singleton::instance('WMUserInterface')->getPath($sType);
	}
	
	public function &getXML()
	{
		return $this->pData->xml;
	}
	
	public function getBody(&$oXML)
	{
		if($this->sTemplateFile){
			return template($this->sTemplateFile,$this->pData->aTemplate);
		}else{
			return '';
		}
	}
	
}



abstract class UIInput extends UIObject
{	
	/*public function __construct($aConstruct)
	{
		parent::__construct($aConstruct);
	}*/
	
	public function create($aConstruct)
	{
		parent::create($aConstruct);
		$oXML = $this->getProperty('xml');
		$this->setTemplateData('variable', strval($oXML['variable']));
		$this->setTemplateData('name', strval($oXML['name']));
		$this->setTemplateData('label',WMUserInterface::getLanguageLabel(strval($oXML['label'])));
		$this->setTemplateData('required',strval($oXML['required'])=='true'?true:false);
		$this->setTemplateData('func',$oXML['func']?strval($oXML['func']):false);
		$this->setTemplateData('type',$oXML['type']?strval($oXML['type']):false);
		$this->setTemplateData('id',$oXML['id']?strval($oXML['id']):false);
		$this->setTemplateData('css',$oXML['css']?strval($oXML['css']):false);
		$this->setTemplateData('body','');
		$this->setTemplateData('value','');

	}
	
	public function getData()
	{
		$WMData = &Singleton::instance('WMData');
		//$UIData = UIData::instance();
		return $WMData->get($this);
	}
	public function getSentData()
	{
		
	}
	public function checkData()
	{
	
	}
}

abstract class UIGraphic extends UIObject
{
	public function __construct($aConstruct)
	{
		parent::__construct($aConstruct);
	}
	
	public function create($aConstruct)
	{
		parent::create($aConstruct);
	}
	
	public function getBody(&$oXML)
	{
		//$t = time();
		//$m = microtime();
		$sBody = '';
		$aBody = array();
		if($oXML) foreach($oXML as $sObjClass => $oObjXML){
			switch($sObjClass){
				case 'config':
				break;
				default:
					//Constructor data
					$aConstruct['objID'] = strval($sObjClass);
					$aConstruct['xml'] = $oObjXML;
					$aConstruct['parent'] = &$this;
					//Load class
					$oObjClass = &WMUserInterface::safeInstance($aConstruct);
					if(is_object($oObjClass)){
						$oObjClass->create($aConstruct);
						$sBody .= $oObjClass->getBody($oObjXML);
					}
				break;
			}
		}
		$this->setTemplateData('body',$sBody);
		$sBody = template($this->sTemplateFile,$this->getProperty('aTemplate'));
		//echo $this->objID.(time()-$t+microtime()-$m)."<br/>";
		return $sBody;
	}
}

abstract class UIForm extends UIObject
{
	/*public function __construct($aConstruct)
	{
		parent::__construct($aConstruct);
	}*/
	
	public function create($aConstruct)
	{
		parent::create($aConstruct);
		$oXML = $this->getProperty('xml');
		$this->setTemplateData('namespace',strval($oXML->config->namespace));
		$this->setTemplateData('action',strval($oXML->config->action));
		$this->setTemplateData('view',strval($oXML->config->view));
		$this->setTemplateData('name',strval($oXML['name']));
		$this->setTemplateData('method',$oXML['method']?strval($oXML['method']):'get');
		$this->setTemplateData('id',strval($oXML['id']));
		$this->setTemplateData('script',strval($oXML['script'])?strval($oXML['script']):$_SERVER['SCRIPT_NAME']);
	}
	
	public function getBody(&$oXML)
	{
		//$fo = time();
		//$fom = microtime();
		$iFieldsetCount = 0;
		if($oXML == null){
			$oXML = $this->getXML();
		}
		if($oXML->fieldset){
			foreach($oXML->fieldset as $val){
				$this->fieldset($val);
			}
		}
		$this->setTemplateData('variable',array('num'=>$this->aVariable));
		$sBody = template($this->sTemplateFile,$this->pData->aTemplate);
		//echo 'Form'.(time()-$fo+microtime()-$fom)."<br/>";
		return $sBody;
	}
	
	private function fieldset(&$oFieldsetXML)
	{
		//$f = time();
		//$fm = microtime();
		$iInputCount = 0;
		$aFieldset = &$this->pData->aTemplate['fieldset']['num'][];
		$aFieldset['legend'] = WMUserInterface::getLanguageLabel(strval($oFieldsetXML["legend"]));

		$aFieldset['css'] = isset($oFieldsetXML["css"])?strval($oFieldsetXML["css"]):false;
		if($oFieldsetXML->input){
			foreach($oFieldsetXML->input as $val){
				$aFieldset['input']['num'][$iInputCount++]['body'] = $this->input($val);
			}
		}
		
		//echo 'Fieldset'.(time()-$f+microtime()-$fm)."<br/>";
	}
			
	private function input(&$oItemXML)
	{
		//$i1 = time();
		//$im1 = microtime();
		$aConstruct['objID'] = strval($oItemXML["obj"]);
		$aConstruct['xml'] = $oItemXML;
		$aConstruct['parent'] = $this;
		$sLibrary = $oItemXML['lib']?strval($oItemXML['lib']):false;
		//$i = time();
		//$im = microtime();
		$oItem = &WMUserInterface::safeInstance($aConstruct,$sLibrary);

		//echo "Item instance".(time()-$i+microtime()-$im)."<br/>";
		//$i = time();
		//$im = microtime();
		$oItem->create($aConstruct);
		//echo "Item create".(time()-$i+microtime()-$im)."<br/>";
		//if(is_object($oItem)){
			$sItem = $oItem->getBody($oItemXML);	
		//}
		//echo "Item".(time()-$i1+microtime()-$im1)."<br/>";

		return $sItem;
	}
	
	protected function addVariable($variable,$value)
	{
		$this->aVariable[$variable]['var_name'] = $variable;
		$this->aVariable[$variable]['var_value'] = $value;
	}
	protected function getVariable($variable,$value)
	{
		return @$this->aVariable[$variable]['var_value'];
	}
	
	
}

require_once('interface/inc/obj/inputs.php');
?>
