#!/bin/bash
#
# IceWarp Merak Mail Server Installation package
# (c) IceWarp Ltd. 2008
#
#	All rights reserved.
#
# http://www.icewarp.com 
#
# file: install.sh - install script
# version: 1.5
#

# defaults

USE_COLORS="0"

NEWVERSION="9.4.1 RHEL5.3"
IMAGENAME="merak-image-9.4.1_RHEL5.3.tar"

# default variables
MERAKDIR="/opt/merak"
USER=$(id -un)
GROUP=$(id -gn)
UUID=$(id -u)
CHROOT="0"
MERAKCONF="${MERAKDIR}/config/merak.conf"
ERRORLOG=~/merak-install-error.log
INSTALLLOG=~/merak-install.log
UPGRADE="0"
OS_PLATFORM="UNKNOWN"
SERVICE_NAME="merak"

# init

SELF=$0
WORKDIR=$(dirname $0)

if  [ $TERM = "linux" ] || [ $TERM = "xterm" ] || [ $TERM = "xterm-color" ] || [ $TERM = "screen" ] && [ "$USE_COLORS" = "1" ]
    then
	GOOD=$'\e[32;01m'
	WARN=$'\e[33;01m'
	BAD=$'\e[31;01m'
	NORMAL=$'\e[0m'
	HILITE=$'\e[37;01m'
	BRACKET=$'\e[34;01m'
    else
	GOOD=$'\e[0;32m'
	BAD=$'\e[0;31m'
	NORMAL=$'\e[0m'
	WARN=${NORMAL}
	HILITE=${NORMAL}
	BRACKET=${NORMAL}
fi

# Detection of host OS platform
# Function tryes to detect OS in simple way
# Result of detection is:
# - "UNKNOWN" if platform is not known
# - "RHEL" if OS is Red Hat Enterprise Linux
# TODO:
# add more precise detection
# detect more platforms
detect_os_platform()
{
    if [ -f "/etc/redhat-release" ]
	then
	    OS_PLATFORM="RHEL"
    fi
}

check_merakd_is_running()
{
	good "Please make sure the Merak Mail Server is not running before installing."
    good "Press ENTER to continue, CTRL+C to quit"
    read -s
    echo ""
}

# Ask question with confirmation
# Parameters:
# $1 - question
# $2 - first choice, it is choosed if user enters only empty string
# $3 - second choice
# Return:
# 0 on error - unknown string entered
# 1 if first choice was selected
# 2 if second choice was selected
ask_with_confirmation()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]
	then
	    return 0
    fi
    LOWER_CHOICE_1=$(echo $2 | tr "A-Z" "a-z")
    LOWER_CHOICE_2=$(echo $3 | tr "A-Z" "a-z")
    echo -e -n "${GOOD}**${NORMAL}\t$1 [${HILITE}$2${NORMAL}/$3]: "
    read PARAM
    PARAM=$(echo $PARAM | tr "A-Z" "a-z")
    if [ "$PARAM" == "" ] || [ "$PARAM" == "$LOWER_CHOICE_1" ]
	then
	    return 1
    fi
    if [ "$PARAM" == "$LOWER_CHOICE_2" ]
	then
	    return 2
    fi
    return 0
}

preconf()
{
    if [ -f $MERAKCONF ]
	then
	    source $MERAKCONF
	    [[ $InstallDir != "" ]] && MERAKDIR=$InstallDir
	    [[ $User != "" ]] && USER=$User
	    [[ $Group != "" ]] && GROUP=$Group
	    [[ $Chroot != "" ]] && CHROOT=$Chroot
	    # hack to fill in version number if empty
	    OLDVERSION=$Version
    fi
}

good()
{
    echo -e "${GOOD}**${NORMAL}\t$*"
}

bad()
{
    echo -e "${BAD}**${NORMAL}\tError: $*"
}

warn()
{
    echo -e "${WARN}**${NORMAL}\tWarning: $*"
}

hilite()
{
    echo -e "${GOOD}**${HILITE}\t$*${NORMAL}"
}

getparam()
{
    echo -e -n "${GOOD}**${NORMAL}\t$1 [$2]${NORMAL}: "
    read PARAM
}

extract_image()
{
    good "Please wait..."
    tar -xf $1 -C $2

    # check if configuration files exist
    # this will create all configuration files in case of new installation
    # and add missing files in case of upgrade
    if [ "$UPGRADE" == "0" ]
	then
	    # copy all configuration files 
	    cp -f $2/config/default/* $2/config/
	    cp -f $2/config/default/db/antispam.db $2/spam/antispam.db
	    cp -f $2/config/default/db/groupware.db $2/calendar/groupware.db
	    cp -f $2/spam/rules/default/local.cf $2/spam/rules/local.cf
	    cp -f $2/webmail/config/default/* $2/webmail/config/			

	else
	    # test if file exist, if not copy default ones

	    # config/cert.pem
	    FNAME_SRC="config/default/cert.pem"
	    FNAME_DST="config/cert.pem"
	    if [ ! -f $2/$FNAME_DST ]
		then
		    good "Configuration file $FNAME_DST does not exist, copying from default."
		    cp -f $2/$FNAME_SRC $2/$FNAME_DST
	    fi
	    # config/webserver.dat
	    FNAME_SRC="config/default/webserver.dat"
	    FNAME_DST="config/webserver.dat"
	    if [ ! -f $2/$FNAME_DST ]
		then
		    good "Configuration file $FNAME_DST does not exist, copying from default."
		    cp -f $2/$FNAME_SRC $2/$FNAME_DST
	    fi
	    # spam/antispam.db
	    FNAME_SRC="config/default/db/antispam.db"
	    FNAME_DST="spam/antispam.db"
	    if [ ! -f $2/$FNAME_DST ]
		then
		    good "Configuration file $FNAME_DST des not exist, copying from default."
		    cp -f $2/$FNAME_SRC $2/$FNAME_DST
	    fi
	    # spam/rules/local.cf
	    FNAME_SRC="spam/rules/default/local.cf"
	    FNAME_DST="spam/rules/local.cf"
	    if [ ! -f $2/$FNAME_DST ]
		then
		    good "Configuration file $FNAME_DST des not exist, copying from default."
		    cp -f $2/$FNAME_SRC $2/$FNAME_DST
	    fi
	    # calendar/groupware.db
	    FNAME_SRC="config/default/db/groupware.db"
	    FNAME_DST="calendar/groupware.db"
	    if [ ! -f $2/$FNAME_DST ]
		then
		    good "Configuration file $FNAME_DST des not exist, copying from default."
		    cp -f $2/$FNAME_SRC $2/$FNAME_DST
	    fi
	    # webmail/config/settings.dat
	    FNAME_SRC="webmail/config/default/settings.dat"
	    FNAME_DST="webmail/config/settings.dat"
	    if [ ! -f $2/$FNAME_DST ]
		then
		    good "Configuration file $FNAME_DST des not exist, copying from default."
		    cp -f $2/$FNAME_SRC $2/$FNAME_DST
	    fi
    fi
    if [ -e info.xml ]
	then
	    mkdir -p $2/xml 2>/dev/null
	    cp -f info.xml $2/xml/
    fi
}

ask_questions()
{
    getparam "Installation prefix" "$MERAKDIR"
    [[ $PARAM != "" ]] && MERAKDIR=$PARAM

    # update merak configuration file name
    MERAKCONF=$MERAKDIR/config/merak.conf

    # sophisticated check for "new installation" or "upgrade"
    # check if installation directory exist
    if [ -d $MERAKDIR ]; then
	UPGRADE="2"
    fi
    # check if installation file exist but it is not directory
    if [ "$UPGRADE" == "0" -a -e $MERAKDIR ]
	then
	    bad "File $MERAKDIR exists but it is not directory."
	    exit 1
    fi
    # check if merak.conf configuration file exist
    if [ -f $MERAKDIR/config/merak.conf ]
	then
	    UPGRADE="1"
    fi

    # if UPGRADE is 2 then MERAKDIR exist but merak.conf does not exist
    if [ "$UPGRADE" == "2" ]; then
	good ""
	good "Directory $MERAKDIR exists but"
	good "configuration file $MERAKDIR/config/merak.conf does not exist."
	good ""
	good "Please select full installation or upgrade."
	good "WARNING: directory $MERAKDIR will be DELETED in full installation!"
	good ""
	ask_with_confirmation "Upgrade or Install?" "U" "i"
	if [ $? -eq 1 ]
	    then
		UPGRADE="1"
	    else
		UPGRADE="0"
	fi
    fi

    # read installed merak.conf in case we are upgrading
    if [ $UPGRADE = "1" ]
	then
	    MERAKCONF="$MERAKDIR/config/merak.conf"	
	    preconf
    fi

    if  [ "$UPGRADE" == "1" ]
	then
	    echo "${GOOD}** Performing upgrade"
	else
	    echo "${GOOD}** Performing install"

	getparam "Run services as user" "$USER"
	[[ $PARAM != "" ]] && USER=$PARAM

	getparam "Run services as group" "$GROUP"
	[[ $PARAM != "" ]] && GROUP=$PARAM

	#getparam "Run services in chroot" "$CHROOT"
	#[[ $PARAM != "" ]] && CHROOT=$PARAM
    fi
}

confirm_params()
{
    good ""
    good "Please check entered informations before continuing:"
    good ""
    if [ "$UPGRADE" == "1" ]
	then
	    good "Installation prefix:\t\t$MERAKDIR (upgrading)"
	    else
		good "Installation prefix:\t\t$MERAKDIR (directory will be created)"
    fi

    good "Services will run as user:\t$USER"
    good "Services will run as group:\t$GROUP"

    if [[ $CHROOT == "1" ]]
	then
	    good "Services will run in chroot environment"
    fi

    good ""
    good "Press ENTER to continue, CTRL+C to quit"
    read -s
    echo ""
}

check_entered_params()
{
    # check if entered user and group exist
    grep -q "^$USER:" /etc/passwd
    if [ $? -ne 0 ]
        then
	    bad "User $USER doesn't exist."
	    exit 1
    fi
    grep -q "^$GROUP:" /etc/group
    if [ $? -ne 0 ]
	then
	    bad "Group $GROUP doesn't exist."
	    exit 1
    fi
    id $USER 2>&1 | grep -q "($GROUP)"
    if [ $? -ne 0 ]
	then
	    bad "User $USER is not a member of group $GROUP!"
	    exit 1
    fi
    # simple check if have permissions to install
    UUID_DST=$(id -u $USER)
    if [ $UUID -ne 0 ]
	then
	    if [ $UUID -ne $UUID_DST ]
		then
		    bad "Permission denied to install with user $USER"
		    exit 1
	    fi
    fi
}

install_package()
{
    if [ "$UPGRADE" == "0" ]
	then
	    good "Creating $MERAKDIR directory"

	    if [ "$MERAKDIR" == "/" ]
	    then
		bad "Rejecting directory removal: $MERAKDIR"
		exit 1
	    fi
	    rm -rf $MERAKDIR
	    mkdir -p $MERAKDIR
	    if [ $? -ne 0 ]
		then
		    bad "Failed! Check user permissions"
		    exit 1
	    fi
	else
	    # remove files which can be new in merak installation package
	    # but not changed by extracting from tar
	    
	    # scripts
	    rm -rf ${MERAKDIR}/scripts
	    rm -f ${MERAKDIR}/startd
	    rm -f ${MERAKDIR}/stopd
	    rm -f ${MERAKDIR}/restartd
    fi
    good "Extracting distribution"
    extract_image "${WORKDIR}/${IMAGENAME}" "$MERAKDIR"
    if [ $? -ne 0 ]
	then
	    bad "Failed, check user permissions and available disk space. 70MB disk space is needed."
	    exit 1
    fi
}

configure_package()
{
	if ( touch $MERAKCONF )
	    then 
		cp $MERAKCONF $MERAKCONF.bak
		DATE=`date`
		echo "## IceWarp Merak Mail Server configuration file
## this file was generated by IceWarp installer on ${DATE}
## InstallDir=/path/to/merak
## Merak installation prefix, absolute path, no useful default

InstallDir=\"${MERAKDIR}\"

## Version=number
## IceWarp Merak Mail Server version, for reference only

Version=\"${NEWVERSION}\"
 
## User=username
## User to run services as, default=root

User=\"${USER}\"
 
## Group=group
## Group to run services as, default=root

Group=\"${GROUP}\"

## Capabilities
## capabilities that are set at startup, see
## man cap_from_text
## for more info
## ADVANCED USERS ONLY!

Capabilities=\"\"

AVENGINE=\"${AVENGINE}\"

# Previous and new version number - used for upgrade purposes

OLDVERSION=\"${OLDVERSION}\"" >$MERAKCONF

	    else
		warn "File ${MERAKCONF} is not writable, skipping it."
	    fi

	    if ( touch ${MERAKDIR}/php/php.ini )
		then
		    echo "eaccelerator.cache_dir = \"${MERAKDIR}/php/tmp/\"
extension_dir = \"${MERAKDIR}/php/ext/\"
error_log = \"${MERAKDIR}/logs/phperror.log\"
open_basedir_ignore = \"${MERAKDIR}/\"
upload_tmp_dir = \"${MERAKDIR}/php/tmp/\"
session.save_path = \"${MERAKDIR}/php/tmp/\"" >>${MERAKDIR}/php/php.ini
		else
		    warn "File ${MERAKDIR}/php/php.ini is not writable, skipping it."
		    warn "Web interface may misbehave."
	    fi
}

run_postconf()
{
    # install links to .so libraries
    
    # add/remove merakd as service
    # this option is available only for root
    SERVICE_INSTALLED="0"
    MERAK_STARTED="0"
    if [ $UUID -eq 0 -a "$OS_PLATFORM" == "RHEL" ]
	then
	    # check if system service is installed
	    good "Checking if Merak Mail Server is added as system service ..."
	    good "Note: Merak Mail Server system service can be reinstalled"
	    good "      by removing already installed service."
	    good ""
	    service $SERVICE_NAME status > /dev/null
	    if [ $? -eq 0 ]
		then
		    SERVICE_INSTALLED="1"
		    good "Merak Mail Server is already added as system service."
		    ask_with_confirmation "Delete service?" "Y" "n"
		    if [ $? -eq 1 ]
			then
			    # remove merakd from service manager
			    good "Removing Merak Mail Server system service."
			    service $SERVICE_NAME stop > /dev/null
			    chkconfig --del $SERVICE_NAME > /dev/null
			    rm -f /etc/init.d/$SERVICE_NAME
			    SERVICE_INSTALLED="0"
		    fi
	    fi

	    # ask and add merakd as service
	    if [ "$SERVICE_INSTALLED" == "0" ]
		then
		    ask_with_confirmation "Do you want to add Merak Mail Server as a system service?" "Y" "n"
		    if [ $? -eq 1 ]
			then
			    good "Adding Merak Mail Server as system service"
			    # create new init script
			    sed "s|%%INSTALL_DIR_VALUE%%|$MERAKDIR|" $MERAKDIR/scripts/setup/$SERVICE_NAME.init > /etc/init.d/$SERVICE_NAME
			    if [ $? -ne 0 ]
				then
				    bad "Cannot create merakd init file."
				    exit 1
			    fi
			    chmod a+x /etc/init.d/$SERVICE_NAME > /dev/null
			    if [ $? -ne 0 ]
				then
				    bad "Cannot change permissions on init file."
				    exit 1
			    fi
			    SERVICE_INSTALLED="1"
			else
			    good "Merak Mail Server will not be added as system service."
		    fi
	    fi

	    # ask and add or remove service to autostart at boot time
	    if [ "$SERVICE_INSTALLED" == "1" ]
		then
		    ask_with_confirmation "Do you want Merak Mail Server to start automatically on reboot?" "Y" "n"
		    if [ $? -eq 1 ]
	    		then
	    		    good "Making service as autostart"
	    		    chkconfig --del $SERVICE_NAME > /dev/null
			    chkconfig --add $SERVICE_NAME > /dev/null
			    if [ $? -ne 0 ]
				then
				    bad "Cannot add service to autostart."
				    exit 1
			    fi
			else
			    good "Merak Mail Server will not start automatically on reboot."
			    chkconfig --del $SERVICE_NAME > /dev/null
		fi
	    fi
    fi

    # run upgrade tool if upgrading
    if [ "$UPGRADE" == "1" ]
	then
	    good ""
	    good "It is possible now to upgrade data to the current version."
	    good "To do this, all services have to be started."
	    good "Note: You can run upgrade process later by command"
	    good "      in $MERAKDIR: ./tool upgrade _old_version_"
	    good ""
	    ask_with_confirmation "Start services and perform the data upgrade now?" "Y" "n"
	    if [ $? -eq 1 ]
		then
		    good "Starting Merak service"
		    ${MERAKDIR}/merakd.sh --start > /dev/null
		    if [ $? -ne 0 ]
			then
			    bad "Cannot start Merak service"
			    exit 1
			else
			    MERAK_STARTED="1"
		    fi
		    ${MERAKDIR}/tool.sh upgrade "$OLD_VERSION" > /dev/null
		    if [ $? -eq 0 ]
			then
			    good "Data upgrade completed successfully"
			else
			    bad "Data upgrade problem!"
		    fi
	    fi
    fi
}

update_configuration_files()
{
    # do not upgrade configuration files when installing
    if [ "$UPGRADE" == "0" ]
	then
	    return 0
    fi

    good ""
    good "Upgrading configuration files:"
    
    # update webserver.dat
    CONFIG_FILE="${MERAKDIR}/config/webserver.dat"
    good $CONFIG_FILE
    sed "s|phpd<|phpd.sh<|" $CONFIG_FILE > "${CONFIG_FILE}_"
    rm -f $CONFIG_FILE
    mv "${CONFIG_FILE}_" $CONFIG_FILE

    return 0    
}

create_uninstall_info()
{
    # create list of all files in tar
    tar -tf "${WORKDIR}/${IMAGENAME}" > ${MERAKDIR}/scripts/setup/content.txt 2> /dev/null

    # split tar content to directories and files
    INSTALLED_DIRS=$(cat ${MERAKDIR}/scripts/setup/content.txt | grep ".*\/$")
    INSTALLED_FILES=$(cat ${MERAKDIR}/scripts/setup/content.txt | grep ".*[^\/]$")

    # create file with installed directories
    rm -f ${MERAKDIR}/scripts/setup/installed_dirs.txt
    touch ${MERAKDIR}/scripts/setup/installed_dirs.txt
    for LINE in $INSTALLED_DIRS
    do
	echo $LINE >> ${MERAKDIR}/scripts/setup/installed_dirs.txt
    done

    # create file with installed files
    # do not count configuration files which comes with installation
    rm -f ${MERAKDIR}/scripts/setup/installed_files.txt
    touch ${MERAKDIR}/scripts/setup/installed_files.txt
    for LINE in $INSTALLED_FILES
    do
	if  [ "$LINE" != "config/cert.pem" ] &&
	    [ "$LINE" != "config/webserver.dat" ] &&
	    [ "$LINE" != "spam/antispam.db" ] &&
	    [ "$LINE" != "spam/rules/local.cf" ] &&
	    [ "$LINE" != "calendar/groupware.db" ] &&
	    [ "$LINE" != "webmail/config/settings.dat" ]
	     then
		echo $LINE >> ${MERAKDIR}/scripts/setup/installed_files.txt
	fi
    done

    # add installed_dirs.txt file
    echo "scripts/setup/installed_dirs.txt" >> ${MERAKDIR}/scripts/setup/installed_files.txt
    # add installed_files.txt file
    echo "scripts/setup/installed_files.txt" >> ${MERAKDIR}/scripts/setup/installed_files.txt

    # remove content file
    rm -f ${MERAKDIR}/scripts/setup/content.txt
}

change_user_owner()
{
	# All files are installed OK, change permissions
    good "Changing permissions"
    chown -R $USER:$GROUP "$MERAKDIR" || bad "Failed! Check user permissions on ${MERAKDIR}"
    chmod +s "${MERAKDIR}/tool.sh"
    chmod +s "${MERAKDIR}/merakd.sh"
}

run_wizard()
{
    # run wizard if not upgrading
    if [ "$UPGRADE" == "0" ]
	then
	    good ""
	    ask_with_confirmation "Do you want to run Merak Mail Server Wizard now?" "Y" "n"
	    if [ $? -eq 1 ]
		then
		    if [ "$MERAK_STARTED" == "0" ]
			then
			    good "Starting Merak service"
			    ${MERAKDIR}/merakd.sh --start > /dev/null
			    if [ $? -ne 0 ]
				then
				    bad "Cannot start Merak service"
				    exit 1
			    fi
		    fi
		    ${MERAKDIR}/wizard.sh
		else
		    good "You can re-run wizard at a later time with script: $MERAKDIR/wizard.sh"
	    fi
    fi
}

copyright()
{
    good ""
    good "IceWarp Merak Mail Server installer"
    good "(c) 1999-2008 IceWarp Ltd. "
    good ""
}

rejoice()
{
    good ""
    good "IceWarp Merak Mail Server was successfully installed or upgraded."
    good ""
    good "Installer log is available in $INSTALLLOG"
    good "Installer error log is available in $ERRORLOG"
}

accept_license()
{
    less "${WORKDIR}/LICENSE"
    echo ""
    good "You must accept this license agreement if you want to continue."
    good "Press ENTER to accept license or CTRL+C to quit"
    read -s
    echo ""
}

getscriptparams()
{
    case $1 in
	"-h"|"--help")
	    copyright
	    good ""
	    good "Usage: $0"
	    good ""
	    good "This script provides functionality for installing or upgrading Merak."
	    good "Default action is to install but if the following conditions are met,"
	    good "only upgrade will be performed:"
	    good "  1. entered destination directory exists"
	    good "  2. file _ENTERED_DESTINATION_DIRECTORY_/config/merak.conf exists"
	    good ""
	    good "Install mode extracts all files and creates configuration files"
	    good "from default configuration file directories."
	    good ""
	    good "Upgrade mode:"
	    good "  1. replaces all files which are not configuration files"
	    good "     (i.e. files not modified by running Merak server)"
	    good "  2. recreates default configuration files that do not exists:"
	    good "       ./config/cert.pem"
	    good "       ./config/webserver.dat"
	    good "       ./spam/antispam.dat"
	    good "       ./spam/rules/local.cf"
	    good "       ./calendar/groupware.dat"
	    good "       ./webmail/config/settings.dat"
	    good ""
	    good "[END]"
	;;
	*)
	    accept_license
	    check_merakd_is_running
	    good "Installation/Upgrade mode entered"
	    good ""
	    ask_questions
	    confirm_params
	    check_entered_params
	    install_package
	    configure_package
	    update_configuration_files
	    run_postconf
	    create_uninstall_info
	    change_user_owner
	    run_wizard
	    rejoice
	;;
    esac
}

# here we go, let's have fun!

preconf

DATE=$(date)

echo "INSTALLATION STARTED ON $DATE" >$INSTALLLOG
echo "INSTALLATION STARTED ON $DATE" >$ERRORLOG
detect_os_platform
getscriptparams $* 2>>$ERRORLOG | tee -a $INSTALLLOG
echo "INSTALLATION FINISHED ON $DATE" >>$INSTALLLOG
echo "INSTALLATION FINISHED ON $DATE" >>$ERRORLOG

exit 0
