#!/bin/bash

# test if have functions included
if [ -z "$FUNCTIONS_INCLUDED" ]
    then
	echo "Script daemonlib.sh requires to have included functions.sh"
	exit 1
fi

# initial variables

# wait time in seconds before sending SIGKILL to daemon
MAX_WAIT_SECONDS="5"
# wait time in micro seconds == 1/5 second
WAIT_TIME="200000"
# max wait time in microseconds == 5 seconds
MAX_WAIT_TIME=$WAIT_TIME
let "MAX_WAIT_TIME*=${MAX_WAIT_SECONDS}*5"

DAEMON_DESC=""
DAEMON=""

# Function returns daemon description string according to name
# Parameters:
#   $1 - name of dameon
# Returns:
#   Sets variables DAEMON_DESC and DAEMON
#   always 0
#
get_daemon_desc()
{
    case "$1" in
	"merakd")
	    DAEMON_DESC="Merak Mail Server Service Helper Daemon"
	    DAEMON="merakd"
	    ;;
	"control")
	    DAEMON_DESC="Merak Mail Server Service Control Daemon"
	    DAEMON="control"
	    ;;
	"smtp")
	    DAEMON_DESC="Merak Mail Server SMTP Service Daemon"
	    DAEMON="smtp"
	    ;;
	"pop3"|"imap")
	    DAEMON_DESC="Merak Mail Server POP3/IMAP4 Service Daemon"
	    DAEMON="pop3"
	    ;;
	"im")
	    DAEMON_DESC="Merak Mail Server Messaging Service Daemon"
	    DAEMON="im"
	    ;;
	"cal"|"calendar")
	    DAEMON_DESC="Merak Mail Server GroupWare Service Daemon"
	    DAEMON="cal"
	    ;;
	"php"|"phpd")
	    DAEMON_DESC="Merak Mail Server PHP"
	    DAEMON="php"
	    ;;
	*)
	    DAEMON_DESC="Unknown daemon '$1'"
	    DAEMON="unknown"
	    return 0
	    ;;
    esac
    return 0
}

startdaemon()
{
    get_daemon_desc "$1"

    pushd $MERAKDIR &>/dev/null

    echo -n "Starting '${DAEMON_DESC}': "
    export LD_LIBRARY_PATH="${MERAK_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export MERAKDIR
    export PHP_FCGI_MAX_REQUESTS=1000
    export PHP_FCGI_CHILDREN=8

    ${MERAKDIR}/${DAEMON} --user=${MERAKUID} --group=${MERAKGID} --capabilities=${MERAKCAP} --installdir=${MERAKDIR} --noservices &> /dev/null
    if [ $? -eq 0 ]
	then
	    echo "OK"
	else
	    echo "FAIL"
    fi

    popd &>/dev/null
    return 0
}

stopdaemon()
{
    get_daemon_desc "$1"

    PID=$(cat "${MERAKDIR}/var/${DAEMON}.pid" 2> /dev/null)

    if [ -z "$PID" ]
	then
	    echo "Can not read PID for daemon ${DAEMON}. Exiting."
	    return 0
    fi

    echo "Shutting down '${DAEMON_DESC}'"
    echo "PID: ${PID}"
    WAITED="0"
    while [ true ]
    do
	kill $PID 2> /dev/null
	usleep $WAIT_TIME

	kill -0 $PID 2> /dev/null
	if [ $? -ne 0 ]
	    then
		echo "OK"
		break
	fi

	let "WAITED+=$WAIT_TIME"
	if [ "$WAITED" == "$MAX_WAIT_TIME" ]
	    then
		echo "Deamon did not exited after ${MAX_WAIT_SECONDS} seconds, killing it."
		kill -9 $PID
		break
	fi
    done

    rm -f "${MERAKDIR}/var/${DAEMON}.pid" 2> /dev/null
    
    # stop PHP when stopping control service
    if [ "$DAEMON" == "control" ]
	then
	    stopdaemon "php"
    fi
    return 0
}

startphp()
{
    pushd $MERAKDIR &> /dev/null

    export LD_LIBRARY_PATH="${MERAK_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export MERAKDIR
    export PHP_FCGI_MAX_REQUESTS=1000
    export PHP_FCGI_CHILDREN=8

    # need to save PID of php process
    ${MERAKDIR}/php/php -c ${MERAKDIR}/php/php.ini -b ${MERAKDIR}/var/phpsocket 2> ${MERAKDIR}/logs/phpstartup.log & echo $! > ${MERAKDIR}/var/php.pid

    popd &> /dev/null
    return 0
}
