#!/bin/bash
#
# IceWarp Merak Mail Server uninstall script
# (c) IceWarp Ltd. 2008
#
#       All rights reserved.
#
# http://www.icewarp.com
#
# file: uninstall.sh - uninstalling script
# version: 1.1
#

# init working dir
pushd `pwd` &> /dev/null
WORK_DIR=$(dirname $0)
cd $WORK_DIR &> /dev/null
if [ $? -ne 0 ]
    then
	echo "Can not change directory to $WORK_DIR"
	exit 1
fi

# include functions
if [ -z "$FUNCTIONS_INCLUDED" ]
    then
	source ./inc/functions.sh
fi

# start uninstall
uninstall_cancelled_quit()
{
    if [ -n "$1" ]
	then
	    echo "$1"
    fi
    echo "Uninstall cancelled."
    exit 1;
}

check_merakd_is_running
if [ $? -eq 0 ]
    then
	bad "Merak server is running."
	bad "Please stop Merak server before uninstalling."
	exit 1
fi

ask_with_confirmation "UNINSTALL MERAK MAILSERVER FROM $MERAKDIR?" "N" "y"
if [ $? -ne 2 ]
    then
	uninstall_cancelled_quit
fi

echo "Uninstalling Merak Mail Server..."

# load directories and files list
if [ -f $MERAKDIR/scripts/setup/installed_dirs.txt ]
    then
	INSTALLED_DIRS=$(cat $MERAKDIR/scripts/setup/installed_dirs.txt)
    else
	uninstall_cancelled_quit "Can not find istalled directories list file."
fi
if [ -f $MERAKDIR/scripts/setup/installed_files.txt ]
    then
	INSTALLED_FILES=$(cat $MERAKDIR/scripts/setup/installed_files.txt)
    else
	uninstall_cancelled_quit "Can not find istalled files list file."
	
fi

# change to merak directory
cd $MERAKDIR &> /dev/null
if [ $? -ne 0 ]
    then
	uninstall_cancelled_quit "Can not change directory to $MERAKDIR"
fi

# remove all installed files
for LINE in $INSTALLED_FILES
do
    rm -f $MERAKDIR/$LINE
done

# remove all installed directories
for LINE in $INSTALLED_DIRS
do
    rmdir -p $MERAKDIR/$LINE &> /dev/null
done

# remove top directory
cd "${MERAKDIR}/.."
rmdir $MERAKDIR
if [ -d $MERAKDIR ]
    then
	echo "Directory $MERAKDIR was not removed."
fi

echo
echo "Uninstall done."

# restore working dir
popd &> /dev/null
exit 0
