#!/bin/bash
#
# IceWarp Merak Mail Server
# (c) IceWarp Ltd. 2008
#
#       All rights reserved.
#
# http://www.icewarp.com
#
# file: license_operations.sh - wizard menu script
# version: 1.0
#

# include functions
if [ -z "$SCRIPTDIR" ]
    then
	echo "This wizard script requires to have \$SCRIPTDIR variable set."
	exit 1
fi

if [ -z "$FUNCTIONS_INCLUDED" ]
    then
	source ${SCRIPTDIR}/inc/functions.sh
fi

fnc_license_info()
{
    TMP_FILE=$(mktemp)
    run_cmd_print_stdout_log "${MERAKDIR}/tool.sh" display system c_license_xmlfilefull > ${TMP_FILE}
    # parse product
    
    TMP_IFS=$IFS
    IFS=$'\n'
    PRODUCTS_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<product/?>" | sed "s|.*<product[^>]*>\([^<]*\).*|\1|g")
    SUBPRODUCTS_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<subproduct/?>" | sed "s|.*<subproduct[^>]*>\([^<]*\).*|\1|g")
    DOMAINS_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<domains/?>" | sed "s|.*<domains[^>]*>\([^<]*\).*|\1|g")
    ACCOUNTS_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<accounts/?>" | sed "s|.*<accounts[^>]*>\([^<]*\).*|\1|g")
    EXPIRESAFTER_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<expiresafter/?>" | sed "s|.*<expiresafter[^>]*>\([^<]*\).*|\1|g")
    VALIDFOR_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<validfor/?>" | sed "s|.*<validfor[^>]*>\([^<]*\).*|\1|g")
    CREATIONDATE_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<creationdate/?>" | sed "s|.*<creationdate[^>]*>\([^<]*\).*|\1|g")
    DEMO_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<demo/?>" | sed "s|.*<demo[^>]*>\([^<]*\).*|\1|g")

    CUSTOMERNAME_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<customername/?>" | sed "s|.*<customername[^>]*>\([^<]*\).*|\1|g")
    CUSTOMEREMAIL_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<customeremail/?>" | sed "s|.*<customeremail[^>]*>\([^<]*\).*|\1|g")
    LICENSEHOLDER_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<licenseholder/?>" | sed "s|.*<licenseholder[^>]*>\([^<]*\).*|\1|g")
    PURCHASEID_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<purchaseid/?>" | sed "s|.*<purchaseid[^>]*>\([^<]*\).*|\1|g")
    PURCHASEDATE_STRING=$(cat ${TMP_FILE} | tr -d "\r" | grep -E "<purchasedate/?>" | sed "s|.*<purchasedate[^>]*>\([^<]*\).*|\1|g")

    PRODUCTS_ARRAY=($PRODUCTS_STRING)
    SUBPRODUCTS_ARRAY=($SUBPRODUCTS_STRING)
    DOMAINS_ARRAY=($DOMAINS_STRING)
    ACCOUNTS_ARRAY=($ACCOUNTS_STRING)
    EXPIRESAFTER_ARRAY=($EXPIRESAFTER_STRING)
    VALIDFOR_ARRAY=($VALIDFOR_STRING)
    CREATIONDATE_ARRAY=($CREATIONDATE_STRING)
    DEMO_ARRAY=($DEMO_STRING)

    NUM_PRODUCTS=${#PRODUCTS_ARRAY[*]}

    echo "Licenses information:"
    echo ""
    echo "Customer: ${CUSTOMERNAME_STRING}"
    echo "Customer email: ${CUSTOMEREMAIL_STRING}"
    echo "License holder: ${LICENSEHOLDER_STRING}"
    echo "Purchase ID: ${PURCHASEID_STRING}"
    echo "Purchase date: ${PURCHASEDATE_STRING}"
    echo ""
    echo "Product                                 Type           Creation date   License expires   Update expires   Domains   Accounts"
    echo "----------------------------------------------------------------------------------------------------------------------------"
    for IDX in $(seq 0 $((${NUM_PRODUCTS} - 1))); do
      if [ "${DOMAINS_ARRAY[$IDX]}" == "0" ]; then
        DOMAINS_ARRAY[$IDX]="Unlimited"
      fi
      if [ "${ACCOUNTS_ARRAY[$IDX]}" == "0" ]; then
        ACCOUNTS_ARRAY[$IDX]="Unlimited"
      fi
      if [ "${VALIDFOR_ARRAY[$IDX]}" == "0" ]; then
        VALIDFOR_ARRAY[$IDX]="Never"
      fi
      if [ "${DEMO_ARRAY[$IDX]}" == "1" ]; then
        DEMO_ARRAY[$IDX]="Evaluation"
        VALIDFOR_ARRAY[$IDX]=${EXPIRESAFTER_ARRAY[$IDX]}
      else
        DEMO_ARRAY[$IDX]="Registered"
      fi

      # compute creation date
      # Days between TDateTime basis (12/31/1899) and Unix time_t basis (1/1/1970)
      # UnixDateDelta = 25569
      CREATION_DATE=$((${CREATIONDATE_ARRAY[$IDX]} - 25569))
      CREATION_DATE=$(date -d "1970-01-01 ${CREATION_DATE} days" +"%Y-%m-%d")

      printf "%-20s%-20s%-15s%-16s%-18s%-17s%-10s%s\n" "${PRODUCTS_ARRAY[$IDX]}" "${SUBPRODUCTS_ARRAY[$IDX]}" "${DEMO_ARRAY[$IDX]}" "$CREATION_DATE" "${VALIDFOR_ARRAY[$IDX]}" "${EXPIRESAFTER_ARRAY[$IDX]}" "${DOMAINS_ARRAY[$IDX]}" "${ACCOUNTS_ARRAY[$IDX]}"
    done

    IFS=$TMP_IFS
    rm -rf $TMP_FILE

    return 0
}

fnc_license_display()
{
    good "XML formatted output follows:"
    echo
    run_cmd_print_stdout_log "${MERAKDIR}/tool" display system c_license
    if [ $? -ne 0 ]
	then
	    bad "License display problem."
	    return 1
    fi
    return 0
}

fnc_license_export()
{
    getparam "Enter file name to save exported license:"
    run_cmd_print_stdout_log "${MERAKDIR}/tool" display system c_license > $PARAM
    if [ $? -eq 0 ]
	then
	    good "License saved to file ${PARAM}"
	else
	    bad "License export problem."
	    return 1
    fi
    return 0
}

fnc_license_online()
{
    getparam "Enter order ID:"
    ORDER_ID=$PARAM
    run_cmd_log "${MERAKDIR}/tool" modify system c_onlinelicense "${ORDER_ID}"
    if [ $? -eq 0 ]
	then
	    good "Request submitted"
	else
	    bad "Request submit problem, try again later."
	    return 1
    fi
    return 0
}

fnc_license_refkey_display()
{
    run_cmd_print_stdout_log "${MERAKDIR}/tool" display system c_reference
    return 0
}

fnc_license_refkey_export()
{
    getparam "Enter file name to save exported reference key:"
    run_cmd_print_stdout_log "${MERAKDIR}/tool" display system c_reference > $PARAM
    if [ $? -eq 0 ]
	then
	    good "Reference key saved to file ${PARAM}"
	else
	    bad "Reference key query problem."
	    return 1
    fi
    return 0
}

SCRIPT_NAME=`basename $0`
good ""
case $SCRIPT_NAME in
    "license_info")
    fnc_license_info
    ;;
    "license_display")
	fnc_license_display
	;;
    "license_export")
	fnc_license_export
	;;
    "license_online")
	fnc_license_online
	;;
    "license_refkey_display")
	fnc_license_refkey_display
	;;
    "license_refkey_export")
	fnc_license_refkey_export
	;;
    *)
	echo "Script called for unknown operation: ${SCRIPT_NAME}"
	exit 1
	;;
esac

exit 0

